/*
 * arch/arm/mach-tcc93xx/board-tcc9300.c
 *
 * Copyright (C) 2010 Telechips, Inc.
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <linux/kernel.h>
#include <linux/init.h>
#include <linux/platform_device.h>
#include <linux/proc_fs.h>
#include <linux/delay.h>

#include <mach/gpio.h>
#include <linux/spi/spi.h>
#include <linux/i2c.h>
#include <linux/i2c/pca953x.h>

#include <asm/mach-types.h>
#include <asm/mach/arch.h>
#include <asm/mach/map.h>
#include <asm/mach/flash.h>

#include <linux/android_pmem.h>

#include <bsp.h>
#include <mach/common.h>
#include <mach/tcc_used_mem.h>

#include "board-tcc9300.h"

#include "devices.h"
#if defined(CONFIG_TCC_SENSOR_DRV)
#include <linux/i2c/sensor_i2c.h>
#endif
#if defined(CONFIG_TCC_OUTPUT_STARTER)
#include <linux/i2c/hdmi_phy.h>
#include <linux/i2c/hdmi_edid.h>
#endif
#if defined(CONFIG_FB_TCC_COMPONENT)
#include <linux/i2c/cs4954.h>
#endif

#if defined(CONFIG_VIDEO_TCCXX_CAMERA) //20100720 ysseung   add to sensor slave id.
#include <media/cam_i2c.h>
#if defined(CONFIG_VIDEO_CAMERA_SENSOR_AIT848_ISP)
#define SENSOR_I2C_SLAVE_ID 		(0x06>>1)
#elif defined(CONFIG_VIDEO_CAMERA_SENSOR_MT9P111)
#define SENSOR_I2C_SLAVE_ID 		(0x78>>1)
#elif defined(CONFIG_VIDEO_CAMERA_SENSOR_MV9317)
#define SENSOR_I2C_SLAVE_ID 		(0x50>>1)
#elif defined(CONFIG_VIDEO_CAMERA_SENSOR_MT9D112)
#define SENSOR_I2C_SLAVE_ID 		(0x7A>>1)
#elif defined(CONFIG_VIDEO_CAMERA_SENSOR_OV3640)
#define SENSOR_I2C_SLAVE_ID 		(0x78>>1)
#elif defined(CONFIG_VIDEO_CAMERA_SENSOR_S5K4BAFB)
#define SENSOR_I2C_SLAVE_ID 		(0x52>>1)
#elif defined(CONFIG_VIDEO_CAMERA_SENSOR_ISX006)
#define SENSOR_I2C_SLAVE_ID 		(0x34>>1)
#elif defined(CONFIG_VIDEO_CAMERA_SENSOR_OV7690)
#define SENSOR_I2C_SLAVE_ID 		(0x42>>1)
#elif defined(CONFIG_VIDEO_ATV_SENSOR_TVP5150)
#define SENSOR_I2C_SLAVE_ID 		(0xB8>>1)
#elif defined(CONFIG_VIDEO_ATV_SENSOR_RDA5888)
#define SENSOR_I2C_SLAVE_ID 		(0xC4>>1)
#endif
#endif // defined(CONFIG_VIDEO_TCCXX_CAMERA)

#if defined(CONFIG_TCC_OUTPUT_STARTER)
#define HDMI_PHY_I2C_SLAVE_ID 		(0x70>>1)
#define HDMI_EDID_I2C_SLAVE_ID 		(0xA0>>1)
#define HDMI_EDID_SEG_I2C_SLAVE_ID	(0x60>>1)
#endif // defined(CONFIG_TCC_OUTPUT_STARTER)

#if defined(CONFIG_FB_TCC_COMPONENT)
#define CS4954_I2C_SLAVE_ID 		(0x08>>1)
#define THS8200_I2C_SLAVE_ID 		(0x40>>1)
#endif // defined(CONFIG_FB_TCC_COMPONENT)


extern void __init tcc9300_irq_init(void);
extern void __init tcc9300_map_common_io(void);


static struct spi_board_info tcc9300_spi0_board_info[] = {
	{
		.modalias = "spidev",
		.bus_num = 0,					// spi channel 0
		.chip_select = 0,
		/* you can modify the following member variables */
		.max_speed_hz = 2*1000*1000,	// default 2Mhz
		.mode = 0						// default 0, you can choose [SPI_CPOL|SPI_CPHA|SPI_CS_HIGH|SPI_LSB_FIRST]
	},
};

static struct spi_board_info tcc9300_spi1_board_info[] = {
	{
		.modalias = "spidev",
		.bus_num = 1,					// spi channel 1
		.chip_select = 0,
		/* you can modify the following member variables */
		.max_speed_hz = 2*1000*1000,	// default 2Mhz
		.mode = 0						// default 0, you can choose [SPI_CPOL|SPI_CPHA|SPI_CS_HIGH|SPI_LSB_FIRST]
	},
};

#if defined(CONFIG_MACH_TCC9300)
static struct pca953x_platform_data pca9539_data1 = {
	.gpio_base	= GPIO_PORTEXT1,
};

static struct pca953x_platform_data pca9539_data2 = {
	.gpio_base	= GPIO_PORTEXT2,
};

static struct pca953x_platform_data pca9539_data3 = {
	.gpio_base	= GPIO_PORTEXT3,
};

static struct pca953x_platform_data pca9539_data4 = {
	.gpio_base	= GPIO_PORTEXT14,
};
#elif defined(CONFIG_MACH_TCC9300CM)
static struct pca953x_platform_data pca9539_data1 = {
	.gpio_base	= GPIO_PORTEXT1,
};

static struct pca953x_platform_data pca9539_data2 = {
	.gpio_base	= GPIO_PORTEXT2,
};

static struct pca953x_platform_data pca9538_data3 = {
	.gpio_base	= GPIO_PORTEXT3,
};
#endif

#if defined(CONFIG_VIDEO_TCCXX_CAMERA)
static struct cam_i2c_platform_data cam_i2c_data1 = {
};
#endif

#if defined(CONFIG_TCC_SENSOR_DRV)
static struct sensor_i2c_platform_data  sensor_i2c_data1 = {
};
#endif

#if defined(CONFIG_TCC_OUTPUT_STARTER)
static struct tcc_hdmi_phy_i2c_platform_data  hdmi_phy_i2c_data = {
};
static struct tcc_hdmi_phy_i2c_platform_data  hdmi_edid_i2c_data = {
};
static struct tcc_hdmi_phy_i2c_platform_data  hdmi_edid_seg_i2c_data = {
};
#endif

#if defined(CONFIG_FB_TCC_COMPONENT)
static struct cs4954_i2c_platform_data  cs4954_i2c_data = {
};
static struct cs4954_i2c_platform_data  ths8200_i2c_data = {
};
#endif

#if defined(CONFIG_TCC_OUTPUT_STARTER)
/* I2C core0 channel 0 devices */
static struct i2c_board_info __initdata i2c_devices0[] = {
    {
		I2C_BOARD_INFO("tcc-hdmi-edid", HDMI_EDID_I2C_SLAVE_ID),
		.platform_data = &hdmi_edid_i2c_data,
	},	
    {
		I2C_BOARD_INFO("tcc-hdmi-edid-seg", HDMI_EDID_SEG_I2C_SLAVE_ID),
		.platform_data = &hdmi_edid_seg_i2c_data,
	},	
};
#endif

/* I2C core0 channel 1 devices */
static struct i2c_board_info __initdata i2c_devices1[] = {
	#if defined(CONFIG_MACH_TCC9300ST) && defined(CONFIG_FB_TCC_COMPONENT)
	{
		I2C_BOARD_INFO("component-cs4954", CS4954_I2C_SLAVE_ID),
		.platform_data = &cs4954_i2c_data,
	},
	{
		I2C_BOARD_INFO("component-ths8200", THS8200_I2C_SLAVE_ID),
		.platform_data = &ths8200_i2c_data,
	},
	#endif
	#if defined(CONFIG_MACH_TCC9300) && defined(CONFIG_VIDEO_TCCXX_CAMERA)
	{
		I2C_BOARD_INFO("tcc-cam-sensor", SENSOR_I2C_SLAVE_ID), //20100716 ysseung   sign-up to sensor slave-id.
		.platform_data = &cam_i2c_data1,
	},
	#endif
};

#if defined(CONFIG_MACH_TCC9300)
/* I2C core1 channel 0 devices */
static struct i2c_board_info __initdata i2c_devices2[] = {
	{
		I2C_BOARD_INFO("pca9539", 0x74),
		.platform_data = &pca9539_data1,
	},
	{
		I2C_BOARD_INFO("pca9539", 0x77),
		.platform_data = &pca9539_data2,
	},
	{
		I2C_BOARD_INFO("pca9539", 0x75),
		.platform_data = &pca9539_data3,
	},
	{
		I2C_BOARD_INFO("pca9538", 0x70),
		.platform_data = &pca9539_data4,
	},	
};
#endif

#if defined (CONFIG_TCC_SENSOR_DRV)
/* I2C core1 channel 1 devices */
static struct i2c_board_info __initdata i2c_devices3[] = {
	{
		I2C_BOARD_INFO("tcc-accel-sensor", 0x0B),
		.platform_data = &sensor_i2c_data1,
	},
};
#endif

/* I2C core2 channel 0 devices */
static struct i2c_board_info __initdata i2c_devices4[] = {
#if defined(CONFIG_MACH_TCC9300CM)
	{
		I2C_BOARD_INFO("pca9539", 0x77),
		.platform_data = &pca9539_data1,
	},
	{
		I2C_BOARD_INFO("pca9539", 0x74),
		.platform_data = &pca9539_data2,
	},
	{
		I2C_BOARD_INFO("pca9538", 0x70),
		.platform_data = &pca9538_data3,
	},
	#if defined(CONFIG_FB_TCC_COMPONENT)
	{
		I2C_BOARD_INFO("component-cs4954", CS4954_I2C_SLAVE_ID),
		.platform_data = &cs4954_i2c_data,
	},	
	{
		I2C_BOARD_INFO("component-ths8200", THS8200_I2C_SLAVE_ID),
		.platform_data = &ths8200_i2c_data,
	},	
	#endif
	#if defined(CONFIG_VIDEO_TCCXX_CAMERA)
	{
		I2C_BOARD_INFO("tcc-cam-sensor", SENSOR_I2C_SLAVE_ID), //20100716 ysseung   sign-up to sensor slave-id.
		.platform_data = &cam_i2c_data1,
	},
	#endif
#endif
};

#if defined(CONFIG_TCC_OUTPUT_STARTER)
/* I2C SMU HDMI PHY devices */
static struct i2c_board_info __initdata i2c_devices7[] = {
	{
		I2C_BOARD_INFO("tcc-hdmi-phy", HDMI_PHY_I2C_SLAVE_ID),
		.platform_data = &hdmi_phy_i2c_data,
	},	
};
#endif

static void __init tcc9300_map_io(void)
{
    tcc9300_map_common_io();
}

static void __init tcc9300_init_irq(void)
{
	tcc9300_irq_init();
}

static struct platform_device *tcc9300_devices[] __initdata = {
#ifdef CONFIG_I2C_TCC_CORE0
	&tcc9300_i2c_core0_device,
#endif
#ifdef CONFIG_I2C_TCC_CORE1
	&tcc9300_i2c_core1_device,
#endif
#ifdef CONFIG_I2C_TCC_CORE2
	&tcc9300_i2c_core2_device,
#endif
#ifdef CONFIG_I2C_TCC_SMU
	&tcc9300_i2c_smu_device,
#endif
#ifdef CONFIG_MTD_NAND_TCC93XX
	&tcc_nand_device,
#endif
#ifdef CONFIG_MMC_TCC_SDHC_CORE0
	&tcc9300_sdhc0_device,
#endif
#ifdef CONFIG_MMC_TCC_SDHC_CORE1
	&tcc9300_sdhc1_device,
#endif
#ifdef CONFIG_MMC_TCC_SDHC_CORE2
	&tcc9300_sdhc2_device,
#endif
#ifdef CONFIG_MMC_TCC_SDHC_CORE3
	&tcc9300_sdhc3_device,
#endif
};

static void tcc9300_add_pmem_devices(void)
{
	pmem_pdata.start = PMEM_SURF_BASE;
	pmem_pdata.size = PMEM_SURF_SIZE;
	platform_device_register(&pmem_device);
	
	pmem_adsp_pdata.start = PMEM_CAM_BASE;
	pmem_adsp_pdata.size = PMEM_CAM_SIZE;
	platform_device_register(&pmem_adsp_device);
}

static struct proc_dir_entry *phys_mem_info;

static inline int phys_mem_proc_info(char *buf, const char *name,
			    unsigned int base_addr, unsigned int size)
{
	return sprintf(buf, "0x%8.8x 0x%8.8x %s\n", base_addr, size, name);
}

static int phys_mem_read_proc(char *page, char **start, off_t off,
			      int count, int *eof, void *data)
{
	int len;

	len = sprintf(page, "%-10s %-10s %s\n", "base_addr", "size", "name");
	len += phys_mem_proc_info(page + len, "pmem",
				  PMEM_SURF_BASE, PMEM_SURF_SIZE);
	len += phys_mem_proc_info(page + len, "viqe",
				  PMEM_VIQE_BASE, PMEM_VIQE_SIZE);
	len += phys_mem_proc_info(page + len, "camera",
				  PMEM_CAM_BASE, PMEM_CAM_SIZE);
	len += phys_mem_proc_info(page + len, "overlay",
				  OVERLAY_MEM_BASE, OVERLAY_MEM_TOTAL_SIZE);
	len += phys_mem_proc_info(page + len, "video",
				  VIDEO_MEM_BASE, VIDEO_MEM_SIZE);
	len += phys_mem_proc_info(page + len, "fb",
				  FB_MEM_BASE, FB_MEM_TOTAL_SIZE);
	len += phys_mem_proc_info(page + len, "jpeg_header",
				  PA_JPEG_HEADER_BASE_ADDR,
				  TCC_JPEG_HEADER_SIZE);
	len += phys_mem_proc_info(page + len, "jpeg_raw",
				  PA_JPEG_RAW_BASE_ADDR, JPEG_RAW_MEM_SIZE);
	len += phys_mem_proc_info(page + len, "jpeg_stream",
				  PA_JPEG_STREAM_BASE_ADDR,
				  JPEG_STREAM_MEM_SIZE);
	return len;
}

static void __init tcc9300_init_machine(void)
{
	phys_mem_info = create_proc_entry("phys_mem", 0444, NULL);
	if (phys_mem_info) {
		phys_mem_info->read_proc = phys_mem_read_proc;
	}

	#if defined(CONFIG_SPI_TCC9300_MASTER) || defined(CONFIG_SPI_TCC9300_MASTER_MODULE)
	spi_register_board_info(tcc9300_spi0_board_info, ARRAY_SIZE(tcc9300_spi0_board_info));
	//spi_register_board_info(tcc9300_spi1_board_info, ARRAY_SIZE(tcc9300_spi1_board_info)); //jhlim
	#endif

	#if defined(CONFIG_TCC_OUTPUT_STARTER)
	i2c_register_board_info(0, i2c_devices0, ARRAY_SIZE(i2c_devices0));
	#endif
	
	#if defined(CONFIG_MACH_TCC9300)
	i2c_register_board_info(2, i2c_devices2, ARRAY_SIZE(i2c_devices2));
	#endif

	#if defined (CONFIG_TCC_SENSOR_DRV)
	i2c_register_board_info(3, i2c_devices3, ARRAY_SIZE(i2c_devices3));
	#endif

	i2c_register_board_info(1, i2c_devices1, ARRAY_SIZE(i2c_devices1));

	i2c_register_board_info(4, i2c_devices4, ARRAY_SIZE(i2c_devices4));

	#if defined(CONFIG_TCC_OUTPUT_STARTER)
	i2c_register_board_info(7, i2c_devices7, ARRAY_SIZE(i2c_devices7));
	#endif

	tcc9300_add_pmem_devices();

	platform_add_devices(tcc9300_devices, ARRAY_SIZE(tcc9300_devices));
}

MACHINE_START(TCC9300, "Telechips TCC9300 Demo Board")
    /* Maintainer: Telechips Linux BSP Team <linux@telechips.com> */
    .phys_io        = 0xb0000000,
    .io_pg_offst    = ((0xf0000000) >> 18) & 0xfffc,
    .boot_params    = PHYS_OFFSET + 0x00000100,
    .map_io         = tcc9300_map_io,
    .init_irq       = tcc9300_init_irq,
    .init_machine   = tcc9300_init_machine,
    .timer          = &tcc9300_timer,
MACHINE_END
