/*
 * arch/arm/mach-tcc9300/pm.h
 *
 * Author:  <linux@telechips.com>
 * Created: April 21, 2008
 * Description: LINUX POWER MANAGEMENT FUNCTIONS
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */
#ifndef __TCC_PM_H__
#define __TCC_PM_H__

/*===========================================================================

                                  MODE

===========================================================================*/

#if defined(CONFIG_SUSPEND_MODE)
#define TCC_PM_SUSPEND_MODE
#elif defined(CONFIG_SLEEP_MODE)
#define TCC_PM_SLEEP_MODE
#endif

/*===========================================================================

                            Suspend/Sleep MAP

===========================================================================*/

#ifdef TCC_PM_SUSPEND_MODE
/*---------------------------------------------------------------------------
 1) suspend (shut down + backup ram boot + sdram self-refresh)

     << sram1 >>
     0xD0000000(0xEFE00000) ------------------
                           |  Enter Shutdown  | 0x400
                     0x400  ------------------
                           |      Stack       | 0x100
                     0x500  ------------------

     << backup ram >>
     0xD8000000(0xEFC00000) ------------------
                           |    Boot code     | 0x400
                     0x400  ------------------
                           |   I/O restore    | 0x200
                     0x600  ------------------
                           |   SDRAM Init     | 0x400
                     0xA00  ------------------
                           | GPIO Backup Data | 0x300
                     0xD00  ------------------

---------------------------------------------------------------------------*/

// SRAM1
#define SHUTDOWN_FUNC_ADDR       0xEFE00000
#define SHUTDOWN_FUNC_SIZE       0x00000400
#define SHUTDOWN_FUNC_PHY        0xD0000000

// Backup RAM
#define BACKUP_RAM_BOOT_ADDR     0xEFC00000
#define BACKUP_RAM_BOOT_SIZE     0x00000400

#define IO_RESTORE_FUNC_ADDR     0xEFC00400
#define IO_RESTORE_FUNC_SIZE     0x00000200

#define SDRAM_INIT_FUNC_ADDR     0xEFC00600
#define SDRAM_INIT_FUNC_SIZE     0x00000400
#define SDRAM_INIT_FUNC_PHY      0xD8000600

#define GPIO_BUF_ADDR            0xEFC00A00
#define GPIO_BUF_PHY             0xD8000A00

/*-------------------------------------------------------------------------*/
#endif

#ifdef TCC_PM_SLEEP_MODE
/*---------------------------------------------------------------------------
 2) sleep (sleep + sdram self-refresh)

     << sram1 >>
     0xD0000000(0xEFE00000) ------------------
                           |   Enter Sleep    | 0x400
                     0x400  ------------------
                           |      Stack       | 0x100
                     0x500  ------------------
                           |   SDRAM Init     | 0x400
                     0x900  ------------------

---------------------------------------------------------------------------*/

// SRAM1
#define SLEEP_FUNC_ADDR          0xEFE00000
#define SLEEP_FUNC_SIZE          0x00000400

#define SDRAM_INIT_FUNC_ADDR     0xEFE00500
#define SDRAM_INIT_FUNC_SIZE     0x00000400

/*-------------------------------------------------------------------------*/
#endif

#ifndef hword_of
#define	hword_of(X)					( *(volatile unsigned short *)((X)) )
#endif

#ifndef byte_of
#define	byte_of(X)					( *(volatile unsigned char *)((X)) )
#endif
#ifndef word_of
#define	word_of(X)					( *(volatile unsigned int *)((X)) )
#endif


#ifdef TCC_PM_SUSPEND_MODE
/*===========================================================================

                             Suspend Mask

===========================================================================*/

#define BSP_SUSPEND_MASK  0x424E4654 /* "TFNB" */

#define DRAM_PHYS_ADDRESS 0x4FF00000
#define DRAM_VIRT_ADDRESS 0x4FF00004
#define DRAM_DATA_ADDRESS 0x4FF0000C



/*===========================================================================

                        Suspend Backup Registers

===========================================================================*/

typedef struct _TCC_REG_{
	unsigned int uMASK;					// 0x00
	
	unsigned int SleepState_WakeAddr;	// 0x04
	
	unsigned int SleepState_SYSCTL;		// 0x08
	unsigned int SleepState_MMUTTB0;	// 0x0C
	unsigned int SleepState_MMUTTB1;	// 0x10
	unsigned int SleepState_MMUTTBCTL;	// 0x14
	unsigned int SleepState_MMUDOMAIN;	// 0x18

	unsigned int SleepState_SVC_SP;		// 0x1C
	unsigned int SleepState_SVC_SPSR;	// 0x20
	unsigned int SleepState_FIQ_SPSR;	// 0x24
	unsigned int SleepState_FIQ_R8;		// 0x28
	unsigned int SleepState_FIQ_R9;		// 0x2C
	unsigned int SleepState_FIQ_R10;	// 0x30
	unsigned int SleepState_FIQ_R11;	// 0x34
	unsigned int SleepState_FIQ_R12;	// 0x38
	unsigned int SleepState_FIQ_SP;		// 0x3C
	unsigned int SleepState_FIQ_LR;		// 0x40
	unsigned int SleepState_ABT_SPSR;	// 0x44
	unsigned int SleepState_ABT_SP;		// 0x48
	unsigned int SleepState_ABT_LR;		// 0x4C
	unsigned int SleepState_IRQ_SPSR;	// 0x50
	unsigned int SleepState_IRQ_SP;		// 0x54
	unsigned int SleepState_IRQ_LR;		// 0x58
	unsigned int SleepState_UND_SPSR;	// 0x5C
	unsigned int SleepState_UND_SP;		// 0x60
	unsigned int SleepState_UND_LR;		// 0x64
	unsigned int SleepState_SYS_SP;		// 0x68
	unsigned int SleepState_SYS_LR;		// 0x70

	unsigned int SleepState_SVC_LR;		// 0x7C
	unsigned int temp;					// 0x80
	
	CKC ckc;
	PIC pic;
	VIC vic;
	TIMER timer;
	PMU pmu;
	SMUCONFIG smuconfig;

	//GPIO gpio;

	I2CMASTER i2c0_0;
	I2CMASTER i2c0_1;
	I2CSLAVE	i2c0_slave;
	//I2CSTATUS i2c0_status;
	
	I2CMASTER i2c1_0;
	I2CMASTER i2c1_1;
	I2CSLAVE	i2c1_slave;
	//I2CSTATUS i2c1_status;
	
	I2CMASTER i2c2_0;
	I2CMASTER i2c2_1;
	I2CSLAVE	i2c2_slave;
	//I2CSTATUS i2c2_status;
	
	SMUI2CMASTER smui2c;
	SMUI2CMASTER smui2c_ch1;
	SMUI2CICLK smui2cclk;
	
    ADMA adma0;
    ADMADAI adma0_dai;
    ADMASPDIFTX adma0_spdiftx;
    
	char		UART0[sizeof(UART)];
	UARTPORTMUX uartportmux;
	TSADC tsadc;
	IOBUSCFG iobuscfg;
//	HSIOBUSCFG hsiobuscfg;
	NFC nfc;

	volatile unsigned int backup_peri_iobus0;
	volatile unsigned int backup_peri_iobus1;
} TCC_REG, *PTCC_REG;
#endif /* TCC_PM_SUSPEND_MODE */

#endif  /*__TCC_PM_H__*/
