
/****************************************************************************
 *   FileName    : tca_fb_output.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif
#include <linux/cpufreq.h>
#include <mach/bsp.h>
#include <mach/tca_ckc.h>

#ifdef __KERNEL__
#	if defined(_TCC79x_)
#		include <mach/tcc79x.h>
#	elif defined(_TCC83x_)
#		include <mach/tcc83x_virt.h>
#	endif
#else
#	if defined(_TCC79x_)
#		include "TCC79x_Virtual.h"
#	elif defined(_TCC83x_)
#		include "TCC83x_Virtual.h"
#	endif
#endif


#include <mach/tcc92xxfb_ioctrl.h>
#include <mach/tcc_grp_ioctrl.h>
#include <mach/tcc_scaler_ctrl.h>
#include <mach/tcc_used_mem.h>
#include <mach/tca_fb_output.h>
#include <mach/tcc_composite_ioctl.h>
#include <mach/tcc_component_ioctl.h>


#if 0
#define dprintk(msg...)	 { printk( "tca_fb_output: " msg); }
#else
#define dprintk(msg...)	 
#endif

//#define FB_OUTPUT_TEST

char M2M_Scaler1_Ctrl_Detail(SCALER_TYPE *scale_img);
void grp_copyblit_ctrl(G2D_BITBLIT_TYPE *g2d_p);
void grp_rotate_ctrl(G2D_BITBLIT_TYPE *g2d_p);

static char *fb_scaler_pbuf0;
static char *fb_scaler_pbuf1;
static char *fb_g2d_pbuf0;
static char *fb_g2d_pbuf1;

#if defined(CONFIG_TCC_OUTPUT_STARTER) && defined(USE_DUALOUTPUT) && defined(USE_DUALBUFFER)
static char *fb_scaler_pbuf2;
static char *fb_scaler_pbuf3;
static char *fb_g2d_pbuf2;
static char *fb_g2d_pbuf3;
#endif

/*****************************************************************************
*
* Defines
*
******************************************************************************/
//#define TCC_FB_UPSCALE

/*****************************************************************************
*
* structures
*
******************************************************************************/
typedef enum{
	M2M_YUV422SEQ0,
	M2M_YUV422SEQ1,
	M2M_YUV422SEP,
	M2M_YUV420SEP,
	M2M_RGB565,
	M2M_RGB555,
	M2M_RGB444,
	M2M_RGB454		
} M2M_FORMAT_TYPE;

typedef struct{
	unsigned char 	intlv_mode; // interleaved mode cb/cr order => cr/cb : 0, cb/cr : 1
	unsigned char 	intlv;	  // YUV type => interleaved : 1, seperate : 0
	M2M_FORMAT_TYPE ImgFormat;	  // Only YUV420/422 format in case of Interleaved configuration!!
}M2M_FORMAT_Info;

typedef struct {
	unsigned int  pSrcBase0;
	unsigned int  pSrcBase1;
	unsigned int  pSrcBase2;
	unsigned int  SrcHsize;
	unsigned int  SrcVsize;
	unsigned int  DstHsize;
	unsigned int  DstVsize;
	unsigned int  pDstBase0;
	unsigned int  pDstBase1;
	unsigned int  pDstBase2;
	M2M_FORMAT_Info SrcImgType;
	M2M_FORMAT_Info DstImgType;
	unsigned int  Hsize_Offset;
	unsigned int  Y_Offset;
	unsigned int  UV_Offset;
	unsigned char  PathType;  // direct_path  1  , Memmory type : 0  //  M2M direct_path
} M2M_SCALER_INFO;

/*****************************************************************************
*
* Variables
*
******************************************************************************/

/*****************************************************************************
*
* Functions
*
******************************************************************************/

static PLCDC pLCDC_OUTPUT[TCC_OUTPUT_MAX] = {NULL, NULL};

static struct clk *scaler1_clk;	 	// video image scale
static struct clk *scaler0_clk;		// fb image scale
static struct clk *lcdc0_output_clk;
static struct clk *lcdc1_output_clk;
static struct clk *g2d_clk;

static char output_lcdc[TCC_OUTPUT_MAX];
static char buf_index = 0;
static unsigned int FBimg_buf_addr;
static unsigned int output_layer_ctrl[TCC_OUTPUT_MAX] = {0, 0};

void TCC_OUTPUT_LCDC_Init(void)
{
	lcdc0_output_clk = clk_get(0, "lcdc0");
	BUG_ON(lcdc0_output_clk == NULL);

	lcdc1_output_clk = clk_get(0, "lcdc1");
	BUG_ON(lcdc1_output_clk == NULL);

	scaler1_clk = clk_get(0, "m2m1");
	BUG_ON(scaler1_clk == NULL);	

	scaler0_clk = clk_get(0, "m2m0");
	BUG_ON(scaler0_clk == NULL);	

	g2d_clk = clk_get(NULL, "overlay");
	BUG_ON(g2d_clk == NULL);

	fb_scaler_pbuf0 = (char *)FB_SCALE_ADDR0;
	fb_scaler_pbuf1 = (char *)FB_SCALE_ADDR1;
	fb_g2d_pbuf0 = (char *)FB_G2D_ADDR0;
	fb_g2d_pbuf1 = (char *)FB_G2D_ADDR1;

	#if defined(CONFIG_TCC_OUTPUT_STARTER) && defined(USE_DUALOUTPUT) && defined(USE_DUALBUFFER)
	fb_scaler_pbuf2 = (char *)FB_SCALE_ADDR2;
	fb_scaler_pbuf3 = (char *)FB_SCALE_ADDR3;
	fb_g2d_pbuf2 = (char *)FB_G2D_ADDR2;
	fb_g2d_pbuf3 = (char *)FB_G2D_ADDR3;
	#endif
}

void TCC_OUTPUT_LCDC_OnOff(char output_type, char output_lcdc_num, char onoff)
{
	dprintk(" %s output_lcdc_num:%d onoff:%d  \n", __func__, output_lcdc_num, onoff);

	if(onoff)
	{
		clk_enable(scaler0_clk);
		M2M_Scaler_SW_Reset(M2M_SCALER0);
		clk_enable(scaler1_clk);
		clk_enable(g2d_clk);

		if(output_lcdc_num)
		{						
			pLCDC_OUTPUT[output_type] = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
			pLCDC_OUTPUT[output_type]->LCTRL = 0;
			pLCDC_OUTPUT[output_type]->LI0C = 0;
			pLCDC_OUTPUT[output_type]->LI2C = 0;
		}
		else
		{
			pLCDC_OUTPUT[output_type] = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
			pLCDC_OUTPUT[output_type]->LCTRL = 0;
			pLCDC_OUTPUT[output_type]->LI0C = 0;
		}
		output_lcdc[output_type] = output_lcdc_num;
	}
	else
	{
		pLCDC_OUTPUT[output_type] = NULL;

		clk_disable(scaler1_clk);
		clk_disable(scaler0_clk);
		clk_disable(g2d_clk);
	}

	output_layer_ctrl[output_type] = 0;
}

void TCC_OUTPUT_LCDC_CtrlLayer(char output_type, char interlace, char format)
{
	dprintk(" %s interlace:%d format:%d  \n", __func__, interlace, format);
	
	output_layer_ctrl[output_type] = HwLIC_IEN | HwLIC_CEN | format;

	if(interlace)
		output_layer_ctrl[output_type] |= HwLIC_INTL;
}

void TCC_OUTPUT_FB_Update(unsigned int width, unsigned int height, unsigned int bits_per_pixel, unsigned int addr, unsigned int type)
{
	unsigned int regl = 0, g2d_rotate_need = 0, g2d_format_need = 0;
	unsigned int lcd_width = 0, lcd_height = 0, lcd_h_pos = 0, lcd_w_pos = 0, scale_x = 0, scale_y = 0;
	unsigned int img_width = 0, img_height = 0;
	unsigned int chromaR = 0, chromaG = 0, chromaB = 0;
	unsigned int max_width = 0, max_height = 0;
	PLCDC_CHANNEL pLCDC_OUTPUT_FB_CH;

	G2D_BITBLIT_TYPE g2d_p;
	SCALER_TYPE fbscaler;

	if(pLCDC_OUTPUT[type] == NULL)
	{
		dprintk("%s - Err: Output LCDC is not valid, type=%d\n", __func__, type);
		return;
	}

	memset(&g2d_p, 0x00, sizeof(G2D_BITBLIT_TYPE));
	memset(&fbscaler, 0x00, sizeof(SCALER_TYPE));

	dprintk(" %s width=%d, height=%d, bpp=%d, type=%d\n", __func__, width, height, bits_per_pixel, type);
	
	FBimg_buf_addr = addr;

	regl = pLCDC_OUTPUT[type]->LDS; // get LCD size 

	lcd_width = (regl & 0xFFFF);
	lcd_height = ((regl>>16) & 0xFFFF);

	if((!lcd_width) || (!lcd_height))
		return;

	chromaR = chromaG = chromaB = 0x00;

	if(output_lcdc[type])
		pLCDC_OUTPUT_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_OUTPUT[type]->LI2C);
	else
		pLCDC_OUTPUT_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_OUTPUT[type]->LI0C);

	if(width < height)
	{
		g2d_rotate_need = 1;
	}

	if(bits_per_pixel == 32)
	{
		if(output_layer_ctrl[type])
		{
			if((output_layer_ctrl[type] & 0x1F) == TCC_LCDC_IMG_FMT_RGB565)
				g2d_format_need = 1;
		}
		else
		{
			g2d_format_need = 1;
		}
	}
		
	if(g2d_rotate_need || g2d_format_need)
	{
		g2d_p.responsetype = G2D_POLLING;
	
		g2d_p.src0 = (unsigned int)addr;
		if(bits_per_pixel == 32)
			g2d_p.srcfm.format = GE_RGB888;
		else
			g2d_p.srcfm.format = GE_RGB565;

		g2d_p.srcfm.data_swap = 0;
		g2d_p.src_imgx = width;
		g2d_p.src_imgy = height;

		if(g2d_rotate_need)
			g2d_p.ch_mode = ROTATE_270;
		else
			g2d_p.ch_mode = NOOP;

		g2d_p.crop_offx = 0;
		g2d_p.crop_offy = 0;
		g2d_p.crop_imgx = width;
		g2d_p.crop_imgy = height;

		#if defined(CONFIG_TCC_OUTPUT_STARTER) && defined(USE_DUALOUTPUT) && defined(USE_DUALBUFFER)
			if(output_lcdc[type])
			{
				if(buf_index)
					g2d_p.tgt0 = fb_g2d_pbuf2;	// destination image address
				else
					g2d_p.tgt0 = fb_g2d_pbuf3;	// destination image address
			}
			else
			{
				if(buf_index)
					g2d_p.tgt0 = fb_g2d_pbuf0;	// destination image address
				else
					g2d_p.tgt0 = fb_g2d_pbuf1;	// destination image address
			}
		#else
			if(buf_index)
				g2d_p.tgt0 = fb_g2d_pbuf0;	// destination image address
			else
				g2d_p.tgt0 = fb_g2d_pbuf1;	// destination image address
		#endif

		g2d_p.tgtfm.format = GE_RGB565;
		g2d_p.tgtfm.data_swap = 0;
		if(g2d_rotate_need)
		{
			g2d_p.dst_imgx = height;
			g2d_p.dst_imgy = width;
		}
		else
		{
			g2d_p.dst_imgx = width;
			g2d_p.dst_imgy = height;
		}

		g2d_p.dst_off_x = 0;
		g2d_p.dst_off_y = 0;
		g2d_p.alpha_value = 255;

		g2d_p.alpha_type = G2d_ALPHA_NONE;
		
		grp_rotate_ctrl(&g2d_p);
//		grp_copyblit_ctrl(&g2d_p);

		FBimg_buf_addr = g2d_p.tgt0;

		if(g2d_rotate_need)
		{
			width = g2d_p.dst_imgx;
			height = g2d_p.dst_imgy;
		}
	}
	
	if(type == TCC_OUTPUT_COMPOSITE)
	{
		max_width 	= TCC_COMPOSITE_MAX_WIDTH;
		max_height 	= TCC_COMPOSITE_MAX_HEIGHT;			
	}
	else
	{
		max_width 	= TCC_COMPONENT_MAX_WIDTH;
		max_height 	= TCC_COMPONENT_MAX_HEIGHT;			
	}

	if(lcd_width > max_width) 
		img_width = lcd_width / 2;
	else
		img_width = lcd_width;

	if(lcd_height > max_height) 
		img_height = lcd_height / 2;
	else
		img_height = lcd_height;

	fbscaler.responsetype = SCALER_NOWAIT;

	fbscaler.src_Yaddr = FBimg_buf_addr;
	fbscaler.src_fmt = SCALER_RGB565;
	fbscaler.src_ImgWidth = width;
	fbscaler.src_ImgHeight = height;

	fbscaler.src_winLeft = 0;
	fbscaler.src_winTop = 0;
	fbscaler.src_winRight = width;
	fbscaler.src_winBottom = height;

	#if defined(CONFIG_TCC_OUTPUT_STARTER) && defined(USE_DUALOUTPUT) && defined(USE_DUALBUFFER)
		if(output_lcdc[type])
		{
			if(buf_index)
				fbscaler.dest_Yaddr = fb_scaler_pbuf2;	// destination image address
			else
				fbscaler.dest_Yaddr = fb_scaler_pbuf3;	// destination image address
		}
		else
		{
			if(buf_index)
				fbscaler.dest_Yaddr = fb_scaler_pbuf0;	// destination image address
			else
				fbscaler.dest_Yaddr = fb_scaler_pbuf1;	// destination image address
		}
	#else
		if(buf_index)
			fbscaler.dest_Yaddr = fb_scaler_pbuf0;	// destination image address
		else
			fbscaler.dest_Yaddr = fb_scaler_pbuf1;	// destination image address
	#endif

	buf_index = !buf_index;

	#if 0
 	if(type == TCC_OUTPUT_COMPOSITE)
	{
		fbscaler.dest_fmt = SCALER_RGB565;				// destination image format
		fbscaler.dest_ImgWidth = COMPOSITE_MAX_WIDTH;	// destination image width
		fbscaler.dest_ImgHeight = COMPOSITE_MAX_HEIGHT; // destination image height
		fbscaler.dest_winLeft = 40;
		fbscaler.dest_winTop = 0;
		fbscaler.dest_winRight = COMPOSITE_MAX_WIDTH;
		fbscaler.dest_winBottom = COMPOSITE_MAX_HEIGHT;
		fbscaler.viqe_onthefly = 0;
	}
	else
	#endif
	{
		fbscaler.dest_fmt = SCALER_RGB565;		// destination image format
		fbscaler.dest_ImgWidth = img_width;		// destination image width
		fbscaler.dest_ImgHeight = img_height; 	// destination image height
		fbscaler.dest_winLeft = 0;
		fbscaler.dest_winTop = 0;
		fbscaler.dest_winRight = img_width;
		fbscaler.dest_winBottom = img_height;
		fbscaler.viqe_onthefly = 0;
	}

	M2M_Scaler1_Ctrl_Detail(&fbscaler);
	FBimg_buf_addr = fbscaler.dest_Yaddr;

//	printk(" %s FB[w:%d h:%d] Image[w:%d h:%d] LCD[w:%d h:%d] addr:0x%x index:%d \n",
//		__func__,width, height, img_width, img_height, lcd_width, lcd_height, FBimg_buf_addr, buf_index);

// lcd image offset
	pLCDC_OUTPUT_FB_CH->LIO = img_width * 2;

// position
	pLCDC_OUTPUT_FB_CH->LIP = ((lcd_h_pos << 16) | (lcd_w_pos));

// scale 2
	if(lcd_width > img_width)
		scale_x = 9;
	else
		scale_x = 0;

	if(lcd_height > img_height)
		scale_y = 9;
	else
		scale_y = 0;

	pLCDC_OUTPUT_FB_CH->LISR =((scale_y<<4) | scale_x);
// size
	pLCDC_OUTPUT_FB_CH->LIS =((img_height << 16) | (img_width));

// image address
//	pLCDC_OUTPUT_FB_CH->LIBA0 = fbscaler.dest_Yaddr;

//channel control
	BITCSET (pLCDC_OUTPUT_FB_CH->LIKR, 0xff <<  0, (chromaR)  <<  0); // key
	BITCSET (pLCDC_OUTPUT_FB_CH->LIKR, 0xff << 16, (0xF8)	<< 16); // keymask
	BITCSET (pLCDC_OUTPUT_FB_CH->LIKG, 0xff <<  0, (chromaG)  <<  0); // key
	BITCSET (pLCDC_OUTPUT_FB_CH->LIKG, 0xff << 16, (0xFC)	<< 16); // keymask
	BITCSET (pLCDC_OUTPUT_FB_CH->LIKB, 0xff <<  0, (chromaB)  <<  0); // key
	BITCSET (pLCDC_OUTPUT_FB_CH->LIKB, 0xff << 16, (0xF8)	<< 16); // keymask

#if 0
	pLCDC_OUTPUT_FB_CH->LIC = HwLIC_INTL | HwLIC_IEN | TCC_LCDC_IMG_FMT_RGB565 | HwLIC_CEN;
#else
	if(output_layer_ctrl[type])
		pLCDC_OUTPUT_FB_CH->LIC = output_layer_ctrl;
	else
		pLCDC_OUTPUT_FB_CH->LIC = HwLIC_INTL | HwLIC_IEN | TCC_LCDC_IMG_FMT_RGB565 | HwLIC_CEN;
#endif

	switch(pLCDC_OUTPUT_FB_CH->LIC & 0x0000001F)
	{
		case TCC_LCDC_IMG_FMT_RGB444:
		case TCC_LCDC_IMG_FMT_RGB565:
		case TCC_LCDC_IMG_FMT_RGB555:
			pLCDC_OUTPUT_FB_CH->LIO = img_width * 2;
			break;

		case TCC_LCDC_IMG_FMT_RGB888:
		case TCC_LCDC_IMG_FMT_RGB666:
			pLCDC_OUTPUT_FB_CH->LIO = img_width * 4;
			break;
		
		case TCC_LCDC_IMG_FMT_YUV420SP:
		case TCC_LCDC_IMG_FMT_YUV422SP:
			pLCDC_OUTPUT_FB_CH->LIO = ((img_width / 2)<<16) | (img_width);
			break;

		case TCC_LCDC_IMG_FMT_YUV422SQ:
			pLCDC_OUTPUT_FB_CH->LIO = img_width * 2;
			break;

		case TCC_LCDC_IMG_FMT_YUV420ITL0:
		case TCC_LCDC_IMG_FMT_YUV420ITL1:
		case TCC_LCDC_IMG_FMT_YUV422ITL0:
		case TCC_LCDC_IMG_FMT_YUV422ITL1:
			pLCDC_OUTPUT_FB_CH->LIO = (img_width << 16) | (img_width);
			break;
	}
		
	pLCDC_OUTPUT[type]->LCTRL	|= (HwLCTRL_LEN | /* HwLCTRL_MSEL|*/ (Hw3+Hw1));
	pLCDC_OUTPUT[type]->LSTATUS	|= HwLSTATUS_RU;
}

void TCC_OUTPUT_FB_UpdateSync(unsigned int type)
{
	PLCDC_CHANNEL pLCDC_OUTPUT_FB_CH;

	if(pLCDC_OUTPUT[type] == NULL)
	{
		dprintk("%s - Err: Output LCDC is not valid, type=%d\n", __func__, type);
		return;
	}

	if(output_lcdc[type])
		pLCDC_OUTPUT_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_OUTPUT[type]->LI2C);
	else
		pLCDC_OUTPUT_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC_OUTPUT[type]->LI0C);

	pLCDC_OUTPUT_FB_CH->LIBA0 = FBimg_buf_addr;
}

