/* 
 *       TITLE: LINUX POWER BUTTON DRIVER
 *   CORE TYPE: TCC83x
 * DRIVER TYPE: kernel driver/module
 *        NAME: tcc83x_pwrbtn.c
 *
 * 2008.04.21 <linux@telechips.com>
 *       VERSION: tcc83x_pwrbtn_v.0.1
 *
 * @Copyright 2008 Telechips, Inc. 
 *
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/moduleparam.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <linux/delay.h>

#include <linux/interrupt.h>
#include <asm/arch/irqs.h>
#include <asm/arch/tcc83x_virt.h>

#include <linux/pm.h>
#include <linux/workqueue.h>

#define PM_DEV_NAME	"pwr"
#define PM_DEV_MAJOR 201

#define IRQ_PWRBTN	INT_EI0

extern void tcc83x_sleep_mode(void);

static int wq_flag = 0;
static void tcc_wq_func(struct work_struct *data);
static DECLARE_WORK(tcc_work_queue, tcc_wq_func);


static void enable_pwrkey_interrupt(void)
{
	// Port Config for EINT1
	BITSET(HwGSEL_B,   HwGSEL_B_N1_EI0 );	// Dedicated function as EINT0
	BITCLR(HwGIOCON_B, Hw4 );				// Port as Input
	BITSET(HwGPIOB_PE, Hw4);				// Pull up|Pull Down Enable
	BITSET(HwGPIOB_PS, Hw4);				// Pull up

	// Interrupt Controller Setting
	BITSET(HwSEL_B,  HwSEL_EI0);			// Normal IRQ
	BITSET(HwMSTS_B, HwMSTS_EI0);
	BITCLR(HwMODE_B, HwMODE_EI0);
	BITSET(HwCLR_B,  HwCLR_EI0);
	BITSET(HwICFG,   HwICFG_DTYP0_HLLevel); // Level Triggered

	while(!(HwGDATA_B & Hw4)) {
		printk("HwGDATA_B_4 = 1\n");
	}

	// Enable EINT1
	BITSET(HwIEN_B, HwIEN_EI0);
}

void tcc_wq_func(struct work_struct *data)
{
	unsigned long flag;

	device_suspend(PMSG_SUSPEND);
	local_irq_save(flag);
	local_irq_disable(); // Before enter Power Down Mode, MUST mask CPSR Irq bit.

	HwWKEN_B = 0xFFFFFFFF;
	HwWKEN   = 0xFFFFFFFF;
	BITCLR(HwWKEN_B, HwWKEN_EI0); //EINT1 wakeup system

	/*
	 * Enter Sleep Mode: arch/arm/mach-tcc83x/pm.c
	 */
	tcc83x_sleep_mode();

	local_irq_restore(flag);
	device_resume();

	// pwr enable irq
	BITSET(HwCLR_B, HwIEN_EI0); 
	BITSET(HwIEN_B, HwIEN_EI0); 

	wq_flag = 0;
}

static irqreturn_t tcc83x_pwrbtn_handler(int irq, void *dev_id)
{
	int i;

	// get rid of noise
	while (!(HwGDATA_B & Hw4));
	for (i = 0; i < 10; i++) {
		mdelay(2);
		while (!(HwGDATA_B & Hw4));
	}

	if (!wq_flag) {
		schedule_work(&tcc_work_queue);
		wq_flag = 1;
	}

	return IRQ_HANDLED;
}

static int tcc83x_pwrbtn_ioctl(struct inode *inode, struct file *filp, 
							unsigned int cmd, unsigned long arg)
{
	return 0;
}

static int tcc83x_pwrbtn_release(struct inode *inode, struct file *filp)
{
	return 0;
}

int tcc83x_pwrbtn_open(struct inode *inode, struct file *filp)
{
	return 0;
}

struct file_operations tcc83x_pwrbtn_fops =
{
	.owner		= THIS_MODULE,
	.open		= tcc83x_pwrbtn_open,
	.ioctl		= tcc83x_pwrbtn_ioctl,
	.release	= tcc83x_pwrbtn_release,
};

static int __init pwrbtn_init(void)
{
	int res;

	res = register_chrdev(PM_DEV_MAJOR, PM_DEV_NAME, &tcc83x_pwrbtn_fops);
	if (res < 0)
		return res;

	enable_pwrkey_interrupt();

	res = request_irq(IRQ_PWRBTN, tcc83x_pwrbtn_handler, IRQF_DISABLED, "pwrbtn", NULL);
	if (res)
		return res;

    printk("pwrbtn: init\n");
    return 0;
}

static void __exit pwrbtn_exit(void)
{
	unregister_chrdev(PM_DEV_MAJOR, PM_DEV_NAME);
	free_irq(IRQ_PWRBTN, NULL);
    printk("pwrbtn: exit\n");
}

module_init(pwrbtn_init);
module_exit(pwrbtn_exit);

MODULE_AUTHOR("Telechips Inc. SYS4-3 linux@telechips.com");
MODULE_DESCRIPTION("TCC83x Power Button driver");
MODULE_LICENSE("GPL");

