/* 
 * linux/drivers/char/tcc_backlight.c
 *
 * Author:  <linux@telechips.com>
 * Created: 10th Jun, 2008 
 * Description: Telechips Linux BACK-LIGHT DRIVER
 *
 * Copyright (c) Telechips, Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <linux/init.h>
#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/moduleparam.h>
#include <linux/device.h>
#include <linux/fs.h>
#include <linux/errno.h>
#include <asm/uaccess.h>

#include "tca_backlight.h"

#define BL_DEV_NAME			"bl"
#define BL_DEV_MAJOR		244
#define BL_DEV_MINOR		0

#define IOCTL_BL_OFF		_IO(BL_DEV_MAJOR, 1)
#define IOCTL_BL_ON			_IO(BL_DEV_MAJOR, 2)
#define IOCTL_BL_GET_LEVEL	_IO(BL_DEV_MAJOR, 3)
#define IOCTL_BL_SET_LEVEL	_IO(BL_DEV_MAJOR, 4)


static void bl_init(void)
{
	
    printk(KERN_INFO "%s\n", __FUNCTION__);

}

static void bl_control(bool flag)
{
    printk(KERN_INFO "%s\n", __FUNCTION__);

	if (flag) {
		tca_bkl_powerup();
	} else {
		tca_bkl_powerdown();
	}
}

static int tcc_bl_ioctl(struct inode *inode, struct file *filp, 
							unsigned int cmd, unsigned long arg)
{
	unsigned long bl_val;
	
    printk(KERN_INFO "%s  (0x%x)  \n", __FUNCTION__, cmd);

	switch (cmd) {
		case IOCTL_BL_OFF:
			bl_control(false);
			break;
		case IOCTL_BL_ON:
			bl_control(true);
			break;

		case IOCTL_BL_GET_LEVEL:
			bl_val = tca_bkl_getpowerval();
			copy_to_user((void *)arg, (const void *)&bl_val, sizeof(unsigned long));
			break;

		case IOCTL_BL_SET_LEVEL:
			copy_from_user((void *)&bl_val, (const void *)arg, sizeof(unsigned long));
			tca_bkl_setpowerval(bl_val);
			break;

		default:
			printk("bl: unrecognized ioctl (0x%x)\n", cmd); 
			return -EINVAL;
			break;
	}

	return 0;
}

static int tcc_bl_release(struct inode *inode, struct file *filp)
{
	return 0;
}

static int tcc_bl_open(struct inode *inode, struct file *filp)
{
	return 0;
}

struct file_operations tcc_bl_fops =
{
	.owner		= THIS_MODULE,
	.open		= tcc_bl_open,
	.release	= tcc_bl_release,
	.ioctl		= tcc_bl_ioctl,
};

static struct class *bl_class;

static int __init tcc_bl_init(void)
{
	int res;

	res = register_chrdev(BL_DEV_MAJOR, BL_DEV_NAME, &tcc_bl_fops);
	if (res < 0)
		return res;
	bl_class = class_create(THIS_MODULE, BL_DEV_NAME);
	device_create(bl_class, NULL, MKDEV(BL_DEV_MAJOR, BL_DEV_MINOR), NULL, BL_DEV_NAME);

    printk(KERN_INFO "%s\n", __FUNCTION__);
	return 0;
}

static void __exit tcc_bl_exit(void)
{
	unregister_chrdev(BL_DEV_MAJOR, BL_DEV_NAME);
    printk(KERN_INFO "%s\n", __FUNCTION__);
}

module_init(tcc_bl_init);
module_exit(tcc_bl_exit);

MODULE_AUTHOR("Telechips Inc. SYS4-3 linux@telechips.com");
MODULE_DESCRIPTION("TCCxxx back-light driver");
MODULE_LICENSE("GPL");

