/*
 * drivers/char/tcc_drminfo.c
 *
 * Copyright (C) 2004 Texas Instruments, Inc. 
 *
 * Video-for-Linux (Version 2) graphic capture driver for
 * the OMAP H2 and H3 camera controller.
 *
 * Adapted from omap24xx driver written by Andy Lowe (source@mvista.com)
 * Copyright (C) 2003-2004 MontaVista Software, Inc.
 * 
 * This package is free software; you can redistribute it and/or modify 
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation. 
 * 
 * THIS PACKAGE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR 
 * IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED 
 * WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE. 
 *
 * History:
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>

#include <linux/poll.h>

#include "tcc_drminfo.h"
#include <mach/tcc_drminfo_ioctrl.h>

#if 0
static int debug	   = 1;
#else
static int debug	   = 0;
#endif

#define dprintk(msg...)	if (debug) { printk( "tcc_drm: " msg); }

#define DEVICE_NAME			"drminfo"
#define MAJOR_ID			220
#define MINOR_ID			1

unsigned int gTCCDCStore[TCC_DCSTORE_MAXSIZE/4];
unsigned int gTCCDCStoreSize = 0;

static unsigned int opened = 0;

void tcc_clear_devicecert(void)
{
	memset((unsigned char*)gTCCDCStore, 0x00, TCC_DCSTORE_MAXSIZE);

	gTCCDCStoreSize = 0;
}

unsigned long tcc_set_devicecert(
	/*IN*/const unsigned char *pbDevCert,
	/*IN*/unsigned long cbDevCert)
{
	//Add Code
	dprintk("tcc_set_devicecert pbDevCert addr [0x%08X] cbDevCert[%d] \n",pbDevCert, cbDevCert);   
	
	if(pbDevCert == 0 || cbDevCert > TCC_DCSTORE_MAXSIZE)
		return -1;

	dprintk("tcc_set_devicecert 2 gTCCDCStore addr [0x%08X] cbDevCert[%d] \n",&gTCCDCStore[0], cbDevCert);  
	memset((unsigned char*)&gTCCDCStore[0], 0x00, TCC_DCSTORE_MAXSIZE);
	memcpy((unsigned char*)&gTCCDCStore[0], pbDevCert, cbDevCert);

	dprintk("tcc_set_devicecert 3\n");  

	gTCCDCStoreSize = cbDevCert;
		
	return 0;
}

unsigned long tcc_get_devicecert(
	/*OUT*/unsigned char *pbDevCert,
	/*IN OUT*/unsigned long *pcbDevCert)
{
	dprintk("tcc_get_devicecert pbDevCert addr [0x%08X] *pcbDevCert[%d] \n",pbDevCert, *pcbDevCert);  

	//Add Code
	if(gTCCDCStoreSize == 0 || *pcbDevCert < gTCCDCStoreSize)
	{
		*pcbDevCert = 0;
		return 0x8004C045;
	}

	memcpy(pbDevCert, (unsigned char*)&gTCCDCStore[0], gTCCDCStoreSize);
	dprintk("tcc_get_devicecert gTCCDCStore addr [0x%08X] gTCCDCStoreSize[%d] \n",&gTCCDCStore[0], gTCCDCStoreSize);  
	*pcbDevCert = gTCCDCStoreSize;
		
	return 0;
}

static int tcc_drminfo_ioctl(struct inode *inode, struct file *file, unsigned int cmd, void *arg)
{
	TCC_DRMINFO_INFO *pDrmInfo;
	unsigned int *pSize = 0;
	unsigned long result;
	
	int ret = 0;
	
	dprintk("drminfo_ioctl IOCTRL[%x] \n", cmd);      
	switch(cmd)
	{
		case TCC_DRMINFO_DATA_SET:
			pDrmInfo = (TCC_DRMINFO_INFO *)arg;
			dprintk("cbDevCert[%d] pbDevCert addr[0x%08X]\n", pDrmInfo->cbDevCert,pDrmInfo->pbDevCert);  
			pDrmInfo->result = tcc_set_devicecert(pDrmInfo->pbDevCert,pDrmInfo->cbDevCert);
			break;
		case TCC_DRMINFO_DATA_GET:
			pDrmInfo = (TCC_DRMINFO_INFO *)arg;

			pDrmInfo->result = tcc_get_devicecert(pDrmInfo->pbDevCert,&pDrmInfo->cbDevCert);
			dprintk("tcc_get_devicecert result[%d] cbDevCert[%d]\n", pDrmInfo->result,pDrmInfo->cbDevCert);
			break;
		case TCC_DRMINFO_DATA_CLEAR:
			tcc_clear_devicecert();
			break;
		case TCC_DRMINFO_DATASIZE_SET:
			pSize = (unsigned int *)arg;
			gTCCDCStoreSize = *pSize;
			break;
		case TCC_DRMINFO_DATASIZE_GET:
			pSize = (unsigned int *)arg;
			*pSize = gTCCDCStoreSize;
			break;
		case TCC_DRMINFO_UID_CHECK:
			break;
		default:
			printk(" Unsupported IOCTL!!!\n");      
			break;			
	}

	return ret;
}

static int tcc_drminfo_release(struct inode *inode, struct file *filp)
{
	opened = 0;
	return 0;
}

static int tcc_drminfo_open(struct inode *inode, struct file *filp)
{	
	int ret;
	
	return 0;
}

static struct file_operations tcc_drminfo_fops = 
{
	.owner			= THIS_MODULE,
	.ioctl			= tcc_drminfo_ioctl,
	.open			= tcc_drminfo_open,
	.release		= tcc_drminfo_release
};

void __exit tccxxx_drminfo_cleanup(void)
{
	unregister_chrdev(MAJOR_ID, DEVICE_NAME);
	
	return;
}

static struct class *drminfo_class;

int __init tccxxx_drminfo_init(void)
{
	//printk(banner);
	
	register_chrdev(MAJOR_ID, DEVICE_NAME, &tcc_drminfo_fops);
	drminfo_class = class_create(THIS_MODULE, DEVICE_NAME);
	device_create(drminfo_class, NULL, MKDEV(MAJOR_ID, MINOR_ID), NULL, DEVICE_NAME);

	return 0;
}


MODULE_AUTHOR("Telechips.");
MODULE_DESCRIPTION("TCC drminfo driver");
MODULE_LICENSE("GPL");


module_init(tccxxx_drminfo_init);
module_exit(tccxxx_drminfo_cleanup);


