/*
 * IR driver for remote controller
 *
 * Copyright (C) 2009 Telechips Inc.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 */


#include <linux/init.h>
#include <linux/fs.h>
#include <linux/major.h>
#include <linux/blkdev.h>
#include <linux/module.h>
#include <linux/capability.h>
#include <linux/uio.h>
#include <linux/cdev.h>
#include <linux/device.h>
#include <linux/mutex.h>
#include <linux/interrupt.h>
#include <linux/delay.h>
#include <linux/kthread.h>

#include <linux/platform_device.h>    
#include <linux/input.h>       
#include <linux/sched.h>
#include <asm/io.h>
#include <linux/device.h>

#include <asm/irq.h>
#include <bsp.h>

#include <linux/cpufreq.h>

/*****************************************************************************
*
* Header Files Include
*
******************************************************************************/
#include "tcc_remocon.h"

#define DEVICE_NAME	"tcc-remote"


#define IR_COUNT		1

#define TCC_IRVERSION 0.001

//#define IR_THREAD
#define IR_TIMER

#ifdef IR_TIMER
struct remocon_cdev
{
    int running;
    char old_key;
    int status;
    int first;
    char val[40];
};
struct remocon_cdev  remocon_cdev;
#endif

struct tcc_remote
{
    struct input_dev *dev;  
    struct work_struct work_q;
};

static struct tcc_remote *rem;
static int startbit_time;
static int prev_startbit_time;

#define IR_IRQ		INT_RMT

#if 0
//#define dbg(x...) printk(KERN_DEBUG "tcc uart: ");
#define dbg printk
#else /* no debug */
#define dbg(x...) do {} while(0)
#endif


/*****************************************************************************
*
* structures
*
******************************************************************************/
typedef struct
{
	unsigned int BitCnt;	//Data Bit Count
	unsigned int Buf;		//Data Buffer
	unsigned int Id;		//Remocon ID
	unsigned int Code;		//Parsing Return Value
	unsigned int Val;		//Parsing Value
	unsigned int Prev;		//Previous Data
	unsigned int RepKey;	//Repeat key		: 0 or 1
	unsigned int T1,T2;		//Repeat Key Time Check
	unsigned int Stat;		//Remocon Status
	unsigned int PwrOff;	//Power Off Mode	: 0 or 1
}REM_DATA;

static REM_DATA Rem;
static char remocon_key;

#if defined(IR_THREAD) || defined(IR_TIMER)
static int rd_data[20];
int rd_valid=0;
#endif

#ifdef IR_TIMER
#define KEY_RELEASED    0
#define KEY_PRESSED     1

static int m_flags = 0;

static struct timer_list remocon_timer;

static void remocon_timer_handler(unsigned long data)
{
    struct remocon_cdev *remocon_data = (struct remocon_cdev *)data;
    struct input_dev *dev = rem->dev;
    
    if(remocon_data->running)
    {
		input_report_key(dev, remocon_data->old_key, 0);
		input_sync(dev);
		dbg("\nnRem=%d released\n",remocon_data->old_key);
		//printk("\nnRem=%d released\n",remocon_data->old_key);
        remocon_data->status = KEY_RELEASED;
        remocon_key = remocon_data->old_key;
        remocon_data->old_key = NO_KEY;
        remocon_data->running = 0;
        m_flags = 1;
		Rem.Code = Rem.Prev = NO_KEY;
    }
}
#endif


//--------------------------------------------------------------------
//
//--------------------------------------------------------------------
static void Init_IR_Port(void)
{
#if defined(CONFIG_ARCH_TCC92X)
	HwGPIOA_FN0 = (HwGPIOA_FN0 & ~(0xF<<20)) | (1<<20);
#elif defined(CONFIG_ARCH_TCC93XX)
	PGPIO pGpioA = (volatile PGPIO)tcc_p2v(HwGPIOA_BASE);
	BITCSET(pGpioA->GPAFN0, (Hw20 | Hw21 | Hw22 | Hw23), Hw20);	//GPIO_A5
#endif /* CONFIG_ARCH_TCC92X */
}

#if 0
/************************************************************************************************
* FUNCTION		: void tca_rem_initvalue(void)
*
* DESCRIPTION	: 
*
************************************************************************************************/
static void tca_rem_initvalue(void)
{
	Rem.Stat = 0;
	Rem.PwrOff=0;
	Rem.RepKey = 0;
	Rem.T1 = Rem.T2 = 0;
}
#endif

/************************************************************************************************
* FUNCTION		: static unsigned int tca_rem_readcode(char ch)
*
* DESCRIPTION	: 
*
************************************************************************************************/
static unsigned int tca_rem_readcode(char ch)
{
	switch (Rem.Stat)
	{
		/* Initialize */
		case STATUS0:
			Rem.BitCnt = Rem.Buf = 0;
			Rem.RepKey = 0;
			Rem.Stat = STATUS1;
#if defined(CONFIG_ARCH_TCC93XX)
		if (ch == 'S')
		{
			Rem.Stat = STATUS2;
			Rem.BitCnt++;
		}
		else if (ch == 'R')
		{
			dbg("R: %x, %x\n", Rem.Code, Rem.Prev);
			//tcnt=12.5ms
			if ((Rem.Prev == SCAN_VOLUP)||(Rem.Prev == SCAN_VOLDN)||(Rem.Prev == SCAN_UP)
				||(Rem.Prev == SCAN_DOWN)||(Rem.Prev == SCAN_LEFT)||(Rem.Prev == SCAN_RIGHT)
#ifdef CONFIG_DARE_IR
				|| (Rem.Prev == SCAN_NEXT)||(Rem.Prev == SCAN_LAST)
#endif
#if defined( CONFIG_CS_2000_IR_X_CANVAS )
			   || (Rem.Prev == SCAN_FB)||(Rem.Prev == SCAN_FF)
			   || (Rem.Prev == SCAN_CENTER)
#endif
		   )
			{
				Rem.RepKey =  1;
				return Rem.Prev;
			}
		}
#endif
			break;
			
		/*Start Bit */
		case  STATUS1:
			if (ch == 'S') 	//tcnt = 15ms 
			{
				//it appears to be HW bug that the 1st repeat pule is always 
				//recognized wrongly as start bit 
		        if (startbit_time != prev_startbit_time)
		        {
			        prev_startbit_time = startbit_time;
			        Rem.Code = 0;	//reset the memorized code if we get a start bit
		        }
				Rem.Stat = STATUS2;
				Rem.BitCnt++;
			}

			//Repeat
			else if (ch == 'R')
			{
				dbg("R: %x, %x\n", Rem.Code, Rem.Prev);
				//tcnt=12.5ms
				if ((Rem.Prev == SCAN_VOLUP)||(Rem.Prev == SCAN_VOLDN)||(Rem.Prev == SCAN_UP)
					||(Rem.Prev == SCAN_DOWN)||(Rem.Prev == SCAN_LEFT)||(Rem.Prev == SCAN_RIGHT)
#ifdef CONFIG_DARE_IR
					|| (Rem.Prev == SCAN_NEXT)||(Rem.Prev == SCAN_LAST)
#endif
#if defined( CONFIG_CS_2000_IR_X_CANVAS )
				   || (Rem.Prev == SCAN_FB)||(Rem.Prev == SCAN_FF)
				   || (Rem.Prev == SCAN_CENTER)
#endif
			   )
				{
					Rem.RepKey =  1;
					return Rem.Prev;
				}
			}
			break;
		
		/* Data Code Check */
		case STATUS2:
			if (ch == '0')			//Bit '0' = 3.5ms
			{
				Rem.Buf =(Rem.Buf << 1) & ~Hw0;
				Rem.BitCnt++;
			}
			else if (ch == '1')		//Bit '1' = 5.5ms
			{
				Rem.Buf = (Rem.Buf << 1) |Hw0;
				Rem.BitCnt++;
			}
			else if (ch == 'S')		//If Receive Start Bit, Return 0;
			{
				return 0;
			}
			else
			{
				Rem.Stat =STATUS0;
			}

			if(Rem.BitCnt == 32)
			{
				dbg("CODE=%x\n", Rem.Buf);
				//printk("CODE=%x\n", Rem.Buf);
				Rem.Id = (Rem.Buf & 0xffff0000) >> 16;

#if defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX)
				if (Rem.Buf == REMOCON_REPEAT)
				{
					if ((Rem.Code == SCAN_VOLUP)||(Rem.Code == SCAN_VOLDN)||(Rem.Code == SCAN_UP)
						||(Rem.Code == SCAN_DOWN)||(Rem.Code == SCAN_LEFT)||(Rem.Code == SCAN_RIGHT)
#ifdef CONFIG_DARE_IR
						|| (Rem.Code == SCAN_NEXT)||(Rem.Code == SCAN_LAST)
#endif
#if defined( CONFIG_CS_2000_IR_X_CANVAS )
						|| (Rem.Code == SCAN_FB)||(Rem.Code == SCAN_FF)
						|| (Rem.Code == SCAN_CENTER)
#endif
					)
					{
						Rem.Stat = STATUS0;
						return Rem.Code;
					}
				}
				else
#endif
				if (Rem.Id == REMOCON_ID)
				{
					Rem.Code = (Rem.Buf & 0x0000ffff);
					Rem.Stat = STATUS0;
					return Rem.Code;
				}
				else
				{
					Rem.Stat = STATUS0;
				}
			}
			break;
	}
	
	return 0;
}

/************************************************************************************************
* FUNCTION		: static int GetKeyCodeByScanCode(unsigned short kc)
*
* DESCRIPTION	: return the keycode, return -1 if no key matched
*
************************************************************************************************/
static int tca_rem_getkeycodebyscancode(unsigned short kc)
{
    int i;
	for (i = 0;i < sizeof(key_mapping)/sizeof(SCANCODE_MAPPING);i++)
      if (kc == key_mapping[i].rcode) 
            return key_mapping[i].vkcode;
    return -1;
}

/************************************************************************************************
* FUNCTION		: void tca_rem_sendremocondata(unsigned int Data)
*
* DESCRIPTION	: 
*
************************************************************************************************/
static void tca_rem_sendremocondata(unsigned int Data)
{
    struct input_dev *dev = rem->dev;
	int nRem;
	
	//Power Control Message
	if (Data == SCAN_PWR)
	{
		nRem = tca_rem_getkeycodebyscancode(Rem.Val);
		if(nRem == -1)
			return;

		remocon_key = (char)nRem;
		dbg("nRem=%d\n", nRem);

		input_report_key(dev, nRem, 1);
        input_sync(dev);

        input_report_key(dev, nRem, 0);
        input_sync(dev);

		//Power Off Mode
		if (Rem.PwrOff == 0)
		{
			Rem.PwrOff = 1;
			printk("PwrOff\n");
		}
		//Power On Mode
		else
		{
			Rem.PwrOff = 0;
			printk("PwrOn\n");
		}
		//reset
		Rem.Val = 0;
	}	

	else
	{
		if (Rem.PwrOff == 0 && Data)
		{
			Rem.Prev = Data;	//For Repeat Key, Current Value Save
			nRem = tca_rem_getkeycodebyscancode(Rem.Val);
			if(nRem == -1)
				return;

			remocon_key = (char)nRem;

			#ifdef IR_TIMER
				if((remocon_cdev.old_key != remocon_key)||(remocon_cdev.status != KEY_PRESSED)){
					dbg("nRem=%d pressed\n", nRem);
					//printk("nRem=%d pressed\n", nRem);
		            remocon_cdev.status = KEY_PRESSED;
		            remocon_cdev.first = 1;		
					
					input_report_key(dev, nRem, 1);
	            	input_sync(dev);		
				}
				del_timer(&remocon_timer);
				remocon_timer.expires = jiffies + msecs_to_jiffies(125);
				add_timer(&remocon_timer);
				remocon_cdev.running = 1;
				remocon_cdev.old_key = nRem;
				m_flags = 1;
            #else
	            dbg("nRem=%d\n", nRem);
				input_report_key(dev, nRem, 1);
	            input_sync(dev);
	            
	            input_report_key(dev, nRem, 0);
	            input_sync(dev);
			#endif
			Rem.Val = 0;	 
		}
	}
}

/*****************************************************************************
* Function Name : tca_rem_process()
******************************************************************************/
static int tca_rem_process(char ch)
{
	Rem.Val = tca_rem_readcode(ch);

	if (Rem.Val)
	{
		if (Rem.RepKey)
		{
			Rem.RepKey = 0;
		}
		tca_rem_sendremocondata(Rem.Val);
		return 0;
	}
	else
		return -1;
}

static void remocon_irq_init(void)
{
	volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	BITSET(pPIC->IEN1, HwINT1_RMT);
	BITSET(pPIC->SEL1, HwINT1_RMT);
	BITSET(pPIC->INTMSK1, HwINT1_RMT);
	BITCLR(pPIC->POL1, HwINT1_RMT);
	BITSET(pPIC->MODE1, HwINT1_RMT);
	BITCLR(pPIC->MODEA1, HwINT1_RMT);
}

#if 0
static void remocon_enable_irq(void)
{
	volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	BITSET(pPIC->IEN1,  HwINT1_RMT);
	BITSET(pPIC->CLR1, HwINT1_RMT);
	BITSET(pPIC->INTMSK1, HwINT1_RMT);
}

static void remocon_disable_irq(void)
{
	volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	BITCLR(pPIC->IEN1,  HwINT1_RMT);
	BITCLR(pPIC->INTMSK1,  HwINT1_RMT);
	BITSET(pPIC->CLR1, HwINT1_RMT);
}
#endif /* 0 */

void rem_getrawdata(void)
{
	int i;
	int nTemp;
	char scan_bit = 'x';

#define FIFO_CNT 20

	int rd_data[FIFO_CNT];

	PREMOCON pRemocon = (volatile PREMOCON)tcc_p2v(Hw_REMOCON_BASE);

	for (i = 0; i < FIFO_CNT; i++)
		rd_data[i] = pRemocon->uPORT.bPORT.PORT;

	dbg("\n");
	//printk("\n");

	for (i = 0; i < FIFO_CNT*2; i++)
	{
		if (!(i % 2))
			nTemp = rd_data[i/2] & 0xffff;
		else
			nTemp = (rd_data[i/2] >> 16) & 0xffff;

		if ((nTemp > LOW_MIN) && (nTemp < LOW_MAX))
		{
			scan_bit = '0';
		}
		else if ((nTemp > HIGH_MIN) && (nTemp < HIGH_MAX))
		{
			scan_bit = '1';
		}
		else if ((nTemp > REPEAT_MIN) && (nTemp < REPEAT_MAX))
		{
			scan_bit = 'R';
		}
		else if ((nTemp > START_MIN) && (nTemp < START_MAX))
		{
			scan_bit = 'S';
			startbit_time = rd_data[i/2]; //get Start bit timing  and the 1st data bit timing
		}
		else
		{
			scan_bit = 'E';
		}

		//printk("[%02d] (%c: %04X)\n",i,  scan_bit, nTemp);

		if (tca_rem_process(scan_bit) == 0)
			break;
	}
	dbg("\n");
	//printk("\n");
}

#if defined(IR_THREAD) || defined(IR_TIMER)
static int remocon_thread(void *data)
{
	int i;
	int low, high;
	char low_bit='x', high_bit='x';

#ifdef IR_THREAD
	while(1)
#endif
	{
		if (rd_valid == 1)
		{
			i=0;
			do
			{
				low = rd_data[i] & 0xffff;
				high = (rd_data[i] >> 16) & 0xffff;

				dbg("%04x|%04x\n", low, high);
				//printk("%04x|%04x\n", low, high);

				if ((low > LOW_MIN) && (low < LOW_MAX))
				{		  
					low_bit='0';
				}
				else if ((low > HIGH_MIN) && (low < HIGH_MAX))
				{
					low_bit='1';
				}
				else if ((low > REPEAT_MIN) && (low < REPEAT_MAX))
				{
					low_bit='R';
				}
				else if ((low > START_MIN) && (low < START_MAX))
				{
					low_bit='S';
					startbit_time= rd_data[i];  //get Start bit timing  and the 1st data bit timing
				}
				else
				{
					low_bit='E';
				}

				if ((high > LOW_MIN) && (high < LOW_MAX))
				{
					high_bit='0';
				}
				else if ((high > HIGH_MIN) && (high < HIGH_MAX))
				{
					high_bit='1';
				}
				else if ((high > REPEAT_MIN) && (high < REPEAT_MAX))
				{
					high_bit='R';
				}
				else if ((high > START_MIN) && (high < START_MAX))
				{
					high_bit='S';
					startbit_time= rd_data[i];  //get Start bit timing  and the 1st data bit timing
				}
				else
				{
					high_bit='E';
				}

				dbg("%c%c", low_bit,high_bit);

				if (tca_rem_process(low_bit) == 0 || tca_rem_process(high_bit) == 0)
				{
					break;
				}
			} while(++i<20);
			
			dbg("\n");
			rd_valid = 0;
		}
		else
		{
			msleep(10);
		}
	}
	return 0;
}
#endif //IR_THREAD

static irqreturn_t remocon_handler(int irq, void *id, struct pt_regs *regs)
{
	int i=0;
	PREMOCON pRemocon = (volatile PREMOCON)tcc_p2v(Hw_REMOCON_BASE);

#ifndef IR_THREAD
#ifndef IR_TIMER
	int rd_data[20];
#endif
#endif

    disable_irq(IR_IRQ);

	Rem.Stat = STATUS0;

#if defined(IR_THREAD) || defined(IR_TIMER)
	#if defined(CONFIG_ARCH_TCC92X)
	if(rd_valid == 0)
	{
		HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV  = 0;
		i=0;
		do
		{
			rd_data[i] = Hw_REMOCON_RDATA;
		}
		while (++i < 20);
		rd_valid = 1;
	}
	#elif defined(CONFIG_ARCH_TCC93XX)
	if(rd_valid == 0)
	{
		pRemocon->uCLKDIVIDE.bCLKDIVIDE.CLK_DIV  = 0;
		i=0;
		do
		{
			rd_data[i] = pRemocon->uPORT.bPORT.PORT;
		}
		while (++i < 20);
		rd_valid = 1;
	}
	#endif
#else
	rem_getrawdata();
#endif

//	udelay(100); //Remocon can't be so faster!!!
//	udelay(0xFF); //Remocon can't be so faster!!!
#ifdef IR_TIMER
	if(rd_valid==1)
		remocon_thread(NULL);
	udelay(100); //Remocon can't be so faster!!!
#else
	i = 0;
	do
	{
		udelay(2000); //Remocon can't be so faster!!!
	}
	while(++i <= 50);
#endif
	RemoconInit ();

    enable_irq(IR_IRQ);

	return IRQ_HANDLED;
}

static int clock_set_enable=0;

extern struct tcc_freq_table_t gtRemoconClockLimitTable[];

static int tcc_remocon_set_clock_table(int enable)
{
	if(clock_set_enable != enable)
	{
		tcc_cpufreq_set_limit_table(&gtRemoconClockLimitTable, TCC_FREQ_LIMIT_REMOCON, enable);
	}

	clock_set_enable = enable;
	ndelay(1);
	return 0;
}

//--------------------------------------------------------------------------
// Name: Device register
// Description: This functions register device
// Arguments:
//--------------------------------------------------------------------------
static int remocon_probe(struct platform_device *pdev)
{
#ifdef IR_THREAD
	struct task_struct * remocon_task;
#endif
	int ret = -1;
	int err = -ENOMEM, i;
	struct input_dev *input_dev;
	PREMOCON pRemocon = (volatile PREMOCON)tcc_p2v(Hw_REMOCON_BASE);

#if defined(CONFIG_ARCH_TCC93XX)
	tca_ckc_setiobus(RB_REMOTECONTROLLER, ENABLE);
#endif

#if defined(CONFIG_ARCH_TCC92X)
	tcc_remocon_set_clock_table(1);
#endif

	rem = kzalloc(sizeof(struct tcc_remote), GFP_KERNEL);
	input_dev = input_allocate_device();

	if (!rem || !input_dev)
	{
		err = -ENOMEM;
		goto error_alloc;
	}
	platform_set_drvdata(pdev, rem);

	rem->dev = input_dev;

	rem->dev->name = "telechips remote controller";
	rem->dev->phys = DEVICE_NAME;
	rem->dev->evbit[0] = BIT(EV_KEY);
	rem->dev->id.version = TCC_IRVERSION;
	input_dev->dev.parent = &pdev->dev;
	for (i = 0; i < ARRAY_SIZE(key_mapping); i++)
	{
		set_bit(key_mapping[i].vkcode & KEY_MAX, rem->dev->keybit);
	}

	Init_IR_Port();

	RemoconConfigure ();

	RemoconStatus();

	RemoconDivide();		//remocon clk divide and end_cout

	RemoconCommandOpen();				

//	RemoconIrqClear ();	
	RemoconIntClear();

	remocon_irq_init();

#if defined(CONFIG_ARCH_TCC92X)                                                                   
	HwREMOCON->uCLKDIVIDE.bCLKDIVIDE.END_CNT  = 100; // data end count value  
#elif defined(CONFIG_ARCH_TCC93XX) 
	pRemocon->uCLKDIVIDE.bCLKDIVIDE.END_CNT  = 100; // data end count value
#endif
 
	//Init IR variable
	remocon_key = NO_KEY;

#ifdef IR_THREAD
	//Remocon thread create & wake up
	rd_valid = 0;
	remocon_task = kthread_create(remocon_thread, NULL, "remocon_task");
	if(remocon_task)
		wake_up_process(remocon_task);
#endif

#ifdef IR_TIMER
    remocon_cdev.running = 0;
    remocon_cdev.old_key = -1;
    remocon_cdev.status = KEY_RELEASED;
    remocon_cdev.first = 0;
    
	rd_valid = 0;
	init_timer(&remocon_timer);
	remocon_timer.data = (unsigned long)(&remocon_cdev);
	remocon_timer.function = remocon_timer_handler;
#endif

	// Enable remocon interrupt
	dbg("IR remote request_irq\n");

	ret =  request_irq(IR_IRQ, remocon_handler, IRQF_DISABLED/*SA_INTERRUPT*/, 
			  DEVICE_NAME, rem);
	if (ret)
	{
		printk("IR remote request_irq error\n");
        goto error_irq;
	}

    ret = input_register_device(rem->dev);
    if (ret) 
        goto error_register;

    return 0;

error_alloc:
    input_free_device(input_dev);
error_irq:
    free_irq(IR_IRQ, rem);
error_register:
    input_unregister_device(rem->dev);

	return ret;
}

static int remocon_remove(struct platform_device *pdev)
{
    disable_irq(IR_IRQ);
    del_timer(&remocon_timer);
	free_irq(IR_IRQ, rem->dev);
    input_unregister_device(rem->dev);
    kfree(rem);

#if defined(CONFIG_ARCH_TCC92X)
    tcc_remocon_set_clock_table(0);
#endif

	return 0;
}

#ifdef CONFIG_PM
static int remocon_suspend(struct platform_device *pdev, pm_message_t state)
{
    DisableRemocon();

	return 0;
}

static int remocon_resume(struct platform_device *pdev)
{
	RemoconInit ();

	return 0;
}
#else
#define remocon_suspend NULL
#define remocon_resume NULL
#endif

static struct platform_driver remocon_driver =
{
	.driver     =
	{
        .name   = DEVICE_NAME,
        .owner  = THIS_MODULE,
    },
	.probe		= remocon_probe,
	.remove		= remocon_remove,
#ifdef CONFIG_PM
	.suspend	= remocon_suspend,
	.resume		= remocon_resume,
#endif
};

static int __init remocon_init(void)
{
	printk("Telechips Remote Controller Driver Init\n");
	return platform_driver_register(&remocon_driver);
}

static void __exit remocon_exit(void)
{
	printk("Telechips Remote Controller Driver Exit \n");
    platform_driver_unregister(&remocon_driver);
}

module_init(remocon_init);
module_exit(remocon_exit);

MODULE_AUTHOR("Linux Team<linux@telechips.com>");
MODULE_DESCRIPTION("IR remote control driver");
MODULE_LICENSE("GPL");
