
/****************************************************************************
 *   FileName    : tca_remocon.h
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

//*******************************************
//	Remote controller define
//*******************************************

//Low Bit:
#define LOW_MIN			0x0600
#define LOW_MAX			0x0BFF
//High Bit
#define HIGH_MIN		0x1600
#define HIGH_MAX		0x1DFF
//Start Bit
#define START_MIN		0x4000
#define START_MAX		0x50FF
//Repeat Bit
#define REPEAT_MIN		0x2000
#define REPEAT_MAX		0x27FF


/*****************************************************************************
*
* IR remote controller scancode
*
******************************************************************************/
#define REM_ID_ERROR	0xFF

#define REMOCON_ID		0x40bf	//vendor ID, prodcut ID

#define SCAN_PWR		0x02fd	//
#define SCAN_LOAD		0xFFFF	//, no mapping
#define SCAN_SUBTITLE	0x906f 	//
#define SCAN_AUDIO		0xd827 	//
#define SCAN_ANGLE		0xFFFF	// no mapping
#define SCAN_INFO		0xe817	//

#define SCAN_KEY1		0x48b7	//"1"
#define SCAN_KEY2		0x6897	//"2"
#define SCAN_KEY3		0x40bf  //"3"
#define SCAN_KEY4		0x58a7 	//"4"
#define SCAN_KEY5		0x7887  //"5"
#define SCAN_KEY6		0xc03f 	//"6"
#define SCAN_KEY7		0xf00f	//"7"
#define SCAN_KEY8		0xd02f	//"8"
#define SCAN_KEY9		0xe01f  //"9"
#define SCAN_KEY0		0x22dd	//"0"
#define SCAN_KEY10		0xFFFF	//"10+" , no mapping

#define SCAN_GOTO		0xf807	//
#define SCAN_TITLE		0x807f	//<<<<<<<<<<<<< not implement
#define SCAN_MENU		0xFFFF	//menu, no mapping
#define SCAN_UP			0x827d	//up
#define SCAN_DOWN		0xa25d	//down
#define SCAN_LEFT		0xb24d	//left
#define SCAN_RIGHT		0xba45	//right
#define SCAN_ENTER		0x8877	//
#define SCAN_SETUP		0x50af	//
#define SCAN_RETURN	    0xFFFF	//, no mapping

#define SCAN_STOP		0x1ae5	//
#define SCAN_PLAY		0x0af5	//
#define SCAN_PAUSE		0xFFFF	//pause, no mapping
#define SCAN_FB			0xFFFF	//fast backward
#define SCAN_FF			0xFFFF	//fast forward
#define SCAN_LAST		0xb847	//
#define SCAN_NEXT		0x00ff	//

#define SCAN_AB			0xFFFF	//A_B
#define SCAN_REPEAT		0xc837  //
#define SCAN_STEP		0xFFFF	//, no mapping
#define SCAN_PBC		0xFFFF	//, no mapping
#define SCAN_ZOOM		0xFFFF	//, no mapping
#define SCAN_MUTE		0x708f	//
#define SCAN_VOLDN		0x9a65	//
#define SCAN_VOLUP		0x8a75	//
#define SCAN_SLOW		0xFFFF	//, no mapping
#define SCAN_RANDOM	    0xFFFF	//, no mapping
#define SCAN_KEYDEC		0xFFFF	//, no mapping
#define SCAN_KEYADD	    0xFFFF	//, no mapping

#define SCAN_PROGRAM	0xFFFF	//, no mapping
#define SCAN_CLEAR		0x12ed	//<<<<<<<<<<<<<< factory reset not implement
#define SCAN_RESUME	    0xFFFF	//, no mapping
#define SCAN_VIDEO		0x10ef	//<<<<<<<<<<<<<VIDEO mode, not implement
#define SCAN_PN			0x6a95	//<<<<<<<<<<< not implement
#define SCAN_DIGEST		0x609f	//<<<<<<<<<<<<PICTURE mode,  not implement
#define SCAN_HDMI		0x32cd	//<<<<<<<<<<< not implement
#define SCAN_FUNCTION	0xa05f	//<<<<<<<<<<< not implement
#define SCAN_EPG		0xFFFF	//, no mapping
#define SCAN_TELETEXT	0xFFFF	//, no mapping
#define SCAN_SERVICE	0xFFFF	//, no mapping
#define SCAN_EXIT		0xFFFF	//, no mapping
#define SCAN_GUIDE		0xFFFF	//, no mapping
#define SCAN_LIST		0xFFFF	//, no mapping
#define SCAN_BROWSE	    0xFFFF	//, no mapping
#define SCAN_RADIO		0x926d	//<<<<<<<<<<<<<<<AUDIO mode,  not implement
#define SCAN_FADEC		0xFFFF	//, no mapping
#define SCAN_FAADD		0xFFFF	//, no mapping


/*****************************************************************************
*
* External Variables
*
******************************************************************************/
#define NO_KEY		0

static SCANCODE_MAPPING key_mapping[] = 
{
	{SCAN_PWR, REM_PWR},	
	{SCAN_LOAD, REM_LOAD},
	{SCAN_SUBTITLE, REM_SUBTITLE},
	{SCAN_AUDIO, REM_AUDIO},	
	{SCAN_ANGLE, REM_ANGLE},
	{SCAN_INFO, REM_INFO},

	{SCAN_KEY1, REM_KEY1},	//"1"
	{SCAN_KEY2, REM_KEY2},	//"2"
	{SCAN_KEY3, REM_KEY3},	//"3"
	{SCAN_KEY4, REM_KEY4},	//"4"
	{SCAN_KEY5, REM_KEY5},	//"5"
	{SCAN_KEY6, REM_KEY6},	//"6"
	{SCAN_KEY7, REM_KEY7},	//"7"
	{SCAN_KEY8, REM_KEY8},	//"8"
	{SCAN_KEY9, REM_KEY9},	//"9"
	{SCAN_KEY0, REM_KEY0},	//"0"
	{SCAN_KEY10, REM_KEY10},//"10+"

	{SCAN_GOTO, REM_GOTO},	//GOTO
	{SCAN_TITLE, REM_TITLE},
	{SCAN_MENU, REM_MENU},
	{SCAN_UP, REM_UP},	
	{SCAN_DOWN, REM_DOWN},	
	{SCAN_LEFT, REM_LEFT},	
	{SCAN_RIGHT, REM_RIGHT},
	{SCAN_ENTER, REM_ENTER},
	{SCAN_SETUP, REM_SETUP},
	{SCAN_RETURN, REM_RETURN},
                    	
	{SCAN_STOP, REM_STOP},
	{SCAN_PLAY, REM_PLAY},
	{SCAN_PAUSE, REM_PAUSE},
	{SCAN_FB, REM_FB},
	{SCAN_FF, REM_FF},
	{SCAN_LAST, REM_LAST},
	{SCAN_NEXT, REM_NEXT},

	{SCAN_AB, REM_AB},
	{SCAN_REPEAT, REM_REPEAT},
	{SCAN_STEP, REM_STEP},	
	{SCAN_PBC, REM_PBC},
	{SCAN_ZOOM, REM_ZOOM},	
	{SCAN_MUTE, REM_MUTE},
	{SCAN_VOLDN, REM_VOLDN},	
	{SCAN_VOLUP, REM_VOLUP},
	{SCAN_SLOW, REM_SLOW},	
	{SCAN_RANDOM, REM_RANDOM},	
	{SCAN_KEYDEC, REM_KEYDEC},	
	{SCAN_KEYADD, REM_KEYADD},	

	{SCAN_PROGRAM, REM_PROGRAM},
	{SCAN_CLEAR, REM_CLEAR},	
	{SCAN_RESUME, REM_RESUME},	
	{SCAN_VIDEO, REM_VIDEO},	
	{SCAN_PN, REM_PN},
	{SCAN_DIGEST, REM_DIGEST},	
	{SCAN_HDMI, REM_HDMI},	
	{SCAN_FUNCTION, REM_FUNCTION},
	{SCAN_EPG, REM_EPG},	
	{SCAN_TELETEXT, REM_TELETEXT},
	{SCAN_SERVICE, REM_SERVICE},	
	{SCAN_EXIT, REM_EXIT},	

	{SCAN_GUIDE, REM_GUIDE},
	{SCAN_LIST, REM_LIST},	
	{SCAN_BROWSE, REM_BROWSE},
	{SCAN_RADIO, REM_RADIO},
	{SCAN_FADEC, REM_FADEC},
	{SCAN_FAADD, REM_FAADD},	
/*
	{SCAN_MODE, REM_MODE},	
	{SCAN_LANG, REM_LANG},
	{SCAN_CHAR, REM_CHAR},	
*/
};
