
/****************************************************************************
 *   FileName    : tca_remocon.h
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

//*******************************************
//	Remote controller define
//*******************************************

//Low Bit
#if (defined(CONFIG_ARCH_TCC92X) || defined(CONFIG_ARCH_TCC93XX))
#define LOW_MIN			0x0700
#define LOW_MAX			0x0B00
//High Bit
#define HIGH_MIN		0x1800
#define HIGH_MAX		0x1D00
//Start Bit
#define START_MIN		0x4000
#define START_MAX		0x5000
//Repeat Bit
#define REPEAT_MIN		0x2000
#define REPEAT_MAX		0x2700
#else
//Low Bit
#define LOW_MIN			0x0500
#define LOW_MAX			0x0B00
//High Bit
#define HIGH_MIN		0x1000
#define HIGH_MAX		0x1D00
//Start Bit
#define START_MIN		0x2500
#define START_MAX		0x5000
#endif


/*****************************************************************************
*
* IR remote controller scancode
*
******************************************************************************/
#define REM_ID_ERROR	0xFF

#define REMOCON_ID		0x007F
#define REMOCON_REPEAT	0x0
#define SCAN_PWR		0xDD22
#define SCAN_LOAD		0xBC43
#define SCAN_SUBTITLE	0xD926
#define SCAN_AUDIO		0xFC03
#define SCAN_ANGLE		0xF807
#define SCAN_INFO		0xB847
#define SCAN_NUM_1		0xC936
#define SCAN_NUM_2		0xE817
#define SCAN_NUM_3		0xA857
#define SCAN_NUM_4		0xC13E
#define SCAN_NUM_5		0xE01F
#define SCAN_NUM_6		0xA05F
#define SCAN_NUM_7		0xD12e
#define SCAN_NUM_8		0xF00F
#define SCAN_NUM_9		0xB04F
#define SCAN_NUM_0		0xCD32
#define SCAN_NUM_10		0xEC13
#define SCAN_GOTO		0xAC53
#define SCAN_TITLE		0xD52A
#define SCAN_UP			0xF40B
#define SCAN_MENU		0xB44B
#define SCAN_LEFT		0xB54A
#define SCAN_ENTER		0xE41B
#define SCAN_RIGHT		0xB946
#define SCAN_SETUP		0xC53A
#define SCAN_DOWN		0xA55A
#define SCAN_RETURN		0xA45B
#define SCAN_STOP		0xAD52
#define SCAN_PLAY		0xA15E
#define SCAN_PAUSE		0xBD42
#define SCAN_FB			0xA956
#define SCAN_FF			0xB14E
#define SCAN_LAST		0xC43B
#define SCAN_NEXT		0x847B
#define SCAN_AB			0xF50A
#define SCAN_REPEAT		0x956A
#define SCAN_STEP		0xD42B
#define SCAN_PBC		0x946B
#define SCAN_ZOOM		0xED12
#define SCAN_MUTE		0x8D72
#define SCAN_VOLDN		0xCC33
#define SCAN_VOLUP		0x8C73
#define SCAN_SLOW		0xF10E
#define SCAN_RANDOM		0x916E
#define SCAN_KEYDEC		0xD02F
#define SCAN_KEYADD		0x906F
#define SCAN_PROGRAM	0xE11E
#define SCAN_CLEAR		0x817E
#define SCAN_RESUME		0xC03F
#define SCAN_VIDEO		0x807F
#define SCAN_PN			0xE916
#define SCAN_DIGEST		0x8976
#define SCAN_HDMI		0xC837
#define SCAN_FUNCTION	0x8877
#define SCAN_EPG		0xF906
#define SCAN_TELETEXT	0x9966
#define SCAN_SERVICE	0xD827
#define SCAN_EXIT		0x9867
#define SCAN_GUIDE		0xFD02
#define SCAN_LIST		0x9D62
#define SCAN_BROWSE		0xDC23
#define SCAN_RADIO		0x9C63
#define SCAN_FADEC		0xE51A
#define SCAN_FAADD		0x857A


/*****************************************************************************
*
* External Variables
*
******************************************************************************/
#define NO_KEY		0

static SCANCODE_MAPPING key_mapping[] = 
{
	{SCAN_PWR,	 		116},	// POWER
//	{SCAN_LOAD,			0},		// OPEN/CLOSE
	{SCAN_SUBTITLE,		207},	// NOTIFICATION
	{SCAN_AUDIO,		237},	// F5
	{SCAN_ANGLE,		236},	// F4
	{SCAN_INFO,			233},	// F1
	{SCAN_NUM_1,		2},		// 1
	{SCAN_NUM_2,		3},		// 2
	{SCAN_NUM_3,		4},		// 3
	{SCAN_NUM_4,		5},		// 4
	{SCAN_NUM_5,		6},		// 5
	{SCAN_NUM_6,		7},		// 6
	{SCAN_NUM_7,		8},		// 7
	{SCAN_NUM_8,		9},		// 8
	{SCAN_NUM_9,		10},	// 9
	{SCAN_NUM_0,		11},	// 0
	{SCAN_NUM_10,		12},	// MINUS
	{SCAN_GOTO,			52},	// PERIOD
//	{SCAN_TITLE,		0},		// TITLE
	{SCAN_UP,			103},	// DPAD_UP
	{SCAN_MENU,			139},	// MENU
	{SCAN_LEFT,			105},	// DPAD_LEFT
#if 0
	{SCAN_ENTER,		232},	// DPAD_CENTER
#else
	{SCAN_ENTER,		28},	// ENTER
#endif
	{SCAN_RIGHT,		106},	// DPAD_RIGHT
	{SCAN_SETUP,		102},	// HOME
	{SCAN_DOWN,			108},	// DPAD_DOWN
	{SCAN_RETURN,		158},	// BACK
	{SCAN_STOP,			199},	// STOP
	{SCAN_PLAY,			198},	// PLAY_PAUSE
	{SCAN_PAUSE,		198},	// PLAY_PAUSE
#if 1
	{SCAN_FB,			201},	// MEDIA_REWIND
	{SCAN_FF,			202},	// MEDIA_FAST_FORWARD
#else
	{SCAN_FB,			204},	// MEDIA_PREVIOUS
	{SCAN_FF,			203},	// MEDIA_NEXT
#endif
	{SCAN_LAST,			206},	// PAGE_DOWN
	{SCAN_NEXT,			205},	// PAGE_UP
//	{SCAN_AB,			13},	// AB
//	{SCAN_REPEAT,		13},	// REPEAT
	{SCAN_STEP,			15},	// TAB
//	{SCAN_PBC,			13},	// PBC
//	{SCAN_ZOOM,			13},	// ZOOM
	{SCAN_MUTE,			200},	// MUTE
	{SCAN_VOLDN,		114},	// VOLDN
	{SCAN_VOLUP,		115},	// VOLUP
//	{SCAN_SLOW,			13},	// SLOW
//	{SCAN_RANDOM,		13},	// RANDOM
//	{SCAN_KEYDEC,		13},	// KEYDEC
//	{SCAN_KEYADD,		13},	// KEYADD
//	{SCAN_PROGRAM,		13},	// PROGRAM
	{SCAN_CLEAR,		14},	// DEL
//	{SCAN_RESUME,		13},	// RESUME
//	{SCAN_VIDEO,		13},	// VIDEO
//	{SCAN_PN,			13},	// P/N
//	{SCAN_DIGEST,		13},	// DIGEST
//	{SCAN_HDMI,			13},	// HDMI
//	{SCAN_FUNCTION,		13},	// FUNCTION
//	{SCAN_EPG,			13},	// EPG
//	{SCAN_TELETEXT,		13},	// TELETEXT
//	{SCAN_SERVICE,		13},	// SERVICE
//	{SCAN_EXIT,			13},	// EXIT
//	{SCAN_GUIDE,		102},	// GUIDE
//	{SCAN_LIST,			13},	// LIST
//	{SCAN_BROWSE,		13},	// BROWSE
//	{SCAN_RADIO,		13},	// TV/RADIO
//	{SCAN_FADEC,		13},	// FADEC
//	{SCAN_FAADD,		13},	// FAADD
};
