/*
 * linux/driver/net/tcc_gmac/tcc93x_gmac.h
 * 	
 * Author : Telechips <linux@telechips.com>
 * Created : June 22, 2010
 * Description : This is the driver for the Telechips MAC 10/100/1000 on-chip Ethernet controllers.  
 *               Telechips Ethernet IPs are built around a Synopsys IP Core.
 *
 * Copyright (C) 2010 Telechips
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms and conditions of the GNU General Public License,
 * version 2, as published by the Free Software Foundation.
 *
 * This program is distributed in the hope it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License for
 * more details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin St - Fifth Floor, Boston, MA 02110-1301 USA.
 *
 * The full GNU General Public License is included in this distribution in
 * the file called "COPYING".
 */ 


#include <bsp.h>

#include <linux/clk.h>

static void tcc93x_gmac_clk_enable(void)
{
	struct clk *hsio_clk=NULL;
	struct clk *gmac_clk=NULL;
	struct clk *gdma_clk=NULL;
	
	hsio_clk = clk_get(NULL, "hsio");
	if(hsio_clk)
		clk_enable(hsio_clk);

	gmac_clk = clk_get(NULL, "gmac");
	if(gmac_clk)
		clk_enable(gmac_clk);

	gdma_clk = clk_get(NULL, "hs_gdma");
	if(gdma_clk)
		clk_enable(gdma_clk);
	//tca_ckc_setcommonhsiobus(HSIOBUS_GMAC, ENABLE);

	clk_set_rate(hsio_clk, 2500000);
	
	tca_ckc_setperi(PERI_GMAC,ENABLE,1250000,PCDIRECTPLL4);
}

static void tcc93x_gmac_clk_disable(void)
{
	struct clk *hsio_clk=NULL;
	struct clk *gmac_clk=NULL;
	struct clk *gdma_clk=NULL;
	
	hsio_clk = clk_get(NULL, "hsio");
	gmac_clk = clk_get(NULL, "gmac");
	gdma_clk = clk_get(NULL, "hs_gdma");
	
	tca_ckc_setperi(PERI_GMAC,DISABLE,1250000,PCDIRECTPLL4);
	//tca_ckc_setcommonhsiobus(HSIOBUS_GMAC, DISABLE);
	if(gdma_clk)
		clk_disable(gdma_clk);
	
	if(gmac_clk)
		clk_disable(gmac_clk);

	if(hsio_clk)
		clk_disable(hsio_clk);

}

static void tcc93x_gmac_phy_pwr_on(void)
{
	volatile PGPIO pGPIO= (PGPIO)tcc_p2v(HwGPIO_BASE);

	BITSET(pGPIO->GPFDAT, Hw29); //Phy Chip VDD on
	msleep(100);
}

static void tcc93x_gmac_phy_pwr_off(void)
{
	volatile PGPIO pGPIO= (PGPIO)tcc_p2v(HwGPIO_BASE);

	BITCLR(pGPIO->GPFDAT, Hw29); //Phy Chip VDD on
}

static void tcc93x_gmac_phy_reset(void)
{
	volatile PGPIO pGPIO= (PGPIO)tcc_p2v(HwGPIO_BASE);

	BITCLR(pGPIO->GPFDAT, Hw27); //Phy Reset On
	msleep(10);
	BITSET(pGPIO->GPFDAT, Hw27); //Phy Reset Off
	msleep(1000);
}

static void tcc93x_gmac_portinit(void)
{
	volatile PHSIOBUSCFG pHSIOCFG= (PHSIOBUSCFG)tcc_p2v(HwHSIOBUSCFG_BASE);
	volatile PGPIO pGPIO= (PGPIO)tcc_p2v(HwGPIO_BASE);


	/* Port Confiugration for GMAC (GPIOF 8~26)*/
//	BITCSET(pGPIO->GPFFN0, Hw32-Hw0, (Hw29|Hw28) | (Hw25|Hw24) | (Hw21|Hw20) | (Hw17|Hw16) 
//									  | (Hw13|Hw12) | (Hw9 |Hw8 ) | (Hw5 |Hw4 ) | (Hw1 |Hw0 ));
	BITCSET(pGPIO->GPFFN1, Hw32-Hw0, (Hw29|Hw28) | (Hw25|Hw24) | (Hw21|Hw20) | (Hw17|Hw16) 
									  | (Hw13|Hw12) | (Hw9 |Hw8 ) | (Hw5 |Hw4 ) | (Hw1 |Hw0 ));
	#if defined(CONFIG_MACH_TCC9300ST)
	//port 17 gpio for SD 
	BITCSET(pGPIO->GPFFN2, Hw32-Hw0, (Hw29|Hw28) | (Hw25|Hw24) | (Hw21|Hw20) | (Hw17|Hw16) 
									  | (Hw13|Hw12) | (Hw9 |Hw8 ) | (Hw1 |Hw0 ));
	#else
	BITCSET(pGPIO->GPFFN2, Hw32-Hw0, (Hw29|Hw28) | (Hw25|Hw24) | (Hw21|Hw20) | (Hw17|Hw16) 
									  | (Hw13|Hw12) | (Hw9 |Hw8 ) | (Hw5 |Hw4 ) | (Hw1 |Hw0 ));
	#endif
	BITCSET(pGPIO->GPFFN3, Hw32-Hw0, (Hw9 |Hw8 ) | (Hw5 |Hw4 ) | (Hw1 |Hw0 ));

#if 0
	BITSET(pGPIO->GPFPD0, Hw32-Hw0); //Pull-up Enable
	BITSET(pGPIO->GPFPD1, Hw22-Hw0); //Pull-up Enable
#endif

#if 0
	BITCSET(pGPIO->GPFPD0, Hw32-Hw0, Hw30|Hw28|Hw26|Hw24|Hw22
									|Hw20|Hw18|Hw16|Hw14|Hw12
									|Hw10|Hw8|Hw6|Hw4|Hw2|Hw0); //Pull-Down Enable
	BITCSET(pGPIO->GPFPD1, Hw22-Hw0, Hw20|Hw18|Hw16|Hw14|Hw12
									|Hw10|Hw8|Hw6|Hw4|Hw2|Hw0); //Pull-Down Enable
#endif

#if 1
	/* Driver Strength */
	//BITSET(pGPIO->GPFCD0, Hw32-Hw0);
	//BITSET(pGPIO->GPFCD1, Hw22-Hw0);
	// 8 ~ 15 settting
	BITCSET(pGPIO->GPFCD0, Hw32-Hw16,  Hw30|Hw28|Hw26|Hw24|Hw22
									|Hw20|Hw18|Hw16);
	// 16 ~ 26 setting
	BITCSET(pGPIO->GPFCD1, Hw22-Hw0,  Hw20|Hw18|Hw16|Hw14|Hw12
									|Hw10|Hw8|Hw6|Hw4|Hw2|Hw0);
#endif

	/* Port Configuration for VDD, Phy Rest pin */
	BITCLR(pGPIO->GPFFN3, (Hw24-Hw20) | (Hw16-Hw12)); //GPIO Mode
	BITSET(pGPIO->GPFEN, Hw29|Hw27); // Output Enable
	BITCLR(pGPIO->GPFIEN, Hw29|Hw27); //Input Disable

	BITCLR(pGPIO->GPFPD1, Hw23|Hw22); //Reset Pin Pull-up/down disable

#if defined(CONFIG_TCC_GMAC_MII_MODE)||defined(CONFIG_TCC_GMAC_GMII_MODE)
	BITCSET(pHSIOCFG->ETHER_CFG, Hw6-Hw0, Hw4|Hw5);
#elif defined(CONFIG_TCC_GMAC_RGMII_MODE)
	BITCSET(pHSIOCFG->ETHER_CFG, Hw6-Hw0, Hw1 | Hw5);
#endif
}

static void tcc93x_gmac_irq_init(void)
{
	volatile PPIC pPIC= (volatile PPIC) tcc_p2v(HwPIC_BASE);

	BITSET(pPIC->INTMSK1, 1 <<(INT_GMAC-32));
	BITSET(pPIC->SEL1, 1 <<(INT_GMAC-32));
	BITCLR(pPIC->POL1, 1 <<(INT_GMAC-32));	// Active Low
	BITSET(pPIC->MODE1, 1 <<(INT_GMAC-32));	// Level Triggered
	BITSET(pPIC->SYNC1, 1 <<(INT_GMAC-32));	// Sync to SMU Clk
}

static void tcc_gmac_set_clk_rate(struct tcc_gmac_priv *priv, unsigned int clk_rate)
{
	switch(clk_rate) {
		case 1250000:
			tca_ckc_setperi(PERI_GMAC,ENABLE,1250000,PCDIRECTPLL4);
			priv->hw->clk_rate = GMII_CLK_RANGE_100_150M;
			break;
		case 250000:
			tca_ckc_setperi(PERI_GMAC,ENABLE,250000,PCDIRECTPLL4);
			priv->hw->clk_rate = GMII_CLK_RANGE_20_35M;
			break;
		case 25000:
			tca_ckc_setperi(PERI_GMAC,ENABLE,25000,PCDIRECTPLL4);
			priv->hw->clk_rate = GMII_CLK_RANGE_20_35M;
			break;
		default:
			tca_ckc_setperi(PERI_GMAC,ENABLE,1250000,PCDIRECTPLL4);
			priv->hw->clk_rate = GMII_CLK_RANGE_100_150M;
			break;
	}
}

#if 0

static void tcc_gmac_debug_gpio_init(void)
{
	volatile PGPIO pGPIO= (PGPIO)tcc_p2v(HwGPIO_BASE);

	BITCLR(pGPIO->GPCFN3, Hw32-Hw16); //GPIO Mode
	BITSET(pGPIO->GPCEN, Hw32-Hw28); // Output Enable
	BITCLR(pGPIO->GPCIEN, Hw32-Hw28); //Input Disable
	BITCLR(pGPIO->GPCDAT, Hw32-Hw28); // Ouput to low
}

static void tcc_gmac_debug_gpio_ctrl(u32 port, u32 val)
{
	volatile PGPIO pGPIO= (PGPIO)tcc_p2v(HwGPIO_BASE);

	if (val) {
		BITSET(pGPIO->GPCDAT, port);
	}
	else {
		BITCLR(pGPIO->GPCDAT, port);
	}
}

#endif

