
/****************************************************************************
 *   FileName    : tca_tccserial.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

/*****************************************************************************
*
* Include
*
******************************************************************************/

#include <bsp.h>
#include "tcc9200_serial.h"


/*****************************************************************************
*
* Defines
*
******************************************************************************/


/*****************************************************************************
* Function Name : tca_uart_channelselectint channel, int mode)
******************************************************************************
* Desription    : tca_serial_portinit
* Parameter     : int channel, int mode
* Return        : 
******************************************************************************/
int tca_uart_channelselect(int channel, int mode)
{
	
	switch(channel){
	case 0:
		return (mode?HwEXTREQ_UART0_RX:HwEXTREQ_UART0_TX);
		break;
	case 1:
		return (mode?HwEXTREQ_UART1_RX:HwEXTREQ_UART1_TX);
		break;
	case 2:
		return (mode?HwEXTREQ_UART2_RX:HwEXTREQ_UART2_TX);
		break;
	case 3:
		return (mode?HwEXTREQ_UART3_RX:HwEXTREQ_UART3_TX);
	default :
		break;
	
	}
	return 0;
}

/*****************************************************************************
* Function Name : tca_serial_portinit(int nCh, int nPort, unsigned long* pvGpioAddr, unsigned long* pvPortMuxAddr)
******************************************************************************
* Desription    : tca_serial_portinit
* Parameter     : int nCh, int nPort, unsigned long* pvGpioAddr, unsigned long* pvPortMuxAddr
* Return        : None
******************************************************************************/
void	tca_serial_portinit(int nCh, int nPort, unsigned long* pvGpioAddr, unsigned long* pvPortMuxAddr)
{

	PGPIO pGpio = (PGPIO)pvGpioAddr;
	PUARTPORTMUX pPortmux = (PUARTPORTMUX)pvPortMuxAddr;
	
	
	// ch5 cannot use DMA (spec.)
	// each value of CHSEL shuld be different
	
	
	
	/*
	if(nPort == 5){ //change ch
		BITCSET(pPortmux->CHSEL, 0x7 <<(4*nCh), nPort << (4*nCh));
		BITCSET(pPortmux->CHSEL, 0x7 <<(4*5), nCh << (4*5));
	}
	*/	


	BITCSET(pPortmux->CHSEL, 0x7 <<(4*0), 0 << (4*0));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*1), 1 << (4*1));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*2), 2 << (4*2));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*3), 5 << (4*3));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*4), 4 << (4*4));  //portmuxinit
	BITCSET(pPortmux->CHSEL, 0x7 <<(4*5), 3 << (4*5));  //portmuxinit
	
	switch(nPort){//do port set
	case 0:
		BITCSET(pGpio->GPEFN0, Hw4-Hw0, Hw0);
		BITCSET(pGpio->GPEFN0, Hw8-Hw4, Hw4);
		break;
	case 1:
		BITCSET(pGpio->GPEFN0, Hw24-Hw20, Hw20);
		BITCSET(pGpio->GPEFN0, Hw20-Hw16, Hw16);
		break;
	case 2:
		BITCSET(pGpio->GPEFN1, Hw4-Hw0, Hw0);
		BITCSET(pGpio->GPEFN1, Hw8-Hw4, Hw4);
		break;
	case 3:
		BITCSET(pGpio->GPDFN1, Hw28-Hw24, Hw24);
		BITCSET(pGpio->GPDFN1, Hw24-Hw20, Hw20);
		break;
	case 4:
		BITCSET(pGpio->GPDFN2, Hw8-Hw4, Hw4);
		BITCSET(pGpio->GPDFN2, Hw12-Hw8, Hw8);
		break;
	case 5:
		BITCSET(pGpio->GPDFN2, Hw8-Hw4, Hw4);
		BITCSET(pGpio->GPDFN2, Hw12-Hw8, Hw8);
		break;
	}
	
}


/*****************************************************************************
* Function Name : tca_serial_dmaclrinterrupt(unsigned nDmanum, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_serial_dmaclrinterrupt
* Parameter     : unsigned nDmanum, unsigned long* pVirtualDmaAddr
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_serial_dmaclrinterrupt(unsigned nDmanum, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(	nDmanum ){
	case 0:
		pDMA->CHCTRL0 |= Hw3; 
		break;
	case 1:
		pDMA->CHCTRL1 |= Hw3; 
		break;
	case 2:
		pDMA->CHCTRL2 |= Hw3; 
		break;
	default:
		return 0;
	}
	return 1;
}

/*****************************************************************************
* Function Name : tca_dma_dmacurrentaddress(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_dmacurrentaddress
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int tca_dma_dmacurrentaddress(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(m_DmaNumber)
	{
	case 0:
		return  (pDMA->C_DADR0);
	case 1:
		return  (pDMA->C_DADR1);
	case 2:
		return  (pDMA->C_DADR2);
	default:
		return 0;
	}
}
/*****************************************************************************
* Function Name : tca_dma_dmadeststartaddress(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_dmadeststartaddress
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int     tca_dma_dmadeststartaddress(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(m_DmaNumber){
	case 0:
		return  (pDMA->ST_DADR0);
	case 1:
		return  (pDMA->ST_DADR1);
	case 2:
		return  (pDMA->ST_DADR2);
	default:
		return 0;
	}
	return 0;
}
/*****************************************************************************
* Function Name : tca_dma_clrien(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_clrien
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int     tca_dma_clrien(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(m_DmaNumber){
	case 0:
		 pDMA->CHCTRL0 &= Hw2; 
		break;
	case 1:
		 pDMA->CHCTRL1 &= Hw2; 
		break;
	case 2:
		 pDMA->CHCTRL2 &= Hw2; 
		break;
	default:
		return 0;
	}
	return 0;
}
/*****************************************************************************
* Function Name : tca_dma_setien(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_setien
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int     tca_dma_setien(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;
	switch(m_DmaNumber){
	case 0:
		 pDMA->CHCTRL0 |= Hw2; 
		break;
	case 1:
		 pDMA->CHCTRL1 |= Hw2; 
		break;
	case 2:
		 pDMA->CHCTRL2 |= Hw2; 
		break;
	default:
		return 0;
	}
	return 0;
}
/*****************************************************************************
* Function Name : tca_dma_ctrl(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
******************************************************************************
* Desription    : tca_dma_ctrl
* Parameter     : int m_DmaNumber, unsigned long* pVirtualDmaAddr
* Return        : 
******************************************************************************/
int    tca_dma_ctrl(int m_DmaNumber, unsigned long* pVirtualDmaAddr)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;

	switch(m_DmaNumber){
	case 0:
		return  (pDMA->CHCTRL0);
	case 1:
		return  (pDMA->CHCTRL1);
	case 2:
		return  (pDMA->CHCTRL2);
	default:
		return 0;
	}
	
}


/*****************************************************************************
* Function Name : tca_dma_setconfig(
	iunsigned uCH,
	void *pSRT, 
	unsigned uSPARAM,
	void *pDST, 
	unsigned uDPARAM,
	unsigned	uCHCTRL,
	unsigned uSize,
	unsigned channel,
	unsigned mode,
	unsigned long* pVirtualDmaAddr
)
******************************************************************************
* Desription    : tca_dma_setconfig
* Parameter     : 
* Return        : 
******************************************************************************/
void tca_dma_setconfig(
    unsigned uCH,
	void* pSRT, 
	unsigned uSPARAM,
	void* pDST, 
	unsigned uDPARAM,
	unsigned	uCHCTRL,
	unsigned uSize,
	unsigned channel,
	unsigned mode,
	unsigned long* pVirtualDmaAddr
)
{
	PGDMACTRL pDMA = (PGDMACTRL)pVirtualDmaAddr;


	switch( uCH )   {
	case 0:
		if (uSize){
        	// Set Source Address & Source Parameter (mask + increment)
			 pDMA->ST_SADR0 = (int)pSRT;
			 pDMA->SPARAM0 = uSPARAM;
        	
			 // Set Dest Address & Dest Parameter (mask + increment)
			 pDMA->ST_DADR0 = (int)pDST;
			 pDMA->DPARAM0 = uDPARAM;
			 pDMA->RPTCTRL0 &= ~Hw31;
			 

			 pDMA->EXTREQ0 = tca_uart_channelselect(channel, mode);
			 pDMA->HCOUNT0 = uSize;
		}

		 pDMA->CHCTRL0 = uCHCTRL;
		 pDMA->CHCTRL0 |= Hw0;
		break;
	case 1:
		if (uSize){
		// Set Source Address & Source Parameter (mask + increment)
			 pDMA->ST_SADR1 = (int)pSRT;
			 pDMA->SPARAM1 = uSPARAM;

        	// Set Dest Address & Dest Parameter (mask + increment)
			 pDMA->ST_DADR1 = (int)pDST;
			 pDMA->DPARAM1 = uDPARAM;
			 pDMA->RPTCTRL1 &= ~Hw31;
			 
			 pDMA->EXTREQ1 = tca_uart_channelselect(channel, mode);
			// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
			 pDMA->HCOUNT1 = uSize;//((uSize)>>0x01)>>0x03;
        	}

		 pDMA->CHCTRL1 = uCHCTRL;
		 pDMA->CHCTRL1 |= Hw0;
        break;
	case 2:
		if (uSize){
		// Set Source Address & Source Parameter (mask + increment)
			 pDMA->ST_SADR2 = (int)pSRT;
			 pDMA->SPARAM2 = uSPARAM;

        	// Set Dest Address & Dest Parameter (mask + increment)
			 pDMA->ST_DADR2 = (int)pDST;
			 pDMA->DPARAM2 = uDPARAM;
			 pDMA->RPTCTRL2 &= ~Hw31;
			 
			 pDMA->EXTREQ2 = tca_uart_channelselect(channel, mode);
			// now one HOP is 1*8/8 bytes, HwHCOUNT determine how many HOPS will fire a interrupt
			 pDMA->HCOUNT2 = uSize;//((uSize)>>0x01)>>0x03;
        	}

		 pDMA->CHCTRL2 = uCHCTRL;
		 pDMA->CHCTRL2 |= Hw0;
        break;
	}
}
