/*
 * linux/drivers/serial/tcc_serial.c
 *
 * Based on: drivers/serial/s3c2410.c and driver/serial/bfin_5xx.c
 * Author:  <linux@telechips.com>
 * Created: MAY 20, 2010
 * Description: Driver for onboard UARTs on the Telechips TCC Series
 *
 * Copyright (C) 2008-2010 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/module.h>
#include <linux/ioport.h>
#include <linux/platform_device.h>
#include <linux/init.h>
#include <linux/sysrq.h>
#include <linux/console.h>
#include <linux/tty.h>
#include <linux/tty_flip.h>
#include <linux/serial_core.h>
#include <linux/serial.h>
#include <linux/delay.h>
#include <linux/clk.h>
#include <linux/serial_reg.h>

#include <asm/io.h>
#include <asm/irq.h>

#include <mach/hardware.h>
#include <mach/irqs.h>

#if defined(CONFIG_MACH_TCC9300) || defined(CONFIG_MACH_TCC9300CM)
#include <bsp.h>
#include <mach/tcc_pca953x.h>
#include "tca_serial.h"
#include "tcc_serial.h"

#define DRV_NAME "tcc-uart"
#define CONSOLE_PORT	0
#define UART1           1     //_TODO_
#define	CONSOLE_BASE	*(volatile unsigned long *)0xF0102000
#define IO_ADDR_OFFSET	0x40000000

#elif defined(CONFIG_MACH_TCC9300ST)
#include <bsp.h>
#include <mach/tcc_pca953x.h>
#include "tca_serial.h"
#include "tcc_serial.h"

#define DRV_NAME "tcc-uart"
#define CONSOLE_PORT	1
#define UART1			1     //_TODO_
#define CONSOLE_BASE    *(volatile unsigned long *)0xF0102100
#define IO_ADDR_OFFSET	0x40000000

#endif

#define BAUDRATE  115200
#if defined(CONFIG_MACH_TCC9300ST)
#define UART_BT_PORT        2
#define UART_CONSOLE_PORT   1
#define UART_DUMMY_PORT		0
#else
#define UART_BT_PORT        1
#define UART_CONSOLE_PORT   0
#endif

//#define TX_RX_DATA_DEBUG      
//#define UART_AND_SLEEP_USE
//#define DMA_AND_NO_SLEEP_USE

#if defined(CONFIG_SERIAL_TCC_DMA)
//BlueZ Serial Driver Limitation [not allowed to use sleep func.]
//#define CONFIG_SERIAL_TCC_DMA_TX
#define CONFIG_SERIAL_TCC_DMA_RX

enum
{
	SERIAL_DMA_IDLE = 0,
	SERIAL_DMA_START,
	SERIAL_DMA_SENDING,
}SERIAL_DMA_STATE;

static int DMA_STATE = SERIAL_DMA_IDLE;
#endif

#if defined (CONFIG_SERIAL_TCC_DMA_TX) || defined (CONFIG_SERIAL_TCC_DMA_RX)
#include <linux/dma-mapping.h>

#define UART1           1
#define DMA_PORT	    UART1   // UART1 
#define DMA_TX_CH_NUM   0   // GDMA1-0
#define DMA_TX_INT	    INT_DMA1_CH0
#define DMA_BUF_SIZE    FIFOSIZE
#define DMA_TX_MODE     0
//#define HwGDMA_BASE     HwGDMA0_BASE  // GDMA1
#define HwGDMA_BASE     HwGDMA1_BASE  // GDMA1
//#define HwGDMA_BASE     HwGDMA2_BASE  // GDMA1
//#define HwGDMA_BASE     HwGDMA3_BASE  // GDMA1
#endif

#ifdef CONFIG_SERIAL_TCC_DMA_RX
#define DMA_CH_NUM_RX   1   // GDMA1-1
#define DMA_RX_MODE     1
#define TIME_STEP       1//(1*HZ)      
#define DMA_RX_BUF_SIZE 0x1000     
#include <linux/time.h>  
#include <linux/timer.h> 
#include <linux/delay.h>   

void *dma_rx_buffer, *dma_rx_buffer_physical;
struct timer_list  *timer;
static int rx_dma_tail=0;
void *kerneltimer_timeover(void *arg );
static int timer_state=0;
static DECLARE_WORK(work_queue, kerneltimer_timeover);
static unsigned long start_value;
#endif

#define TIMEOUT     jiffies_to_msecs(1000)

#define UART_IER_ELSI   UART_IER_RLSI 

/* UARTx_LCR Masks                                              */
#define WLS(x)      (((x)-5) & 0x03)    /* Word Length Select */
#define STB         0x04                /* Stop Bits            */
#define PEN         0x08                /* Parity Enable        */
#define EPS         0x10                /* Even Parity Select   */
#define SP          0x20                /* Stick Parity         */
#define SB          0x40                /* Set Break            */
#define DLAB        0x80                /* Divisor Latch Access */

/* UARTx_MCR Mask                                       */
#define AFE         0x20    /* Auto Flow Control Enable */
#define LOOP        0x10    /* Loopback Mode Enable     */
#define RTS         0x02    /* Request To Sent          */

/* UARTx_LSR Masks                                      */
#define DR          0x01    /* Data Ready               */
#define OE          0x02    /* Overrun Error            */
#define PE          0x04    /* Parity Error             */
#define FE          0x08    /* Framing Error            */
#define BI          0x10    /* Break Interrupt          */
#define THRE        0x20    /* THR Empty                */
#define TEMT        0x40    /* TSR and UART_THR Empty   */

/* UARTx_IER Masks */
#define ERXI    0x01
#define ETXI    0x02
#define ELSI    0x04

#define OFFSET_THR    0x00	/* Transmit Holding register            */
#define OFFSET_RBR    0x00	/* Receive Buffer register              */
#define OFFSET_DLL    0x00	/* Divisor Latch (Low-Byte)             */
#define OFFSET_IER    0x04	/* Interrupt Enable Register            */
#define OFFSET_DLM    0x04	/* Divisor Latch (High-Byte)            */
#define OFFSET_IIR    0x08	/* Interrupt Identification Register    */
#define OFFSET_FCR    0x08	/* FIFO Control Register                */
#define OFFSET_LCR    0x0C	/* Line Control Register                */
#define OFFSET_MCR    0x10	/* Modem Control Register               */
#define OFFSET_LSR    0x14	/* Line Status Register                 */
#define OFFSET_MSR    0x18	/* Modem Status Register                */
#define OFFSET_SCR    0x1C	/* SCR Scratch Register                 */
#define OFFSET_AFT    0x20	/* AFC Trigger Level Register           */
#define OFFSET_UCR    0x24	/* UART Control Register                */

#define portaddr(port, reg) ((port)->membase + (reg))

#define rd_regb(port, reg) (__raw_readb(portaddr(port, reg)))
#define rd_regl(port, reg) (__raw_readl(portaddr(port, reg)))

#define wr_regl(port, reg, val) \
    do { __raw_writel(val, portaddr(port, reg)); } while(0)
#define wr_regb(port, reg, val) \
    do { __raw_writeb(val, portaddr(port, reg)); } while(0)



/* structures */

struct tcc_uart_clksrc {
	const char	    *name;
	unsigned int	divisor;
	unsigned int	min_baud;
	unsigned int	max_baud;
};

struct tcc_uart_info {
	char			*name;
	unsigned int	type;
	unsigned int	fifosize;
};

struct tcc_reg_info {
	unsigned long	bDLL;
	unsigned long	bIER;
	unsigned long	bDLM;
	unsigned long	bLCR;
	unsigned long	bMCR;
	unsigned long	bSCR;
	unsigned long	bFCR;
	unsigned long	bMSR;
	unsigned long	bAFT;
	unsigned long	bUCR;
};

#ifdef CONFIG_SERIAL_TCC_DMA_TX
typedef struct _tcc_dma_buf_t {
    char *addr;
    dma_addr_t dma_addr;
    int  buf_size; // total size of DMA
} tcc_dma_buf_t;
#endif

struct tcc_uart_port {
	unsigned char			rx_claimed;
	unsigned char			tx_claimed;

	struct tcc_uart_info	*info;
	struct tcc_uart_clksrc	*clksrc;
	struct clk				*clk;
	struct clk				*baudclk;
	struct uart_port		port;
	struct semaphore 		sem;
    wait_queue_head_t wait_q;
////    int    fifosize;
    unsigned long	fifosize;			//// 100511
    char *name;
#ifdef CONFIG_SERIAL_TCC_DMA_TX
    wait_queue_head_t wait_dma_q;
	tcc_dma_buf_t	dma_tx_buffer;
	tcc_dma_buf_t	dma_tx_buffer_test;
	int tx_done;					////////////////// 100510
#endif
	struct tcc_reg_info		reg;	// for suspend/resume
};

/* configuration defines */

#if 0
//#define dbg(x...) printk(KERN_DEBUG "tcc uart: ");
//#define dbg printk
#define dbg
#else /* no debug */
#define dbg(x...) do {} while(0)
#endif

/* UART name and device definitions */

#define TCC_SERIAL_NAME	"ttySAC"
#define TCC_SERIAL_MAJOR	204
#define TCC_SERIAL_MINOR	64

/* conversion functions */

#define tcc_dev_to_port(__dev) (struct uart_port *)dev_get_drvdata(__dev)

/* Number of serial ports */
#define NR_PORTS (6)

#define FIFOSIZE (16) /* defend on FCR */

#define tx_enabled(port)	((port)->unused[0])
#define rx_enabled(port)	((port)->unused[1])
#define port_used(port)		((port)->unused1)

static void tcc_serial_console_putchar(struct uart_port *port, int ch);
extern void tcc_serial_uart_putchar(struct uart_port *port, int ch);

int tcc_serial_init_ports(struct tcc_uart_port *ourport, struct tcc_uart_info *info, struct platform_device *pdev);

static void tcc_serial_tx(struct uart_port *port);

static struct uart_ops tcc_serial_ops;

static struct tcc_uart_port tcc_serial_ports[NR_PORTS] = {
	[0] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[0].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART0,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 0,
		},
        .name = "uart0"
	},
	[1] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[1].port.lock),
			.iotype		= UPIO_MEM,
			.uartclk	= 0,
			.irq		= INT_UART1,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 1,
		},
        .name = "uart1"
	},
	[2] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[2].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART2,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 2,
		},
        .name = "uart2"
	},
	[3] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[3].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART3,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 3,
		},
        .name = "uart3"
	},
	[4] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[4].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART4,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 4,
		},
        .name = "uart4"
	},
	[5] = {
		.port = {
			.lock		= __SPIN_LOCK_UNLOCKED(tcc_uart_port[5].port.lock),
			.iotype		= UPIO_MEM,
			.irq		= INT_UART5,
			.uartclk	= 0,
			.fifosize	= FIFOSIZE,
			.ops		= &tcc_serial_ops,
			.flags		= UPF_BOOT_AUTOCONF,
			.line		= 5,
		},
        .name = "uart5"
	},

};


#ifdef CONFIG_SERIAL_TCC_DMA_RX
void kerneltimer_registertimer(struct timer_list* ptimer, unsigned long timeover,struct uart_port *port )
{
     init_timer( ptimer );
     ptimer->expires  = get_jiffies_64() + timeover;
     ptimer->data     = (unsigned long)port;
     ptimer->function = (void *)kerneltimer_timeover;
     add_timer( ptimer);
}

void my_uart_rx_process(unsigned long arg)
{
    volatile int current_tail,current_head, i, rts_state;
    struct uart_port *port;
    unsigned int ch = 0;
    unsigned uerstat = 0, flag;
    struct tty_struct *tty;
    unsigned char *buffer;

    volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);
    int cur_addr = tca_dma_dmacurrentaddress(DMA_CH_NUM_RX, (unsigned long *)pDMA);

    port = (struct uart_port *)arg;
    buffer = (unsigned char *)dma_rx_buffer;
    current_tail = rx_dma_tail & (DMA_RX_BUF_SIZE-1);
    current_head = cur_addr & (DMA_RX_BUF_SIZE-1);

    if (current_head != current_tail) 
    {
        if (current_head > current_tail) 
            rts_state = current_head - current_tail;
        else 
            rts_state = DMA_RX_BUF_SIZE - current_tail - current_head;

        if (rts_state > 2048) 
        {
            printk("RTS ON[0x%x : 0x%x]\n", cur_addr, current_tail);
            wr_regl(port, OFFSET_MCR, rd_regl(port, OFFSET_MCR) & ~Hw1);
        }

      spin_lock(port->lock);
      tty = port->info->port.tty;

      flag = TTY_NORMAL;
      port->icount.rx++;

      if (uerstat & UART_LSR_BI) {
          port->icount.brk++;
          goto out;
      }

      if (uerstat & UART_LSR_FE)
          port->icount.frame++;
      if (uerstat & UART_LSR_OE)
          port->icount.overrun++;

      uerstat &= port->read_status_mask;

      if (uerstat & UART_LSR_BI)
          flag = TTY_BREAK;
      else if (uerstat & UART_LSR_PE)
          flag = TTY_PARITY;
      else if (uerstat & ( UART_LSR_FE | UART_LSR_OE))
          flag = TTY_FRAME;      
      
      if (current_head < current_tail)
        current_head += (DMA_RX_BUF_SIZE);

      //printk("[#### BT ####] DMA_RX size[%d] head[%d] tail[%d]\n", (current_head-current_tail), current_head, current_tail);

      for(i=current_tail;i<current_head;i++) 
      {
          if (timer_state == 0) break;
          if (i >= DMA_RX_BUF_SIZE)
            ch = buffer[i-DMA_RX_BUF_SIZE];
          else
            ch = buffer[i];

          if (uart_handle_sysrq_char(port, ch))
              goto out;
          /* put the received char into UART buffer */
          uart_insert_char(port, uerstat, UART_LSR_OE, ch, flag);
          tty_flip_buffer_push(tty);
          rx_dma_tail++;
      }

      if (rts_state > 2048) 
      {
        printk("RTS OFF\n");
        wr_regl(port, OFFSET_MCR, rd_regl(port, OFFSET_MCR) | Hw1);
      }
out:
    spin_unlock(port->lock);

    }
}

void *kerneltimer_timeover(void *arg )
{
    volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);
    int cur_addr = tca_dma_dmacurrentaddress(DMA_CH_NUM_RX, (unsigned long *)pDMA);

    
    if (start_value == cur_addr) 
    {
        if (start_value == 0xC0000000) 
            my_uart_rx_process((unsigned long)arg);
    } 
    else 
    {
        if (start_value != 0xC0000000) 
        {
            rx_dma_tail = (int) dma_rx_buffer_physical;
            start_value = 0xC0000000;
        }

        my_uart_rx_process((unsigned long)arg);
    }

    if (timer_state == 1)
        kerneltimer_registertimer( timer, TIME_STEP,arg );

    return 0;
}

int kerneltimer_init(struct uart_port *port)
{
    timer= kmalloc( sizeof( struct timer_list ), GFP_KERNEL );
    if( timer== NULL ) return -ENOMEM;
    memset( timer, 0, sizeof( struct timer_list) );
    kerneltimer_registertimer( timer,TIME_STEP,port );

    return 0;
}
#endif

static inline struct tcc_uart_port *to_ourport(struct uart_port *port)
{
	return container_of(port, struct tcc_uart_port, port);
}

/* translate a port to the device name */

static inline const char *tcc_serial_portname(struct uart_port *port)
{
	return to_platform_device(port->dev)->name;
}

static void tcc_serial_stop_tx(struct uart_port *port)
{
	if(port->line == UART_BT_PORT)
	    printk("[#### BT ####] tcc_serial_stop_tx port[%d]\n", port->line);

#ifdef CONFIG_SERIAL_TCC_DMA_TX
	tcc_serial_ports[port->line].tx_done = 1;	//// 100510
#endif	

    while(!(rd_regl(port, OFFSET_LSR)) & TEMT)
        continue;

    if(tx_enabled(port)){
        disable_irq(port->irq);
#ifdef CONFIG_SERIAL_TCC_DMA_TX
       // disable_irq(DMA_TX_INT);
#endif
        tx_enabled(port) = 0;
    }
}

static void tcc_serial_start_tx(struct uart_port *port)
{
	if(!tx_enabled(port)) {
	        enable_irq(port->irq);
        tx_enabled(port) = 1;
    }
#ifdef CONFIG_SERIAL_TCC_DMA_TX
	if (port->line == DMA_PORT) {
		while (!(rd_regl(port, OFFSET_LSR) & THRE))
			barrier();
		
		if(tcc_serial_ports[port->line].tx_done){
			tcc_serial_tx(port);
			wr_regl(port, OFFSET_IER, rd_regl(port, OFFSET_IER) | Hw1);
		}
	}
	else {
		tcc_serial_tx(port);
		wr_regl(port, OFFSET_IER, rd_regl(port, OFFSET_IER) | Hw1);
	}	
#else		
	tcc_serial_tx(port);
	wr_regl(port, OFFSET_IER, rd_regl(port, OFFSET_IER) | Hw1);
#endif
}


static void tcc_serial_tx(struct uart_port *port)
{
        struct circ_buf *xmit = &port->info->xmit;
#ifdef CONFIG_SERIAL_TCC_DMA_TX
    volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);
		tcc_serial_ports[port->line].tx_done = 0;	//// 100510
#endif
		tcc_serial_ports[port->line].fifosize = uart_circ_chars_pending(xmit);
		if(tcc_serial_ports[port->line].fifosize > FIFOSIZE) {
			tcc_serial_ports[port->line].fifosize = FIFOSIZE;
			if((xmit->tail + tcc_serial_ports[port->line].fifosize) > UART_XMIT_SIZE ) {
				tcc_serial_ports[port->line].fifosize = UART_XMIT_SIZE - xmit->tail;
			}
		}
#ifdef CONFIG_SERIAL_TCC_DMA_TX
		if (uart_circ_empty(xmit)||uart_tx_stopped(port)) {
			wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) & ~Hw1));
			tcc_serial_ports[port->line].tx_done = 1;	//// 100510		

			if(port->line == DMA_PORT) {
////				printk("Empty!!! \n");
			}
			
			if (uart_circ_chars_pending(xmit) < WAKEUP_CHARS) {
				uart_write_wakeup(port);
			}			
            return;
        }		
#endif		
        if (port->x_char) {
            tcc_serial_uart_putchar(port, port->x_char);
            port->icount.tx++;
            port->x_char = 0;
            tcc_serial_ports[port->line].fifosize--;
        }
#ifdef CONFIG_SERIAL_TCC_DMA_TX		
		if(port->line == DMA_PORT)
		{
		////	printk("P:%d, fifo : %d\n", port->line,tcc_serial_ports[DMA_PORT].fifosize );
		     if(tcc_serial_ports[DMA_PORT].fifosize > 0)
		     {
		         wr_regl(port, OFFSET_UCR, rd_regl(port, OFFSET_UCR) & ~Hw0);
		         memcpy(tcc_serial_ports[DMA_PORT].dma_tx_buffer.addr, (xmit->buf+xmit->tail),tcc_serial_ports[DMA_PORT].fifosize );

		         //DMA Setting
		         tca_dma_setconfig(DMA_TX_CH_NUM,
		                           (void *)tcc_serial_ports[DMA_PORT].dma_tx_buffer.dma_addr, 	
		                           0x1, /* src Param */
		                           portaddr(port, 0x0) - IO_ADDR_OFFSET, 
		                           0x0, /* dest Param */
		                           HwCHCTRL_SYNC_EN        |
		                           HwCHCTRL_TYPE_SL        |
		                           HwCHCTRL_BSIZE_1        |
		                           HwCHCTRL_WSIZE_8        |
		                           HwCHCTRL_IEN_ON         |
		                           HwCHCTRL_FLAG           ,
		                           tcc_serial_ports[DMA_PORT].fifosize	,
		                           DMA_PORT, 
		                           DMA_TX_MODE, /* mode */
		                           (unsigned long *)pDMA); 
				 wr_regl(port, OFFSET_UCR, rd_regl(port, OFFSET_UCR) |Hw0);      
		     }
		}
		else {
#endif
			while (tcc_serial_ports[port->line].fifosize > 0){	
				if (uart_circ_empty(xmit)|| uart_tx_stopped(port)) {						
					wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) & ~Hw1));
					break;
				}
				else {
					tcc_serial_uart_putchar(port, xmit->buf[xmit->tail]);
					xmit->tail = (xmit->tail + 1) & (UART_XMIT_SIZE - 1);
					port->icount.tx++;
					tcc_serial_ports[port->line].fifosize--;
				}											
			} //// while				             
			if (uart_circ_chars_pending(xmit) < WAKEUP_CHARS) {
					uart_write_wakeup(port);				
			}
#ifdef CONFIG_SERIAL_TCC_DMA_TX			
		}
#endif		
}

static void tcc_serial_stop_rx(struct uart_port *port)
{
	dbg("%s\n", __func__);
    if (rx_enabled(port)) {
        disable_irq(port->irq);
        mdelay(10);
#ifdef CONFIG_SERIAL_TCC_DMA_TX
       // disable_irq(DMA_TX_INT);
#endif
        rx_enabled(port) = 0;
    }

    if(port->line == UART_BT_PORT){
#if defined(CONFIG_MACH_TCC9300)
        //tcc_pca953x_setup(PCA9539_U1_SLAVE_ADDR, BT_ON, OUTPUT, LOW, SET_DIRECTION|SET_VALUE);		
       // tcc_pca953x_config(GXP_BT_ON, OUTPUT, LOW, SET_DIRECTION|SET_VALUE);		
#endif
    }

}

static void tcc_serial_enable_ms(struct uart_port *port)
{
	dbg("%s\n", __func__);
}

static inline struct tcc_uart_info *tcc_port_to_info(struct uart_port *port)
{
	return to_ourport(port)->info;
}

#ifdef CONFIG_SERIAL_TCC_DMA_TX
static irqreturn_t tcc_serial_interrupt_dma(int irq, void *id)
{
	struct uart_port *port = id;
    volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);

    struct circ_buf *xmit = &port->info->xmit;

    spin_lock(port->lock);
    if (port->line == DMA_PORT) {
        /* XXX */
        xmit->tail = (xmit->tail + tcc_serial_ports[DMA_PORT].fifosize) & (UART_XMIT_SIZE - 1);
        port->icount.tx+= tcc_serial_ports[port->line].fifosize;	                     
        tcc_serial_ports[port->line].fifosize = 0;

        tca_dma_clrien(DMA_TX_CH_NUM, (unsigned long *)pDMA);
        tca_serial_dmaclrinterrupt(DMA_TX_CH_NUM, (unsigned long *)pDMA);

		tcc_serial_tx(port);

	    if (uart_circ_chars_pending(xmit) < WAKEUP_CHARS){
			uart_write_wakeup(port);
	    }
    }	
    spin_unlock(port->lock);
	return IRQ_HANDLED;
}
#endif


////DECLARE_WORK(n, f);

static irqreturn_t tcc_serial_interrupt(int irq, void *id)
{
	unsigned int ch = 0;
	struct uart_port *port = id;
	unsigned uerstat, flag;
	struct tty_struct *tty = port->info->port.tty;
    unsigned int iir_data = 0;

    uerstat = 0;

    spin_lock(port->lock);

    iir_data = rd_regl(port, OFFSET_IIR);
    iir_data = (iir_data & 0x0E) >> 1;
    if (iir_data == 2 || iir_data == 6) {
        ch = rd_regb(port, OFFSET_RBR);
		
#ifdef CONFIG_SERIAL_TCC_DMA_RX
	    if(port->line != DMA_PORT)
		{
#endif
	        flag = TTY_NORMAL;
	        port->icount.rx++;

	        if (uerstat & UART_LSR_BI) {
	            dbg("break!\n");
	            port->icount.brk++;
	            goto out;
	        }

	        if (uerstat & UART_LSR_FE)
	            port->icount.frame++;
	        if (uerstat & UART_LSR_OE)
	            port->icount.overrun++;

	        uerstat &= port->read_status_mask;

	        if (uerstat & UART_LSR_BI)
	            flag = TTY_BREAK;
	        else if (uerstat & UART_LSR_PE)
	            flag = TTY_PARITY;
	        else if (uerstat & ( UART_LSR_FE | UART_LSR_OE))
	            flag = TTY_FRAME;

	        if (uart_handle_sysrq_char(port, ch))
	            goto out;
	        /* put the received char into UART buffer */
	        uart_insert_char(port, uerstat, UART_LSR_OE, ch, flag);

	        tty_flip_buffer_push(tty);			
#ifdef CONFIG_SERIAL_TCC_DMA_RX
	    }
#endif

    }
	else if (iir_data == 1) {
#ifdef CONFIG_SERIAL_TCC_DMA_TX
			if (port->line == DMA_PORT) {
				while (!(rd_regl(port, OFFSET_LSR) & THRE))
						barrier();
			}
			else {
				tcc_serial_tx(port);
			}
#else
			tcc_serial_tx(port);
#endif
	}	
out:
    spin_unlock(port->lock);
	return IRQ_HANDLED;
}


static unsigned int tcc_serial_tx_empty(struct uart_port *port)
{
	unsigned short lsr;

	lsr = rd_regl(port, OFFSET_LSR);
	if (lsr & TEMT)
		return TIOCSER_TEMT;
	else
		return 0;

}

static unsigned int tcc_serial_get_mctrl(struct uart_port *port)
{
	dbg("%s\n", __func__);
    return TIOCM_CTS | TIOCM_DSR | TIOCM_CAR;
}

static void tcc_serial_set_mctrl(struct uart_port *port, unsigned int mctrl)
{
	dbg("%s\n", __func__);
	/* todo - possibly remove AFC and do manual CTS */
}

static void tcc_serial_break_ctl(struct uart_port *port, int break_state)
{
	dbg("%s\n", __func__);
}

static void tcc_serial_shutdown(struct uart_port *port)
{
	printk("[#### BT ####] tcc_serial_shutdown port[%d]\n", port->line);

	#if defined(CONFIG_CPU_FREQ_TCC92X) || defined (CONFIG_CPU_FREQ_TCC93XX)
	if(port->line == 1)
	;//	tcc_cpufreq_set_limit_table(&gtBtClockLimitTable, TCC_FREQ_LIMIT_BT, 0);
	#endif

#ifdef CONFIG_SERIAL_TCC_DMA_RX
    if(port->line == DMA_PORT)
    {      
      timer_state = 0;
      mdelay(1);
      kfree(timer);
    }
#endif

    wr_regl(port, OFFSET_IER, 0x0);
    free_irq(port->irq, port);
#ifdef CONFIG_SERIAL_TCC_DMA_TX
	if(port->line == DMA_PORT)
    	free_irq(DMA_TX_INT, port);
#endif
	port_used(port) = 0;	// for suspend/resume
}

#if defined(CONFIG_MACH_TCC8900)
int pwr_ioctl_uart(int onoff, struct uart_port *port )
{
	static int flag=1;

    volatile PGPIO pGPIO = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);
    volatile PPIC tPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);

    if (onoff) // ON 
    {
        tx_enabled(port) = 1;
        rx_enabled(port) = 1;
        port_used(port) = 1;	// for suspend/resume

        if(port->line == UART1){
            tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, BT_ON, OUTPUT, HIGH, SET_DIRECTION|SET_VALUE);
            dbg("tcc_uart: BT power %s\n", (tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, 0, OUTPUT, 0, GET_VALUE)&BT_ON)?"ON":"OFF");
        }

        tca_ckc_setiobus(port->line + RB_UARTCONTROLLER0 , 1);
        tca_ckc_setperi(port->line + PERI_UART0,ENABLE,520000,PCDIRECTPLL2);

        if(port->line == 0) {
            pGPIO->GPEEN    |= (Hw1|Hw0);
            pGPIO->GPEFN0   |= (Hw4|Hw0); // UTXD0, URXD0

        } else if(port->line == 1) {
            pGPIO->GPEEN    |= (Hw4|Hw5|Hw6|Hw7);
            BITCLR(pGPIO->GPEFN0, Hw31-Hw16);
            pGPIO->GPEFN0   |= (Hw28|Hw24|Hw20|Hw16); // UTXD1, URXD1, 
        }

        tPIC->INTMSK1   |= Hw15;

		flag = 1;
    } 
    else  // OFF
    {
		if (!flag)
			return 0;

        tx_enabled(port) = 0;
        rx_enabled(port) = 0;
        port_used(port) = 0;	// for suspend/resume
 
        if(port->line == UART1){
            tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, BT_ON, OUTPUT, LOW, SET_DIRECTION|SET_VALUE);
            dbg("tcc_uart: BT power %s\n", (tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, 0, OUTPUT, 0, GET_VALUE)&BT_ON)?"ON":"OFF");
        }

        if(port->line == 0) {
            pGPIO->GPEEN    &= ~(Hw1|Hw0);
            BITCLR(pGPIO->GPEFN0, Hw8-Hw0);
            BITCLR(pGPIO->GPEDAT, Hw4-Hw0);
        } else if(port->line == 1) {
            pGPIO->GPEEN    &= ~(Hw4|Hw5|Hw6|Hw7);
            BITCLR(pGPIO->GPEFN0, Hw31-Hw16);
            BITCLR(pGPIO->GPEDAT, Hw8-Hw4);
        }

        tPIC->INTMSK1   &= ~Hw15;

		flag = 0;
    }

	return 0;
}

static stpwrinfo pwrinfo = {PWR_STATUS_ON};

static int uart_pwr_ctl(void *h_private, int cmd, void *p_out)
{
    switch (cmd) {
    case PWR_CMD_OFF:
        dbg("PWR_CMD_OFF command ==> [%d]\n", cmd);
        pwr_ioctl_uart(0, h_private);
        pwrinfo.status = PWR_STATUS_OFF;
        break;
    case PWR_CMD_ON:
        dbg("PWR_CMD_ON command ==> [%d]\n", cmd);
        pwr_ioctl_uart(1, h_private);
        pwrinfo.status = PWR_STATUS_ON;
        break;
    case PWR_CMD_GETSTATUS:
        dbg("PWR_CMD_GETSTATUS command ==> [%d], status:[%d]\n", cmd, pwrinfo.status);
        memcpy(p_out, &pwrinfo, sizeof(stpwrinfo));
        break;
//  case PWR_CMD_MAX:
//      dbg ("PWR_CMD_MAX command ==> [%d]\n", cmd);
//      break;
    default:
        printk("unknown pwr command !!! ==> [%d]\n", cmd);
        break;
    }
    return 0;
}
#endif


/* while application opening the console device, this function will invoked
 * This function will initialize the interrupt handling 
 */
static int tcc_serial_startup(struct uart_port *port)
{
	int retval=0;
	unsigned int lcr;
	
    volatile PPIC tPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);
    volatile PGPIO pGPIO = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);
    volatile PUARTPORTMUX pPORTMUX = (volatile PUARTPORTMUX)tcc_p2v(HwUARTPORTMUX_BASE);
#ifdef CONFIG_SERIAL_TCC_DMA_RX
	volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);
#endif

#ifdef CONFIG_MACH_TCC9300ST
	if(port->line == UART_DUMMY_PORT)
		return retval;
#endif

    if(port->line == UART_BT_PORT)
	{
	    printk("[#### BT ####] tcc_serial_startup port[%d]\n", port->line);
#if defined(CONFIG_MACH_TCC9300)
        //tcc_pca953x_setup(PCA9539_U1_SLAVE_ADDR, BT_ON, OUTPUT, HIGH, SET_DIRECTION|SET_VALUE);		
		//dbg("tcc_uart: BT power %s\n", (tcc_pca953x_setup(PCA9539_U1_SLAVE_ADDR, 0, OUTPUT, 0, GET_VALUE)&BT_ON)?"ON":"OFF");
#endif
	}

    tx_enabled(port) = 1;
    rx_enabled(port) = 1;
	port_used(port) = 1;	// for suspend/resume

    /* Set default clock of UARTx: 480kHz */
    tca_ckc_setperi(port->line + PERI_UART0, ENABLE, 480000, PCDIRECTPLL1);

#if defined(CONFIG_SERIAL_TCC_DMA) && defined(CONFIG_MACH_TCC9300) 
    tca_ckc_setiobus(RB_GDMA1CONTROLLER , 1);
#endif

	if(port->line == UART_BT_PORT)
    {
    	#if defined(CONFIG_CPU_FREQ_TCC92X) || defined(CONFIG_CPU_FREQ_TCC93XX)
		//tcc_cpufreq_set_limit_table(&gtBtClockLimitTable, TCC_FREQ_LIMIT_BT, 1);
		#endif
	}	

    tca_serial_portinit(0, port->line, pGPIO, pPORTMUX);

    wr_regl(port, OFFSET_IER, 0x0);
    
    lcr = rd_regl(port, OFFSET_LCR);
    wr_regl(port, OFFSET_LCR, (lcr | Hw7));
    
    wr_regl(port, OFFSET_FCR, 0x07);
    tcc_serial_ports[port->line].fifosize = FIFOSIZE;
    tcc_serial_ports[port->line].reg.bFCR = 0x07;		// for resume restore

#ifdef CONFIG_SERIAL_TCC_DMA_TX
	tcc_serial_ports[port->line].tx_done  = 1;			//// 100510
#endif


    wr_regl(port, OFFSET_LCR, (lcr & (~Hw7)));
    rd_regl(port, OFFSET_IIR);
    
    retval = request_irq(port->irq, tcc_serial_interrupt, IRQF_SHARED, tcc_serial_ports[port->line].name , port);
    dbg("request %s serial irq:%d,retval:%d\n", tcc_serial_ports[port->line].name, port->irq, retval);

#ifdef CONFIG_SERIAL_TCC_DMA_TX
    if(port->line == DMA_PORT)
    	retval = request_irq(DMA_TX_INT, tcc_serial_interrupt_dma, IRQF_SHARED, "uart1_dma" , port);
    dbg("request serial dma irq:%d,retval:%d\n", DMA_TX_INT, retval);
    //pDMA->CHCTRL0 |= Hw2;
#endif

#ifdef CONFIG_SERIAL_TCC_DMA_RX
{
    int cur_addr = tca_dma_dmacurrentaddress(DMA_CH_NUM_RX, (unsigned long *)pDMA);

    if(port->line == DMA_PORT) 
    {
      printk("rx_dma_tail : %x\n",rx_dma_tail);

        wr_regl(port, OFFSET_MCR, rd_regl(port, OFFSET_MCR) | Hw5);
        wr_regl(port, OFFSET_MCR, rd_regl(port, OFFSET_MCR) | Hw1);
        wr_regl(port, OFFSET_AFT, 0x00000021);
    
        // Set Source Address & Source Parameter (mask + increment)

        wr_regl(port, OFFSET_UCR, rd_regl(port, OFFSET_UCR) & ~Hw1);

        tca_dma_setconfig(DMA_CH_NUM_RX,
                          portaddr(port, 0x0) - IO_ADDR_OFFSET, 
                          0x0, /* src Param */
                          dma_rx_buffer_physical,
                          0x1, /* dest Param */
                          //HwCHCTRL_CONT_C         |
                          HwCHCTRL_SYNC_EN        |
                          //HwCHCTRL_TYPE_SL        |   // sync 92 nucleus                          
                          HwCHCTRL_BSIZE_1        |
                          HwCHCTRL_WSIZE_8        |
                       //   HwCHCTRL_IEN_ON         |
                          HwCHCTRL_REP_EN         |
                          HwCHCTRL_FLAG           ,
                          DMA_RX_BUF_SIZE ,
                          DMA_PORT, 
                          DMA_RX_MODE, /* mode */
                          (unsigned long *)pDMA);

      wr_regl(port, OFFSET_UCR, rd_regl(port, OFFSET_UCR) |Hw1);

      start_value = cur_addr;
      if (cur_addr != 0xC0000000)
        rx_dma_tail = cur_addr;   			
      else
        rx_dma_tail = (int) dma_rx_buffer_physical;
      timer_state=1;
      kerneltimer_init(port);
    }
}
#endif

    wr_regl(port, OFFSET_IER, ERXI);							

#ifdef CONFIG_SERIAL_TCC_DMA_RX
    if(port->line == DMA_PORT) {
        printk("line %d IER tx Only\n", port->line);
		wr_regl(port, OFFSET_IER, 0);
    }
#endif

    //tca_serial_intrdone();
    tPIC->INTMSK1   |= Hw15;

    dbg("==== %s() IER[0x%04x] FCR[0x%04x] LCR[0x%04x] MCR[0x%04x] UCR[0x%04x]\n", __func__, 
            rd_regl(port, OFFSET_IER), rd_regl(port, OFFSET_FCR), rd_regl(port, OFFSET_LCR), rd_regl(port, OFFSET_MCR), rd_regl(port, OFFSET_UCR));
    dbg("==== CHSEL[0x%08x] \n", (*(volatile unsigned int *)(0xF0102600)));
	return retval;
}

/* power power management control */

static void tcc_serial_pm(struct uart_port *port, unsigned int level, unsigned int old)
{
}


static inline int tcc_serial_setsource(struct uart_port *port, struct tcc_uart_clksrc *c)
{
    dbg("%s() \n", __func__);
	return 0;
}

static void tcc_serial_set_termios(struct uart_port *port,
				       struct ktermios *termios,
				       struct ktermios *old)
{
	unsigned long flags;
	unsigned int baud, quot;
	unsigned int ulcon;
	//unsigned int umcon;
	unsigned int umcon, lsr;
    int uart_clk = 0;

#ifdef CONFIG_MACH_TCC9300ST
	if(port->line == UART_DUMMY_PORT)
		return;
#endif

	/*
	 * We don't support modem control lines.
	 */
	termios->c_cflag &= ~(HUPCL | CMSPAR);
	termios->c_cflag |= CLOCAL;

	/*
	 * Ask the core to calculate the divisor for us.
	 */

	printk("tcc_serial_set_termios port[%d]\n", port->line);
	
	baud = uart_get_baud_rate(port, termios, old, 0, 3500000);
	if(port->line == UART_BT_PORT)	// for BT Baudrate change while working
	{
		if(baud == 3000000)
		{
			tca_ckc_setiobus(port->line + RB_UARTCONTROLLER0, 1);
			tca_ckc_setperi(port->line + PERI_UART0, ENABLE, 480000, PCDIRECTPLL1);  // Console:115.2K BT:3M
		}
		else
		{
			tca_ckc_setiobus(port->line + RB_UARTCONTROLLER0, 1);
			tca_ckc_setperi(port->line + PERI_UART0, ENABLE, 147456, PCDIRECTPLL1);  // Console:115.2K BT:921.6K
		}
	}
	uart_clk = (unsigned int)tca_ckc_getperi(port->line + PERI_UART0);
	quot = (uart_clk*100)/(16*baud); 

    /*
     * set byte size
     */
	switch (termios->c_cflag & CSIZE) {
	case CS5:
		dbg("config: 5bits/char\n");
		ulcon = 0;
		break;
	case CS6:
		dbg("config: 6bits/char\n");
		ulcon = 1;
		break;
	case CS7:
		dbg("config: 7bits/char\n");
		ulcon = 2;
		break;
	case CS8:
	default:
		dbg("config: 8bits/char\n");
		ulcon = 3;
		break;
	}

	/* preserve original lcon IR settings */
	
	if (termios->c_cflag & CSTOPB)
		ulcon |= (1<<2); /* HwUART_LCR_STB_ONE */

	umcon = (termios->c_cflag & CRTSCTS) ? (Hw5|Hw1) : 0; /* HwUART_MCR_RTS_ON */

	if (termios->c_cflag & PARENB) {
		if (termios->c_cflag & PARODD)
			ulcon |= ((0<<4)|(1<<3));
		else
			ulcon |= ((1<<4)|(1<<3));
	} else {
		ulcon |= (0<<3);
	}

	spin_lock_irqsave(&port->lock, flags);

	do {
		lsr = rd_regl(port, OFFSET_LSR);
	} while (!(lsr & TEMT));

	//dbg("setting ulcon to %08x, brddiv to %d\n", ulcon, quot);
	dbg("setting ulcon to uclk[%d] %08x, brddiv to %d [%d]\n", uart_clk, ulcon, quot, baud);

    wr_regl(port, OFFSET_MCR, umcon);
    wr_regl(port, OFFSET_LCR, (ulcon | (1<<7)));
    if(quot > 0xFF) {
        wr_regl(port, OFFSET_DLL, quot & 0x00FF);
        wr_regl(port, OFFSET_DLM, quot >> 8);
    }else if (quot > 0) {
        wr_regl(port, OFFSET_DLL, quot);
        wr_regl(port, OFFSET_DLM, 0x0);
    }
    wr_regl(port, OFFSET_LCR, (ulcon & (~(1<<7))));

	/*
	 * Update the per-port timeout.
	 */
	uart_update_timeout(port, termios->c_cflag, baud);

	/*
	 * Which character status flags are we interested in?
	 */
	port->read_status_mask = 0;
	if (termios->c_iflag & INPCK)
		port->read_status_mask |= 0;

	/*
	 * Which character status flags should we ignore?
	 */
	port->ignore_status_mask = 0;
	if (termios->c_iflag & IGNPAR)
		port->ignore_status_mask |= 0;
	if (termios->c_iflag & IGNBRK && termios->c_iflag & IGNPAR)
		port->ignore_status_mask |= 0;

	/*
	 * Ignore all characters if CREAD is not set.
	 */
	if ((termios->c_cflag & CREAD) == 0)
		port->ignore_status_mask |= 0;

	spin_unlock_irqrestore(&port->lock, flags);
}

static const char *tcc_serial_type(struct uart_port *port)
{
    dbg("%s() \n", __func__);
    return tcc_serial_ports[port->line].name;
}

#define MAP_SIZE (0x100)

static void tcc_serial_release_port(struct uart_port *port)
{
    dbg("%s() \n", __func__);
    /* TODO */
	//release_mem_region(port->mapbase, MAP_SIZE);
}

static int tcc_serial_request_port(struct uart_port *port)
{
    dbg("%s() \n", __func__);
    return 0;
    /*
	return request_mem_region(port->mapbase, MAP_SIZE, "tcc7901") ? 0 : -EBUSY;
    */
}

static void tcc_serial_config_port(struct uart_port *port, int flags)
{
    dbg("%s() \n", __func__);
	if (flags & UART_CONFIG_TYPE && tcc_serial_request_port(port) == 0)
        port->type = PORT_TCC;
}

/*
 * verify the new serial_struct (for TIOCSSERIAL).
 */
static int tcc_serial_verify_port(struct uart_port *port, struct serial_struct *ser)
{
	struct tcc_uart_info *info = tcc_port_to_info(port);

    dbg("%s() \n", __func__);
	if (ser->type != PORT_UNKNOWN && ser->type != info->type)
		return -EINVAL;
	return 0;
}


#ifdef CONFIG_SERIAL_TCC_CONSOLE

static struct console tcc_serial_console;

#define TCC_SERIAL_CONSOLE &tcc_serial_console
#else
#define TCC_SERIAL_CONSOLE NULL
#endif

static struct uart_ops tcc_serial_ops = {
	.pm				= tcc_serial_pm,
	.tx_empty		= tcc_serial_tx_empty,
	.get_mctrl		= tcc_serial_get_mctrl,
	.set_mctrl		= tcc_serial_set_mctrl,
	.stop_tx		= tcc_serial_stop_tx,
	.start_tx		= tcc_serial_start_tx,
	.stop_rx		= tcc_serial_stop_rx,
	.enable_ms		= tcc_serial_enable_ms,
	.break_ctl		= tcc_serial_break_ctl,
	.startup		= tcc_serial_startup,
	.shutdown		= tcc_serial_shutdown,
	.set_termios	= tcc_serial_set_termios,
	.type			= tcc_serial_type,
	.release_port	= tcc_serial_release_port,
	.request_port	= tcc_serial_request_port,
	.config_port	= tcc_serial_config_port,
	.verify_port	= tcc_serial_verify_port,
};


static struct uart_driver tcc_uart_drv = {
	.owner		= THIS_MODULE,
	.dev_name	= "tcc-uart",
	.nr		    = NR_PORTS,
	.cons		= TCC_SERIAL_CONSOLE,
	.driver_name    = TCC_SERIAL_NAME,
	.major		= TCC_SERIAL_MAJOR,
	.minor		= TCC_SERIAL_MINOR,
};


/*  initialise  serial port information */
/* cpu specific variations on the serial port support */

#if 0
static int tcc_serial_getsource(struct uart_port *port,
				    struct tcc_uart_clksrc *clk)
{
	return 0;
}
#endif

static struct tcc_uart_info tcc_uart_inf = {
	.name		    = "Telechips UART",
	.type	    	= PORT_TCC,
	.fifosize   	= FIFOSIZE,
};

#ifdef CONFIG_SERIAL_TCC_DMA_TX
static void *tcc_free_dma_buf(tcc_dma_buf_t *dma_buf)
{
	dbg("%s\n", __func__);
    if (dma_buf) {
        if (dma_buf->dma_addr != 0) {
            dma_free_writecombine(0, dma_buf->buf_size, dma_buf->addr, dma_buf->dma_addr);
        }
        memset(dma_buf, 0, sizeof(tcc_dma_buf_t));
    }
    return NULL;
}

static void *tcc_malloc_dma_buf(tcc_dma_buf_t *dma_buf, int buf_size)
{


	dbg("%s\n", __func__);
    if (dma_buf) {
        tcc_free_dma_buf(dma_buf);
        dma_buf->buf_size = buf_size;
        dma_buf->addr = dma_alloc_writecombine(0, dma_buf->buf_size, &dma_buf->dma_addr, GFP_KERNEL);
        dbg("Malloc DMA buffer @0x%X(Phy=0x%X), size:%d\n",
               (unsigned int)dma_buf->addr,
               (unsigned int)dma_buf->dma_addr,
               dma_buf->buf_size);
        return dma_buf->addr;
    }
    return NULL;
}
#endif



static int tcc_uart_probe(struct platform_device *dev, struct tcc_uart_info *info)
{
	int ret;
    struct tcc_uart_port *ourport;
	char clk_name[10];
	struct clk *clk;

    volatile PPIC pPIC = (volatile PPIC)tcc_p2v(HwPIC_BASE);
#ifdef CONFIG_SERIAL_TCC_DMA_TX
    volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);
#endif

	dbg("\n\n%s: dev->id = %d \n", __func__, dev->id);

    /* Check UART# */
    if(dev->id > (NR_PORTS-1)) {
        printk("%s: UART device # is out of range.\n", __func__);
        return -1;
    }

    //tca_serial_intrinit();
    pPIC->SEL1      |= Hw15;
    pPIC->INTMSK1   |= Hw15;
    pPIC->MODE1     |= Hw15;    // Level trigger

    ourport = &tcc_serial_ports[dev->id];

#ifdef CONFIG_MACH_TCC9300ST
    if((&ourport->port)->line == UART_DUMMY_PORT)
    {
        dbg("port is dummy!!\n");
		return -1;
	}
#endif

    ret = tcc_serial_init_ports(ourport, info, dev);
    if (ret < 0){
        dbg("tcc_serial_init_ports failure\n");
        goto probe_err;
    }

    init_waitqueue_head(&(ourport->wait_q));
    init_MUTEX(&(ourport->sem));
#ifdef CONFIG_SERIAL_TCC_DMA_TX
    init_waitqueue_head(&(ourport->wait_dma_q));
#endif

    ret = uart_add_one_port(&tcc_uart_drv, &ourport->port);
    if (ret){
        dbg("uart_add_one_port failure\n");
        return ret;
    }

    platform_set_drvdata(dev, &ourport->port);

#ifdef CONFIG_SERIAL_TCC_DMA_TX
    if((&ourport->port)->line == DMA_PORT) 
    {        
        if (!tcc_malloc_dma_buf(&(tcc_serial_ports[DMA_PORT].dma_tx_buffer), DMA_BUF_SIZE)) {
            dbg("Unable to attach UART TX DMA 1 channel\n");
                ret = -ENOMEM;
            goto probe_err;
		}

        if (!tcc_malloc_dma_buf(&(tcc_serial_ports[DMA_PORT].dma_tx_buffer_test), DMA_BUF_SIZE)) {
	        dbg("Unable to attach UART TX DMA 1 channel\n");
	            ret = -ENOMEM;
	        goto probe_err;
        }

    }
    tca_dma_setien(DMA_TX_CH_NUM, (unsigned long *)pDMA);
#endif

#ifdef CONFIG_SERIAL_TCC_DMA_RX
    if((&ourport->port)->line == DMA_PORT)
    {        
        dma_rx_buffer = dma_alloc_writecombine(0, DMA_RX_BUF_SIZE, &dma_rx_buffer_physical, GFP_KERNEL);

        printk("[#### BT ####] tcc_uart_probe dma_rx_buffer[0x%x][0x%x]\n", dma_rx_buffer, dma_rx_buffer_physical);
        if (dma_rx_buffer == 0) {
            dbg("Unable to attach UART RX DMA 5 channel\n");
                return -ENOMEM;
        }
        rx_dma_tail = (int) dma_rx_buffer_physical;
    }
#endif

    /* Bus Clock Enable of UARTx */
    ourport->clk = clk_get(NULL, ourport->name);
    if(ourport->clk != NULL)
        clk_enable(ourport->clk);
    dbg(" clock enable: %s \n", ourport->name);

	return ret;

probe_err:
    dbg("probe_err\n");
    if(ourport->clk != NULL)
        clk_disable(ourport->clk);
	return ret;
}


static int tcc_serial_remove(struct platform_device *dev)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);
	struct clk *clk;
#ifdef CONFIG_SERIAL_TCC_DMA_RX
    volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);
#endif

#ifdef CONFIG_MACH_TCC9300ST
	if(port->line == UART_DUMMY_PORT)
		return 0;
#endif

    printk("[#### BT ####] tcc_serial_remove Port[%d]\n", port->line);

#if  defined(CONFIG_MACH_TCC9300)
    if(port->line == UART1){
        //tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, BT, OUTPUT, LOW, SET_DIRECTION|SET_VALUE);
        //dbg("tcc_uart: BT power %s\n", (tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, 0, OUTPUT, 0, GET_VALUE)&BT)?"ON":"OFF");
    }
#endif 

	if (port)
		uart_remove_one_port(&tcc_uart_drv, port);

#ifdef CONFIG_SERIAL_TCC_DMA_TX
    if(port->line == DMA_PORT)
        tcc_free_dma_buf(&(tcc_serial_ports[DMA_PORT].dma_tx_buffer));

	if(port->line == DMA_PORT)
        tcc_free_dma_buf(&(tcc_serial_ports[DMA_PORT].dma_tx_buffer_test));

#endif

#ifdef CONFIG_SERIAL_TCC_DMA_RX
    if(port->line == DMA_PORT)
    {     
        dma_free_writecombine(0,DMA_RX_BUF_SIZE,dma_rx_buffer,dma_rx_buffer_physical);
        wr_regl(port, OFFSET_UCR, rd_regl(port, OFFSET_UCR) & ~Hw1);
        pDMA->CHCTRL1 &= Hw0;
    }
#endif

	clk = to_ourport(port)->clk;
    if(clk != NULL) {
    	clk_disable(clk);
    	clk_put(clk);
    }

	return 0;
}

/* UART power management code */

#ifdef CONFIG_PM
/*-------------------------------------------------
 * TODO: handling DMA_PORT suspend/resume (TCC79X)
 *       DMA stop and start ...
 *-------------------------------------------------*/
static int tcc_serial_suspend(struct platform_device *dev, pm_message_t state)
{
#if defined(CONFIG_MACH_TCC8900)
	struct uart_port *port = tcc_dev_to_port(&dev->dev);

	if (port) {
		port->suspended = 1;

		uart_suspend_port(&tcc_uart_drv, port);

        if(port->line == UART1){
            tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, BT_ON, OUTPUT, LOW, SET_DIRECTION|SET_VALUE);
        }
    }
#elif defined (CONFIG_MACH_TCC9300) || defined (CONFIG_MACH_TCC9300CM)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);
	struct tcc_reg_info *reg = &tcc_serial_ports[port->line].reg;    
    
	if (port && port_used(port)) {
	    printk("[#### CONFIG_PM ####] tcc_serial_suspend Port[%d]\n", port->line);	    
		port->suspended = 1;

		#ifdef CONFIG_SERIAL_TCC_DMA_RX
        if(port->line == DMA_PORT)
        {
            int current_tail, current_head;
            volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);
            int cur_addr = tca_dma_dmacurrentaddress(DMA_CH_NUM_RX, (unsigned long *)pDMA);
            current_tail = rx_dma_tail & (DMA_RX_BUF_SIZE-1);
            current_head = cur_addr & (DMA_RX_BUF_SIZE-1);
            
            printk("[SUSPEND][RX_DMA Info] head[%d] tail[%d]\n", current_head, current_tail);
            // because of DMA_CUR_ADDR reset..
            rx_dma_tail = 0;
        }
        #endif

#ifndef CONFIG_PM_CONSOLE_NOT_SUSPEND
		// disable interrupt
		wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) & (~UART_IER_ELSI)));

		reg->bLCR = rd_regl(port, OFFSET_LCR);

		// DLAB = 0
		wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
		reg->bIER = rd_regl(port, OFFSET_IER);

		// DLAB = 1
		wr_regl(port, OFFSET_LCR, (reg->bLCR | Hw7));
//		reg->bFCR = 0x07;						// set in tcc_serial_startup()			
		reg->bDLL = rd_regl(port, OFFSET_DLL);
		reg->bDLM = rd_regl(port, OFFSET_DLM);

		reg->bMCR = rd_regl(port, OFFSET_MCR);
		reg->bAFT = rd_regl(port, OFFSET_AFT);
		reg->bUCR = rd_regl(port, OFFSET_UCR);
#endif
	}

	//BITCSET(HwGPIOE->GPFN0, HwPORTCFG_GPFN0_MASK|HwPORTCFG_GPFN1_MASK, HwPORTCFG_GPFN0(0)|HwPORTCFG_GPFN1(0));	// functions : GPIO	
}

#endif
	
	return 0;
}

static int tcc_serial_resume(struct platform_device *dev)
{
#if defined(CONFIG_MACH_TCC8900)
	struct uart_port *port = tcc_dev_to_port(&dev->dev);

    if(port) {
		if (port->suspended) {

            if(port->line == UART1){
                tcc_pca953x_setup(PCA9539_U3_SLAVE_ADDR, BT_ON, OUTPUT, HIGH, SET_DIRECTION|SET_VALUE);
            }

            uart_resume_port(&tcc_uart_drv, port);

			port->suspended = 0;
        }
    }
#elif defined (CONFIG_MACH_TCC9300) || defined (CONFIG_MACH_TCC9300CM)
{
	struct uart_port *port = tcc_dev_to_port(&dev->dev);
	struct tcc_reg_info *reg = &tcc_serial_ports[port->line].reg;

	//BITCSET(HwGPIOE->GPFN0, HwPORTCFG_GPFN0_MASK|HwPORTCFG_GPFN1_MASK, HwPORTCFG_GPFN0(1)|HwPORTCFG_GPFN1(1));	// functions : GPIO

	if (port && port_used(port)) {
		if (port->suspended) {
		    printk("[#### CONFIG_PM ####] tcc_serial_resume Port[%d]\n", port->line);

    		#ifdef CONFIG_SERIAL_TCC_DMA_RX
            if(port->line == DMA_PORT)
            {
                int current_tail, current_head;
                volatile PGDMACTRL pDMA = (volatile PGDMACTRL)tcc_p2v(HwGDMA_BASE);
                int cur_addr = tca_dma_dmacurrentaddress(DMA_CH_NUM_RX, (unsigned long *)pDMA);
                current_tail = rx_dma_tail & (DMA_RX_BUF_SIZE-1);
                current_head = cur_addr & (DMA_RX_BUF_SIZE-1);
                printk("[RESUME][RX_DMA Info] head[%d] tail[%d]\n", current_head, current_tail);
            }
            #endif
        
#ifndef CONFIG_PM_CONSOLE_NOT_SUSPEND
			// DLAB = 0
			wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
			wr_regl(port, OFFSET_IER, reg->bIER);

			// DLAB = 1
			wr_regl(port, OFFSET_LCR, (reg->bLCR | Hw7));
			wr_regl(port, OFFSET_FCR, reg->bFCR);
			wr_regl(port, OFFSET_DLL, reg->bDLL);
			wr_regl(port, OFFSET_DLM, reg->bDLM);

			wr_regl(port, OFFSET_MCR, reg->bMCR);
			wr_regl(port, OFFSET_AFT, reg->bAFT);
			wr_regl(port, OFFSET_UCR, reg->bUCR);

			port->suspended = 0;
			// DLAB = 0
			wr_regl(port, OFFSET_LCR, (reg->bLCR & (~Hw7)));
			wr_regl(port, OFFSET_IER, (rd_regl(port, OFFSET_IER) | UART_IER_ELSI));

			wr_regl(port, OFFSET_LCR, reg->bLCR);
#endif
		}
	}
	
	return 0;
}
#endif

	return 0;
}

#else
#define tcc_serial_suspend NULL
#define tcc_serial_resume  NULL
#endif

static int tcc_serial_probe(struct platform_device *dev)
{
	return tcc_uart_probe(dev, &tcc_uart_inf);
}

static struct platform_driver tcc_serial_drv = {
	.probe			= tcc_serial_probe,
	.remove			= tcc_serial_remove,
	.suspend		= tcc_serial_suspend,
	.resume			= tcc_serial_resume,
//	.suspend_late	= tcc_serial_suspend,
//	.resume_early	= tcc_serial_resume,
	.driver		= {
		.name	= DRV_NAME,
		.owner	= THIS_MODULE,
	},
};

static int tcc_serial_init(struct platform_driver *drv,
			       struct tcc_uart_info *info)
{
	dbg("%s(%p,%p)\n", __func__, drv, info);

	return platform_driver_register(drv);
}

static inline void tcc_serial_exit(void)
{
    //remove_pwm_node(DEVICE_UART);
	platform_driver_unregister(&tcc_serial_drv);
}


/* module initialisation code */
static int __init tcc_serial_modinit(void)
{
	int ret;
	
	ret = uart_register_driver(&tcc_uart_drv);
	if (ret < 0) {
		dbg(KERN_ERR "failed to register UART driver\n");
		return -1;
	}
	//printk("Serial driver %s registered.\n", tcc_uart_drv.driver_name);
	
	tcc_serial_init(&tcc_serial_drv, &tcc_uart_inf);

	return 0;
}

static void __exit tcc_serial_modexit(void)
{
	tcc_serial_exit();

	uart_unregister_driver(&tcc_uart_drv);
}


module_init(tcc_serial_modinit);
module_exit(tcc_serial_modexit);

/************************************************************
 *
 * The following is Console driver 
 *
 ************************************************************/

#ifdef CONFIG_SERIAL_TCC_CONSOLE

static struct uart_port *cons_uart;

static void tcc_serial_console_putchar(struct uart_port *port, int ch)
{
	while (!(rd_regl(port, OFFSET_LSR) & THRE))
		barrier();

    wr_regb(port, OFFSET_THR, ch);

}

void tcc_serial_uart_putchar(struct uart_port *port, int ch)
{
	while (!(rd_regl(port, OFFSET_LSR) & THRE))
		barrier();

    wr_regb(port, OFFSET_THR, ch);
}


static void tcc_console_write(struct console *co, const char *s,
			     unsigned int count)
{
	struct uart_port *port;
	unsigned int t_ier, b_ier; 

	port = &tcc_serial_ports[co->index].port;

    t_ier = rd_regl(port, OFFSET_IER);
    b_ier = t_ier;

    wr_regl(port, OFFSET_IER, t_ier & ~ETXI);
    uart_console_write(cons_uart, s, count, tcc_serial_console_putchar);
    wr_regl(port, OFFSET_IER, b_ier);
}

static void __init
tcc_serial_get_options(struct uart_port *port, int *baud,
			   int *parity, int *bits)
{
}


/* tcc_serial_init_ports
 * initialise the serial ports from the machine provided initialisation data.
 */
int tcc_serial_init_ports(struct tcc_uart_port *ourport,
					struct tcc_uart_info *info,
					struct platform_device *pdev)
{
    struct uart_port *port = &ourport->port;
    struct resource *res;
    int ret;

    dbg("tcc_serial_init_ports: initialising ports...\n");

    port->iotype	= UPIO_MEM;
    port->flags     = UPF_BOOT_AUTOCONF;
    port->ops	    = &tcc_serial_ops;
    port->fifosize  = FIFOSIZE;
    port->line	    = pdev->id;
    port->dev	    = &pdev->dev;
    port->type      = PORT_TCC;

    res = platform_get_resource(pdev, IORESOURCE_MEM, 0);
    if (res == NULL) {
        printk(KERN_ERR "failed to find memory resource for uart\n");
        return -EINVAL;
    }

    //port->mapbase	= pdev->resource[0].start;
    port->mapbase	= res->start;
    //port->membase	= ioremap_nocache(port->mapbase, MAP_SIZE);
    port->membase	= (unsigned char __iomem *)port->mapbase;
    port->irq       = platform_get_irq(pdev, 0);

    ret = (int)port->line;

    return ret;
}

static int __init
tcc_console_setup(struct console *co, char *options)
{
	struct uart_port *port;
	int baud = BAUDRATE;
	int bits = 8;
	int parity = 'n';
	int flow = 'n';

	dbg("tcc_serial_console_setup: co=%p (%d), %s\n", co, co->index, options);

	/* is this a valid port */

	if (co->index == -1 || co->index >= NR_PORTS)
		co->index = CONSOLE_PORT;

	port = &tcc_serial_ports[co->index].port;

	/* is the port configured? */

	if (port->mapbase == 0x0) {
        port->mapbase = (resource_size_t)&(CONSOLE_BASE);
        port->membase = (unsigned char __iomem *)port->mapbase;
		co->index = CONSOLE_PORT;
		port = &tcc_serial_ports[co->index].port;
	}
    
	cons_uart = port;

	if (options) {
        dbg("uart_parse_options\n");
		uart_parse_options(options, &baud, &parity, &bits, &flow);
    } else {
        dbg("tcc_serial_get_options\n");
		tcc_serial_get_options(port, &baud, &parity, &bits);
    }
	
	dbg("tcc_serial_console_setup: port=%p (%d)\n", port, co->index);
	return uart_set_options(port, co, baud, parity, bits, flow);
}

/* tcc_console_init
 *
 * initialise the console from one of the uart drivers
*/

static struct console tcc_serial_console =
{
	.name		= TCC_SERIAL_NAME,
	.device		= uart_console_device,
	.flags		= CON_PRINTBUFFER | CON_ENABLED,
	.index		= -1,
	.write		= tcc_console_write,
	.setup		= tcc_console_setup,
};

static int tcc_console_init(void)
{
	struct tcc_uart_info *info;
	
	dbg("%s\n", __func__);
	
	info = &tcc_uart_inf;
	
	tcc_serial_console.data = &tcc_uart_drv;
	register_console(&tcc_serial_console);
	
	return 0;
}

console_initcall(tcc_console_init);

#endif /* CONFIG_SERIAL_TCC_CONSOLE */

MODULE_LICENSE("GPL");
MODULE_AUTHOR("linux <linux@telechips.com>");
MODULE_DESCRIPTION("Telechips TCC Serial port driver");

