/****************************************************************************
 *   FileName    :  tcc_tsif_module_hwset.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

#include <linux/module.h>
#include <mach/io.h>

#include <bsp.h>
#include "tca_tsif_module_hwset.h"

#define TSIF_USE_DEBUG 1

static void tcc_tsif_module_gpio_debug(volatile PGPIO regs)
{
#if TSIF_USE_DEBUG
	printk("## GPEFN debug ################\n");
	printk("GPEFN0(0x%08x): 0x%08x\n", &regs->GPEFN0, regs->GPEFN0);
	printk("GPEFN1(0x%08x): 0x%08x\n", &regs->GPEFN1, regs->GPEFN1);
	printk("GPEFN2(0x%08x): 0x%08x\n", &regs->GPEFN2, regs->GPEFN2);
	printk("GPEFN3(0x%08x): 0x%08x\n", &regs->GPEFN3, regs->GPEFN3);
	printk("###############################\n");
#endif
}

static void tcc_tsif_module_dma_debug(volatile PGDMACTRL regs, int ch)
{
#if TSIF_USE_DEBUG
	printk("## DMA debug ##################\n");	
	if(ch == 0) {
		printk("ST_SADR0-%d(0x%08x): 0x%08x\n", ch, &regs->ST_SADR0, regs->ST_SADR0);
		printk("SPARAM0-%d (0x%08x): 0x%08x\n", ch, &regs->SPARAM0,  regs->SPARAM0);
		printk("ST_DADR0-%d(0x%08x): 0x%08x\n", ch, &regs->ST_DADR0, regs->ST_DADR0);
		printk("DPARAM0-%d (0x%08x): 0x%08x\n", ch, &regs->DPARAM0, 	regs->DPARAM0);
		printk("C_DADR0-%d (0x%08x): 0x%08x\n", ch, &regs->C_DADR0, 	regs->C_DADR0);
		printk("HCOUNT0-%d (0x%08x): 0x%08x\n", ch, &regs->HCOUNT0, 	regs->HCOUNT0);
		printk("CHCTRL0-%d (0x%08x): 0x%08x\n", ch, &regs->CHCTRL0, 	regs->CHCTRL0);
		printk("RPTCTRL0-%d(0x%08x): 0x%08x\n", ch, &regs->RPTCTRL0, regs->RPTCTRL0);
		printk("EXTREQ0-%d (0x%08x): 0x%08x\n", ch, &regs->EXTREQ0, 	regs->EXTREQ0);
	} else if(ch == 1) {
		printk("ST_SADR1-%d(0x%08x): 0x%08x\n", ch, &regs->ST_SADR1, regs->ST_SADR1);
		printk("SPARAM1-%d (0x%08x): 0x%08x\n", ch, &regs->SPARAM1,  regs->SPARAM1);
		printk("ST_DADR1-%d(0x%08x): 0x%08x\n", ch, &regs->ST_DADR1, regs->ST_DADR1);
		printk("DPARAM1-%d (0x%08x): 0x%08x\n", ch, &regs->DPARAM1, 	regs->DPARAM1);
		printk("C_DADR1-%d (0x%08x): 0x%08x\n", ch, &regs->C_DADR1, 	regs->C_DADR1);
		printk("HCOUNT1-%d (0x%08x): 0x%08x\n", ch, &regs->HCOUNT1, 	regs->HCOUNT1);
		printk("CHCTRL1-%d (0x%08x): 0x%08x\n", ch, &regs->CHCTRL1, 	regs->CHCTRL1);
		printk("RPTCTRL1-%d(0x%08x): 0x%08x\n", ch, &regs->RPTCTRL1, regs->RPTCTRL1);
		printk("EXTREQ1-%d (0x%08x): 0x%08x\n", ch, &regs->EXTREQ1, 	regs->EXTREQ1);
	} else {
		printk("ST_SADR2-%d(0x%08x): 0x%08x\n", ch, &regs->ST_SADR2, regs->ST_SADR2);
		printk("SPARAM2-%d (0x%08x): 0x%08x\n", ch, &regs->SPARAM2,  regs->SPARAM2);
		printk("ST_DADR2-%d(0x%08x): 0x%08x\n", ch, &regs->ST_DADR2, regs->ST_DADR2);
		printk("DPARAM2-%d (0x%08x): 0x%08x\n", ch, &regs->DPARAM2, 	regs->DPARAM2);
		printk("CHCTRL2-%d (0x%08x): 0x%08x\n", ch, &regs->CHCTRL2, 	regs->CHCTRL2);
		printk("RPTCTRL2-%d(0x%08x): 0x%08x\n", ch, &regs->RPTCTRL2, regs->RPTCTRL2);
		printk("EXTREQ2-%d (0x%08x): 0x%08x\n", ch, &regs->EXTREQ2, 	regs->EXTREQ2);
	}
	printk("CHCONFIG0 (0x%08x): 0x%08x\n", &regs->CHCONFIG, regs->CHCONFIG);
	printk("###############################\n");
#endif
}

static void tcc_tsif_module_reg_debug(volatile struct tcc_tsif_regs* regs, int id)
{
#if TSIF_USE_DEBUG
	printk("## TSIF debug #################\n");
	printk("TSCR%d(0x%08x)  : 0x%08x\n", id, &regs->TSCR, regs->TSCR);
	//printk("TSPID%d(0x%08x) : 0x%08x\n", id, &regs->TSPID, regs->TSPID);
	printk("TSIC%d(0x%08x): 0x%08x\n", id, &regs->TSIC, regs->TSIC);
	//print("TSCHS(0x%08x)  : 0x%08x\n", &HwTSIFPORTSEL_BASE, HwTSIFPORTSEL_BASE);
	printk("###############################\n");
#endif
}

static void tcc_tsif_module_clearfifopacket(struct tcc_tsif_module_handle *h)
{
	//BITSET(h->regs->TSCR, Hw7);		//set to Hw7 : empties RX Fifo

	BITSET(h->regs->TSCR, Hw30);
	BITCLR(h->regs->TSCR, Hw30);

	printk("%s\n", __func__);
}

static int tca_tsif_module_set_port(struct tcc_tsif_module_handle *h)
{
	volatile PGPIO gpio_regs = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);
    volatile unsigned long* TSIFPORT = (volatile unsigned long *)tcc_p2v(HwTSIFPORTSEL_BASE);
	int ret = 0;

	switch (h->gpio_port)
	{
		case 0xC:
		{
			/* port config for serial test */
		    BITCSET(gpio_regs->GPCFN3, Hw32-Hw16, (Hw29|Hw28|Hw25|Hw24|Hw21|Hw20|Hw17|Hw16));
     
			BITCSET(gpio_regs->GPCPD1, Hw27|Hw26, Hw27|Hw26);		// Clk port Internal Pull-up;
			BITCSET(gpio_regs->GPCCD1, Hw27|Hw26, Hw27|Hw26);

			BITCSET(gpio_regs->GPCPD1, Hw29|Hw28, Hw29|Hw28);		// Data port Internal Pull-up;
			BITCSET(gpio_regs->GPCCD1, Hw29|Hw28, Hw29|Hw28);
	
			/* tsif ch config */ 
			if(h->id == 0) {
				BITCSET(*TSIFPORT, Hw3|Hw2, Hw3|Hw2);
    		} else {
				BITCSET(*TSIFPORT, Hw1|Hw0, Hw1|Hw0);
			}
		}
		break;

		case 0xE:
		{
			/* port config for module test */ 
			BITCSET(gpio_regs->GPEFN1, Hw32-Hw16, (Hw29|Hw28|Hw25|Hw24|Hw21|Hw20|Hw17|Hw16));
			BITCSET(gpio_regs->GPEFN2, Hw28-Hw0,  (Hw25|Hw24|Hw21|Hw20|Hw17|Hw16|Hw13|Hw12|Hw9|Hw8|Hw5|Hw4|Hw1|Hw0));

			BITCSET(gpio_regs->GPEPD1, Hw9|Hw8, Hw9|Hw8);		// Clk port Internal Pull-up;
			BITCSET(gpio_regs->GPECD1, Hw9|Hw8, Hw9|Hw8);

			/* tsif ch config */
			if(h->id == 0) {
				BITCSET(*TSIFPORT, Hw3|Hw2, Hw2);
		    } else {
				BITCSET(*TSIFPORT, Hw1|Hw0, Hw0);
			}
		}
		break;
 
		case 0xF:
		{
			/* port config for module test */
			BITCSET(gpio_regs->GPFFN0, Hw32-Hw0, (Hw30|Hw26|Hw22|Hw18|Hw14|Hw10|Hw6|Hw2));
			BITCSET(gpio_regs->GPFFN1, Hw12-Hw0, (Hw10|Hw6|Hw2));

			BITCSET(gpio_regs->GPFPD1, Hw19|Hw18, Hw19|Hw18);		// Clk port Internal Pull-up;
			BITCSET(gpio_regs->GPFCD1, Hw19|Hw18, Hw19|Hw18);
			 
			/* tsif ch config */ 
			if(h->id == 0) {
				BITCLR(*TSIFPORT, Hw3|Hw2);
    		} else {
				BITCLR(*TSIFPORT, Hw1|Hw0);
			}
		}
		break;
		
		default:
		{
			ret = -1;
		}
		break;
	}
	
		return ret;
}

static void tcc_tsif_module_release_port(struct tcc_tsif_module_handle *h)
{
	volatile PGPIO gpio_regs = (volatile PGPIO)tcc_p2v(HwGPIO_BASE);
    volatile unsigned long* TSIFPORT = (volatile unsigned long *)tcc_p2v(HwTSIFPORTSEL_BASE);

	switch (h->gpio_port)
	{
		case 0xC:
		{
			/* port release */
		    BITCLR(gpio_regs->GPCFN3, Hw32-Hw16);
     
			BITCLR(gpio_regs->GPCPD1, Hw27|Hw26);
			BITCLR(gpio_regs->GPCCD1, Hw27|Hw26);

			BITCLR(gpio_regs->GPCPD1, Hw29|Hw28);
			BITCLR(gpio_regs->GPCCD1, Hw29|Hw28);
		}
		break;

		case 0xE:
		{
			/* port release */
			BITCLR(gpio_regs->GPEFN1, Hw32-Hw16);
			BITCLR(gpio_regs->GPEFN2, Hw28-Hw0);

			BITCLR(gpio_regs->GPEPD1, Hw9|Hw8);
			BITCLR(gpio_regs->GPECD1, Hw9|Hw8);
		}
		break;

		case 0xF:
		{
			/* port release */
			BITCLR(gpio_regs->GPFFN0, Hw32-Hw0);
			BITCLR(gpio_regs->GPFFN1, Hw12-Hw0);

			BITCLR(gpio_regs->GPFPD1, Hw19|Hw18);
			BITCLR(gpio_regs->GPFCD1, Hw19|Hw18);
		}
		break;
		
		default:
		{
			printk("error release port\n");
		}
		break;
	}

    /* tsif ch release */
	if(h->id == 0) {
		BITCLR(*TSIFPORT, Hw3|Hw2);
	} else {
		BITCLR(*TSIFPORT, Hw1|Hw0);
	}
}

static void tcc_tsif_module_set(struct tcc_tsif_module_handle *h)
{
#ifdef GDMA
#else //GDMA
	BITSET(h->regs->TSCR, Hw17);
#endif //GDMA

	if(h->msb_first)		BITSET(h->regs->TSCR, Hw16);
	else					BITCLR(h->regs->TSCR, Hw16);

	if(h->big_endian)		BITSET(h->regs->TSCR, Hw15);
	else					BITCLR(h->regs->TSCR, Hw15);
		
	if(h->serial_mode) {
		BITSET(h->regs->TSCR, Hw14);
		BITSET(h->regs->TSCR, Hw4|Hw3|Hw1);
	} 
	else {
		BITCLR(h->regs->TSCR, Hw14);
		BITSET(h->regs->TSCR, Hw2|Hw0);
	}

	if(h->clk_polarity)		BITSET(h->regs->TSCR, Hw13);
	else					BITCLR(h->regs->TSCR, Hw13);

	if(h->valid_polarity)	BITSET(h->regs->TSCR, Hw12);
	else					BITCLR(h->regs->TSCR, Hw12);

	if(h->sync_polarity)	BITSET(h->regs->TSCR, Hw11);
	else					BITCLR(h->regs->TSCR, Hw11);

	if(h->sync_delay)		BITSET(h->regs->TSCR, Hw5);
	else					BITCLR(h->regs->TSCR, Hw5);
	
	//BITSET(h->regs->TSCR, (h->sync_delay & 0x1f));

	printk("%s : mode set : %s, %s, %s, %s, %s, %s, sync delay %d port 0x%X\n", __func__, 
																		(h->msb_first		? "msb first" 		: "lsb first"),
																		(h->big_endian 		? "big endian" 		: "little endian"),
																		(h->serial_mode		? "serial mode" 	: "parallel mode"),
																		(h->clk_polarity	? "falling edge" 	: "rising edge"),
																		(h->valid_polarity 	? "valid high active": "valid low active"),
																		(h->sync_polarity	? "sync high active" : "sync low active"),
																		 h->sync_delay, h->gpio_port );
}

#ifdef GDMA
static void tcc_tsif_module_isr(struct tcc_tsif_module_handle *h)
{
	volatile PGDMACTRL dma_regs;
	
	if(h->dma_controller == 0)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
	else if(h->dma_controller == 1)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA1_BASE);
	else if(h->dma_controller == 2)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA2_BASE);
	else if(h->dma_controller == 3)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA3_BASE);

	if(h->dma_ch == 0)
	{
		BITSET(dma_regs->CHCTRL0, Hw3);		//dma done flag set
		h->dma_phy_curpos = dma_regs->C_DADR0;
	}
	else if(h->dma_ch == 1)
	{
		BITSET(dma_regs->CHCTRL1, Hw3);		//dma done flag set
		h->dma_phy_curpos = dma_regs->C_DADR1;
	}
	else
	{
		BITSET(dma_regs->CHCTRL2, Hw3);		//dma done flag set
		h->dma_phy_curpos = dma_regs->C_DADR2;
	}

	h->dma_intr_cnt++;

}

static void tcc_tsif_module_dma_init(struct tcc_tsif_module_handle *h)
{
	volatile PGDMACTRL dma_regs;
	unsigned int mask_addr;

	if(h->dma_controller == 0)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
	else if(h->dma_controller == 1)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA1_BASE);
	else if(h->dma_controller == 2)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA2_BASE);
	else if(h->dma_controller == 3)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA3_BASE);
	
	printk("[%s]: dma set dest addr 0x%08x:%dbyte\n", __func__, h->rx_dma.dma_addr, h->rx_dma.buf_size);

	if(h->dma_ch == 0) {
		BITCSET(dma_regs->ST_SADR0, (Hw32-Hw0), &(h->regs->TSDI));		//source addr set
		BITCLR(dma_regs->SPARAM0, (Hw32-Hw0));

		BITCSET(dma_regs->ST_DADR0, (Hw32-Hw0), h->rx_dma.dma_addr);	//destination addr set
		mask_addr = TSIF_MASK_SIZE << 8;								// 1mbyte mask set
		BITCSET(dma_regs->DPARAM0, (Hw32-Hw0), (mask_addr|TSIF_DMA_INCREASE_SIZE));		//inc & mask addr set

		BITCSET(dma_regs->HCOUNT0, (Hw32-Hw0), TSIF_DMA_HOPE_CNT(h->dma_intr_packet_cnt));		//hop count set
		BITCLR(dma_regs->CHCTRL0, (Hw32-Hw0));							//repeat ctrl reg set

		if(h->id == 0)	BITCSET(dma_regs->EXTREQ0, (Hw32-Hw0), Hw3);	//ext request set tsif0
		else			BITCSET(dma_regs->EXTREQ0, (Hw32-Hw0), Hw7);	//ext request set tsif1

		BITCSET(dma_regs->CHCTRL0, (Hw32-Hw0), (Hw15|Hw11|Hw9|Hw8|Hw5|Hw4|Hw2|Hw1));		//channel ctrl reg set
	} else if(h->dma_ch == 1) {
		BITCSET(dma_regs->ST_SADR1, (Hw32-Hw0), &(h->regs->TSDI));		//source addr set
		BITCLR(dma_regs->SPARAM1, (Hw32-Hw0));

		BITCSET(dma_regs->ST_DADR1, (Hw32-Hw0), h->rx_dma.dma_addr);	//destination addr set
		mask_addr = TSIF_MASK_SIZE << 8;						    	// 1mbyte mask set
		BITCSET(dma_regs->DPARAM1, (Hw32-Hw0), (mask_addr|TSIF_DMA_INCREASE_SIZE));		//inc & mask addr set

		BITCSET(dma_regs->HCOUNT1, (Hw32-Hw0), TSIF_DMA_HOPE_CNT(h->dma_intr_packet_cnt));		//hop count set
		BITCLR(dma_regs->CHCTRL1, (Hw32-Hw0));							//repeat ctrl reg set

		if(h->id == 0)	BITCSET(dma_regs->EXTREQ1, (Hw32-Hw0), Hw3);	//ext request set tsif0
		else			BITCSET(dma_regs->EXTREQ1, (Hw32-Hw0), Hw7);	//ext request set tsif1

		BITCSET(dma_regs->CHCTRL1, (Hw32-Hw0), (Hw15|Hw11|Hw9|Hw8|Hw5|Hw4|Hw2|Hw1));			//channel ctrl reg set
	} else { 
		//BITCSET(dma_regs->ST_SADR2, (Hw32-Hw0), &(h->regs->TSDI));		//source addr set
		BITCSET(dma_regs->ST_SADR2, (Hw32-Hw0), &HwTSIF0_BASE);	            //source addr set
		BITCLR(dma_regs->SPARAM2, (Hw32-Hw0));

		BITCSET(dma_regs->ST_DADR2, (Hw32-Hw0), h->rx_dma.dma_addr);	//destination addr set
		mask_addr = TSIF_MASK_SIZE << 8;								// 1mbyte mask set
		BITCSET(dma_regs->DPARAM2, (Hw32-Hw0), (mask_addr|TSIF_DMA_INCREASE_SIZE));		//inc & mask addr set

		BITCSET(dma_regs->HCOUNT2, (Hw32-Hw0), TSIF_DMA_HOPE_CNT(h->dma_intr_packet_cnt));		//hop count set
		BITCLR(dma_regs->CHCTRL2, (Hw32-Hw0));							//repeat ctrl reg set

		if(h->id == 0)	BITCSET(dma_regs->EXTREQ2, (Hw32-Hw0), Hw3);	//ext request set tsif0
		else			BITCSET(dma_regs->EXTREQ2, (Hw32-Hw0), Hw7);	//ext request set tsif1

		BITCSET(dma_regs->CHCTRL2, (Hw32-Hw0), (Hw15|Hw11|Hw9|Hw8|Hw7|Hw5|Hw4|Hw2|Hw1));			//channel ctrl reg set
	}
}

static int tcc_tsif_module_dmastart(struct tcc_tsif_module_handle *h)
{
	volatile PGDMACTRL dma_regs;

	if(h->dma_controller == 0)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
	else if(h->dma_controller == 1)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA1_BASE);
	else if(h->dma_controller == 2)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA2_BASE);
	else if(h->dma_controller == 3)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA3_BASE);

	h->dma_phy_rdpos 	= h->rx_dma.dma_addr;
	h->dma_phy_curpos 	= h->dma_phy_rdpos;
	h->dma_phy_maxaddr	= h->dma_phy_curpos + h->dma_total_size;
	h->dma_virt_rdpos 	= (unsigned int)h->rx_dma.v_addr;
	h->dma_virt_curpos 	= h->dma_virt_rdpos;
	h->dma_virt_maxaddr	= h->dma_virt_curpos + h->dma_total_size;
	h->dma_recv_size	= 0;
	h->dma_intr_cnt		= 0;

	tcc_tsif_module_dma_init(h);

	if(h->dma_ch == 0)	 	BITSET(dma_regs->CHCTRL0, 	Hw0);	//dma enable
	else if(h->dma_ch == 1)	BITSET(dma_regs->CHCTRL1, 	Hw0);	//dma enable
	else					BITSET(dma_regs->CHCTRL2, 	Hw0);	//dma enable
		
	tcc_tsif_module_dma_debug(dma_regs, h->dma_ch);

	BITSET(h->regs->TSCR, Hw31);								//tsif enable

	printk("%s\n", __func__);

	tcc_tsif_module_reg_debug(h->regs, h->id);

	return 0;
}

static int tcc_tsif_module_dmastop(struct tcc_tsif_module_handle *h)
{
	volatile PGDMACTRL dma_regs;

	if(h->dma_controller == 0)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
	else if(h->dma_controller == 1)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA1_BASE);
	else if(h->dma_controller == 2)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA2_BASE);
	else if(h->dma_controller == 3)
		dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA3_BASE);

	if(h->dma_ch == 0)		BITCLR(dma_regs->CHCTRL0, 	Hw0);	//dma disable
	else if(h->dma_ch == 1)	BITCLR(dma_regs->CHCTRL1, 	Hw0);	//dma disable
	else					BITCLR(dma_regs->CHCTRL2, 	Hw0);	//dma disable

	BITCLR(h->regs->TSCR, Hw31);								//tsif disable

	printk("%s\n", __func__);
	return 0;
}

static void tcc_tsif_module_dma_deinit(struct tcc_tsif_module_handle *h)
{
	volatile PGDMACTRL dma_regs;

	dma_regs = (volatile PGDMACTRL)tcc_p2v(HwGDMA0_BASE);
}
#else //GDMA
static void tcc_tsif_module_isr(struct tcc_tsif_module_handle *h)
{
	volatile PGPSB dma_regs;
	unsigned long dma_done_reg = 0;

	if(h->id == 0)
		dma_regs = (volatile PGPSB)tcc_p2v(HwGPSBCH0_BASE);
	else if(h->id == 1)
		dma_regs = (volatile PGPSB)tcc_p2v(HwGPSBCH1_BASE);
/*
if( g_testflag == TRUE )
{
	unsigned int packet_cnt = (h->dma_total_packet_cnt & 0x1FFF) -1;
	unsigned int packet_size = (MPEG_PACKET_SIZE & 0x1FFF);
	unsigned int intr_packet_cnt = (h->dma_intr_packet_cnt & 0x1FFF) -1;

    dma_regs->PACKET = ( packet_cnt << 16) | packet_size;
	BITCSET(dma_regs->DMAICR, 0x1FFF, intr_packet_cnt);	

	g_testflag = FALSE;
}	
*/
	dma_done_reg = dma_regs->DMAICR;
	if( dma_done_reg & (Hw29|Hw28) ) {
		BITSET(dma_regs->DMAICR, (Hw29|Hw28));
		h->cur_q_pos = (int)(dma_regs->DMASTR >> 17);
	}

	return;
}

static void tcc_tsif_module_dma_init(struct tcc_tsif_module_handle *h)
{
	volatile PGPSB dma_regs;
	
	if(h->id == 0)
		dma_regs = (volatile PGPSB)tcc_p2v(HwGPSBCH0_BASE);
	else if(h->id == 1)
		dma_regs = (volatile PGPSB)tcc_p2v(HwGPSBCH1_BASE);

	//BITCSET(dma_regs->TXBASE, (Hw32-Hw0), h->regs->TSDI);
	BITCSET(dma_regs->RXBASE, (Hw32-Hw0), h->rx_dma.dma_addr);

	//BITCSET(dma_regs->PACKET, (Hw32-Hw0), MPEG_PACKET_SIZE);
	//BITSET(dma_regs->PACKET, ((h->dma_total_size/MPEG_PACKET_SIZE)&0x1FFF)<<16);

	BITCLR(dma_regs->DMACTR, Hw28); //0:little endian, 1:Big endian

}

static int tcc_tsif_module_dmastart(struct tcc_tsif_module_handle *h)
{
	volatile PGPSB dma_regs;
	unsigned int packet_cnt = (h->dma_total_packet_cnt & 0x1FFF) -1;
	unsigned int packet_size = (MPEG_PACKET_SIZE & 0x1FFF);
	unsigned int intr_packet_cnt = (h->dma_intr_packet_cnt & 0x1FFF) -1;

	if(h->id == 0)
		dma_regs = (volatile PGPSB)tcc_p2v(HwGPSBCH0_BASE);
	else if(h->id == 1)
		dma_regs = (volatile PGPSB)tcc_p2v(HwGPSBCH1_BASE);
/*
{
    dma_regs->PACKET = ( 1 << 16) | 192;
	BITCSET(dma_regs->DMAICR, 0x1FFF, 1);	

	g_testflag = TRUE;
}
*/
 
	dma_regs->PACKET = ( packet_cnt << 16) | packet_size;
	BITCSET(dma_regs->DMAICR, 0x1FFF, intr_packet_cnt);	

	BITSET(dma_regs->DMAICR, Hw16); 			//enable DMA packet interrupt
	//BITSET(dma_regs->DMAICR, Hw17);				//enable DMA Done Interrupt
	BITCLR(dma_regs->DMAICR, Hw20);				//set RX interrupt
	BITSET(dma_regs->DMAICR, Hw16);

	//BITSET(dma_regs->DMACTR, Hw2);
	//BITCLR(dma_regs->DMACTR, Hw2);

	BITSET(dma_regs->DMACTR, Hw30);				//enable recv DMA requeset 

	BITSET(dma_regs->DMACTR, Hw19|Hw18);		//enable PID & Sync Byte match

	//BITCSET(dma_regs->DMACTR, Hw5|Hw4, Hw29);
	BITCSET(dma_regs->DMACTR, Hw5|Hw4, Hw4);	//00:normal mode, 01:MPEG2_TS mode
	BITSET(dma_regs->DMACTR, Hw29);				//enable continues


	BITSET(dma_regs->DMACTR, Hw1);				//enable TSSEL
	BITSET(dma_regs->DMACTR, Hw0);				//enable DMA

	BITSET(h->regs->TSCR, Hw31);
}

static int tcc_tsif_module_dmastop(struct tcc_tsif_module_handle *h)
{
	volatile PGPSB dma_regs;
	
	if(h->id == 0)
		dma_regs = (volatile PGPSB)tcc_p2v(HwGPSBCH0_BASE);
	else if(h->id == 1)
		dma_regs = (volatile PGPSB)tcc_p2v(HwGPSBCH1_BASE);

	BITCLR(dma_regs->DMACTR, Hw30); //disable DMA receive
	BITSET(dma_regs->DMAICR, Hw29|Hw28);
	BITCLR(dma_regs->DMACTR, Hw0);
	
	BITCLR(h->regs->TSCR, Hw31);

	return 0;
}

static void tcc_tsif_module_dma_deinit(struct tcc_tsif_module_handle *h)
{
}
#endif //GDMA

#if 0
static void tcc_tsif_module_irq_deinit(struct tcc_tsif_module_handle *h)
{
	volatile PPIC pic_regs = (volatile PPIC)tcc_p2v(HwPIC_BASE);

	//priority interrupt ctrl reg set
	BITCLR(pic_regs->INTMSK1,  	(HwINT1_TSIF0 << h->id));		//irq output mask
	BITCLR(pic_regs->IEN1,  	(HwINT1_TSIF0 << h->id));		//enable
}
#endif

static void tcc_tsif_module_hw_init(struct tcc_tsif_module_handle *h)
{
	unsigned int reg_val;
	int i;

	tca_tsif_module_set_port(h);

	if(TSIF_BYTE_SIZE == 4)			reg_val = Hw17 | Hw9 | Hw8 | Hw7;
	else if(TSIF_BYTE_SIZE == 2)	reg_val = Hw17 | Hw8 | Hw7;
	else							reg_val = Hw17 | Hw7;
	BITCSET(h->regs->TSCR, 	(Hw32-Hw0), reg_val);
	
	//20100120 koo atsc   little endian  Ű tsif reg 0x47 detect  dma buf 4byte align ڹٲ . Ͽ 0x47 detect mode clear
	BITCLR(h->regs->TSCR, Hw17);
	
	reg_val = (TSIF_RXFIFO_THRESHOLD << 5) | Hw4;
	BITCSET(h->regs->TSIC,(Hw32-Hw0), reg_val);
	
	for(i=0; i<16; i++)
	{	
		BITCLR(h->regs->TSPID[i], 	(Hw32-Hw0));
	}

	tcc_tsif_module_dma_init(h);
	
	printk("%s\n", __func__);
}

static void tcc_tsif_module_hw_deinit(struct tcc_tsif_module_handle *h)
{
	tcc_tsif_module_release_port(h);

	BITCLR(h->regs->TSCR, 	(Hw32-Hw0));
	BITCLR(h->regs->TSIC,	(Hw32-Hw0));

	tcc_tsif_module_dma_deinit(h);

	printk("%s\n", __func__);
}

/******************************
 * return value
 *
 * ret == 0: success
 * ret > 0 or ret < 0: fail
 ******************************/
int tca_tsif_module_init(struct tcc_tsif_module_handle *h,
					volatile struct tcc_tsif_regs *regs,
					dma_alloc_f tea_dma_alloc,
					dma_free_f tea_dma_free,
					int dma_size,
					int tsif_ch,
					int dma_controller,
					int dma_ch,
                    int port)
{
	int ret = 1;
	
	if(regs)
	{
        h->regs             = (volatile struct tcc_tsif_regs *)tcc_p2v(*regs);
		h->dma_controller	= dma_controller;
		h->id 				= tsif_ch;
		h->dma_ch			= dma_ch;
#ifdef GDMA
		h->irq 				= (INT_DMA0_BASE + 3*dma_controller + dma_ch); //INT_DMA
#else //GDMA
		h->irq				= (tsif_ch == 0) ? INT_GPSB0_DMA : INT_GPSB1_DMA;
#endif //GDMA
		h->gpio_port 		= port;

		h->dma_stop 		= tcc_tsif_module_dmastop;
		h->dma_start 		= tcc_tsif_module_dmastart;
		h->clear_fifo_packet= tcc_tsif_module_clearfifopacket;
		h->tsif_set			= tcc_tsif_module_set;
		h->tsif_isr			= tcc_tsif_module_isr;

		h->tea_dma_alloc 	= tea_dma_alloc;
		h->tea_dma_free 	= tea_dma_free;

#ifdef VPU_DMA_USE        
		h->dma_total_size	= h->dma_phy_size;
#else
		h->dma_total_size 	= dma_size;
#endif

		if(h->tea_dma_alloc) {
			if(h->tea_dma_alloc(&(h->rx_dma), h->dma_total_size) == 0) {
				ret = 0;
			}
		} else {
			/* Already, tsif has rx_dma buf */
			ret = 0;
		}

		tcc_tsif_module_hw_init(h);
		
		if(ret)
			tca_tsif_module_clean(h);
	}

	return ret;
}

void tca_tsif_module_clean(struct tcc_tsif_module_handle *h)
{
	if(h) {
		if(h->tea_dma_free) {
			h->tea_dma_free(&(h->rx_dma));
		}

		tcc_tsif_module_hw_deinit(h);
	}
}


EXPORT_SYMBOL(tca_tsif_module_init);
EXPORT_SYMBOL(tca_tsif_module_clean);


