/****************************************************************************
 *   FileName    : TSDEMUX_ref_timer.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/

/****************************************************************************

  Revision History

 ****************************************************************************/

#include <linux/kernel.h>
#include <linux/delay.h>
#include <mach/bsp.h>
#include <mach/hardware.h>
#include "TSDEMUX_ref_timer.h"
//#define	PRINTF printk
#define	PRINTF

#define	TCC_MEMORY_DEVICE	"/dev/mem"
#define	TCCTIMER_VALUE_MAX		0x100000 // 20bit max
static volatile PTIMER pTCCTIMER = NULL;
static volatile PCKC pTCCCKC = NULL;
static unsigned int guiRefTime, guiPrevTime, guiOffsetTime; //ms

int TCCREFTIME_Open(void)
{
	PRINTF("%s 0x%X, 0x%X\n", __func__, &HwTMR_BASE, &HwCLK_BASE);
	pTCCTIMER	= (volatile PTIMER)tcc_p2v(HwTMR_BASE);
	pTCCCKC = (volatile PCKC)tcc_p2v(HwCLK_BASE);

	BITCLR(pTCCTIMER->TCFG4, Hw0);	
	msleep(10);
	#if	defined(CONFIG_ARCH_TCC92X)
	pTCCCKC->PCLK_TCT = 0x14000176;  // 12000000/(374+1) = 32000
	#endif
	#if	defined(CONFIG_ARCH_TCC93XX)
	pTCCCKC->PCLK_TCT = 0x24000176;  // 12000000/(374+1) = 32000
	#endif
	
	pTCCTIMER->TCFG4 = 0x00000040; // 32000/2^(4+1) = 1000  1khz
	pTCCTIMER->TCNT4 = 0x00000000;
	pTCCTIMER->TCNT4 = 0x000fffff; // 20bit
	guiRefTime = 0;
	guiPrevTime = 0;
	guiOffsetTime = 0;
	BITSET(pTCCTIMER->TCFG4, Hw0);
	msleep(10);
	return 0;
}

int TCCREFTIME_Close(void)
{
	PRINTF("%s\n", __func__);
	if(pTCCTIMER == NULL || pTCCCKC == NULL )
		return 0;
	
	BITCLR(pTCCTIMER->TCFG4, Hw0);	
	pTCCTIMER = NULL;
	pTCCCKC = NULL;
	return 0;
}

unsigned int TCCREFTIME_GetTime(void)
{
	if(pTCCTIMER == NULL)
		return 0;
	
	guiRefTime = pTCCTIMER->TCNT4;
	if(guiRefTime < guiPrevTime )
		guiOffsetTime += 0x100000;		
	guiPrevTime = guiRefTime;

	//PRINTF("%s :: %d",__func__, (guiRefTime+guiOffsetTime));	
	return (guiRefTime+guiOffsetTime);
}

void TCCREFTIME_TestMain(void)
{
	int i, start, check, sec;
	TCCREFTIME_Open();		
	
	//PRINTF("0x%x", pTCCTIMER->TCFG4);	
	//for(i=0;i<50;i++)
	start = TCCREFTIME_GetTime();
	sec = 0;
	while(1)
	{
		//PRINTF("0x%x :: %d\n", pTCCTIMER->TCNT4, pTCCTIMER->TCNT4);
		//PRINTF("REAL :: 0x%x :: %d", TCCREFTIME_GetTime(), TCCREFTIME_GetTime());		
		check = TCCREFTIME_GetTime();
		if( (check - start) == 1000 )
		{
			PRINTF("%d second\n", ++sec);
			start = check;
		}		
		if(sec == 30)
			break;
	}
	TCCREFTIME_Close();
}

