/*
 * drivers/usb/gadget/tcc-udc-otg-hs.c
 * Samsung S3C on-chip full/high speed USB OTG 2.0 device controllers
 *
 * $Id: tcc-udc-otg-hs.c,v 1.4 2007/08/13 09:25:24 dasan Exp $*
 *
 * Copyright (C) 2006 for Samsung Electronics 
 * 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include <linux/platform_device.h>
#include <linux/clk.h>
#include <linux/kthread.h>
#include <linux/usb.h>
#if defined(CONFIG_REGULATOR)
#include <linux/regulator/consumer.h>
#endif

#include "tcc_udc_otg.h"
#include "tcc/tcc_otg_regs.h"
#include "tcc/tcc_otg.h"
#include "tcc/usbphy.h"
#include "tcc/otgregs.h"
#if defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
/* VBUS Voltage Control Code, sgbaek */
#include <mach/tcc_pca953x.h>
#endif

#if 1
#undef DEBUG_S3C_UDC_SETUP
#undef DEBUG_S3C_UDC_EP0
#undef DEBUG_S3C_UDC_ISR
#undef DEBUG_S3C_UDC_EP1
#undef DEBUG_S3C_UDC_EP2
#undef DEBUG_S3C_UDC
#else
#define DEBUG_S3C_UDC_SETUP
#define DEBUG_S3C_UDC_EP0
#define DEBUG_S3C_UDC_ISR
#define DEBUG_S3C_UDC_EP1
#define DEBUG_S3C_UDC_EP2
#define DEBUG_S3C_UDC
#endif

//#define USE_VBUS

PUSB20OTG	pOTG =NULL;
PUSBOTGCFG  pOTGCFG=NULL;
PGPIO		rGPIO=NULL;
PPIC		rPIC =NULL;
PIOBUSCFG	pOTG_IOBUSCFG = NULL;


// USB Device DMA support
#define RX_DMA_MODE 0 
#define TX_DMA_MODE 0

#if TX_DMA_MODE
static int tx_dmaStart = 0;
#endif
static int rx_dmaStart = 0;

#if defined(DEBUG_S3C_UDC_ISR) || defined(DEBUG_S3C_UDC_EP0)
static char *state_names[] = {
	"WAIT_FOR_SETUP",
	"DATA_STATE_XMIT",
	"DATA_STATE_NEED_ZLP",
	"WAIT_FOR_OUT_STATUS",
	"DATA_STATE_RECV"
	};
#endif

#ifdef DEBUG_S3C_UDC_SETUP
#define DEBUG_SETUP(fmt,args...) printk(fmt, ##args)
#else
#define DEBUG_SETUP(fmt,args...) do {} while(0)
#endif

#ifdef DEBUG_S3C_UDC_EP0
#define DEBUG_EP0(fmt,args...) printk(fmt, ##args)
#else
#define DEBUG_EP0(fmt,args...) do {} while(0)
#endif

#undef DEBUG
#ifdef DEBUG_S3C_UDC
#define DEBUG(fmt,args...) printk(fmt, ##args)
#else
#define DEBUG(fmt,args...) do {} while(0)
#endif

#ifdef DEBUG_S3C_UDC_ISR
#define DEBUG_ISR(fmt,args...) printk(fmt, ##args)
#else
#define DEBUG_ISR(fmt,args...) do {} while(0)
#endif

#ifdef DEBUG_S3C_UDC_EP1
#define DEBUG_EP1(fmt,args...) printk(fmt, ##args)
#else
#define DEBUG_EP1(fmt,args...) do {} while(0)
#endif

#ifdef DEBUG_S3C_UDC_EP2
#define DEBUG_EP2(fmt,args...) printk(fmt, ##args)
#else
#define DEBUG_EP2(fmt,args...) do {} while(0)
#endif


#define	DRIVER_DESC		"Samsung Dual-speed USB 2.0 OTG Device Controller"
#define	DRIVER_VERSION		__DATE__

struct tcc_udc	*the_controller;

static const char driver_name[] = "tcc-udc";
static const char driver_desc[] = DRIVER_DESC;
static const char ep0name[] = "ep0-control";

// Max packet size
static u32 ep0_fifo_size = 64;
static u32 ep_fifo_size =  512;
static u32 ep_fifo_size2 = 1024;

static struct notifier_block *notifier = NULL;

/*
  Local declarations.
*/
static int tcc_ep_enable(struct usb_ep *ep,
			     const struct usb_endpoint_descriptor *);
static int tcc_ep_disable(struct usb_ep *ep);
static struct usb_request *tcc_alloc_request(struct usb_ep *ep, gfp_t gfp_flags);
static void tcc_free_request(struct usb_ep *ep, struct usb_request *);
//static void *tcc_alloc_buffer(struct usb_ep *ep, unsigned, dma_addr_t *, gfp_t gfp_flags);
//static void tcc_free_buffer(struct usb_ep *ep, void *, dma_addr_t, unsigned);
static int tcc_queue(struct usb_ep *ep, struct usb_request *, gfp_t gfp_flags);
static int tcc_dequeue(struct usb_ep *ep, struct usb_request *);
static int tcc_set_halt(struct usb_ep *ep, int);
static int tcc_fifo_status(struct usb_ep *ep);
static void tcc_fifo_flush(struct usb_ep *ep);
//static void tcc_ep0_kick(struct tcc_udc *dev, struct tcc_ep *ep);
static void tcc_handle_ep0(struct tcc_udc *dev);

static void done(struct tcc_ep *ep, struct tcc_request *req,
		 int status);
static void stop_activity(struct tcc_udc *dev,
			  struct usb_gadget_driver *driver);
static int udc_enable(struct tcc_udc *dev);
static void udc_set_address(struct tcc_udc *dev, unsigned char address);
static void reconfig_usbd(void);

static __inline__ u32 usb_read(u32 port, u8 ind) 
{
	return tcc_readl(port);
}

static __inline__ void usb_write(u32 val, u32 port, u8 ind) 
{	
	tcc_writel(val, port);
}

static __inline__ void usb_set(u32 val, u32 port, u8 ind)
{
	tcc_writel(tcc_readl(port) | val, port);
}

static __inline__ void usb_clear(u32 val, u32 port, u8 ind)
{
	tcc_writel(tcc_readl(port) & ~val, port);
}


static struct usb_ep_ops tcc_ep_ops = {
	.enable = tcc_ep_enable,
	.disable = tcc_ep_disable,

	.alloc_request = tcc_alloc_request,
	.free_request = tcc_free_request,

#if 0
	.alloc_buffer = tcc_alloc_buffer,
	.free_buffer = tcc_free_buffer,
#endif

	.queue = tcc_queue,
	.dequeue = tcc_dequeue,

	.set_halt = tcc_set_halt,
	.fifo_status = tcc_fifo_status,
	.fifo_flush = tcc_fifo_flush,
};

/* Inline code */
static __inline__ int write_packet(struct tcc_ep *ep,
				   struct tcc_request *req, int max)
{
	u32 *buf;
	int length, count;
	u32 fifo = ep->fifo;//, in_ctrl;
	u8 iEP = ep_index(ep);

	buf = req->req.buf + req->req.actual;
	prefetch(buf);

	length = req->req.length - req->req.actual;
	length = min(length, max);
	req->req.actual += length;

	DEBUG("%s: Write %d (max %d), fifo=0x%x\n",
		__func__, length, max, fifo);

	if(iEP == 0) {
		//usb_write((1<<19)|(length<<0), (u32) S3C_UDC_OTG_DIEPTSIZ0, 0);
		//
		//in_ctrl =  tcc_readl(S3C_UDC_OTG_DIEPCTL0);
		//usb_write(DEPCTL_EPENA|DEPCTL_CNAK|(BULK_IN_EP<<11)| in_ctrl, (u32) S3C_UDC_OTG_DIEPCTL0, 0);
		//
		//DEBUG_EP0("%s:(DIEPTSIZ0):0x%x, (DIEPCTL0):0x%x, (GNPTXSTS):0x%x\n", __func__,
		//	tcc_readl(S3C_UDC_OTG_DIEPTSIZ0),tcc_readl(S3C_UDC_OTG_DIEPCTL0),
		//	tcc_readl(S3C_UDC_OTG_GNPTXSTS));
		//udelay(20);
		//_loc_Disable_IN_EP(0);
		//BITSET(pOTG->DIEPINT0, DIEPINT_XferCompl);
		if( length )
			pOTG->DIEPTSIZ0 = DIEPTSIZ0_PktCnt(1)|DIEPTSIZ0_XferSize(length);
		else
			pOTG->DIEPTSIZ0 = DIEPTSIZ0_PktCnt(1)|0;
		pOTG->DIEPCTL0 |= DIEPCTL0_EPEna|DIEPCTL0_CNAK;
		DEBUG_EP0("%s:EP 0 length %d\n", __func__, length);
	}
	else {
		//usb_write((1<<29)|(1<<19)|(length<<0), (u32) S3C_UDC_OTG_DIEPTSIZ2, 2);
		//
		//in_ctrl =  tcc_readl(S3C_UDC_OTG_DIEPCTL2);
		//usb_write(DEPCTL_EPENA|DEPCTL_CNAK|(BULK_IN_EP<<11)| in_ctrl, (u32) S3C_UDC_OTG_DIEPCTL2, 2);
		//
		//DEBUG_EP2("%s:(DIEPTSIZ2):0x%x, (DIEPCTL2):0x%x, (GNPTXSTS):0x%x\n", __func__,
		//	tcc_readl(S3C_UDC_OTG_DIEPTSIZ2),tcc_readl(S3C_UDC_OTG_DIEPCTL2),
		//	tcc_readl(S3C_UDC_OTG_GNPTXSTS));
		//
		//udelay(10);
		//BITSET(pOTG->DEVINENDPT[iEP-1][DIEPINT_INDEX], DIEPINT_XferCompl);
		//while((pOTG->DEVINENDPT[iEP-1][DIEPTSIZ_INDEX]&(DIEPTSIZ_PktCnt_MASK|DIEPTSIZ_XferSize_MASK)) != 0 );
		if( length )
			pOTG->DEVINENDPT[iEP-1][DIEPTSIZ_INDEX] = DIEPTSIZ_PktCnt(1)|DIEPTSIZ_XferSize(length);
		else
			pOTG->DEVINENDPT[iEP-1][DIEPTSIZ_INDEX] = DIEPTSIZ_PktCnt(1)|0;
		pOTG->DEVINENDPT[iEP-1][DIEPCTL_INDEX] |= DIEPCTL_EPEna|DIEPCTL_CNAK;
	}

	for (count=0;count<length;count+=4) {
	  	tcc_writel(*buf++, fifo);
	}
	
	return length;
}

#ifdef CONFIG_USB_GADGET_DEBUG_FILES

static const char proc_node_name[] = "driver/udc";

static int
udc_proc_read(char *page, char **start, off_t off, int count,
	      int *eof, void *_dev)
{
	char *buf = page;
	struct tcc_udc *dev = _dev;
	char *next = buf;
	unsigned size = count;
	unsigned long flags;
	int t;

	if (off != 0)
		return 0;

	local_irq_save(flags);

	/* basic device status */
	t = scnprintf(next, size,
		      DRIVER_DESC "\n"
		      "%s version: %s\n"
		      "Gadget driver: %s\n"
		      "\n",
		      driver_name, DRIVER_VERSION,
		      dev->driver ? dev->driver->driver.name : "(none)");
	size -= t;
	next += t;

	local_irq_restore(flags);
	*eof = 1;
	return count - size;
}

#define create_proc_files() \
	create_proc_read_entry(proc_node_name, 0, NULL, udc_proc_read, dev)
#define remove_proc_files() \
	remove_proc_entry(proc_node_name, NULL)

#else	/* !CONFIG_USB_GADGET_DEBUG_FILES */

#define create_proc_files() do {} while (0)
#define remove_proc_files() do {} while (0)

#endif	/* CONFIG_USB_GADGET_DEBUG_FILES */

/*
 * 	udc_disable - disable USB device controller
 */
static void udc_disable(struct tcc_udc *dev)
{
	DEBUG_SETUP("%s: %p\n", __func__, dev);

	udc_set_address(dev, 0);
	*(volatile unsigned long*)0xF05F5028 = 0x2940;	
	dev->ep0state = WAIT_FOR_SETUP;
	dev->gadget.speed = USB_SPEED_UNKNOWN;
	dev->usb_address = 0;
	//tcc_writel(tcc_readl(S3C_USBOTG_PHYPWR)|(0x7<<1), S3C_USBOTG_PHYPWR);
	USBPHY_SetMode(USBPHY_MODE_RESET);
}

/*
 * 	udc_reinit - initialize software state
 */
static void udc_reinit(struct tcc_udc *dev)
{
	u32 i;

	DEBUG_SETUP("%s: %p\n", __func__, dev);

	/* device/ep0 records init */
	INIT_LIST_HEAD(&dev->gadget.ep_list);
	INIT_LIST_HEAD(&dev->gadget.ep0->ep_list);
	dev->ep0state = WAIT_FOR_SETUP;

	/* basic endpoint records init */
	for (i = 0; i < S3C_MAX_ENDPOINTS; i++) {
		struct tcc_ep *ep = &dev->ep[i];
		if (i != 0)
			list_add_tail(&ep->ep.ep_list, &dev->gadget.ep_list);

		ep->desc = 0;
		ep->stopped = 0;
		INIT_LIST_HEAD(&ep->queue);
		ep->pio_irqs = 0;
	}

	/* the rest was statically initialized, and is read-only */
}

#define BYTES2MAXP(x)	(x / 8)
#define MAXP2BYTES(x)	(x * 8)

/* until it's enabled, this UDC should be completely invisible
 * to any USB host.
 */
static int udc_enable(struct tcc_udc *dev)
{
//	u32 uTemp;
	DEBUG_SETUP("%s: %p\n", __func__, dev);

//	uTemp = tcc_readl(S3C_OTHERS);
//	uTemp |= (1<<16);   // USB_SIG_MASK
//	tcc_writel(uTemp, S3C_OTHERS);

	// 1. Initializes OTG Phy.
	//tcc_writel(0x0, S3C_USBOTG_PHYPWR);
	//tcc_writel(0x20, S3C_USBOTG_PHYCLK);
	//tcc_writel(0x1, S3C_USBOTG_RSTCON);
	//
	//udelay(20);
	//tcc_writel(0x0, S3C_USBOTG_RSTCON);
	//udelay(20);

	USBPHY_SetMode(USBPHY_MODE_NONE);	// hds090925
	{
		volatile unsigned int i=1000;
		while(i--);
	}
	OTGCORE_Init();
	OTGDEV_IO_Init();
	OTGCORE_SetID(1);
	//USBPHY_SetMode(USB_MODE_DEVICE);

	//reconfig_usbd();

	DEBUG_SETUP("S3C6400 USB 2.0 OTG Controller Core Initialized : 0x%x\n",  tcc_readl(S3C_UDC_OTG_GINTMSK));
	
	dev->gadget.speed = USB_SPEED_UNKNOWN;

	return 0;
}

//static void udc_ep0_complete(struct usb_ep *ep, struct usb_request *req)
//{
//}

static struct usb_request *tcc_alloc_request(struct usb_ep *ep, gfp_t gfp_flags);
/*
  Register entry point for the peripheral controller driver.
*/
int usb_gadget_register_driver(struct usb_gadget_driver *driver)
{
	struct tcc_udc *dev = the_controller;
	int retval;

	DEBUG_SETUP("%s: %s\n", __func__, driver->driver.name);
#if 0
	if (!driver || (driver->speed != USB_SPEED_FULL && driver->speed != USB_SPEED_HIGH)
	    || !driver->bind || !driver->unbind || !driver->disconnect || !driver->setup) {
	    
		printk("-->return failed.\n");
		return -EINVAL;
	}
#endif
	if (!dev) {
		return -ENODEV;
	}
	if (dev->driver) {
		return -EBUSY;
	}	

	/* first hook up the driver ... */
	dev->driver = driver;
	dev->gadget.dev.driver = &driver->driver;


	retval = device_add(&dev->gadget.dev);

	if(retval) { /* TODO */
		printk("target device_add failed, error %d\n", retval);
		return retval;
	}
		
	retval = driver->bind(&dev->gadget);
	if (retval) {
		printk("%s: bind to driver %s --> error %d\n", dev->gadget.name,
		       driver->driver.name, retval);
		device_del(&dev->gadget.dev);

		dev->driver = 0;
		dev->gadget.dev.driver = 0;
		return retval;
	}
	//if( dev->ep0req )
	//{
	//	printk("%s: ep0req is allocated already!\n", __func__);
	//	return -EBUSY;
	//}
	//dev->ep0req = tcc_alloc_request(&dev->ep[0].ep, GFP_KERNEL);
	//if (!dev->ep0req) {
	//	printk("%s: insufficient memory error for ep0req\n", __func__);
	//	return -ENOMEM;
 //   }
	//dev->ep0req->buf = kmalloc(256, GFP_KERNEL);
	//if (!dev->ep0req->buf) {
	//	printk("%s: insufficient memory error for ep0req's buf\n", __func__);
	//	return -ENOMEM;
 //   }
	//dev->ep0req->context = NULL;
	//dev->ep0req->complete = udc_ep0_complete;

	printk("Registered gadget driver '%s'\n", driver->driver.name);
	//udc_enable(dev);

	enable_irq(IRQ_OTG);

	return 0;
}

EXPORT_SYMBOL(usb_gadget_register_driver);

static void tcc_free_request(struct usb_ep *ep, struct usb_request *_req);
/*
  Unregister entry point for the peripheral controller driver.
*/
int usb_gadget_unregister_driver(struct usb_gadget_driver *driver)
{
	struct tcc_udc *dev = the_controller;
	unsigned long flags;

	if (!dev)
		return -ENODEV;
	if (!driver || driver != dev->driver)
		return -EINVAL;

	spin_lock_irqsave(&dev->lock, flags);
	dev->driver = 0;
	stop_activity(dev, driver);
	spin_unlock_irqrestore(&dev->lock, flags);

	/* Free the request and buffer for endpoint 0 */
	if (dev->ep0req) {
		kfree(dev->ep0req->buf);
		usb_ep_free_request(&dev->ep[0].ep, dev->ep0req);
		dev->ep0req = NULL;
	}

	driver->unbind(&dev->gadget);
	device_del(&dev->gadget.dev);

	disable_irq(IRQ_OTG);

	printk("Unregistered gadget driver '%s'\n", driver->driver.name);
	return 0;
}

EXPORT_SYMBOL(usb_gadget_unregister_driver);

/*-------------------------------------------------------------------------*/

/** Write request to FIFO (max write == maxp size)
 *  Return:  0 = still running, 1 = completed, negative = errno
 */
//static int write_fifo(struct tcc_ep *ep, struct tcc_request *req)
//{
//	u32 max;//, gintmsk;
//	unsigned count;
//	int is_last = 0, is_short = 0;
//
//	//gintmsk = tcc_readl(S3C_UDC_OTG_GINTMSK);
//
//#if TX_DMA_MODE
//	if(tx_dmaStart) return 0;
//	
//	if ((req->req.actual == 0) && (req->req.length > ep->ep.maxpacket) && (!tx_dmaStart)) {
//
//           /* To Do - TX_DMA handling code  */
//	   
//	}
//	else {
//#endif
//		max = le16_to_cpu(ep->desc->wMaxPacketSize);
//		count = write_packet(ep, req, max);
//
//		/* last packet is usually short (or a zlp) */
//		if (unlikely(count != max))
//			is_last = is_short = 1;
//		else {
//			if (likely(req->req.length != req->req.actual)
//			    || req->req.zero)
//				is_last = 0;
//			else
//				is_last = 1;
//			/* interrupt/iso maxpacket may not fill the fifo */
//			is_short = unlikely(max < ep_maxpacket(ep));
//		}
//
//		DEBUG_EP2("%s: wrote %s %d bytes%s%s req %p %d/%d\n", __func__,
//	      	ep->ep.name, count,
//	     	 is_last ? "/L" : "", is_short ? "/S" : "",
//	      	req, req->req.actual, req->req.length);
//
//#if TX_DMA_MODE	
//	}
//#endif
//
//	/* requests complete when all IN data is in the FIFO */
//	if (is_last) {
//		if(!ep_index(ep)){
//			DEBUG("%s: --> Error EP0 must not come here!\n",
//				__func__);
//			BUG();
//		}
//
//		//tcc_writel(gintmsk&(~INT_TX_FIFO_EMPTY), S3C_UDC_OTG_GINTMSK);
//		BITSET(pOTG->DAINTMSK, DAINTMSK_InEPInt(ep_index(ep)));
//		done(ep, req, 0);
//		if( !count )
//		{
//			//mdelay(1);
//			//printk("zero length data is sent!!!\n");
//		}
//		return 1;
//	}
//
//	// Unmask USB OTG 2.0 interrupt source : INT_TX_FIFO_EMPTY
//	//tcc_writel(gintmsk | INT_TX_FIFO_EMPTY, S3C_UDC_OTG_GINTMSK);
//	BITSET(pOTG->DAINTMSK, DAINTMSK_InEPInt(ep_index(ep)));
//	return 0;
//}



/** Read to request from FIFO (max read == bytes in fifo)
 *  Return:  0 = still running, 1 = completed, negative = errno
 */
static int read_fifo(struct tcc_ep *ep, struct tcc_request *req)
{
	u32 csr, gintmsk;
	u32 *buf;
	u32 bufferspace, count, count_bytes, is_short = 0;
	u32 fifo = ep->fifo;

	csr = tcc_readl(S3C_UDC_OTG_GRXSTSP);
	count_bytes = (csr & 0x7ff0)>>4;

	gintmsk = tcc_readl(S3C_UDC_OTG_GINTMSK);
	
	if(!count_bytes) {
		DEBUG_EP1("%s: count_bytes %d bytes\n", __func__, count_bytes);
		
		// Unmask USB OTG 2.0 interrupt source : INT_RX_FIFO_NOT_EMPTY
		tcc_writel(gintmsk | INT_RX_FIFO_NOT_EMPTY, S3C_UDC_OTG_GINTMSK);
		return 0;
	}

	buf = req->req.buf + req->req.actual;
	prefetchw(buf);
	bufferspace = req->req.length - req->req.actual;

	count = count_bytes / 4;
	if(count_bytes%4) count = count + 1;


#if RX_DMA_MODE
	if(rx_dmaStart) return 0;
	
	if ((req->req.actual == 0) && (req->req.length > ep->ep.maxpacket) && (!rx_dmaStart)) {

		/* To Do - RX_DMA handling code */
		
		rx_dmaStart = 1;	
	}
	else {
#endif	
		req->req.actual += min(count_bytes, bufferspace);

		is_short = (count_bytes < ep->ep.maxpacket);
		DEBUG_EP1("%s: read %s, %d bytes%s req %p %d/%d GRXSTSP:0x%x\n",
			__func__,
			ep->ep.name, count_bytes,
			is_short ? "/S" : "", req, req->req.actual, req->req.length, csr);

		while (likely(count-- != 0)) {
			u32 byte = (u32) tcc_readl(fifo);

			if (unlikely(bufferspace == 0)) {
				/* this happens when the driver's buffer
			 	* is smaller than what the host sent.
			 	* discard the extra data.
			 	*/
				if (req->req.status != -EOVERFLOW)
					printk("%s overflow %d\n", ep->ep.name, count);
				req->req.status = -EOVERFLOW;
			} else {
				*buf++ = byte;
				bufferspace-=4;
			}
	 	 }

#if RX_DMA_MODE	
	}
#endif

	// Unmask USB OTG 2.0 interrupt source : INT_RX_FIFO_NOT_EMPTY
	tcc_writel(gintmsk | INT_RX_FIFO_NOT_EMPTY, S3C_UDC_OTG_GINTMSK);

	/* completion */
	if (is_short || req->req.actual == req->req.length) {
		done(ep, req, 0);
		return 1;
	}

	/* finished that packet.  the next one may be waiting... */
	return 0;
}

/*
 *	done - retire a request; caller blocked irqs
 */
static void done(struct tcc_ep *ep, struct tcc_request *req, int status)
{
	unsigned int stopped = ep->stopped;

	DEBUG("%s: tcc_ep%d-%p\n", __func__, ep_index(ep), ep);
	list_del_init(&req->queue);

	if (likely(req->req.status == -EINPROGRESS))
		req->req.status = status;
	else
		status = req->req.status;

	if (status && status != -ESHUTDOWN)
		DEBUG("complete %s req %p stat %d len %u/%u\n",
			ep->ep.name, &req->req, status,
			req->req.actual, req->req.length);

	/* don't modify queue heads during completion callback */
	ep->stopped = 1;

	spin_unlock(&ep->dev->lock);
	req->req.complete(&ep->ep, &req->req);
	spin_lock(&ep->dev->lock);

	ep->stopped = stopped;
}

/*
 * 	nuke - dequeue ALL requests
 */
void nuke(struct tcc_ep *ep, int status)
{
	struct tcc_request *req;

	DEBUG("%s: tcc_ep%d-%p\n", __func__, ep_index(ep), ep);

	/* called with irqs blocked */
	while (!list_empty(&ep->queue)) {
		req = list_entry(ep->queue.next, struct tcc_request, queue);
		done(ep, req, status);
	}
}

static void udc_ep_in_start_transfer(struct tcc_ep *ep)
{
	volatile unsigned int *pDIEPTSIZ,*pDIEPCTL;
	unsigned int PktCnt,XferSize;

	//printk("start_transfer ep%d length=%d\n",ep_index(ep),ep->xfer_len);

	if(ep_index(ep)==0)
	{
		pDIEPTSIZ = &pOTG->DIEPTSIZ0;
		pDIEPCTL = &pOTG->DIEPCTL0;
		BITCLR(*pDIEPTSIZ,DIEPTSIZ0_PktCnt(0x3)|DIEPTSIZ0_XferSize(0x7F));
	}
	else
	{
		pDIEPTSIZ = &pOTG->DEVINENDPT[ep_index(ep)-1][DIEPTSIZ_INDEX];
		pDIEPCTL = &pOTG->DEVINENDPT[ep_index(ep)-1][DIEPCTL_INDEX];
		BITCLR(*pDIEPTSIZ,DIEPTSIZ_PktCnt(0x3FF)|DIEPTSIZ_XferSize(0x7FFFF));
	}

	if(ep->xfer_len==0)
	{
		XferSize=0;
		PktCnt=1;
	}
	else
	{
		XferSize=ep->xfer_len;
		PktCnt=(XferSize+ep->ep.maxpacket-1)/ep->ep.maxpacket;
	}

	BITSET(*pDIEPTSIZ,DIEPTSIZ_PktCnt(PktCnt)|DIEPTSIZ_XferSize(XferSize));
	BITSET(*pDIEPCTL,DIEPCTL_EPEna|DIEPCTL_CNAK);

	if(ep->xfer_len>0)
	{
		/* Enable the Tx FIFO Empty Interrupt for this EP */
		BITSET(pOTG->DIEPEMPMSK,DIEPEMPMSK_InEpTxfEmpMsk(ep_index(ep)));
	}
}

static void udc_ep_in_start_next_request(struct tcc_ep *ep)
{
	struct tcc_request *req = 0;

	if (!list_empty(&ep->queue))
	{
		//printk("start next request\n");
		req = list_entry(ep->queue.next, 
			   struct tcc_request, queue);
			
		/* Setup and start the Transfer */
		ep->xfer_len = req->req.length;
		ep->xfer_count = 0;
		ep->xfer_buff = req->req.buf;
		udc_ep_in_start_transfer( ep );
	}
}

//static int tcc_ep0_write(struct tcc_udc *dev);
/**
 * tcc_in_epn - handle IN interrupt
 */
//static void tcc_in_epn(struct tcc_udc *dev, u32 ep_idx)
//{
//	struct tcc_ep *ep = &dev->ep[ep_idx];
//	struct tcc_request *req;
//	//volatile unsigned int *pDIEPINT;
//	//if( ep_idx == 0 )
//	//	pDIEPINT = &pOTG->DIEPINT0;
//	//else
//	//	pDIEPINT = &pOTG->DEVINENDPT[ep_idx-1][DIEPINT_INDEX];
//
//	if (list_empty(&ep->queue))
//		req = 0;
//	else
//		req = list_entry(ep->queue.next, struct tcc_request, queue);
//
//	if (unlikely(!req)) {
//		DEBUG("%s: NULL REQ on IN EP-%d\n", __func__, ep_idx);
//		//BITSET(*pDIEPINT, DIEPINT_XferCompl);	//clear
//		//printk("%s: NULL REQ on IN EP-%d\n", __func__, ep_idx);
//		BITCLR(pOTG->DAINTMSK, DAINTMSK_InEPInt(ep_index(ep)));
//		return;
//	}
//	else {
//		//if( *pDIEPINT & DIEPINT_XferCompl )
//		{
//			//BITSET(*pDIEPINT, DIEPINT_XferCompl);	//clear
//			if( ep_idx == 0 )
//			{
//				tcc_ep0_write(dev);
//			}
//			else
//			{
//				write_fifo(ep, req);
//			}
//		}
//	}
//}

static void complete_ep_in(struct tcc_udc *dev,struct tcc_ep *ep)
{
	struct tcc_request *req;

	//printk("complete in ep%d\n",ep_index(ep));

	if (list_empty(&ep->queue))
		req = 0;
	else
		req = list_entry(ep->queue.next, struct tcc_request, queue);

	if(req)
	{
		req->req.actual = ep->xfer_count;
		//printk("done\n");
		done(ep,req,0);
	}

	udc_ep_in_start_next_request(ep);
}

static void write_tx_fifo(struct tcc_ep *ep)
{
	unsigned int count;
	unsigned int length;
	length = min(ep->xfer_len-ep->xfer_count,(unsigned int)ep->ep.maxpacket);
	//printk("write tx fifo ep%d size=%d\n",ep_index(ep),length);
	for (count=0;count<length;count+=4) {
	  	tcc_writel(*(ep->xfer_buff)++, ep->fifo);
	}
	ep->xfer_count += length;
}

static void tcc_in_ep_irq(struct tcc_udc *dev, u32 ep_idx)
{
	struct tcc_ep *ep = &dev->ep[ep_idx];
	volatile unsigned int *pDIEPINT;
	if( ep_idx==0 )
	{
		pDIEPINT = &pOTG->DIEPINT0;
	}
	else
	{
		pDIEPINT = &pOTG->DEVINENDPT[ep_idx-1][DIEPINT_INDEX];
	}

	if(*pDIEPINT&DIEPINT_XferCompl)
	{
		/* Disable the Tx FIFO Empty Interrupt for this EP */
		BITCLR(pOTG->DIEPEMPMSK,DIEPEMPMSK_InEpTxfEmpMsk(ep_idx));
		*pDIEPINT = DIEPINT_XferCompl;
		if(ep_idx==0)
		{
			dev->ep0state = WAIT_FOR_SETUP;
			complete_ep_in(dev,ep);
		}
		else
		{
			complete_ep_in(dev,ep);
		}
	}
	if(*pDIEPINT&DIEPINT_TxFEmp_MASK)
	{
		write_tx_fifo(ep);
	}
}

/* ********************************************************************************************* */
/* Bulk OUT (recv)
 */

static void tcc_out_epn(struct tcc_udc *dev, u32 ep_idx)
{
	struct tcc_ep *ep = &dev->ep[ep_idx];
	struct tcc_request *req;

	if (unlikely(!(ep->desc))) {
		/* Throw packet away.. */
		printk("%s: No descriptor?!?\n", __func__);
		return;
	}

	if(!rx_dmaStart) {
		if (list_empty(&ep->queue))
			req = 0;
		else
			req = list_entry(ep->queue.next,
					struct tcc_request, queue);

		if (unlikely(!req)) {
			DEBUG("%s: NULL REQ on OUT EP-%d\n", __func__, ep_idx);
			return;
			
		} else {
			read_fifo(ep, req);
		}
	}

}

static void stop_activity(struct tcc_udc *dev,
			  struct usb_gadget_driver *driver)
{
	int i;

	/* don't disconnect drivers more than once */
	if (dev->gadget.speed == USB_SPEED_UNKNOWN)
		driver = 0;
	dev->gadget.speed = USB_SPEED_UNKNOWN;

	/* prevent new request submissions, kill any outstanding requests  */
	for (i = 0; i < S3C_MAX_ENDPOINTS; i++) {
		struct tcc_ep *ep = &dev->ep[i];
		ep->stopped = 1;
		nuke(ep, -ESHUTDOWN);
	}

	/* report disconnect; the driver is already quiesced */
	if (driver) {
		spin_unlock(&dev->lock);
		driver->disconnect(&dev->gadget);
		spin_lock(&dev->lock);
	}

	/* re-init driver-visible data structures */
	udc_reinit(dev);
}

static void reconfig_usbd(void)
{
	//printk("---> %s\n", __func__);
#if 0
	// 2. Soft-reset OTG Core and then unreset again.
	u32 uTemp = tcc_writel(CORE_SOFT_RESET, S3C_UDC_OTG_GRSTCTL);

#if 0
	tcc_writel(	0<<15			// PHY Low Power Clock sel
			|1<<14			// Non-Periodic TxFIFO Rewind Enable
			|0x5<<10		// Turnaround time
			|0<<9|0<<8		// [0:HNP disable, 1:HNP enable][ 0:SRP disable, 1:SRP enable] H1= 1,1							
			|0<<7			// Ulpi DDR sel
			|0<<6			// 0: high speed utmi+, 1: full speed serial
			|0<<4			// 0: utmi+, 1:ulpi
			|1<<3			// phy i/f  0:8bit, 1:16bit
			|0x7<<0			// HS/FS Timeout*
				,S3C_UDC_OTG_GUSBCFG);
#endif
	
	// 3. Put the OTG device core in the disconnected state.
	uTemp = tcc_readl(S3C_UDC_OTG_DCTL);
	uTemp |= SOFT_DISCONNECT;
	tcc_writel(uTemp, S3C_UDC_OTG_DCTL);
	
	udelay(20);

	// 4. Make the OTG device core exit from the disconnected state.
	uTemp = tcc_readl(S3C_UDC_OTG_DCTL);
	uTemp = uTemp & ~SOFT_DISCONNECT;
	tcc_writel(uTemp, S3C_UDC_OTG_DCTL);	

	// 5. Configure OTG Core to initial settings of device mode.
	tcc_writel(1<<18|0x0<<0, S3C_UDC_OTG_DCFG);		// [][1: full speed(30Mhz) 0:high speed]

	// 6. Unmask the core interrupts
	tcc_writel(INT_RESUME|INT_ENUMDONE|INT_RESET|INT_SUSPEND|INT_RX_FIFO_NOT_EMPTY
				,S3C_UDC_OTG_GINTMSK);	
	
	// 7. Set NAK bit of EP0, EP1, EP2
	//u32 uTemp = tcc_readl(S3C_UDC_OTG_DOEPCTL0);
	tcc_writel(DEPCTL_EPDIS|DEPCTL_SNAK|(0<<0), S3C_UDC_OTG_DOEPCTL0); /* EP0: Control OUT */
	tcc_writel(DEPCTL_EPDIS|DEPCTL_SNAK|(0<<0), S3C_UDC_OTG_DIEPCTL0); /* EP0: Control IN */

	//uTemp = tcc_readl(S3C_UDC_OTG_DOEPCTL1);
	tcc_writel(DEPCTL_EPDIS|DEPCTL_SNAK|DEPCTL_BULK_TYPE|(0<<0), S3C_UDC_OTG_DOEPCTL1); /* EP1:Data OUT */
	tcc_writel(DEPCTL_EPDIS|DEPCTL_SNAK|DEPCTL_BULK_TYPE|(0<<0), S3C_UDC_OTG_DIEPCTL2); /* EP1:Data IN */

	// 8. Unmask EP interrupts on IN EPs : 0, 2
	//        		      OUT EPs : 0, 1
	tcc_writel(((1<<BULK_OUT_EP)|(1<<CONTROL_EP))<<16|((1<<BULK_IN_EP)|(1<<CONTROL_EP)), S3C_UDC_OTG_DAINTMSK);

	// 9. Unmask device IN EP common interrupts
	tcc_writel(AHB_ERROR, S3C_UDC_OTG_DOEPMSK);

	// 10. Unmask device OUT EP common interrupts
	tcc_writel(NON_ISO_IN_EP_TIMEOUT|AHB_ERROR, S3C_UDC_OTG_DIEPMSK);

	// 11. Set Rx FIFO Size
	tcc_writel(RX_FIFO_SIZE, S3C_UDC_OTG_GRXFSIZ);	
	
	// 12. Set Non Periodic Tx FIFO Size
	tcc_writel(NPTX_FIFO_SIZE<<16| NPTX_FIFO_START_ADDR<<0, S3C_UDC_OTG_GNPTXFSIZ);		

	// 13. Clear NAK bit of EP0, EP1, EP2
	// For Slave mode
	tcc_writel(DEPCTL_EPDIS|DEPCTL_CNAK|(0<<0), S3C_UDC_OTG_DOEPCTL0); /* EP0: Control OUT */

	// 14. Initialize OTG Link Core.
	tcc_writel(PTXFE_HALF|NPTXFE_HALF|MODE_SLAVE|BURST_INCR16|GBL_INT_UNMASK, S3C_UDC_OTG_GAHBCFG);
#endif

	OTGDEV_IO_Reset();
}

void set_max_pktsize(struct tcc_udc *dev, enum usb_device_speed speed)
{
	u32 ep_ctrl;
	
	if (speed == USB_SPEED_HIGH) {
		ep0_fifo_size = 64;
		ep_fifo_size = 512;
		ep_fifo_size2 = 1024;
		dev->gadget.speed = USB_SPEED_HIGH;
	} else {
		ep0_fifo_size = 64;
		ep_fifo_size = 64;
		ep_fifo_size2 = 64;
		dev->gadget.speed = USB_SPEED_FULL;
	}

	dev->ep[0].ep.maxpacket = ep0_fifo_size;
	dev->ep[1].ep.maxpacket = ep_fifo_size;
	dev->ep[2].ep.maxpacket = ep_fifo_size;
	dev->ep[3].ep.maxpacket = ep_fifo_size;
	dev->ep[4].ep.maxpacket = ep_fifo_size;
	dev->ep[5].ep.maxpacket = ep_fifo_size2;
	dev->ep[6].ep.maxpacket = ep_fifo_size2;
	dev->ep[7].ep.maxpacket = ep_fifo_size2;
	dev->ep[8].ep.maxpacket = ep_fifo_size2;


	if (speed == USB_SPEED_HIGH) {
		// EP0 - Control IN (64 bytes)
		ep_ctrl = tcc_readl(S3C_UDC_OTG_DIEPCTL0);
		usb_write(ep_ctrl|(0<<0), (u32) S3C_UDC_OTG_DIEPCTL0, 0);

		// EP0 - Control OUT (64 bytes)
		ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL0);
		usb_write(ep_ctrl|(0<<0), (u32) S3C_UDC_OTG_DOEPCTL0, 0);
	} else {
		// EP0 - Control IN (8 bytes)
		ep_ctrl = tcc_readl(S3C_UDC_OTG_DIEPCTL0);
		usb_write(ep_ctrl|(3<<0), (u32) S3C_UDC_OTG_DIEPCTL0, 0);

		// EP0 - Control OUT (8 bytes)
		ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL0);
		usb_write(ep_ctrl|(3<<0), (u32) S3C_UDC_OTG_DOEPCTL0, 0);	
	}
		

	// EP1 - Bulk Data OUT (512 bytes)
	ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL1);
	usb_write(ep_ctrl|(ep_fifo_size<<0), (u32) S3C_UDC_OTG_DOEPCTL1, 1);

	// EP2 - Bulk Data IN (512 bytes)
	ep_ctrl = tcc_readl(S3C_UDC_OTG_DIEPCTL2);
	usb_write(ep_ctrl|(ep_fifo_size<<0), (u32) S3C_UDC_OTG_DIEPCTL2, 2);


	// jmlee 
	// EP3 - Bulk Data OUT (512 bytes)
	ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL3);
	usb_write(ep_ctrl|(ep_fifo_size<<0), (u32) S3C_UDC_OTG_DOEPCTL3, 3);

	// EP4 - Bulk Data IN (512 bytes)
	ep_ctrl = tcc_readl(S3C_UDC_OTG_DIEPCTL4);
	usb_write(ep_ctrl|(ep_fifo_size<<0), (u32) S3C_UDC_OTG_DIEPCTL4, 4);
	
}

static int reset_available = 1;
/*
 *	elfin usb client interrupt handler.
 */
static irqreturn_t tcc_udc_irq(int irq, void *_dev)
{
	struct tcc_udc *dev = _dev;
	u32 intr_status, grx_status, fifoCntByte = 0;
	u32 usb_status, ep_ctrl, gintmsk;
	
	spin_lock(&dev->lock);
	//printk("===>> %s, %p, gadget is %p\n", __func__, _dev, &dev->gadget);

	intr_status = tcc_readl(S3C_UDC_OTG_GINTSTS);
	gintmsk = tcc_readl(S3C_UDC_OTG_GINTMSK);
	
	DEBUG_ISR("GINTSTS=0x%x(on state %s), GINTMSK : 0x%x\n",
			intr_status, state_names[dev->ep0state], gintmsk);
	
	if (!intr_status) {
		spin_unlock(&dev->lock);
		return IRQ_HANDLED;
	}

	if (intr_status) {
		if (intr_status & INT_RESET) {
			usb_status = tcc_readl(S3C_UDC_OTG_GOTGCTL);
			DEBUG_SETUP("####################################\n");
			DEBUG_SETUP("    %s: Reset interrupt - (GOTGCTL):0x%x\n", __func__, usb_status);
			tcc_writel(INT_RESET, S3C_UDC_OTG_GINTSTS);

			//if((usb_status & 0xc0000) == (0x3 << 18)) {
				//if(reset_available) {
					DEBUG_SETUP("     ===> OTG core got reset (%d)!! \n", reset_available);
					//printk("---->>>>>>>>> reset %p\n", _dev);
					reconfig_usbd();
					//printk("---->>>>>>>>> after reset %p\n", _dev);
					dev->ep0state = WAIT_FOR_SETUP;
				//	reset_available = 0;
				//}
			//}
			//else {
			//	reset_available = 1;
			//	DEBUG_SETUP("      RESET handling skipped : reset_available : %d\n", reset_available);
			//}
		}

		if (intr_status & INT_ENUMDONE) {
			DEBUG_SETUP("####################################\n");
			DEBUG_SETUP("    %s: Speed Detection interrupt\n",
					__func__);
			tcc_writel(INT_ENUMDONE, S3C_UDC_OTG_GINTSTS);
			
			usb_status = (tcc_readl(S3C_UDC_OTG_DSTS) & 0x6);

			if (usb_status & (USB_FULL_30_60MHZ | USB_FULL_48MHZ)) {
				DEBUG_SETUP("    %s: Full Speed Detection\n",__func__);
				set_max_pktsize(dev, USB_SPEED_FULL);
				
			} else {
				DEBUG_SETUP("    %s: High Speed Detection : 0x%x\n", __func__, usb_status);
				set_max_pktsize(dev, USB_SPEED_HIGH);
			}

			OTGDEV_IO_EnumDone();
		}

		if (intr_status & INT_EARLY_SUSPEND) {
			DEBUG_SETUP("####################################\n");
			DEBUG_SETUP("    %s:Early suspend interrupt\n", __func__);
			tcc_writel(INT_EARLY_SUSPEND, S3C_UDC_OTG_GINTSTS);
		}

		if (intr_status & INT_SUSPEND) {
			usb_status = tcc_readl(S3C_UDC_OTG_DSTS);
			DEBUG_SETUP("####################################\n");
			DEBUG_SETUP("    %s:Suspend interrupt :(DSTS):0x%x\n", __func__, usb_status);
			tcc_writel(INT_SUSPEND, S3C_UDC_OTG_GINTSTS);
			
			if (dev->gadget.speed != USB_SPEED_UNKNOWN
			    && dev->driver
			    && dev->driver->suspend) {
				dev->driver->suspend(&dev->gadget);
			}
		}

		if (intr_status & INT_RESUME) {
			DEBUG_SETUP("####################################\n");
			DEBUG_SETUP("    %s: Resume interrupt\n", __func__);
			tcc_writel(INT_RESUME, S3C_UDC_OTG_GINTSTS);
			
			if (dev->gadget.speed != USB_SPEED_UNKNOWN
			    && dev->driver
			    && dev->driver->resume) {
				dev->driver->resume(&dev->gadget);
			}
		}

	}

	if (intr_status & INT_RX_FIFO_NOT_EMPTY) {
		u8 iEP;
		// Mask USB OTG 2.0 interrupt source : INT_RX_FIFO_NOT_EMPTY
		gintmsk &= ~INT_RX_FIFO_NOT_EMPTY;
		tcc_writel(gintmsk, S3C_UDC_OTG_GINTMSK);

		grx_status = tcc_readl(S3C_UDC_OTG_GRXSTSR);
		DEBUG_ISR("    INT_RX_FIFO_NOT_EMPTY(GRXSTSR):0x%x, GINTMSK:0x%x\n", grx_status, gintmsk);

		iEP = grx_status&0xF;
		fifoCntByte = (grx_status & 0x7ff0)>>4;

		if(fifoCntByte) {

			if ((grx_status & 0x1E0000) == SETUP_PKT_RECEIVED)  {
				DEBUG_EP0("      => A SETUP data packet received : %d bytes\n", fifoCntByte);
				tcc_handle_ep0(dev);

				// Unmask USB OTG 2.0 interrupt source : INT_RX_FIFO_NOT_EMPTY
				gintmsk |= INT_RX_FIFO_NOT_EMPTY;	
			}
			else if ((grx_status & 0x1E0000) == OUT_PKT_RECEIVED)
			{
				if( iEP!=0 ) {
					//ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL1);
					//DEBUG_ISR("      => A Bulk OUT data packet received : %d bytes, (DOEPCTL1):0x%x\n", 
					//	fifoCntByte, ep_ctrl);
					//printk("bulk out is received iEP=%d length=%d\n",iEP,fifoCntByte);
					tcc_out_epn(dev, iEP);
					gintmsk = tcc_readl(S3C_UDC_OTG_GINTMSK);
					pOTG->DEVOUTENDPT[iEP-1][DOEPCTL_INDEX] |= DOEPCTL_CNAK; //tcc_writel(ep_ctrl | DEPCTL_CNAK, S3C_UDC_OTG_DOEPCTL1);
				}
				else /*if (grx_status & CONTROL_EP)*/ {
					DEBUG_EP0("      => A CONTROL OUT data packet received : %d bytes\n", 
						fifoCntByte);	
				}
				//else {
				//	DEBUG_ISR("      => Unused EP: %d bytes, (GRXSTSR):0x%x\n", fifoCntByte, grx_status);
				//}
			}
			else {
				grx_status = tcc_readl(S3C_UDC_OTG_GRXSTSP);
				
				// Unmask USB OTG 2.0 interrupt source : INT_RX_FIFO_NOT_EMPTY
				gintmsk |= INT_RX_FIFO_NOT_EMPTY;

				DEBUG_ISR("      => A reserved packet received : %d bytes\n", fifoCntByte);
			}
		}
		else {
			if ((grx_status & 0x1E0000) == SETUP_TRANSACTION_COMPLETED)
			{	
				ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL0);
				DEBUG_EP0("      => A SETUP transaction completed (DOEPCTL0):0x%x\n", ep_ctrl);
				tcc_writel(ep_ctrl | DEPCTL_CNAK, S3C_UDC_OTG_DOEPCTL0);
			} 
			else if ( (grx_status & 0x1E0000)==OUT_TRANSFER_COMPLELTED && iEP!=0 )
			{
				ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL1);
				DEBUG_ISR("      => An OUT transaction completed (DOEPCTL1):0x%x\n", ep_ctrl);
				pOTG->DEVOUTENDPT[iEP-1][DOEPCTL_INDEX] |= DOEPCTL_CNAK; //tcc_writel(ep_ctrl | DEPCTL_CNAK, S3C_UDC_OTG_DOEPCTL1);
			}
			else if ((grx_status & 0x1E0000) == OUT_PKT_RECEIVED)
			{
				DEBUG_ISR("      => A  OUT PACKET RECEIVED (NO FIFO CNT BYTE)...(GRXSTSR):0x%x\n", grx_status);
			}
			else
			{
				DEBUG_ISR("      => A RESERVED PACKET RECEIVED (NO FIFO CNT BYTE)...(GRXSTSR):0x%x\n", grx_status);
			}

			grx_status = tcc_readl(S3C_UDC_OTG_GRXSTSP);

			// Unmask USB OTG 2.0 interrupt source : INT_RX_FIFO_NOT_EMPTY
			gintmsk |= INT_RX_FIFO_NOT_EMPTY;
		}

		// Un/Mask USB OTG 2.0 interrupt sources
		tcc_writel(gintmsk, S3C_UDC_OTG_GINTMSK);

		//spin_unlock(&dev->lock);
		//return IRQ_HANDLED;
	}

	// It's not available for TCC
	//if (intr_status & INT_TX_FIFO_EMPTY) {
	//	DEBUG_ISR("    INT_TX_FIFO_EMPTY(GNPTXSTS):0x%x, (DIEPCTL2):0x%x, GINTMSK:0x%x\n", 
	//			tcc_readl(S3C_UDC_OTG_GNPTXSTS), tcc_readl(S3C_UDC_OTG_DIEPCTL2),
	//			tcc_readl(S3C_UDC_OTG_GINTMSK));
	//	tcc_in_epn(dev, 2);
	//}
	if( intr_status & GINTSTS_IEPInt_MASK )
	{
		unsigned int i;

		gintmsk &= ~GINTSTS_IEPInt_MASK;
		tcc_writel(gintmsk, S3C_UDC_OTG_GINTMSK);

		for(i=0;i<16;i++)
		{
			volatile unsigned int DAINT_masked = pOTG->DAINT & pOTG->DAINTMSK;
			if( DAINT_masked & (1<<i) )
			{
				//volatile unsigned int *pDIEPINT;
				//unsigned int DIEPINT;
				//
				//if( i == 0 )
				//	pDIEPINT = &pOTG->DIEPINT0;
				//else
				//	pDIEPINT = &pOTG->DEVINENDPT[i-1][DIEPINT_INDEX];
				//
				//DIEPINT = *pDIEPINT;
				//
				////printk("irq: EP%d DIEPINT=%X\n",i,DIEPINT);
				//
				//if( DIEPINT & DIEPINT_INTknTXFEmp )
				//{
				//	tcc_in_epn(dev,i);
				//	*pDIEPINT = DIEPINT_INTknTXFEmp;
				//}
				tcc_in_ep_irq(dev,i);
			}
		}

		gintmsk |= GINTSTS_IEPInt_MASK;
		tcc_writel(gintmsk, S3C_UDC_OTG_GINTMSK);
	}
	spin_unlock(&dev->lock);

	return IRQ_HANDLED;
}

unsigned int OTGDEV_IO_EP_Enable(struct tcc_ep *ep, unsigned int enable)
{
	unsigned int DIOEPCTL;
	unsigned int iEndPoint = ep_index(ep);

	if( iEndPoint == 0 )
		return FALSE;

	if( enable )
	{
		DIOEPCTL=DIOEPCTL_SetD0PID|DIOEPCTL_USBActEP|DIOEPCTL_MPS(ep_maxpacket(ep));

		switch( ep->bmAttributes )
		{
			case USB_ENDPOINT_XFER_CONTROL:
				DIOEPCTL |= DIOEPCTL_EPType_CONTROL;
				break;
			case USB_ENDPOINT_XFER_ISOC:
				DIOEPCTL |= DIOEPCTL_EPType_ISOCHRONOUS;
				break;
			case USB_ENDPOINT_XFER_BULK:
				DIOEPCTL |= DIOEPCTL_EPType_BULK;
				break;
			case USB_ENDPOINT_XFER_INT:
				DIOEPCTL |= DIOEPCTL_EPType_INTERRUPT;
				break;
		}

		if( ep_is_in(ep) )
		{
			// IN
			//pOTG->DIEPEMPMSK |= 1<<iEndPoint;
			BITSET(pOTG->DAINTMSK,DAINTMSK_InEPInt(ep_index(ep)));
			pOTG->DEVINENDPT[iEndPoint-1][DIEPCTL_INDEX] = DIOEPCTL|DIEPCTL_TxFNum(iEndPoint);
		}
		else
		{
			// OUT
			pOTG->DEVOUTENDPT[iEndPoint-1][DOEPCTL_INDEX] = DIOEPCTL|DOEPCTL_EPEna|DOEPCTL_CNAK;
		}

		return TRUE;
	}
	else
	{
		if( ep_is_in(ep) )
		{
			_loc_Disable_IN_EP(iEndPoint);
			BITCLR(pOTG->DAINTMSK,DAINTMSK_InEPInt(ep_index(ep)));
		}
	}

	return TRUE;
}

static int tcc_ep_enable(struct usb_ep *_ep, const struct usb_endpoint_descriptor *desc)
{
	struct tcc_ep *ep;
	struct tcc_udc *dev;
	unsigned long flags;

	DEBUG("%s: %p\n", __func__, _ep);

	ep = container_of(_ep, struct tcc_ep, ep);

	if (!_ep || !desc || ep->desc || _ep->name == ep0name
	    || desc->bDescriptorType != USB_DT_ENDPOINT
	    || (ep->bEndpointAddress&0x4) != (desc->bEndpointAddress&0x4)
	    || ep_maxpacket(ep) < le16_to_cpu(desc->wMaxPacketSize)) {
		printk("%s: bad ep or descriptor\n", __func__);
		return -EINVAL;
	}

	/* xfer types must match, except that interrupt ~= bulk */
	if (ep->bmAttributes != desc->bmAttributes
	    && ep->bmAttributes != USB_ENDPOINT_XFER_BULK
	    && desc->bmAttributes != USB_ENDPOINT_XFER_INT) {
		printk("%s: %s type mismatch\n", __func__, _ep->name);
		return -EINVAL;
	}

	/* hardware _could_ do smaller, but driver doesn't */
	if ((desc->bmAttributes == USB_ENDPOINT_XFER_BULK
	     && le16_to_cpu(desc->wMaxPacketSize) != ep_maxpacket(ep))
	    || !desc->wMaxPacketSize) {
		printk("%s: bad %s maxpacket\n", __func__, _ep->name);
		return -ERANGE;
	}

	dev = ep->dev;
	if (!dev->driver || dev->gadget.speed == USB_SPEED_UNKNOWN) {
		printk("%s: bogus device state\n", __func__);
		return -ESHUTDOWN;
	}

	spin_lock_irqsave(&ep->dev->lock, flags);

	ep->stopped = 0;
	ep->desc = desc;
	ep->pio_irqs = 0;
	ep->ep.maxpacket = le16_to_cpu(desc->wMaxPacketSize);
	ep->bEndpointAddress = desc->bEndpointAddress;

	/* Reset halt state */
	//tcc_set_halt(_ep, 0);
	//printk("ep enable index=%d max=%d attr=%d isIn=%d\n",ep_index(ep),ep_maxpacket(ep),ep->bmAttributes,ep_is_in(ep));
	OTGDEV_IO_EP_Enable(ep,TRUE);

	spin_unlock_irqrestore(&ep->dev->lock, flags);

	DEBUG("%s: enabled %s\n", __func__, _ep->name);
	return 0;
}

/** Disable EP
 */
static int tcc_ep_disable(struct usb_ep *_ep)
{
	struct tcc_ep *ep;
	unsigned long flags;

	DEBUG("%s: %p\n", __func__, _ep);

	ep = container_of(_ep, struct tcc_ep, ep);
	if (!_ep || !ep->desc) {
		DEBUG("%s: %s not enabled\n", __func__,
		      _ep ? ep->ep.name : NULL);
		return -EINVAL;
	}

	spin_lock_irqsave(&ep->dev->lock, flags);

	/* Nuke all pending requests */
	nuke(ep, -ESHUTDOWN);
	OTGDEV_IO_EP_Enable(ep,FALSE);

	ep->desc = 0;
	ep->stopped = 1;

	spin_unlock_irqrestore(&ep->dev->lock, flags);

	DEBUG("%s: disabled %s\n", __func__, _ep->name);
	return 0;
}

static struct usb_request *tcc_alloc_request(struct usb_ep *ep,	 gfp_t gfp_flags)
{
	struct tcc_request *req;

	DEBUG("%s: %s %p\n", __func__, ep->name,ep);

	req = kmalloc(sizeof *req, gfp_flags);
	if (!req)
		return 0;

	memset(req, 0, sizeof *req);
	INIT_LIST_HEAD(&req->queue);

	return &req->req;
}

static void tcc_free_request(struct usb_ep *ep, struct usb_request *_req)
{
	struct tcc_request *req;

	DEBUG("%s: %s %p\n", __func__, ep->name, ep);

	req = container_of(_req, struct tcc_request, req);
	WARN_ON(!list_empty(&req->queue));
	kfree(req);
}

#if 0
static void *tcc_alloc_buffer(struct usb_ep *ep, unsigned bytes,
				  dma_addr_t * dma, gfp_t gfp_flags)
{
	char *retval;

	DEBUG("%s (%p, %d, %d)\n", __func__, ep, bytes, gfp_flags);

	retval = kmalloc(bytes, gfp_flags & ~(__GFP_DMA | __GFP_HIGHMEM));
	if (retval)
		*dma = virt_to_dma(NULL, retval);
	return retval;
}

static void tcc_free_buffer(struct usb_ep *ep, void *buf, dma_addr_t dma,
				unsigned bytes)
{
	DEBUG("%s: %p\n", __func__, ep);
	kfree(buf);
}
#endif

static void tcc_ep0_read(struct tcc_udc *dev);
/** Queue one request
 *  Kickstart transfer if needed
 */
static int tcc_queue(struct usb_ep *_ep, struct usb_request *_req, gfp_t gfp_flags)
{
	struct tcc_request *req;
	struct tcc_ep *ep;
	struct tcc_udc *dev;
	unsigned long flags;

	DEBUG("%s: %s %p\n", __func__, _ep->name, _ep);

	req = container_of(_req, struct tcc_request, req);
	if (unlikely
	    (!_req || !_req->complete || !_req->buf
	     || !list_empty(&req->queue))) {

		if (list_empty(&req->queue) == NULL) {
			printk("-->queue is NULL.\n");
		}
		 
		DEBUG("%s: bad params\n", __func__);
		return -EINVAL;
	}

	ep = container_of(_ep, struct tcc_ep, ep);
	if (unlikely(!_ep || (!ep->desc && ep->ep.name != ep0name))) {
		DEBUG("%s: bad ep\n", __func__);
		return -EINVAL;
	}

	dev = ep->dev;
	if (unlikely(!dev->driver || dev->gadget.speed == USB_SPEED_UNKNOWN)) {
		DEBUG("%s: bogus device state %p\n", __func__, dev->driver);
		return -ESHUTDOWN;
	}

	DEBUG("%s: %s queue req %p, len %d buf %p\n",
		__func__, _ep->name, _req, _req->length, _req->buf);
	//printk("%s: %s queue req %p, len %d buf %p\n",
	//	__func__, _ep->name, _req, _req->length, _req->buf);

	spin_lock_irqsave(&dev->lock, flags);

	_req->status = -EINPROGRESS;
	_req->actual = 0;

	/* kickstart this i/o queue? */
	DEBUG("%s: Add to ep=%d, Q empty=%d, stopped=%d\n",
		__func__, ep_index(ep), list_empty(&ep->queue), ep->stopped);
	if (list_empty(&ep->queue) && likely(!ep->stopped)) {
		u32 csr;

		if (unlikely(ep_index(ep) == 0)) {
			/* EP0 */
			list_add_tail(&req->queue, &ep->queue);
			if (ep_is_in(ep)) {
				dev->ep0state = DATA_STATE_XMIT;
				//printk("DAINTMSK=%X\n",pOTG->DAINTMSK);
				//if( !(pOTG->DAINTMSK & DAINTMSK_InEPInt(0)) )
				{
					//tcc_ep0_write(dev);
					//BITSET(pOTG->DIEPCTL0, DIEPCTL0_CNAK);
					//BITSET(pOTG->DAINTMSK, DAINTMSK_InEPInt(0));
					ep->xfer_len= _req->length;
					ep->xfer_count = 0;
					ep->xfer_buff = _req->buf;
					udc_ep_in_start_transfer(ep);
				}
				//printk("tcc_ep0_kick\n");
				//printk("DIEPCTL0=%X\n",pOTG->DIEPCTL0);
			} else {
				dev->ep0state = DATA_STATE_RECV;
				tcc_ep0_read(dev);
			}
			//tcc_ep0_kick(dev, ep);
			req = 0;

		} else if (ep_is_in(ep)) {
			//csr = usb_read((u32) S3C_UDC_OTG_GINTSTS, ep_index(ep));
			//DEBUG("%s: ep_is_in, S3C_UDC_OTG_GINTSTS=0x%x\n",
			//	__func__, csr);
			//
			//if((csr & INT_TX_FIFO_EMPTY)
			//   && (write_fifo(ep, req) == 1))
			//	req = 0;
			//else
			//	DEBUG("++++ IN-list_add_taill::req=%p, DIEPCTL2:0x%x\n", req,
			//		tcc_readl(S3C_UDC_OTG_DIEPCTL2));
			//mdelay(10);
			//if( pOTG->DEVINENDPT[ep_index(ep)-1][DIEPINT_INDEX] & DIEPINT_TxFEmp_MASK
			//	&& !(pOTG->DAINTMSK & DAINTMSK_InEPInt(ep_index(ep))) )
			//	if( write_fifo(ep,req) == 1 )
			//		req = 0;
			//if( req )
			//{
			//	DEBUG("++++ IN-list_add_taill::req=%p, DIEPINT%d:0x%x\n", req,ep_index(ep),
			//		pOTG->DEVINENDPT[ep_index(ep)-1][DIEPINT_INDEX]);
			//}
			//BITSET(pOTG->DEVINENDPT[ep_index(ep)-1][DIEPCTL_INDEX], DIEPCTL_CNAK);
			//printk("queue ep%d DIEPINT=%X\n",ep_index(ep),pOTG->DEVINENDPT[ep_index(ep)-1][DIEPINT_INDEX]);
			//BITSET(pOTG->DAINTMSK, DAINTMSK_InEPInt(ep_index(ep)));
			ep->xfer_len= _req->length;
			ep->xfer_count = 0;
			ep->xfer_buff = _req->buf;
			udc_ep_in_start_transfer(ep);
		} else {
			csr = usb_read((u32) S3C_UDC_OTG_GINTSTS, ep_index(ep));
			DEBUG("%s: ep_is_out, S3C_UDC_OTG_GINTSTS=0x%x\n",
				__func__, csr);
			
			if((csr & INT_RX_FIFO_NOT_EMPTY)
			   && (read_fifo(ep, req) == 1))
				req = 0;
			else
				DEBUG("++++ OUT-list_add_taill::req=%p, DOEPCTL1:0x%x\n", req,
					tcc_readl(S3C_UDC_OTG_DOEPCTL1));
		}
	}

	/* pio or dma irq handler advances the queue. */
	if (likely(req != 0)) 
	{
		list_add_tail(&req->queue, &ep->queue);
		//printk("IN list add tail ep%d length=%d\n",ep_index(ep),_req->length);
	}

	spin_unlock_irqrestore(&dev->lock, flags);

	return 0;
}

/* dequeue JUST ONE request */
static int tcc_dequeue(struct usb_ep *_ep, struct usb_request *_req)
{
	struct tcc_ep *ep;
	struct tcc_request *req;
	unsigned long flags;

	DEBUG("%s: %p\n", __func__, _ep);

	ep = container_of(_ep, struct tcc_ep, ep);
	if (!_ep || ep->ep.name == ep0name)
		return -EINVAL;

	spin_lock_irqsave(&ep->dev->lock, flags);

	/* make sure it's actually queued on this endpoint */
	list_for_each_entry(req, &ep->queue, queue) {
		if (&req->req == _req)
			break;
	}
	if (&req->req != _req) {
		spin_unlock_irqrestore(&ep->dev->lock, flags);
		return -EINVAL;
	}

	done(ep, req, -ECONNRESET);

	spin_unlock_irqrestore(&ep->dev->lock, flags);
	return 0;
}

/** Halt specific EP
 *  Return 0 if success
 */
static int tcc_set_halt(struct usb_ep *_ep, int value)
{
	struct tcc_ep *ep;
	unsigned long flags;

	ep = container_of(_ep, struct tcc_ep, ep);
	if (!_ep)
		return -EINVAL;

	spin_lock_irqsave(&ep->dev->lock, flags);

	/*
	 * We can't halt IN endpoints while there are still data to be
	 * transferred
	*/
	if( !list_empty(&ep->queue) )
	{
		return -EAGAIN;
	}
	else
	{
		if( value )
		{
			//OTGDEV_IO_EP_Stall(ep->bEndpointAddress);
		}
		else
		{
			//OTGDEV_IO_EP_Unstall(ep->bEndpointAddress);
		}
	}

	spin_unlock_irqrestore(&ep->dev->lock, flags);
	return 0;
}

/** Return bytes in EP FIFO
 */
static int tcc_fifo_status(struct usb_ep *_ep)
{
	int count = 0;
	struct tcc_ep *ep;

	ep = container_of(_ep, struct tcc_ep, ep);
	if (!_ep) {
		DEBUG("%s: bad ep\n", __func__);
		return -ENODEV;
	}

	DEBUG("%s: %d\n", __func__, ep_index(ep));

	/* LPD can't report unclaimed bytes from IN fifos */
	if (ep_is_in(ep))
		return -EOPNOTSUPP;

	return count;
}

/** Flush EP FIFO
 */
static void tcc_fifo_flush(struct usb_ep *_ep)
{
	struct tcc_ep *ep;

	ep = container_of(_ep, struct tcc_ep, ep);
	if (unlikely(!_ep || (!ep->desc && ep->ep.name != ep0name))) {
		DEBUG("%s: bad ep\n", __func__);
		return;
	}
	
	DEBUG("%s: %d\n", __func__, ep_index(ep));
}

/****************************************************************/
/* End Point 0 related functions                                */
/****************************************************************/

/* return:  0 = still running, 1 = completed, negative = errno */
//static int write_fifo_ep0(struct tcc_ep *ep, struct tcc_request *req)
//{
//	u32 max;
//	unsigned count;
//	int is_last;
//
//	max = ep_maxpacket(ep);
//
//	DEBUG_EP0("%s: max = %d\n", __func__, max);
//
//	count = write_packet(ep, req, max);
//
//	/* last packet is usually short (or a zlp) */
//	if (unlikely(count != max))
//		is_last = 1;
//	else {
//		if (likely(req->req.length != req->req.actual) || req->req.zero)
//			is_last = 0;
//		else
//			is_last = 1;
//	}
//
//	DEBUG_EP0("%s: wrote %s %d bytes%s %d left %p\n", __func__,
//		  ep->ep.name, count,
//		  is_last ? "/L" : "", req->req.length - req->req.actual, req);
//
//	/* requests complete when all IN data is in the FIFO */
//	if (is_last) {
//		return 1;
//	}
//
//	return 0;
//}

static __inline__ int tcc_fifo_read(struct tcc_ep *ep, u32 *cp, int max)
{
	int bytes;
	int count;
	u32 grx_status = tcc_readl(S3C_UDC_OTG_GRXSTSP);
	bytes = (grx_status & 0x7ff0)>>4;
	
	DEBUG_EP0("%s: GRXSTSP=0x%x, bytes=%d, ep_index=%d, fifo=0x%x\n",
			__func__, grx_status, bytes, ep_index(ep), ep->fifo);

	// 32 bits interface
	count = bytes / 4;
	
	while (count--) {
		*cp++ = (u32) tcc_readl(S3C_UDC_OTG_EP0_FIFO);
	}
	
	return bytes;
}

static int read_fifo_ep0(struct tcc_ep *ep, struct tcc_request *req)
{
	u32 csr;
	u32 *buf;
	unsigned bufferspace, count, is_short, bytes;
	u32 fifo = ep->fifo;

	DEBUG_EP0("%s\n", __func__);

	csr = tcc_readl(S3C_UDC_OTG_GRXSTSP);
	bytes = (csr & 0x7ff0)>>4;

	buf = req->req.buf + req->req.actual;
	prefetchw(buf);
	bufferspace = req->req.length - req->req.actual;

	/* read all bytes from this packet */
	if (likely((csr & 0xf) & CONTROL_EP)) {
		count = bytes / 4;
		req->req.actual += min(bytes, bufferspace);
		
	} else	{		// zlp 
		count = 0;
		bytes = 0;
	}

	is_short = (bytes < ep->ep.maxpacket);
	DEBUG_EP0("%s: read %s %02x, %d bytes%s req %p %d/%d\n",  __func__,  ep->ep.name, csr, bytes,
		  is_short ? "/S" : "", req, req->req.actual, req->req.length);

	while (likely(count-- != 0)) {
		u32 byte = (u32) tcc_readl(fifo);

		if (unlikely(bufferspace == 0)) {
			/* this happens when the driver's buffer
			 * is smaller than what the host sent.
			 * discard the extra data.
			 */
			if (req->req.status != -EOVERFLOW)
				DEBUG_EP0("%s overflow %d\n", ep->ep.name,
					  count);
			req->req.status = -EOVERFLOW;
		} else {
			*buf++ = byte;
			bufferspace = bufferspace - 4;
		}
	}
	
	/* completion */
	if (is_short || req->req.actual == req->req.length) {
		return 1;
	}

	return 0;
}

/**
 * udc_set_address - set the USB address for this device
 * @address:
 *
 * Called from control endpoint function
 * after it decodes a set address setup packet.
 */
static void udc_set_address(struct tcc_udc *dev, unsigned char address)
{	
	u32 tmp = tcc_readl(S3C_UDC_OTG_DCFG);
	tcc_writel(address << 4 | tmp, S3C_UDC_OTG_DCFG);

	tmp = tcc_readl(S3C_UDC_OTG_DIEPCTL0);
	tcc_writel((1<<31)|DEPCTL_CNAK|tmp, S3C_UDC_OTG_DIEPCTL0); /* EP0: Control IN */
	
	DEBUG_EP0("%s: USB OTG 2.0 Device address=%d, DCFG=0x%x\n",	__func__, address, tcc_readl(S3C_UDC_OTG_DCFG));
	
	dev->usb_address = address;
}



static int first_time = 1;

static void tcc_ep0_read(struct tcc_udc *dev)
{
	struct tcc_request *req;
	struct tcc_ep *ep = &dev->ep[0];
	int ret;

	if (!list_empty(&ep->queue))
		req = list_entry(ep->queue.next, struct tcc_request, queue);
	else {
		DEBUG_EP0("%s: ---> BUG !!! \n", __func__);
		BUG();	//logic ensures		-jassi
		return;
	}
	
	DEBUG_EP0("%s: req.length = 0x%x, req.actual = 0x%x\n", __func__, req->req.length, req->req.actual);
	
	if(req->req.length == 0) {
		dev->ep0state = WAIT_FOR_SETUP;
		first_time = 1;
		done(ep, req, 0);
		return;
	}

	if(!req->req.actual && first_time){	//for SetUp packet
		first_time = 0;
		return;
	}

	ret = read_fifo_ep0(ep, req);
	if (ret) {
		dev->ep0state = WAIT_FOR_SETUP;
		first_time = 1;
		done(ep, req, 0);
		return;
	}

}

/*
 * DATA_STATE_XMIT
 */
//static int tcc_ep0_write(struct tcc_udc *dev)
//{
//	struct tcc_request *req;
//	struct tcc_ep *ep = &dev->ep[0];
//	int ret, need_zlp = 0;
//
//	DEBUG_EP0("%s: ep0 write\n", __func__);
//
//	if (list_empty(&ep->queue))
//		req = 0;
//	else
//		req = list_entry(ep->queue.next, struct tcc_request, queue);
//
//	if (!req) {
//		DEBUG_EP0("%s: NULL REQ\n", __func__);
//		return 0;
//	}
//
//	DEBUG_EP0("%s: req.length = 0x%x, req.actual = 0x%x\n",
//		__func__, req->req.length, req->req.actual);
//	
//	if (req->req.length == 0) {	
//		dev->ep0state = WAIT_FOR_SETUP;
//	   	done(ep, req, 0);
//		return 1;
//	}
//
//	if (req->req.length - req->req.actual == ep0_fifo_size) {
//		/* Next write will end with the packet size, */
//		/* so we need Zero-length-packet */
//		need_zlp = 1;
//	}
//
//	ret = write_fifo_ep0(ep, req);
//
//	if ((ret == 1) && !need_zlp) {
//		/* Last packet */
//		DEBUG_EP0("%s: finished, waiting for status\n", __func__);
//		dev->ep0state = WAIT_FOR_SETUP;
//	} else {
//		DEBUG_EP0("%s: not finished\n", __func__);
//	}
//
//	if (need_zlp) {
//		DEBUG_EP0("%s: Need ZLP!\n", __func__);
//		dev->ep0state = DATA_STATE_NEED_ZLP;
//	}
//
//	if(ret)
//		done(ep, req, 0);
//
//	//BITSET(pOTG->DAINTMSK, DAINTMSK_InEPInt(0));
//
//	return 1;
//}

/*
 * WAIT_FOR_SETUP (OUT_PKT_RDY)
 */
static void tcc_ep0_setup(struct tcc_udc *dev)
{

	struct tcc_ep *ep = &dev->ep[0];
	struct usb_ctrlrequest ctrl;

	int i, bytes, is_in;
	u32 ep_ctrl;

	/* Nuke all previous transfers */
	nuke(ep, -EPROTO);

	/* read control req from fifo (8 bytes) */
	bytes = tcc_fifo_read(ep, (u32 *)&ctrl, 8);

	DEBUG_SETUP("Read CTRL REQ %d bytes\n", bytes);
	DEBUG_SETUP("CTRL.bRequestType = 0x%x (is_in %d)\n", ctrl.bRequestType, ctrl.bRequestType & USB_DIR_IN);
	DEBUG_SETUP("CTRL.bRequest = 0x%x\n", ctrl.bRequest);
	DEBUG_SETUP("CTRL.wLength = 0x%x\n", ctrl.wLength);
	DEBUG_SETUP("CTRL.wValue = 0x%x (%d)\n", ctrl.wValue, ctrl.wValue >> 8);
	DEBUG_SETUP("CTRL.wIndex = 0x%x\n", ctrl.wIndex);

	/* Set direction of EP0 */
	if (likely(ctrl.bRequestType & USB_DIR_IN)) {
		ep->bEndpointAddress |= USB_DIR_IN;
		is_in = 1;
	} else {
		ep->bEndpointAddress &= ~USB_DIR_IN;
		is_in = 0;
	}

	dev->req_pending = 1;

	/* Handle some SETUP packets ourselves */
	switch (ctrl.bRequest) {
		case USB_REQ_GET_STATUS:
		{
			if (ctrl.bRequestType == (USB_DIR_IN | USB_RECIP_DEVICE)) {
				*dev->status_buf = 0; 
			}
			else if (ctrl.bRequestType == (USB_DIR_IN | USB_RECIP_INTERFACE)) {
				*dev->status_buf = __constant_cpu_to_le16(0);
			}
			else if (ctrl.bRequestType == (USB_DIR_IN | USB_RECIP_ENDPOINT)) {
				*dev->status_buf = 0;
				if ( OTGDEV_IO_IsStalled((unsigned char)le16_to_cpu(ctrl.wIndex)) )
					*dev->status_buf |= __constant_cpu_to_le16(1);
			}
			else
				break;
			/* Write directly to the FIFO. No queueing is done. */
			if (ctrl.wLength != __constant_cpu_to_le16(sizeof(unsigned short)))
			{
				OTGDEV_IO_EP_Stall(0x80);
				dev->ep0state = WAIT_FOR_SETUP;
				return;
			}

			//dev->ep0req->length = 2;
			//dev->ep0req->zero = 0;
			//memcpy(dev->ep0req->buf, &status, 2);
			//tcc_queue(&ep->ep, dev->ep0req, GFP_ATOMIC);
			
			dev->ep0state = DATA_STATE_XMIT;
			{
				ep->xfer_len = sizeof(unsigned short);
				ep->xfer_count = 0;
				ep->xfer_buff = (u32*)dev->status_buf;
				udc_ep_in_start_transfer(ep);
			}
			return;
		}

		case USB_REQ_CLEAR_FEATURE:
		{
			if (ctrl.bRequestType == USB_RECIP_DEVICE) {
				//if (feature_is_dev_remote_wakeup(crq))
				//	udc->devstatus
				//		&= ~(1 << USB_DEVICE_REMOTE_WAKEUP);
				//else
				////	/* Can't CLEAR_FEATURE TEST_MODE */
				//	goto stall;
			} else if (ctrl.bRequestType == USB_RECIP_ENDPOINT) {
				u32 tmp;
				if (ctrl.wLength != __constant_cpu_to_le16(0)||
					ctrl.wValue  != __constant_cpu_to_le16(USB_ENDPOINT_HALT))
				{
					OTGDEV_IO_EP_Stall(0x80);
					dev->ep0state = WAIT_FOR_SETUP;
				}

				OTGDEV_IO_EP_Unstall((unsigned char)le16_to_cpu(ctrl.wIndex));

				tmp = tcc_readl(S3C_UDC_OTG_DIEPCTL0);
				tcc_writel((1<<31)|DEPCTL_CNAK|tmp, S3C_UDC_OTG_DIEPCTL0); /* EP0: Control IN */
			}
			else {
				break;
			}
			return;
		}

		case USB_REQ_SET_FEATURE:
			return;

		case USB_REQ_SET_ADDRESS:
			if (ctrl.bRequestType != (USB_TYPE_STANDARD | USB_RECIP_DEVICE))
				break;

			DEBUG_SETUP("%s: USB_REQ_SET_ADDRESS (%d)\n", __func__, ctrl.wValue);
			udc_set_address(dev, ctrl.wValue);
			return;
			
		case USB_REQ_SET_CONFIGURATION :
			DEBUG_SETUP("============================================\n");
			DEBUG_SETUP("%s: USB_REQ_SET_CONFIGURATION (%d)\n",	__func__, ctrl.wValue);

			// Just to send ZLP(Zero length Packet) to HOST in response to SET CONFIGURATION
			ep_ctrl = tcc_readl(S3C_UDC_OTG_DIEPCTL0);
			tcc_writel(DEPCTL_EPENA|DEPCTL_CNAK|ep_ctrl, S3C_UDC_OTG_DIEPCTL0); /* EP0: Control IN */

			// For Startng EP1 on this new configuration
			ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL1);
			tcc_writel(DEPCTL_EPDIS|DEPCTL_CNAK|DEPCTL_BULK_TYPE|DEPCTL_USBACTEP|ep_ctrl, S3C_UDC_OTG_DOEPCTL1); /* EP1: Bulk OUT */

			// For starting EP2 on this new configuration
			ep_ctrl = tcc_readl(S3C_UDC_OTG_DIEPCTL2);
			tcc_writel(DEPCTL_BULK_TYPE|DEPCTL_USBACTEP|ep_ctrl, S3C_UDC_OTG_DIEPCTL2); /* EP2: Bulk IN */

			// jmlee-090427 : ep3
			// For Startng EP3 on this new configuration
			ep_ctrl = tcc_readl(S3C_UDC_OTG_DOEPCTL3);
			tcc_writel(DEPCTL_EPDIS|DEPCTL_CNAK|DEPCTL_BULK_TYPE|DEPCTL_USBACTEP|ep_ctrl, S3C_UDC_OTG_DOEPCTL3); /* EP3: Bulk OUT */

			// For starting EP4 on this new configuration
			ep_ctrl = tcc_readl(S3C_UDC_OTG_DIEPCTL4);
			tcc_writel(DEPCTL_BULK_TYPE|DEPCTL_USBACTEP|ep_ctrl, S3C_UDC_OTG_DIEPCTL4); /* EP4: Bulk IN */

			DEBUG_SETUP("%s:(DOEPCTL1):0x%x, (DIEPCTL2):0x%x\n", __func__, tcc_readl(S3C_UDC_OTG_DOEPCTL1), tcc_readl(S3C_UDC_OTG_DIEPCTL2));
			DEBUG_SETUP("============================================\n");

			reset_available = 1;
			break;
			
		default:
			DEBUG_SETUP("%s: Default.....\n", __func__);
			break;
	}

	if (likely(dev->driver)) {
		/* device-2-host (IN) or no data setup command,
		 * process immediately */
		spin_unlock(&dev->lock);
		i = dev->driver->setup(&dev->gadget, &ctrl);
		spin_lock(&dev->lock);

		if (i < 0) {
			/* setup processing failed, force stall */
			printk("%s:  --> ERROR: gadget setup FAILED (stalling), setup returned %d\n",__func__, i);
			/* ep->stopped = 1; */
			OTGDEV_IO_EP_Stall(0x80);
			dev->ep0state = WAIT_FOR_SETUP;
		}
	}
}

/*
 * handle ep0 interrupt
 */
static void tcc_handle_ep0(struct tcc_udc *dev)
{	
	//u32 csr;
	//struct tcc_ep *ep = &dev->ep[0];
	
	if (dev->ep0state == WAIT_FOR_SETUP) {
		DEBUG_EP0("%s: WAIT_FOR_SETUP\n", __func__);
		tcc_ep0_setup(dev);
	} else {
		DEBUG_EP0("%s: strange state!!(state = %s)\n", __func__, state_names[dev->ep0state]);
	}
}


#if 0
static void tcc_ep0_kick(struct tcc_udc *dev, struct tcc_ep *ep)
{
	DEBUG_EP0("%s: ep_is_in = %d\n", __func__, ep_is_in(ep));
	if (ep_is_in(ep)) {
		dev->ep0state = DATA_STATE_XMIT;
		//printk("DAINTMSK=%X\n",pOTG->DAINTMSK);
		//if( !(pOTG->DAINTMSK & DAINTMSK_InEPInt(0)) )
		{
			//tcc_ep0_write(dev);
			//BITSET(pOTG->DIEPCTL0, DIEPCTL0_CNAK);
			//BITSET(pOTG->DAINTMSK, DAINTMSK_InEPInt(0));
			udc_ep_in_start_transfer(ep);
		}
		//printk("tcc_ep0_kick\n");
		//printk("DIEPCTL0=%X\n",pOTG->DIEPCTL0);
	} else {
		dev->ep0state = DATA_STATE_RECV;
		tcc_ep0_read(dev);
	}
}
#endif
/* ---------------------------------------------------------------------------
 * 	device-scoped parts of the api to the usb controller hardware
 * ---------------------------------------------------------------------------
 */

static int tcc_udc_get_frame(struct usb_gadget *_gadget)
{
	/*fram count number [21:8]*/
	u32 frame = tcc_readl(S3C_UDC_OTG_DSTS);
	
	DEBUG("%s: %p\n", __func__, _gadget);
	return (frame & 0x3ff00);
}

static int tcc_udc_wakeup(struct usb_gadget *_gadget)
{
	return -ENOTSUPP;
}

static const struct usb_gadget_ops tcc_udc_ops = {
	.get_frame = tcc_udc_get_frame,
	.wakeup = tcc_udc_wakeup,
	/* current versions must always be self-powered */
};

static void nop_release(struct device *dev)
{
	DEBUG("%s %s\n", __func__, dev->bus_id);
}

static struct tcc_udc memory = {
	.usb_address = 0,

	.gadget = {
		   .ops = &tcc_udc_ops,
		   .ep0 = &memory.ep[0].ep,
		   .name = driver_name,
		   .dev = {
			   .bus_id = "gadget",
			   .release = nop_release,
			   },
		   },

	/* control endpoint */
	.ep[0] = {
		  .ep = {
			 .name = ep0name,
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP0_FIFO_SIZE,
			 },
		  .dev = &memory,

		  .bEndpointAddress = 0,
		  .bmAttributes = 0,

		  .ep_type = ep_control,
		  .fifo = (u32) S3C_UDC_OTG_EP0_FIFO,
		  },

	/* first group of endpoints */
	.ep[1] = {
		  .ep = {
			 .name = "ep1-bulk",
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP_FIFO_SIZE,
			 },
		  .dev = &memory,

		  .bEndpointAddress = 1,
		  .bmAttributes = USB_ENDPOINT_XFER_BULK,

		  .ep_type = ep_bulk_out,
		  .fifo = (u32) S3C_UDC_OTG_EP1_FIFO,
		  },

	.ep[2] = {
		  .ep = {
			 .name = "ep2-bulk",
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP_FIFO_SIZE,
			 },
		  .dev = &memory,

		  .bEndpointAddress = USB_DIR_IN | 2,
		  .bmAttributes = USB_ENDPOINT_XFER_BULK,

		  .ep_type = ep_bulk_in,
		  .fifo = (u32) S3C_UDC_OTG_EP2_FIFO,
		  },

	.ep[3] = {				/* jmlee-090416 : for android adb bulk endpoint 3 */
		  .ep = {
			 .name = "ep3-bulk",
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP_FIFO_SIZE,
			 },
		  .dev = &memory,

		  .bEndpointAddress = USB_DIR_OUT | 3,
		  .bmAttributes = USB_ENDPOINT_XFER_BULK,

		  .ep_type = ep_bulk_out,
		  .fifo = (u32) S3C_UDC_OTG_EP3_FIFO,
		  },
	.ep[4] = {             /* jmlee-090416 : for android adb bulk endpoint 4 */
		  .ep = {
			 .name = "ep4-bulk",
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP_FIFO_SIZE,
			 },
		  .dev = &memory,

		  .bEndpointAddress = USB_DIR_IN | 4,
		  .bmAttributes = USB_ENDPOINT_XFER_BULK,

		  .ep_type = ep_bulk_in,
		  .fifo = (u32) S3C_UDC_OTG_EP4_FIFO,
		  },
	.ep[5] = {				// Though NOT USED XXX
		  .ep = {
			 .name = "ep5-int",
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP_FIFO_SIZE2,
			 },
		  .dev = &memory,

		  .bEndpointAddress = USB_DIR_IN | 5,
		  .bmAttributes = USB_ENDPOINT_XFER_INT,

		  .ep_type = ep_interrupt,
		  .fifo = (u32) S3C_UDC_OTG_EP5_FIFO,
		  },
	.ep[6] = {				// Though NOT USED XXX
		  .ep = {
			 .name = "ep6-int",
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP_FIFO_SIZE2,
			 },
		  .dev = &memory,

		  .bEndpointAddress = USB_DIR_IN | 6,
		  .bmAttributes = USB_ENDPOINT_XFER_INT,

		  .ep_type = ep_interrupt,
		  .fifo = (u32) S3C_UDC_OTG_EP6_FIFO,
		  },
	.ep[7] = {				// Though NOT USED XXX
		  .ep = {
			 .name = "ep7-int",
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP_FIFO_SIZE2,
			 },
		  .dev = &memory,

		  .bEndpointAddress = USB_DIR_IN | 7,
		  .bmAttributes = USB_ENDPOINT_XFER_INT,

		  .ep_type = ep_interrupt,
		  .fifo = (u32) S3C_UDC_OTG_EP7_FIFO,
		  },
	.ep[8] = {				// Though NOT USED XXX
		  .ep = {
			 .name = "ep8-int",
			 .ops = &tcc_ep_ops,
			 .maxpacket = EP_FIFO_SIZE2,
			 },
		  .dev = &memory,

		  .bEndpointAddress = USB_DIR_IN | 8,
		  .bmAttributes = USB_ENDPOINT_XFER_INT,

		  .ep_type = ep_interrupt,
		  .fifo = (u32) S3C_UDC_OTG_EP8_FIFO,
		  },

	.ep0req = NULL,
};

extern void fsg_change_state(int state);

#define USE_VBUS

static int tcc_udc_vbus_thread(void* _udc)
{
#define IsUSBDET()		( rPIC->STS0 & (1<<IRQ_VBON) )

	struct tcc_udc *dev = _udc;

#ifndef USE_VBUS
#if defined(CONFIG_MACH_TCC9200S)
	#define IsUSBDEV_DET()		( HwGPIOC->GPDAT & (1<<28) )
#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
	#define IsUSBDEV_DET()		( HwGPIOA->GPDAT & (1<<13) )
#endif
#endif

#ifndef USE_VBUS
#if defined(CONFIG_MACH_TCC9200S)
	/* Setting Port for USB_DET(VBUS) (GPIOC_28)*/
	BITCSET(HwGPIOC->GPFN3, HwPORTCFG_GPFN4_MASK, HwPORTCFG_GPFN4(0));	// functions : GPIO
	BITCLR(HwGPIOC->GPEN, Hw28);				// direction: input
	BITCLR(HwGPIOC->GPPD1, Hw25|Hw24);			// disable pull-up/pull-down
#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
	/* Setting Port for USB_DET(VBUS) (GPIOA_13)*/
	BITCSET(HwGPIOA->GPFN1, HwPORTCFG_GPFN6_MASK, HwPORTCFG_GPFN6(0));	// functions : GPIO
	BITCLR(HwGPIOA->GPEN, Hw13);				// direction: input
	BITCLR(HwGPIOA->GPPD0, Hw27|Hw26);			// disable pull-up/pull-down
#endif
#endif

#ifdef USE_VBUS
	#define IsUSBDET()			( rPIC->STS0 & (1<<IRQ_VBON) )
	// EINT0 -> USB_VBON
	BITCSET(rGPIO->EINTSEL0, (Hw6-HwZERO), 62);
#endif
	dev->flagVBUS = 0;

	for(;;) {
		msleep_interruptible(200);
#ifdef USE_VBUS
		if( IsUSBDET() && dev->driver )
#else
#if defined(CONFIG_MACH_TCC9200S)
		if( IsUSBDEV_DET() && dev->driver )
#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)
		if( IsUSBDEV_DET() && dev->driver )
#endif
#endif
		{
			if( dev->flagVBUS == 0 )
			{
				spin_lock(&dev->lock);
				dev->flagVBUS = 1;
				printk("\n usb connected.\n");
				//printk("vbus on\n");
				//BITSET(pOTG_IOBUSCFG->HCLKEN0, HwIOBUSCFG_USB);
				clk_enable(dev->clk);
				udc_enable(dev);
				USBHY_DEVICE_VBUS(TRUE);
#ifdef CONFIG_USB_ANDROID								
				fsg_change_state(1);
#endif
				spin_unlock(&dev->lock);
				// XXX
				if (notifier && notifier->notifier_call) {
					notifier->notifier_call(0, USB_BUS_ADD, 0);
				}

//				spin_unlock(&dev->lock);
			}
		}
		else
		{
			if( dev->flagVBUS != 0 )
			{
				spin_lock(&dev->lock);
				dev->flagVBUS = 0;
				printk("\n usb disconnected.\n");
				//printk("vbus off\n");
				udc_disable(dev);
				USBHY_DEVICE_VBUS(FALSE);
				clk_disable(dev->clk);				
#ifdef CONFIG_USB_ANDROID								
				fsg_change_state(0);
#endif
				spin_unlock(&dev->lock);
				// XXX
				if (notifier && notifier->notifier_call) {
					notifier->notifier_call(0, USB_BUS_REMOVE, 0);
				}

//				spin_unlock(&dev->lock);
			}
		}
	}
	return 0;
}

/*
 * 	probe - binds to the platform device
 */
//static struct clk	*otg_clock = NULL;

static int tcc_udc_probe(struct platform_device *pdev)
{
	struct tcc_udc *dev = &memory;
	int retval;

	DEBUG("=====>>>>> %s: %p, %p\n", __func__, pdev, dev);

	spin_lock_init(&dev->lock);
	dev->dev = pdev;

	device_initialize(&dev->gadget.dev);
	dev->gadget.dev.parent = &pdev->dev;

	dev->gadget.is_dualspeed = 1;	// Hack only
	dev->gadget.is_otg = 0;
	dev->gadget.is_a_peripheral = 0;
	dev->gadget.b_hnp_enable = 0;
	dev->gadget.a_hnp_support = 0;
	dev->gadget.a_alt_hnp_support = 0;
	dev->status_buf = kmalloc(sizeof (unsigned short), GFP_KERNEL);
	if( dev->status_buf==NULL )
		return -ENOMEM;

	the_controller = dev;
	platform_set_drvdata(pdev, dev);

	///////////////////////////////////////////////////
	// base address
	pOTG	 = (PUSB20OTG)tcc_p2v(HwUSB20OTG_BASE);
	pOTGCFG  = (PUSBOTGCFG)tcc_p2v(HwUSBOTGCFG_BASE);
	rPIC	 = (PPIC)tcc_p2v(HwPIC_BASE);
	rGPIO	 = (PGPIO)tcc_p2v(HwGPIO_BASE);
	pOTG_IOBUSCFG = (PIOBUSCFG)tcc_p2v(HwIOBUSCFG_BASE);

	// init
	//USBPHY_SetMode(USB_MODE_DEVICE);
	//OTGCORE_Init();
	//OTGDEV_IO_Init();
	//OTGCORE_SetID(1);
	//////////////////////////////////////////////////


//	otg_clock = clk_get(&pdev->dev, "otg");
//	if (otg_clock == NULL) {
//		printk(KERN_INFO "failed to find otg clock source\n");
//		return -ENOENT;
//	}
//	clk_enable(otg_clock);

#if defined(CONFIG_REGULATOR)
	dev->vdd_usb33 = regulator_get(NULL, "vdd_usb33");
	if (dev->vdd_usb33 == ERR_PTR(-ENODEV)) {
		pr_warning("%s: failed to get USB 3.3V regulator\n", driver_name);
	} else {
		regulator_enable(dev->vdd_usb33);
	}
	dev->vdd_usb12 = regulator_get(NULL, "vdd_usb12");
	if (dev->vdd_usb12 == ERR_PTR(-ENODEV)) {
		pr_warning("%s: failed to get USB 1.2V regulator\n", driver_name);
	} else {
		regulator_enable(dev->vdd_usb12);
	}
#endif
	udc_reinit(dev);

	local_irq_disable();

	/* irq setup after old hardware state is cleaned up */
	retval = request_irq(IRQ_OTG, tcc_udc_irq, IRQF_DISABLED, driver_name, dev);
	if (retval != 0) {
		DEBUG(KERN_ERR "%s: can't get irq %i, err %d\n", driver_name,
		      IRQ_OTG, retval);
		return -EBUSY;
	}
	printk("tcc_udc_otg: set irq number %d\n", IRQ_OTG);
	dev->vbus_thread_task = kthread_create(tcc_udc_vbus_thread, dev,"tcc-udc-vbus-thread");
	if (IS_ERR(dev->vbus_thread_task)) {
		printk("\nPTR ERR %p", dev->vbus_thread_task);
		return -EINVAL;
	}

	/* Tell the thread to start working */
	wake_up_process(dev->vbus_thread_task);
	dev->clk = clk_get(NULL, "usb_otg");

	disable_irq(IRQ_OTG);
	local_irq_enable();
	create_proc_files();
	
	return retval;
}
EXPORT_SYMBOL(the_controller);

static int tcc_udc_remove(struct platform_device *pdev)
{
	struct tcc_udc *dev = platform_get_drvdata(pdev);

	DEBUG("%s: %p\n", __func__, pdev);

//	if (otg_clock != NULL) {
//		clk_disable(otg_clock);
//		clk_put(otg_clock);
//		otg_clock = NULL;
//	}
	kfree(dev->status_buf);

	udc_disable(dev);

#if defined(CONFIG_REGULATOR)
	if (dev->vdd_usb33)
		regulator_put(dev->vdd_usb33);
	if (dev->vdd_usb12)
		regulator_put(dev->vdd_usb12);
#endif

	remove_proc_files();
	usb_gadget_unregister_driver(dev->driver);

	free_irq(IRQ_OTG, dev);

	platform_set_drvdata(pdev, 0);
	
	the_controller = 0;

	return 0;
}

#ifdef CONFIG_PM
static int tcc_udc_suspend(struct platform_device *dev, pm_message_t state)
{
	struct tcc_udc *pdev = platform_get_drvdata(dev);

	udc_disable(dev);
#if defined(CONFIG_REGULATOR)
	if (pdev->vdd_usb33)
		regulator_disable(pdev->vdd_usb33);
	if (pdev->vdd_usb12)
		regulator_disable(pdev->vdd_usb12);
#endif
	return 0;
}

static int tcc_udc_resume(struct platform_device *dev)
{
	struct tcc_udc *pdev = platform_get_drvdata(dev);
	clk_enable(pdev->clk);
#if defined(CONFIG_REGULATOR)
	if (pdev->vdd_usb33)
		regulator_enable(pdev->vdd_usb33);
	if (pdev->vdd_usb12)
		regulator_enable(pdev->vdd_usb12);
#endif
	udc_enable(dev);
	return 0;
}
#else
#define tcc_udc_suspend NULL
#define tcc_udc_resume  NULL
#endif /* CONFIG_PM */

/*-------------------------------------------------------------------------*/
static struct platform_driver tcc_udc_driver = {
//	.probe		= tcc_udc_probe,
	.remove		= tcc_udc_remove,
	.suspend	= tcc_udc_suspend,
	.resume		= tcc_udc_resume,
	.driver		= {
		.owner	= THIS_MODULE,
		.name	= "tcc-udc",
	},
};




void usb_register_notifier(struct notifier_block *nb)
{
	//	blocking_notifier_chain_register(&usb_notifier_list, nb);
	notifier = nb;
}
EXPORT_SYMBOL_GPL(usb_register_notifier);
#if (defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)) && defined(USE_VBUS)
/* USB Otg clock Enable */
struct clk *pUdc_clk;
#endif
static int __init udc_init(void)
{
	int ret;
#if (defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)) && defined(USE_VBUS)
    /* USB Otg clock Enable, sgbaek */
    pUdc_clk = clk_get(NULL, "usb_otg");
    if (pUdc_clk == NULL)
    {
        printk("\n\t\t===========> %s() clock enable failed..\n", __func__);
    }
    clk_enable(pUdc_clk);

    /* VBUS Voltage Control Code, sgbaek */
	tcc_pca953x_setup(PCA9538_U4_SLAVE_ADDR, DVBUS, OUTPUT, LOW, SET_DIRECTION | SET_VALUE);

#endif

	//ret = platform_driver_register(&tcc_udc_driver);
	ret = platform_driver_probe(&tcc_udc_driver, tcc_udc_probe);
	if(!ret)
	   printk("%s: init\n", driver_name);

	return ret;
}

static void __exit udc_exit(void)
{
	platform_driver_unregister(&tcc_udc_driver);
	printk("%s: exit\n", driver_name);
}

module_init(udc_init);
module_exit(udc_exit);

MODULE_DESCRIPTION(DRIVER_DESC);
MODULE_AUTHOR("Telechips Inc. Team BSP <linux@telechips.com>");
MODULE_LICENSE("GPL");
