/*
 * drivers/usb/gadget/tcc-udc.h
 * Samsung S3C on-chip full/high speed USB device controllers
 * $Id: tcc-udc.h,v 1.1 2007/08/10 07:00:56 dasan Exp $*
 * Copyright (C) 2005 for Samsung Electronics 
 * 		- by Jaswinder Singh Brar <jassi@samsung.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#ifndef __S3C_USB_GADGET
#define __S3C_USB_GADGET

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/ioport.h>
#include <linux/types.h>
#include <linux/version.h>
#include <linux/errno.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/init.h>
#include <linux/timer.h>
#include <linux/list.h>
#include <linux/interrupt.h>
#include <linux/proc_fs.h>
#include <linux/mm.h>
#include <linux/device.h>
#include <linux/dma-mapping.h>
#include <linux/clk.h>
#include <asm/byteorder.h>
#include <asm/dma.h>
#include <asm/io.h>
#include <asm/irq.h>
#include <asm/system.h>
#include <asm/unaligned.h>

#include <linux/usb/ch9.h>
#include <linux/usb/gadget.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
#include <mach/bsp.h>
#include <mach/hardware.h>


#define IRQ_OTG		INT_UOTG
#define IRQ_VBON	INT_EI0

// Max packet size
#ifdef CONFIG_USB_GADGET_S3C_FS
#define EP0_FIFO_SIZE		8
#define EP_FIFO_SIZE		64
#define S3C_MAX_ENDPOINTS	5
#else
#define EP0_FIFO_SIZE		64
#define EP_FIFO_SIZE		512
#define EP_FIFO_SIZE2		1024
#define S3C_MAX_ENDPOINTS	9
#endif

#define WAIT_FOR_SETUP          0
#define DATA_STATE_XMIT         1
#define DATA_STATE_NEED_ZLP     2
#define WAIT_FOR_OUT_STATUS     3
#define DATA_STATE_RECV         4

/* ********************************************************************************************* */
/* IO
 */

typedef enum ep_type {
	ep_control, ep_bulk_in, ep_bulk_out, ep_interrupt
} ep_type_t;

struct tcc_ep {
	struct usb_ep ep;
	struct tcc_udc *dev;

	const struct usb_endpoint_descriptor *desc;
	struct list_head queue;
	unsigned long pio_irqs;

	u8 stopped;
	u8 bEndpointAddress;
	u8 bmAttributes;

	ep_type_t ep_type;
	u32 fifo;
#ifdef CONFIG_USB_GADGET_S3C_FS
	u32 csr1;
	u32 csr2;
#endif
	u32 xfer_len;
	u32 xfer_count;
	u32 *xfer_buff;
};

struct tcc_request {
	struct usb_request req;
	struct list_head queue;
};

struct regulator;
struct tcc_udc {
	struct usb_gadget gadget;
	struct usb_gadget_driver *driver;
	//struct device *dev;
	struct platform_device *dev;
	spinlock_t lock;
	struct clk              *clk;
	int ep0state;
	unsigned short *status_buf;
	struct tcc_ep ep[S3C_MAX_ENDPOINTS];
	struct usb_request	*ep0req;	// For internal control responses

	struct usb_request *ep2req;
	
	unsigned char usb_address;

	unsigned req_pending:1, req_std:1, req_config:1;

	u8 flagVBUS;
	struct task_struct	*vbus_thread_task;
#if defined(CONFIG_REGULATOR)
	struct regulator *vdd_usb33;
	struct regulator *vdd_usb12;
#endif
#ifdef CONFIG_HAS_EARLYSUSPEND
    struct early_suspend early_suspend;
#endif	
 	
};

extern struct tcc_udc *the_controller;

#define ep_is_in(EP) 		(( (EP)->bEndpointAddress & USB_DIR_IN) == USB_DIR_IN )
#define ep_index(EP) 		((EP)->bEndpointAddress&0xF)
#define ep_maxpacket(EP) 	((EP)->ep.maxpacket)

#endif
