

#if defined(CONFIG_ARCH_TCC79X)
#include <mach/tcc79x.h>
#elif defined(CONFIG_ARCH_TCC83X)
#include <mach/tcc83x_virt.h>
#endif

#include "lms350df01_lcdc.h"

/*
 * Setting IMGx Display On/Off.
 *
 * onoff : 0 - disable
 *         1 - enable
 *
 */
 
#define TRUE 1
#define FALSE 0


void LCDC_SET_INTTERUPT(char onoff)
{

	if(onoff) 
	{
		BITSET(HwLSTATUS, HwLSTATUS_ITY);
		BITCLR(HwLIM, HwLIM_MDD); 

	    BITCLR(HwIEN,  HwINT_LCD);
	    BITSET(HwSEL,  HwINT_LCD);
	    BITSET(HwMODE, HwINT_LCD);
	    BITSET(HwIEN,  HwINT_LCD);
	}  
	else
	{
		BITCLR(HwLSTATUS, HwLSTATUS_ITY);
		BITSET(HwLIM, HwLIM_MDD); 
		BITCLR(HwIEN,  HwINT_LCD);		
		BITCLR(HwSEL,  HwINT_LCD);
	}  
}

void	LCDC_SetSize (unsigned int uiType, unsigned int uiWidth, unsigned int uiHeight)
{
	
	switch (uiType) 
	{
		case	LCD_CH :
			BITCSET(HwLDS, 0x0000FFFF, uiWidth);
			BITCSET(HwLDS, 0xFFFF0000, (uiHeight << 16));
			break;
		case	IMG_CH0 :
			BITCSET(HwLI0S, 0xFFFF0000, (uiHeight << 16));
			BITCSET(HwLI0S, 0x0000FFFF, uiWidth);
			break;
		case	IMG_CH1 :
			BITCSET(HwLI1S, 0xFFFF0000, (uiHeight << 16));
			BITCSET(HwLI1S, 0x0000FFFF, uiWidth);
			break;
		case	IMG_CH2 :
			BITCSET(HwLI2S, 0xFFFF0000, (uiHeight << 16));
			BITCSET(HwLI2S, 0x0000FFFF, uiWidth);
			break;
		default :
			break;
	}
	
}


///////////////////////////////////////////////////
//
//	LCDC_SetHTime
//		: Setting the Horizontal Timing Register
//	Parameters
//		: nPulseWidth	: Horizontal Line Pulse Width
//		: nStartCount	: Horizontal Line Start Count
//		: nWidth		: Width of the Displaying Window
//		: nEndCount	: Horizontal Line End Count
//
///////////////////////////////////////////////////
void LCDC_SetHTime (unsigned int uiPulseWidth, unsigned int uiStartCount, unsigned int uiWidth, unsigned int uiEndCount)
{
	BITCSET(HwLHTIME1, 0x0000FFFF, (uiWidth - 1));
	BITCSET(HwLHTIME1, 0x01FF0000, (uiPulseWidth << 16));
	BITCSET(HwLHTIME2, 0x000000FF, uiEndCount);
	BITCSET(HwLHTIME2, 0x00FF0000, (uiStartCount << 16));
}


///////////////////////////////////////////////////
//
//	LCDC_SetVTime
//		: Setting the Vertical Timing Register
//	Parameters
//		: nVSyncPulseWidth		: Number of Clock Pulse Width for VSYNC
//		: nHeight					: Height of the Displaying Window
//		: nFrameStartWaitCycle		: Number of Wait Cycles after Starting the New Frame
//		: nFrameEndWaitCycle		: Number of Wait Cycles before Starting the New Frame
//		: nVSyncPulseWidth1		: Number of Clock Pulse Width for VSYNC
//		: nHeight1				: Height of the Displaying Window
//		: nFrameStartWaitCycle1	: Number of Wait Cycles after Starting the New Frame
//		: nFrameEndWaitCycle1		: Number of Wait Cycles before Starting the New Frame
//		: nVSyncDelay			: Number of VSYNC Delay after HSYNC
//
///////////////////////////////////////////////////

void LCDC_SetVTime (unsigned int uiVSyncPulseWidth, unsigned int uiFrameStartWaitCycle, unsigned int	uiHeight, unsigned int uiFrameEndWaitCycle, unsigned int uiVSyncPulseWidth1, unsigned int uiFrameStartWaitCycle1, unsigned int uiHeight1, unsigned int	uiFrameEndWaitCycle1, unsigned int uiFVSyncDelay, unsigned int uiBVSyncDelay)
{
	BITCSET(HwLVTIME1, 0x0000FFFF, (uiHeight - 1));
	BITCSET(HwLVTIME1, 0x003F0000, (uiVSyncPulseWidth << 16));
	BITCSET(HwLVTIME2, 0xFFFF0000, (uiFrameStartWaitCycle << 16));
	BITCSET(HwLVTIME2, 0x0000FFFF, uiFrameEndWaitCycle);
	BITCSET(HwLVTIME3, 0x0000FFFF, (uiHeight1 - 1));
	BITCSET(HwLVTIME3, 0xFFFF0000, (uiVSyncPulseWidth1 << 16));
	BITCSET(HwLVTIME4, 0xFFFF0000, (uiFrameStartWaitCycle1 << 16));
	BITCSET(HwLVTIME4, 0x0000FFFF, uiFrameEndWaitCycle1);
	BITCSET(HwLVTIME1, 0x03C00000, (uiFVSyncDelay << 22));
	BITCSET(HwLVTIME1, 0xF8000000, (uiBVSyncDelay << 27));
}

///////////////////////////////////////////////////
//
//	LCDC_SetClipping
//		: Setting the Clipping Informations
//	Parameters
//		: nUpperLimitY		: Upper Boundary for Y Channel Data
//		: nLowerLimitY		: Lower Boundary for Y Channel Data
//		: nUpperLimitUV		: Upper Boundary for UV Channel Data
//		: nLowerLimitUV		: Lower Boundary for UV Channel Data
//
///////////////////////////////////////////////////
void LCDC_SetClipping (unsigned int	uiUpperLimitY, unsigned int uiLowerLimitY, unsigned int uiUpperLimitUV, unsigned int uiLowerLimitUV)
{
	BITCSET(HwLCP1, 0xFF000000, (uiLowerLimitUV << 24)); 	// CLP2L Clipping U/G below this value. (standard value is 0)
	BITCSET(HwLCP1, 0x00FF0000, (uiUpperLimitUV << 16)); 	// CLP2 Clipping U/G upper this value. (standard value is 255)
	BITCSET(HwLCP1, 0x0000FF00, (uiLowerLimitY << 8));		// CLP1L Clipping Y/R below this value.. (standard value is 0
	BITCSET(HwLCP1, 0x000000FF, uiUpperLimitY);				// CLP1 Clipping Y/R upper this value. (standard value is 255)

	BITCSET(HwLCP2, 0x000000FF, uiUpperLimitUV);				// CLP3 Clipping V/B upper this value. (standard value is 255)
	BITCSET(HwLCP2, 0x0000FF00, (uiLowerLimitUV << 8));		// CLP3L Clipping V/B below this value.. (standard value is 0)
}


void LCDC_SetMaskAndPolarity (unsigned int uiFlag, unsigned int uiHsync, unsigned int uiVsync, unsigned int uiClockPulse, unsigned int uiACBias, unsigned int uiDd, unsigned int uiFu, unsigned int uiRu)
{
	if(uiFlag & SET_LCD_POLARITY)
	{
		BITCSET(HwLCTRL, HwLCTRL_IH_INVERTED, (uiHsync << 13));
		BITCSET(HwLCTRL, HwLCTRL_IV_INVERTED, (uiVsync << 14));
		BITCSET(HwLCTRL, HwLCTRL_IP_FEDGE, (uiClockPulse << 12));
		BITCSET(HwLCTRL, HwLCTRL_ID_INVERTED, (uiACBias << 15));
	}

	if(uiFlag & SET_LCD_MASK)
	{
		BITCSET(HwLIM, HwLIM_MDD, (uiDd << 4));
		BITCSET(HwLIM, HwLIM_MFU, uiFu);
		BITCSET(HwLIM, HwLIM_MRU, (uiRu << 3));
	}
}

void LCDC_SetClock (unsigned int uiCS, unsigned int uiPxClkDiv, unsigned int uiLclkDiv,  unsigned int uiACDiv)
{
	BITCSET(HwLCLKDIV, HwLCLKDIV_CS, uiCS);
	
	BITCSET(HwLCLKDIV, 0x00FF0000, (uiACDiv << 16));
	BITCSET(HwLCLKDIV, 0x0000FF00, (uiLclkDiv << 8));    
	BITCSET(HwLCLKDIV, 0x000000FF, uiPxClkDiv);
}

///////////////////////////////////////////////////
//
//	LCDC_SetImageBase
//	Parameters
//		: nType		: IMG_CH0, IMG_CH1, IMG_CH2
//		: nBase0		: Base Address for Y Channel or RGB Channel
//					: Valid for 3 Channel
//		: nBase1		: Base Address for Cb(U)
//					: Valid for IMG_CH0
//		: nBase2		: Base Address for Cb(U) Channel
//					: Valid for IMG_CH0
//
///////////////////////////////////////////////////
void LCDC_SetImageBase (unsigned int uiType, unsigned int uiBase0, unsigned int uiBase1, unsigned int uiBase2)
{
	switch (uiType) 
	{
		case	IMG_CH0 :
			BITCSET(HwLI0BA0, 0xFFFFFFFF, uiBase0);
			BITCSET(HwLI0BA1, 0xFFFFFFFF, uiBase1);
			BITCSET(HwLI0BA2, 0xFFFFFFFF, uiBase2);
			break;

		case	IMG_CH1 :
			BITCSET(HwLI1BA0, 0xFFFFFFFF, uiBase0);
			break;

		case	IMG_CH2 :
			BITCSET(HwLI2BA0, 0xFFFFFFFF, uiBase0);
			break;

		default	:
			break;
	}
}

///////////////////////////////////////////////////
//
//	LCDC_SetImageInfo
//	Parameters
//		: nType		: IMG_CH0, IMG_CH1, IMG_CH2
//		: nHPos		: Horizontal Position
//		: nVPos		: Vertical Position
//		: nOffset0	: Horizontal Pixel Offset for Y Channel or RGB Channel
//		: nOffset1	: Horizontal Pixel Offset for UV Channel
//					: Valid for IMG_CH0
//		: nHScale		: Horizontal Scale Factor
//		: nVScale		: Vertical Scale Factor
//		: nFormat	: Endian / YUV / BPP Setting
//					: [3:0]	: BPP Information
//					: [6:4]	: YUV Format Information
//					: [7]		: Endian Information
//
///////////////////////////////////////////////////
unsigned int LCDC_SetImageInfo (unsigned int uiFlag, unsigned int uiType, unsigned int uiWidth, unsigned int uiHeight, unsigned int uiHPos, unsigned int uiVPos, unsigned int uiHOffset0, unsigned int uiHOffset1, unsigned int uiHScale, unsigned int uiVScale, unsigned int uiFormat)
{
	unsigned int uiCH0Position = 0;
	
	switch (uiType) 
	{
		case	LCD_CH :
			if(uiFlag & SET_IMAGE_SIZE)
			{
				BITCSET(HwLDS, 0x0000FFFF, uiWidth);
				BITCSET(HwLDS, 0xFFFF0000, (uiHeight << 16));
			}	
			break;

		case	IMG_CH0 :
			if(uiFlag & SET_IMAGE_SIZE)
			{
				BITCSET(HwLI0S, 0xFFFF0000, (uiHeight << 16));
				BITCSET(HwLI0S, 0x0000FFFF, uiWidth);
			}

			if(uiFlag & READ_IMAGE_POSITION)
			{
				uiCH0Position = HwLI0P;
			}

			if(uiFlag & SET_IMAGE_POSITION)
			{
				BITCSET(HwLI0P, 0x0000FFFF, uiHPos);
				BITCSET(HwLI0P, 0xFFFF0000, (uiVPos << 16));
			}	

			if(uiFlag & SET_IMAGE_OFFSET)
			{
				BITCSET(HwLI0O, 0x0000FFFF, uiHOffset0);
				BITCSET(HwLI0O, 0xFFFF0000, (uiHOffset1 << 16));
			}	

			if(uiFlag & SET_IMAGE_SCALE)
			{
				BITCSET(HwLI0SCALE, 0x0000000F, uiHScale);
				BITCSET(HwLI0SCALE, 0x000000F0, (uiVScale << 4));
			}
			
			if(uiFlag & SET_IMAGE_FORMAT)
			{

				BITCSET(HwLI0C, 0x000080FF, uiFormat);

				if ( (uiFormat & HwLI0C_YUV_422P) != 0 )
				{
					BITSET(HwLCTRL, HwLCTRL_Y2R0_EN);
				}	
				else
				{
					BITCLR(HwLCTRL, HwLCTRL_Y2R0_EN);
				}
			}	
			break;
			
		case	IMG_CH1 :
			if(uiFlag & SET_IMAGE_SIZE)
			{
				BITCSET(HwLI1S, 0xFFFF0000, (uiHeight << 16));
				BITCSET(HwLI1S, 0x0000FFFF, uiWidth);
			}	

			if(uiFlag & SET_IMAGE_POSITION)
			{
				BITCSET(HwLI1P, 0x0000FFFF, uiHPos);
				BITCSET(HwLI1P, 0xFFFF0000, (uiVPos << 16));
			}	

			if(uiFlag & SET_IMAGE_OFFSET)
			{
				BITCSET(HwLI1O, 0x0000FFFF, uiHOffset0);
			}	

			if(uiFlag & SET_IMAGE_SCALE)
			{
				BITCSET(HwLI1SCALE, 0x0000000F, uiHScale);
				BITCSET(HwLI1SCALE, 0x000000F0, (uiVScale << 4));
			}	

			if(uiFlag & SET_IMAGE_FORMAT)
			{
				BITCSET(HwLI1C, 0x000080FF, uiFormat);
				BITCLR(HwLCTRL, HwLCTRL_Y2R1_EN);
			}	
			break;
			
		case	IMG_CH2 :
			if(uiFlag & SET_IMAGE_SIZE)
			{
				BITCSET(HwLI2S, 0xFFFF0000, (uiHeight << 16));
				BITCSET(HwLI2S, 0x0000FFFF, uiWidth);
			}

			if(uiFlag & SET_IMAGE_POSITION)
			{
				BITCSET(HwLI2P, 0x0000FFFF, uiHPos);
				BITCSET(HwLI2P, 0xFFFF0000, (uiVPos << 16));
			}

			if(uiFlag & SET_IMAGE_OFFSET)
			{
				BITCSET(HwLI2O, 0x0000FFFF, uiHOffset0);
			}

			if(uiFlag & SET_IMAGE_SCALE)
			{
				BITCSET(HwLI2SCALE, 0x0000000F, uiHScale);
				BITCSET(HwLI2SCALE, 0x000000F0, (uiVScale << 4));
			}

			if(uiFlag & SET_IMAGE_FORMAT)
			{

				BITCSET(HwLI2C, 0x000080FF, uiFormat);
				BITCLR(HwLCTRL, HwLCTRL_Y2R2_EN);
			}	
			break;

		default	:
			break;
	}
	return uiCH0Position;
}

#define	HwLK1_SA1						(Hw31)
#define	HwLK2_SA2						(Hw31)
void	LCDC_SetAlphatype (unsigned int uiType, LCDC_AlphaType uiAlpha_type)
{
	switch(uiType)
	{
		case IMG_CH1:
			BITCSET(HwLK1, HwLK1_SA1, (uiAlpha_type << 31));
			break;

		case IMG_CH2:
			BITCSET(HwLK2, HwLK2_SA2, (uiAlpha_type << 31));
			break;
	}

}

#define	HwY2RP0_CH2_TYPE0			HwZERO					// YUV to RGB converter type 0
#define	HwY2RP0_CH2_TYPE1			Hw4						// YUV to RGB converter type 1 
#define	HwY2RP0_CH2_TYPE2			Hw5						// YUV to RGB converter type 2
#define	HwY2RP0_CH2_TYPE3			(Hw5+Hw4)				// YUV to RGB converter type 3 
#define	HwY2RP0_CH1_TYPE0			HwZERO					// YUV to RGB converter type 0
#define	HwY2RP0_CH1_TYPE1			(Hw2)					// YUV to RGB converter type 1 
#define	HwY2RP0_CH1_TYPE2			(Hw3)					// YUV to RGB converter type 2
#define	HwY2RP0_CH1_TYPE3			(Hw3+Hw2)				// YUV to RGB converter type 3
#define	HwY2RP0_CH0_TYPE0			HwZERO					// YUV to RGB converter type 0
#define	HwY2RP0_CH0_TYPE1			(Hw0)					// YUV to RGB converter type 1
#define	HwY2RP0_CH0_TYPE2			(Hw1)					// YUV to RGB converter type 2
#define	HwY2RP0_CH0_TYPE3			(Hw1+Hw0)				// YUV to RGB converter type 3
/*----------------------------------------------------
 	YUVtoRGB Format Converter Type 0
-	R = Y + 1.371 ( V - 128 )
-	G = Y - 0.336 ( U - 128 ) - 0.698 ( V - 128 )
-	B = Y + 1.732 ( U - 128 )	
 	YUVtoRGB Format Converter Type 1
-	R = 1.164 ( Y - 16 ) + 1.159 ( V - 128 )
-	G = 1.164 ( Y - 16 ) - 0.391 ( U - 128 ) - 0.813 ( V - 128 )
-	B = 1.164 ( Y - 16 ) + 2.018 ( U - 128 )
 	YUVtoRGB Format Converter Type 2
-	R = Y + 1.540 ( V - 128 )
-	G = Y - 0.183 ( U - 128 ) - 0.459 ( V - 128 )
-	B = Y + 1.816 ( U - 128 )	
 	YUVtoRGB Format Converter Type 3
-	R = 1.164 ( Y - 16 ) + 1.793 ( V - 128 )
-	G = 1.164 ( Y - 16 ) - 0.213 ( U - 128 ) - 0.534 ( V - 128 )
-	B = 1.164 ( Y - 16 ) + 2.115 ( U - 128 )

--------------------------------------------------------*/
void	LCDC_SetYUV2RGBtype (unsigned int uiType, LCDC_YUV2RGBType coverter_type)
{
	switch(uiType)
	{
		case IMG_CH0:
			if(coverter_type == YUV2RGBType0)		BITCSET(HwY2RP0, HwY2RP0_CH0_TYPE3, HwY2RP0_CH0_TYPE0);
			else if(coverter_type == YUV2RGBType1)	BITCSET(HwY2RP0, HwY2RP0_CH0_TYPE3, HwY2RP0_CH0_TYPE1);				
			else if(coverter_type == YUV2RGBType2)	BITCSET(HwY2RP0, HwY2RP0_CH0_TYPE3, HwY2RP0_CH0_TYPE2);				
			else if(coverter_type == YUV2RGBType3)	BITCSET(HwY2RP0, HwY2RP0_CH0_TYPE3, HwY2RP0_CH0_TYPE3);				

			break;


		case IMG_CH1:
			if(coverter_type == YUV2RGBType0)		BITCSET(HwY2RP0, HwY2RP0_CH1_TYPE3, HwY2RP0_CH1_TYPE0);
			else if(coverter_type == YUV2RGBType1)	BITCSET(HwY2RP0, HwY2RP0_CH1_TYPE3, HwY2RP0_CH1_TYPE1);				
			else if(coverter_type == YUV2RGBType2)	BITCSET(HwY2RP0, HwY2RP0_CH1_TYPE3, HwY2RP0_CH1_TYPE2);				
			else if(coverter_type == YUV2RGBType3)	BITCSET(HwY2RP0, HwY2RP0_CH1_TYPE3, HwY2RP0_CH1_TYPE3);				

			break;

		case IMG_CH2:
			if(coverter_type == YUV2RGBType0)		BITCSET(HwY2RP0, HwY2RP0_CH2_TYPE3, HwY2RP0_CH2_TYPE0);
			else if(coverter_type == YUV2RGBType1)	BITCSET(HwY2RP0, HwY2RP0_CH2_TYPE3, HwY2RP0_CH2_TYPE1);				
			else if(coverter_type == YUV2RGBType2)	BITCSET(HwY2RP0, HwY2RP0_CH2_TYPE3, HwY2RP0_CH2_TYPE2);				
			else if(coverter_type == YUV2RGBType3)	BITCSET(HwY2RP0, HwY2RP0_CH2_TYPE3, HwY2RP0_CH2_TYPE3);				

			break;
	}

}
void	LCDC_SetImageOverlayKey (unsigned int uiFlag, unsigned int uiType, LCDC_AlphaValue uiAlpha, unsigned int uiKeyRed, unsigned int uiKeyGreen, unsigned int uiKeyBlue, LCDC_AlphaValue uiAlphaMask, unsigned int uiKeyMaskRed, unsigned int uiKeyMaskGreen, unsigned int uiKeyMaskBlue)
{
	if(uiType & IMG_CH1)
	{

		if(uiFlag & SET_LCD_ALPHA_VALUE)
		{
			BITCSET(HwLK1, HwLK1_A10_MAX, (uiAlpha << 24));
		}

		if(uiFlag & SET_LCD_KEY_BLUE)
		{
			BITCSET(HwLK1, 0x000000FF, uiKeyBlue);
		}

		if(uiFlag & SET_LCD_KEY_GREEN)
		{
			BITCSET(HwLK1, 0x0000FF00, (uiKeyGreen << 8));
		}

		if(uiFlag & SET_LCD_KEY_RED)
		{
			BITCSET(HwLK1, 0x00FF0000, (uiKeyRed << 16));
		}

		if(uiFlag & SET_LCD_ALPHAMASK_VALUE)
		{
			BITCSET(HwLKM1, HwLKM1_A11_MAX, (uiAlphaMask << 24));
		}

		if(uiFlag & SET_LCD_KEYMASK_BLUE)
		{
			BITCSET(HwLKM1, 0x000000FF, uiKeyMaskBlue);
		}

		if(uiFlag & SET_LCD_KEYMASK_GREEN)
		{
			BITCSET(HwLKM1, 0x0000FF00, (uiKeyMaskGreen << 8));
		}

		if(uiFlag & SET_LCD_KEYMASK_RED)
		{
			BITCSET(HwLKM1, 0x00FF0000, (uiKeyMaskRed << 16));
		}
	}

	if(uiType & IMG_CH2)
	{

		if(uiFlag & SET_LCD_ALPHA_VALUE)
		{
			BITCSET(HwLK2, HwLK2_A20_MAX, (uiAlpha << 24));			
		}

		if(uiFlag & SET_LCD_KEY_BLUE)
		{
			BITCSET(HwLK2, 0x000000FF, uiKeyBlue);
		}

		if(uiFlag & SET_LCD_KEY_GREEN)
		{
			BITCSET(HwLK2, 0x0000FF00, (uiKeyGreen << 8));
		}

		if(uiFlag & SET_LCD_KEY_RED)
		{
			BITCSET(HwLK2, 0x00FF0000, (uiKeyRed << 16));
		}

		if(uiFlag & SET_LCD_ALPHAMASK_VALUE)
		{
			BITCSET(HwLKM2, HwLKM2_A21_MAX, (uiAlphaMask << 24));
		}

		if(uiFlag & SET_LCD_KEYMASK_BLUE)
		{
			BITCSET(HwLKM2, 0x000000FF, uiKeyMaskBlue);
		}

		if(uiFlag & SET_LCD_KEYMASK_GREEN)
		{
			BITCSET(HwLKM2, 0x0000FF00, (uiKeyMaskGreen << 8));
		}

		if(uiFlag & SET_LCD_KEYMASK_RED)
		{
			BITCSET(HwLKM2, 0x00FF0000, (uiKeyMaskRed << 16));
		}
	}
}

void LCDC_SetBackgroundColor(unsigned int uiColorRV, unsigned int uiColorGU, unsigned int uiColorBY)
{
	BITCSET(HwLBC, 0x000000FF, uiColorRV);
	BITCSET(HwLBC, 0x0000FF00, (uiColorGU << 8));
	BITCSET(HwLBC, 0x00FF0000, (uiColorBY << 16));
}

#define	HwLCTRL_PXDW_888_RGB_GRB	(Hw19+Hw17)				// Pixel Data Width = 888 pxdw (delta type, RGB - GRB)
#define	HwLCTRL_PXDW_888_24BIT		(Hw19+Hw18)				// Pixel Data Width = 888 pxdw (24 bit mode) - NOT USED in TCC83X
#define	HwLCTRL_PXDW_888_RGB_DUMMY	(Hw19+Hw18+Hw16)		// Pixel Data Width = 888 pxdw (RGB-dummy mode)
#define	HwLCTRL_PXDW_888_GRB_RGB	(Hw19+Hw17+Hw16)		// Pixel Data Width = 888 pxdw (delta type, GRB - RGB)
void LCDC_SetControlInfo(unsigned int uiPage, unsigned int uiFlag)
{
	unsigned	uLCTRL	= HwLCTRL;

	if(uiPage == 0)
	{
		if(uiFlag & SET_LCD_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_LEN_EN);
		}

		if(uiFlag & SET_LCD_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_LEN_EN);
		}	

		if(uiFlag & SET_CH0_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_IEN0_EN);
		}

		if(uiFlag & SET_CH0_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_IEN0_EN);
		}
			
		if(uiFlag & SET_CH1_ENABLE)
		{
	            BITSET(uLCTRL, HwLCTRL_IEN1_EN);
		}

		if(uiFlag & SET_CH1_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_IEN1_EN);
		}
			
		if(uiFlag & SET_CH2_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_IEN2_EN);
		}
			
		if(uiFlag & SET_CH2_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_IEN2_EN);
		}
			
		if(uiFlag & SET_TFT_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_TFT);
		}

		if(uiFlag & SET_TFT_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_TFT);
		}

		if(uiFlag & SET_TVOUT_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_TV);
		}

		if(uiFlag & SET_TVOUT_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_TV);
		}

		if(uiFlag & SET_STN_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_STN);
		}
			
		if(uiFlag & SET_STN_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_STN);
		}

		if(uiFlag & SET_NOTINTERLACE_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_NI_PMODE);				// Not-Interlace Field ( 1 for Progressive )		
		}
			
		if(uiFlag & SET_NOTINTERLACE_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_NI_PMODE);				// Not-Interlace Field ( 1 for Progressive )		
		}

		if(uiFlag & SET_DP2_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_DP_2);					// 1 Pixel Data / 2 Pixel Clocks ( 0 for 1 Data/Clock )
		}

		if(uiFlag & SET_DP2_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_DP_2);					// 1 Pixel Data / 2 Pixel Clocks ( 0 for 1 Data/Clock )
		}

		if(uiFlag & SET_RGB2YUV_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_R2Y_EN);				// YUV to RGB Output
		}
			
		if(uiFlag & SET_RGB2YUV_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_R2Y_EN);				// YUV to RGB Output
		}

		if(uiFlag & SET_CCIR656_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_656_EN);				// CCIR 656 or 601 ( 0 for CCIR-601 )
		}
			
		if(uiFlag & SET_CCIR656_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_656_EN);				// CCIR 656 or 601 ( 0 for CCIR-601 )
		}

		if(uiFlag & SET_CH0YUV2RGB_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_Y2R0_EN);
		}

		if(uiFlag & SET_CH0YUV2RGB_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_Y2R0_EN);
		}
			
		if(uiFlag & SET_CH1CHROMA_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_CEN1_EN);
		}

		if(uiFlag & SET_CH1CHROMA_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_CEN1_EN);
		}
			
		if(uiFlag & SET_CH2CHROMA_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_CEN2_EN);
		}

		if(uiFlag & SET_CH2CHROMA_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_CEN2_EN);
		}

		if(uiFlag & SET_CH1ALPHA_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_AEN1_EN);
		}

		if(uiFlag & SET_CH1ALPHA_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_AEN1_EN);
		}

		if(uiFlag & SET_CH2ALPHA_ENABLE)
		{
			BITSET(uLCTRL, HwLCTRL_AEN2_EN);
		}

		if(uiFlag & SET_CH2ALPHA_DISABLE)
		{
			BITCLR(uLCTRL, HwLCTRL_AEN2_EN);
		}
	}		

	if(uiPage == 1)
	{
		if(uiFlag & SET_PXDW4_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_4);
		}

		if(uiFlag & SET_PXDW8_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_8);
		}

		if(uiFlag & SET_PXDW888_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_888);
		}

		if(uiFlag & SET_PXDW565_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_565);
		}

		if(uiFlag & SET_PXDW555_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_555);
		}

		if(uiFlag & SET_PXDW18_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_18);
		}

		if(uiFlag & SET_PXDW8UY_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_8UY);
		}

		if(uiFlag & SET_PXDW8VY_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_8VY);
		}

		if(uiFlag & SET_PXDW16YU_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_16YU);
		}

		if(uiFlag & SET_PXDW16YV_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_16YV);
		}

		if(uiFlag & SET_PXDW888_RGB_GRB_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_888_RGB_GRB);
		}

		if(uiFlag & SET_PXDW888_GRB_RGB_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_888_GRB_RGB);
		}		

		if(uiFlag & SET_PXDW888_24BIT_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_888_24BIT);
		}

		if(uiFlag & SET_PXDW888_DUMMY_ENABLE)
		{
			BITCSET(uLCTRL, (HwLCTRL_PXDW_16YU|HwLCTRL_PXDW_8VY), HwLCTRL_PXDW_888_RGB_DUMMY);
		}		

		if(uiFlag & SET_BPP1_ENABLE)
		{
			BITCSET(uLCTRL, (Hw23|HwLCTRL_BPP_MASK), HwLCTRL_BPP_1);						// RGB565
		}

		if(uiFlag & SET_BPP2_ENABLE)
		{
			BITCSET(uLCTRL, (Hw23|HwLCTRL_BPP_MASK), HwLCTRL_BPP_2);						// RGB565
		}

		if(uiFlag & SET_BPP4_ENABLE)
		{
			BITCSET(uLCTRL, (Hw23|HwLCTRL_BPP_MASK), HwLCTRL_BPP_4);						// RGB565
		}

		if(uiFlag & SET_BPP332_ENABLE)
		{
			BITCSET(uLCTRL, (Hw23|HwLCTRL_BPP_MASK), HwLCTRL_BPP_332);						// RGB565
		}

		if(uiFlag & SET_BPP444_ENABLE)
		{
			BITCSET(uLCTRL, (Hw23|HwLCTRL_BPP_MASK), HwLCTRL_BPP_444);						// RGB565
		}

		if(uiFlag & SET_BPP565_ENABLE)
		{
			BITCSET(uLCTRL, (Hw23|HwLCTRL_BPP_MASK), HwLCTRL_PXDW_565);						// RGB565
		}

		if(uiFlag & SET_BPP555_ENABLE)
		{
			BITCSET(uLCTRL, (Hw23|HwLCTRL_BPP_MASK), HwLCTRL_PXDW_555);						// RGB565
		}

		if(uiFlag & SET_BPP888_ENABLE)
		{
			BITCSET(uLCTRL, (Hw23|HwLCTRL_BPP_MASK), HwLCTRL_BPP_MASK);						// RGB565
		}

		if(uiFlag & SET_MASTER_DMA0)
		{
			BITCLR(uLCTRL, HwLCTRL_MSEL);						// RGB565
		}	
		
		if(uiFlag & SET_MASTER_DMA1)
		{
			BITSET(uLCTRL, HwLCTRL_MSEL);						// RGB565
		}	
	
	}	
	HwLCTRL	= uLCTRL;
}   

void LCDC_SetOverlayPriority(unsigned int uiPriority)
{
	
	BITCSET(HwLI0C,(Hw12+Hw13+Hw14) , (uiPriority&7)<<12 );
	
}
 

#define	HwLCDSICTRL1_BW1		        	Hw31										// LCDXD data width is 8bits. Prefix W means writing operation
#define	HwLCDSICTRL1_BW0		        	Hw15										// LCDXD data width is 8bits. Prefix R means reading operation
#define	HwLCDSICTRL2_BW1		        	Hw31										// LCDXD data width is 8bits. Prefix W means writing operation
#define	HwLCDSICTRL2_BW0		        	Hw15										// LCDXD data width is 8bits. Prefix R means reading operation
#define	HwLCDSICTRL3_BW1		        	Hw31										// LCDXD data width is 8bits. Prefix W means writing operation
#define	HwLCDSICTRL3_BW0				Hw15									// LCDXD data width is 8bits. Prefix R means reading operation
#define	HwLCDSICTRL4_BW1		        	Hw31										// LCDXD data width is 8bits. Prefix W means writing operation
#define	HwLCDSICTRL4_BW0		        	Hw15										// LCDXD data width is 8bits. Prefix R means reading operation
void InitLCDSI_Time(unsigned int uiCS, unsigned int uiRW, unsigned int uiBw, unsigned int uiStp, unsigned int uiPw, unsigned int uiHld)
{
	
	if (uiCS & LCDSI_CS00)
	{


		
		if (uiRW & LCDSI_READ)
		{
			BITCSET(HwLCDSICTRL1, HwLCDSICTRL1_BW0, ((uiBw&1) << 15));
			BITCSET(HwLCDSICTRL1, 0x00007000, (uiStp << 12));
			BITCSET(HwLCDSICTRL1, 0x00000FF8, (uiPw << 3));
			BITCSET(HwLCDSICTRL1, 0x00000007, uiHld);
		} 

		if(uiRW & LCDSI_WRITE) 
		{
			BITCSET(HwLCDSICTRL1, HwLCDSICTRL1_BW1, ((uiBw) << 31));	

			BITCSET(HwLCDSICTRL1, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSICTRL1, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSICTRL1, 0x00070000, (uiHld << 16));
		}
	}

	if (uiCS & LCDSI_CS01)
	{
		if (uiRW & LCDSI_READ)
		{
			BITCSET(HwLCDSICTRL2, HwLCDSICTRL2_BW0, ((uiBw&1) << 15)); 
			BITCSET(HwLCDSICTRL2, 0x00007000, (uiStp << 12));
			BITCSET(HwLCDSICTRL2, 0x00000FF8, (uiPw << 3));
			BITCSET(HwLCDSICTRL2, 0x00000007, uiHld);
		}

		if(uiRW & LCDSI_WRITE)
		{
			BITCSET(HwLCDSICTRL2, HwLCDSICTRL2_BW1, ((uiBw&1) << 31));			
			BITCSET(HwLCDSICTRL2, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSICTRL2, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSICTRL2, 0x00070000, (uiHld << 16));
		}
	}

	if (uiCS & LCDSI_CS10)
	{

		if (uiRW & LCDSI_READ)
		{
			BITCSET(HwLCDSICTRL3, HwLCDSICTRL3_BW0, ((uiBw) << 15));    
			BITCSET(HwLCDSICTRL3, 0x00007000, (uiStp << 12));
			BITCSET(HwLCDSICTRL3, 0x00000FF8, (uiPw << 3));
			BITCSET(HwLCDSICTRL3, 0x00000007, uiHld);
		}

		if(uiRW & LCDSI_WRITE)
		{
			BITCSET(HwLCDSICTRL3, HwLCDSICTRL3_BW1, ((uiBw) << 31));		

			BITCSET(HwLCDSICTRL3, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSICTRL3, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSICTRL3, 0x00070000, (uiHld << 16));
		}
	}

	if (uiCS & LCDSI_CS11)
	{
		
	
		 
		if (uiRW & LCDSI_READ)
		{
			BITCSET(HwLCDSICTRL4, HwLCDSICTRL4_BW0, ((uiBw) << 15));   
			BITCSET(HwLCDSICTRL4, 0x00007000, (uiStp << 12));
			BITCSET(HwLCDSICTRL4, 0x00000FF8, (uiPw << 3));
			BITCSET(HwLCDSICTRL4, 0x00000007, uiHld);
		}

		if(uiRW & LCDSI_WRITE)
		{
			BITCSET(HwLCDSICTRL4, HwLCDSICTRL4_BW1, ((uiBw) << 31));		

			BITCSET(HwLCDSICTRL4, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSICTRL4, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSICTRL4, 0x00070000, (uiHld << 16));
		}
	}

}


void InitLCDSI_Time_(unsigned int uiCS, unsigned int uiBw, unsigned int uiStp, unsigned int uiPw, unsigned int uiHld)
{
	
	if (uiCS & LCDSI_CS00)
	{
		
		if(uiBw) // 16Bit 
			BITSCLR(HwLCDSICTRL5, HwLCDSICTRL5_WBW, uiBw << 31);	
		else //8bit
			BITCSET(HwLCDSICTRL5, HwLCDSICTRL5_WBW, uiBw << 31);	
		BITCSET(HwLCDSICTRL5, 0x70000000, (uiStp << 28));
		BITCSET(HwLCDSICTRL5, 0x0FF80000, (uiPw << 19));
		BITCSET(HwLCDSICTRL5, 0x00070000, (uiHld << 16));
	}

	if (uiCS & LCDSI_CS01)
	{
#ifndef FEATURE_LCD_IF_8P8
		if(uiBw)
			BITSCLR(HwLCDSICTRL6, HwLCDSICTRL6_WBW, uiBw << 31);	
		else
#endif
			BITCSET(HwLCDSICTRL6, HwLCDSICTRL6_WBW, uiBw << 31);
	
			BITCSET(HwLCDSICTRL6, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSICTRL6, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSICTRL6, 0x00070000, (uiHld << 16));
	}

	if (uiCS & LCDSI_CS10)
	{
		if(uiBw)
			BITSCLR(HwLCDSICTRL7, HwLCDSICTRL7_WBW, uiBw << 31);
		else
			BITCSET(HwLCDSICTRL7, HwLCDSICTRL7_WBW, uiBw << 31);
			BITCSET(HwLCDSICTRL7, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSICTRL7, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSICTRL7, 0x00070000, (uiHld << 16));
	}

	if (uiCS & LCDSI_CS11)
	{
		
		if(uiBw)
			BITSCLR(HwLCDSICTRL8, HwLCDSICTRL8_WBW, uiBw<< 31);		 
		else
			BITCSET(HwLCDSICTRL8, HwLCDSICTRL8_WBW, uiBw<< 31);		 
			BITCSET(HwLCDSICTRL8, 0x70000000, (uiStp << 28));
			BITCSET(HwLCDSICTRL8, 0x0FF80000, (uiPw << 19));
			BITCSET(HwLCDSICTRL8, 0x00070000, (uiHld << 16));
	}
}


unsigned int LCDSI_CTRL (unsigned int uiFlag, unsigned int uiOM, unsigned int uiIM, unsigned int uiFmt, unsigned int uiRsp, unsigned int uiCs, unsigned int uiIa, unsigned int uiIvs)
{
	unsigned int uiReadIM = 0;
	
	if(uiFlag & SET_LCDSI_OUTPUTMODE)
	{
		BITCSET(HwLCDSICTRL0, HwLCDSICTRL0_OM, (uiOM << 1));
	}

	if(uiFlag & READ_LCDSI_INPUTMODE)
	{
		uiReadIM = (HwLCDSICTRL0 & HwLCDSICTRL0_IM);
	}

	if(uiFlag & SET_LCDSI_INPUTMODE)
	{
		BITCSET(HwLCDSICTRL0, HwLCDSICTRL0_IM, uiIM);
	}

	if(uiFlag & SET_LCDSI_FORMAT)
	{
		BITCSET(HwLCDSICTRL0, Hw2+Hw3+Hw4+Hw5, (uiFmt << 2));
	}

	if(uiFlag & SET_LCDSI_RESPONCE)
	{
		BITCSET(HwLCDSICTRL0, HwLCDSICTRL0_RSP_HIGH, (uiRsp << 6));
	}

	if(uiFlag & SET_LCDSI_CS)
	{
		BITCSET(HwLCDSICTRL0, HwLCDSICTRL0_CS_1, (uiCs << 7));
	}

	if(uiFlag & SET_LCDSI_IA)
	{
		BITCSET(HwLCDSICTRL0, HwLCDSICTRL0_IA_LOW, (uiIa << 15));
	}

	if(uiFlag & SET_LCDSI_IVS)
	{
		BITCSET(HwLCDSICTRL0, HwLCDSICTRL0_IVS_LOW, (uiIvs << 14));
	}

	return uiReadIM;
}


void LCDC_LookUpTable_Init(void)
{
	unsigned int i = 0, tmp = 0;

	volatile unsigned int *fb_lookup_table;

	fb_lookup_table = &HwLCDLUT;	

	for (i=0;i<256;i++) { 
		if (i < 128) {
			tmp = i << 1;
			fb_lookup_table[i] = (tmp << 16) | (tmp << 8) | tmp;
		}
		else {
			tmp = (i - 128) << 1;
			fb_lookup_table[i] = (tmp << 16) | (tmp << 8) | tmp;
		}
	}
}

void LCDC_SetLookUpTable(unsigned int uiType)
{
	switch (uiType) 
	{
		case	IMG_CH0 :
			BITCSET(HwLI2C, 0x00000600, 1<<9);
			break;

		case	IMG_CH1 :
			BITCSET(HwLI2C, 0x00000600, 2<<9);
			break;

		case	IMG_CH2 :
			BITCSET(HwLI2C, 0x00000600, 3<<9);
			break;

		default	:
			BITCSET(HwLI2C, 0x00000600, 0);
			break;
	}

}

void LCD_DELAY(unsigned int	delay)
{
	unsigned int loop;

	for(loop=0; loop<delay; loop++)
	{
		//__asm(nop);
		asm volatile ("nop");
	}
}




void LCDSI_DATA(unsigned int uiCS, unsigned long usData)
{   

  switch(uiCS) {

    case LCDSI_CS00:
      HwLCDSICS0RS0 = usData;
      break;
    case LCDSI_CS01:
      HwLCDSICS0RS1 = usData;
      break;
	  
    default:
      break;
  }
  #if 0  
            
	if (uiCS == LCDSI_CS00)
	{
		HwLCDSICS0RS0 = usData;
	}

	if (uiCS == LCDSI_CS01)
	{
		HwLCDSICS0RS1 = usData;
	}

	if (uiCS == LCDSI_CS10)
	{
		HwLCDSICS1RS0 = usData;
	}

	if (uiCS == LCDSI_CS11)
	{
		HwLCDSICS1RS1 = usData;
	}
	#endif
}

unsigned int LCDSI_DATA_Read(unsigned int uiCS)
{
	unsigned int usData;

	if (uiCS & LCDSI_CS00)
		usData	= HwLCDSICS0RS0 ;
	
	if (uiCS & LCDSI_CS01)
		usData	= HwLCDSICS0RS1;
	
	if (uiCS & LCDSI_CS10)
		usData	= HwLCDSICS1RS0;
	
	if (uiCS & LCDSI_CS11)
		usData	= HwLCDSICS1RS1;

	return usData;
	
}

	


