/*
 * linux/drivers/video/tccfb.c
 *
 * Based on:    Based on s3c2410fb.c, sa1100fb.c and others
 * Author:  <linux@telechips.com>
 * Created: June 10, 2008
 * Description: TCC LCD Controller Frame Buffer Driver
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>
#include <linux/cpufreq.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif
#include <mach/tcc_used_mem.h>
#include <asm/tlbflush.h>
#include <asm/cacheflush.h>

#include <mach/bsp.h>
#include <mach/tca_ckc.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
#include "tcc92xxfb.h"
#include <mach/tcc_scaler_ctrl.h>
#include <mach/tcc92xxfb_ioctrl.h>
#include <mach/tcc_composite_ioctl.h>
#include <mach/tcc_component_ioctl.h>

#include <mach/tca_fb_hdmi.h>
#include <mach/tca_fb_output.h>

/* Debugging stuff */
static int debug = 0;
#define dprintk(msg...)	if (debug) { printk( "tcc92fb: " msg); }

static int screen_debug = 0;
#define sprintk(msg...)	if (screen_debug) { printk( "tcc92fb scr: " msg); }
#define TCC_FB_DOUBLE
#define TCC_LCDC1_USE

#undef LOCK_SCREEN
/* define vsync methods */
#undef USE_VSYNC_INTERRUPT
#ifdef USE_VSYNC_INTERRUPT 
#define USE_EXT_INTERRUPT
#define FB_M2M_COPY
#endif//
#undef USE_VSYNC_POLLING

unsigned int last_framebufferaddr;

//#define POST_SET
#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
#define FB_HDMI_TEST
#endif


#define CONFIG_FB_TCC_DEVS_MAX	3	// do not change!
#define CONFIG_FB_TCC_DEVS		1

#if (CONFIG_FB_TCC_DEVS > CONFIG_FB_TCC_DEVS_MAX)
	#undef CONFIG_FB_TCC_DEVS
	#define CONFIG_FB_TCC_DEVS	CONFIG_FB_TCC_DEVS_MAX
#endif


#define SCREEN_DEPTH_MAX	32	// 32 or 16 
//								 : 32 - 32bpp(alpah+rgb888)
//								 : 16 - 16bpp(rgb565)


#ifdef FB_M2M_COPY
extern unsigned char tccxxx_overlay_use(void);
char M2M_Scaler0_Ctrl_External(SCALER_TYPE *scale_img);

dma_addr_t		Gmap_dma;	/* physical */
u_char *		Gmap_cpu;	/* virtual */

typedef struct{
	unsigned int update;
	unsigned int left;
	unsigned int top; 
	unsigned int eright;
	unsigned int ebottom;
	struct tccfb_info fbi;
}tcc_patial_info;
tcc_patial_info tccfb_parial;

static struct clk *scaler0_clk;		// fb image scale
#endif//


const unsigned int default_scn_depth[CONFIG_FB_TCC_DEVS_MAX] = 
{
/* fb0, Layer0: bottom */  (16), // 32 or 16
/* fb1, Layer1: middle */  (16), //  "
/* fb2, Layer2: top    */  (16)  //  "
};


struct lcd_struct {
	spinlock_t lock;
	wait_queue_head_t waitq;
	char state;
};
static struct lcd_struct lcdc_struct;


char 		Fb_Lcdc_num = 0xFF;
PLCDC		Fb_pLCDC;
static PLCDC_CHANNEL pLCDC_FB_CH;
static char HDMI_pause;
static char HDMI_video_mode = 0;
static char HDMI_lcdc_num = 1;

OUTPUT_SELECT_MODE	Output_SelectMode =  OUTPUT_SELECT_NONE;
static unsigned int Output_BaseAddr;

static unsigned int HDMI_video_width = 0;
static unsigned int HDMI_video_height = 0;

static struct clk *lcdc0_clk;
static struct clk *lcdc1_clk;
static struct clk *ddi_cache;
#ifdef CONFIG_LCD_CPU_INTERFACE
static struct clk *lcd_si;
#endif//
char fb_power_state;
static char fb_updating;
static unsigned int backlight_level;

// for frame buffer clear
static u_char *fb_mem_vaddr[CONFIG_FB_TCC_DEVS]= {0,};
static u_int   fb_mem_size [CONFIG_FB_TCC_DEVS]= {0,};

#if defined(CONFIG_FB_TCC89XX_XGA_CLAA104XA01CW) || defined(CONFIG_FB_TCC89XX_WXGA_HT121WX2)
static char fb_lvds_state = 1;
#else
static char fb_lvds_state = 0;
#endif


static int already_resumed;

static irqreturn_t tcc_lcd_handler(int irq, void *dev_id);
void tcc_lcd_interrupt_reg(char lcdc, char SetClear);

extern void tca_lcdc_port_setting(char Nlcdc);
extern void tca_bkl_init(char lcdctrl_num);
extern void tca_bkl_powerup(unsigned int last_buf);
extern void tca_bkl_powerdown(void);
extern void tca_bkl_setpowerval(unsigned int inValue);
extern unsigned int tca_bkl_getpowerval(void);

extern int hdmi_set_setting_flag(unsigned int hdmi_setting);
extern int hdmi_get_setting_flag(unsigned int *hdmi_setting);

extern struct tcc_freq_table_t stFBClockLimitTable;
extern struct display_platform_data tcc_display_data;

char TCC_Set_pLCDC(char Nlcdc)
{

	dprintk("%s  nuw_mode:%d , before_mode:%d \n",__func__, Nlcdc, Fb_Lcdc_num);

	if(Nlcdc == Fb_Lcdc_num)
		return FALSE;
	
	if(Nlcdc)
	{
		Fb_Lcdc_num = 1;
		Fb_pLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
		pLCDC_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(Fb_pLCDC->LI2C);
	}
	else
	{
		Fb_Lcdc_num = 0;
		Fb_pLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
		pLCDC_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(Fb_pLCDC->LI0C);
	}
	
	tca_bkl_init(Fb_Lcdc_num);
	
	return TRUE;
}


static unsigned char tcc_LCDC_Wait_signal_disable(char lcdc)
{
	unsigned int loop = 0;
	volatile PLCDC	uiPLCD;

	if(lcdc == 0)
	{
		uiPLCD = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);

		if(ISSET(HwDDI_CONFIG->PWDN, HwDDIC_PWDN_LCDC0))
			return FALSE;
	}
	else
	{
		uiPLCD = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);

		if(ISSET(HwDDI_CONFIG->PWDN, HwDDIC_PWDN_LCDC1))
			return FALSE;
	}

	if( ISSET(uiPLCD->LCTRL, HwLCTRL_LEN) )
	{
		BITSET(uiPLCD->LSTATUS, HwLSTATUS_DD);
		BITCLR(uiPLCD->LCTRL, HwLCTRL_LEN);

		while(TRUE && (loop < 100000)) //check wheather is lcd on
		{
			if((ISZERO(uiPLCD->LSTATUS, HwLSTATUS_BUSY)) && (ISSET(uiPLCD->LSTATUS, HwLSTATUS_DD)))
				break;
			msleep(1);
		}
	}

	return TRUE;
}

static unsigned int tcc_LCDC_enable(unsigned lcdc, unsigned onoff)
{
	volatile PLCDC pstLCDC;

	if(lcdc == 0)
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	}
	else
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
	}

	if(onoff)
	{
		BITSET(pstLCDC->LCTRL, HwLCTRL_LEN);
		BITSET(pstLCDC->LSTATUS, HwLSTATUS_RU);
	}
	else
	{
		BITCLR(pstLCDC->LCTRL, HwLCTRL_LEN);
		BITCLR(pstLCDC->LSTATUS, HwLSTATUS_RU);
	}

	return 1;
}

static unsigned int tcc_LCDC_ctrl_layer(unsigned lcdc, unsigned layer, unsigned onoff)
{
	volatile PLCDC pstLCDC;
	volatile PLCDC_CHANNEL pstLCDC_layer;

	if(lcdc == 0)
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
		pstLCDC_layer = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI0C);
	}
	else
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
		switch(layer)
		{
			case 0:
				pstLCDC_layer = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI0C);
				break;
			case 1:
				pstLCDC_layer = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI1C);
				break;
			case 2:
				pstLCDC_layer = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI2C);
				break;
		}
	}

	if(onoff)
	{
		BITSET(pstLCDC_layer->LIC, (1<<28)); /* channel image enable */
	}
	else
	{
		BITCLR(pstLCDC_layer->LIC, (1<<28)); /* channel image disalbe */
	}

	return 1;
}

static unsigned int tcc_LCDC_set_alpha(unsigned lcdc, unsigned layer, unsigned alpha)
{
	volatile PLCDC pstLCDC;
	volatile PLCDC_CHANNEL pstLCDC_CH;

	if(lcdc == 0)
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
		pstLCDC_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI0C);
	}
	else
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
		switch(layer)
		{
			case 0:
				pstLCDC_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI0C);
				break;
			case 1:
				pstLCDC_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI1C);
				break;
			case 2:
				pstLCDC_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pstLCDC->LI2C);
				break;
		}
	}

	alpha &= 0xff;

	BITSET(pstLCDC_CH->LIC, Hw30);						/* AEN bit */
	BITCLR(pstLCDC_CH->LIC, Hw24);						/* ASEL bit */

	BITCSET(pstLCDC_CH->LIA, 0x000000ff, (alpha));		/* A0 bits */
	BITCSET(pstLCDC_CH->LIA, 0x00ff0000, (alpha)<<16);	/* A1 bits */
	
	return 1;
}

static unsigned int tcc_LCDC_set_bgcolor(unsigned lcdc, unsigned red, unsigned green, unsigned blue)
{
	volatile PLCDC pstLCDC;
	unsigned uRegValue;

	if(lcdc == 0)
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	}
	else
	{
		pstLCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
	}

	uRegValue = ((blue & 0xff)<<16) | ((green & 0xff)<<8) | (red & 0xff);

	pstLCDC->LBC = uRegValue;

	return 1;
}

void tcc_ddi_cache_setting(void)
{
	dprintk("%s  \n",__func__);

	HwDDI_CACHE->DDIC_CFG0 = HwDDIC_CFG27(DDIC_LCD0_DMA0_0) | HwDDIC_CFG26(DDIC_LCD1_DMA0_0);
	HwDDI_CACHE->DDIC_CFG1 = HwDDIC_CFG31(DDIC_LCD1_DMA0_1) | HwDDIC_CFG30(DDIC_LCD1_DMA2)
							| HwDDIC_CFG29(DDIC_MSC0_DMA_0) | HwDDIC_CFG28(DDIC_MSC1_DMA_0);
		
	HwDDI_CACHE->DDIC_CTRL = (1<<DDIC_LCD0_DMA0_0)|(1<<DDIC_LCD1_DMA0_0)
							|(1<<DDIC_LCD1_DMA0_1)|(1<<DDIC_LCD1_DMA2)
							|(1<<DDIC_MSC0_DMA_0) |(1<<DDIC_MSC1_DMA_0)
								|(1<< DDIC_BW);

	dprintk("DDIC_CFG0 : 0x%08X, DDIC_CFG1 : 0x%08X DDIC_CTRL : %08X\n", HwDDI_CACHE->DDIC_CFG0, HwDDI_CACHE->DDIC_CFG1, HwDDI_CACHE->DDIC_CTRL);

}



void TCC_LCDC_Change_CtrlReg(PLCDC SrcLCDC, PLCDC DestLCDC)
{
	dprintk("%s  \n",__func__);

	DestLCDC->LBC = SrcLCDC->LBC;				//   0x04  R/W  0x00000000  LCD Background Color Register 
	DestLCDC->LCLKDIV = SrcLCDC->LCLKDIV;			//   0x08  R/W  0x00000000  LCD Clock Divider Register 
	DestLCDC->LHTIME1 = SrcLCDC->LHTIME1;			//   0x0C  R/W  0x00000000  LCD Horizontal Timing Register 1 
	DestLCDC->LHTIME2 = SrcLCDC->LHTIME2;			//   0x10  R/W  0x00000000  LCD Horizontal Timing Register 2 
	DestLCDC->LVTIME1 = SrcLCDC->LVTIME1;			//   0x14  R/W  0x00000000  LCD Vertical Timing Register 1 
	DestLCDC->LVTIME2 = SrcLCDC->LVTIME2;			//   0x18  R/W  0x00000000  LCD Vertical Timing Register 2 
	DestLCDC->LVTIME3 = SrcLCDC->LVTIME3;			//   0x1C  R/W  0x00000000  LCD Vertical Timing Register 3 
	DestLCDC->LVTIME4 = SrcLCDC->LVTIME4;			//   0x20  R/W  0x00000000  LCD Vertical Timing Register 4 
	DestLCDC->LLUTR = SrcLCDC->LLUTR;				//   0x24  R/W  0x00000000  LCD Lookup Register for Red 
	DestLCDC->LLUTG = SrcLCDC->LLUTG;				//   0x28  R/W  0x00000000  LCD Lookup Register for Green 
	DestLCDC->LLUTB = SrcLCDC->LLUTB;				//   0x2C  R/W  0x00000000  LCD Lookup Register for Blue 
	DestLCDC->LDP7L = SrcLCDC->LDP7L;				//   0x30  R/W  0x4d2b3401  LCD Modulo 7 Dithering Pattern (low) 
	DestLCDC->LDP7H = SrcLCDC->LDP7H;				//   0x34  R/W  0x0000003f  LCD Modulo 7 Dithering Pattern (high) 
	DestLCDC->LDP5 = SrcLCDC->LDP5;				//   0x38  R/W  0x1d0b0610  LCD Modulo 5 Dithering Pattern Register 
	DestLCDC->LDP4 = SrcLCDC->LDP4;				//   0x3C  R/W  0x00000768  LCD Modulo 4 Dithering Pattern Register 
	DestLCDC->LDP3 = SrcLCDC->LDP3;				//   0x40  R/W  0x00000034  LCD 3-bit Dithering Pattern Register 
	DestLCDC->LCP1 = SrcLCDC->LCP1;				//   0x44  R/W  0x000000ff  LCD Clipping Register1 
	DestLCDC->LCP2 = SrcLCDC->LCP2;				//   0x48  R/W  0x000000ff  LCD Clipping Register2 
	DestLCDC->LDS = SrcLCDC->LDS;				//   0x4C  R/W  0x00000000  LCD Display Size Register 
	DestLCDC->LSTATUS = SrcLCDC->LSTATUS;			//   0x50  R/CLR  0x00000000  LCD Status Register 
	DestLCDC->LIM = SrcLCDC->LIM;				//   0x54  R/W  0x0000001f  LCD Interrupt Register. 
	DestLCDC->LGR0 = SrcLCDC->LGR0;				//   0x58  R/W  0x00000000  LCD Gamma Correction Register 0 for Red Color 
	DestLCDC->LGR1 = SrcLCDC->LGR1;				//   0x5C  R/W  0x00000000  LCD Gamma Correction Register 1 for Red Color 
	DestLCDC->LGG0 = SrcLCDC->LGG0;				//   0x60  R/W  0x00000000  LCD Gamma Correction Register 0 for Green Color 
	DestLCDC->LGG1 = SrcLCDC->LGG1;				//   0x64  R/W  0x00000000  LCD Gamma Correction Register 1 for Green Color 
	DestLCDC->LGB0 = SrcLCDC->LGB0;				//   0x68  R/W  0x00000000  LCD Gamma Correction Register 0 for Blue Color 
	DestLCDC->LGB1 = SrcLCDC->LGB1;				//   0x6C  R/W  0x00000000  LCD Gamma Correction Register 1 for Blue Color 
	DestLCDC->LENH = SrcLCDC->LENH;				//   0x70  R/W  0x00000020  LCD Color Enhancement Register 
	

	DestLCDC->LCTRL = SrcLCDC->LCTRL;

}



void TCC_LCDC_Change_ChannelReg(PLCDC_CHANNEL SrcLCDC_CH, PLCDC_CHANNEL DestLCDC_CH)
{
	dprintk("%s   \n",__func__);

	DestLCDC_CH->LIP = SrcLCDC_CH->LIP;	
	DestLCDC_CH->LIS = SrcLCDC_CH->LIS;	
	DestLCDC_CH->LIBA0 = SrcLCDC_CH->LIBA0;
	DestLCDC_CH->LICA = SrcLCDC_CH->LICA;
	DestLCDC_CH->LIBA1 = SrcLCDC_CH->LIBA1;
	DestLCDC_CH->LIBA2 = SrcLCDC_CH->LIBA2;
	DestLCDC_CH->LIO = SrcLCDC_CH->LIO;	
	DestLCDC_CH->LISR = SrcLCDC_CH->LISR;
	DestLCDC_CH->LIA = SrcLCDC_CH->LIA;	
	DestLCDC_CH->LIKR = SrcLCDC_CH->LIKR;
	DestLCDC_CH->LIKG = SrcLCDC_CH->LIKG;
	DestLCDC_CH->LIKB = SrcLCDC_CH->LIKB;
	DestLCDC_CH->LIEN = SrcLCDC_CH->LIEN;

	DestLCDC_CH->LIC = SrcLCDC_CH->LIC;	
}


char TCC_FB_LCDC_NumSet(char NumLcdc)
{
	volatile int ttt= 0;
	unsigned int regl;

	PLCDC pLCDC0, pLCDC1;
	PLCDC_CHANNEL pLCDC0_FB_CH, pLCDC1_FB_CH;
	
	PCKC	pCKC = (CKC *)&HwCLK_BASE;
	PDDICONFIG		pDDICfg 	= (DDICONFIG *)&HwDDI_CONFIG_BASE;

	pLCDC0 = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	pLCDC0_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC0->LI0C);

	pLCDC1 = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
	pLCDC1_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC1->LI2C);

	dprintk("%s set_num:%d before_num:%d \n",__func__, NumLcdc, Fb_Lcdc_num);

	if(NumLcdc != Fb_Lcdc_num)
	{
		if(NumLcdc)
		{
			clk_enable(lcdc1_clk);
			regl = readl(&pCKC->PCLK_LCD0);
			writel(regl | (0x10000000), &pCKC->PCLK_LCD1); // EN: 0
		}
		else
		{
			clk_enable(lcdc0_clk);
			regl = readl(&pCKC->PCLK_LCD1);
			writel(regl | (0x10000000), &pCKC->PCLK_LCD0); // EN: 0
		}

		if(NumLcdc)
		{
			pLCDC0->LCTRL &= (~HwLCTRL_LEN);
			pLCDC0->LI0C &= (~HwLIC_IEN);

			pLCDC1->LCTRL &= (~HwLCTRL_LEN);
			pLCDC1->LI0C &= (~HwLIC_IEN);
			pLCDC1->LI1C &= (~HwLIC_IEN);
			pLCDC1->LI2C &= (~HwLIC_IEN);		

			msleep(30);
			
			while((ISSET(pLCDC0->LSTATUS, HwLSTATUS_BUSY))|| (ISSET(pLCDC1->LSTATUS, HwLSTATUS_BUSY)))
					ttt++;								

			if(ttt)
				printk("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  %d~~~~~~~~~~~~~~~~~~~~~~~ \n", ttt);
			
			TCC_LCDC_Change_ChannelReg(pLCDC0_FB_CH, pLCDC1_FB_CH);
			TCC_LCDC_Change_CtrlReg(pLCDC0, pLCDC1);
		}
		else
		{
			pLCDC1->LCTRL &= (~HwLCTRL_LEN);
			pLCDC1->LI0C &= (~HwLIC_IEN);
			pLCDC1->LI1C &= (~HwLIC_IEN);
			pLCDC1->LI2C &= (~HwLIC_IEN);

			pLCDC0->LCTRL &= (~HwLCTRL_LEN);
			pLCDC0->LI0C &= (~HwLIC_IEN);

			msleep(30);
			
			while((ISSET(pLCDC0->LSTATUS, HwLSTATUS_BUSY))|| (ISSET(pLCDC1->LSTATUS, HwLSTATUS_BUSY)))
					ttt++;								

			if(ttt)
				printk("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~  %d~~~~~~~~~~~~~~~~~~~~~~~ \n", ttt);

			TCC_LCDC_Change_ChannelReg(pLCDC1_FB_CH, pLCDC0_FB_CH);
			TCC_LCDC_Change_CtrlReg(pLCDC1, pLCDC0);
		}	

		TCC_Set_pLCDC(NumLcdc);

		tca_lcdc_port_setting(NumLcdc);


		if(NumLcdc)
		{
			pLCDC1_FB_CH->LIC |= HwLIC_IEN;
			pLCDC1->LCTRL |= HwLCTRL_LEN;
		}
		else
		{
			pLCDC0_FB_CH->LIC |= HwLIC_IEN;
			pLCDC0->LCTRL |= HwLCTRL_LEN;
		}

		if(fb_lvds_state)
			BITCSET(pDDICfg->LVDS_CTRL, (1<<0), NumLcdc); // lcdc1 -> LVDS
		
		if(NumLcdc)
			clk_disable(lcdc0_clk);
		else
			clk_disable(lcdc1_clk);
		

		Fb_Lcdc_num = NumLcdc;
		return 1;
	}
	
	return 0;
}

char TCC_FB_LCDC_ChangeReg(char NumLcdc, char OnOff)
{
	unsigned int regl;
	PLCDC pLCDC0, pLCDC1;
	PLCDC_CHANNEL pLCDC0_FB_CH, pLCDC1_FB_CH;
	PCKC pCKC = (CKC *)&HwCLK_BASE;

	// Set the address for LCDC0
	pLCDC0 = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	pLCDC0_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC0->LI0C);

	// Set the address for LCDC1
	pLCDC1 = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
	pLCDC1_FB_CH = (volatile PLCDC_CHANNEL)tcc_p2v(pLCDC1->LI2C);

	dprintk("%s set_num:%d, onoff:%d \n",__func__, NumLcdc, OnOff);

	if(NumLcdc)
	{
		// Save the setting value for clock from LCDC0 to LCDC1
		clk_enable(lcdc1_clk);
		regl = readl(&pCKC->PCLK_LCD0);
		writel(regl | (0x10000000), &pCKC->PCLK_LCD1); // EN: 0
		
		// Disable LCDC0 and image
		pLCDC0->LCTRL &= (~HwLCTRL_LEN);
		pLCDC0_FB_CH->LIC &= (~HwLIC_IEN);

		// Disable LCDC1 and each images
		pLCDC1->LCTRL &= (~HwLCTRL_LEN);
		pLCDC1->LI0C &= (~HwLIC_IEN);
		pLCDC1->LI1C &= (~HwLIC_IEN);
		pLCDC1->LI2C &= (~HwLIC_IEN);

		msleep(30);

		// Save the setting value from LCDC0 to LCDC1
		TCC_LCDC_Change_ChannelReg(pLCDC0_FB_CH, pLCDC1_FB_CH);
		TCC_LCDC_Change_CtrlReg(pLCDC0, pLCDC1);

		if(OnOff)
		{
			pLCDC1_FB_CH->LIC |= HwLIC_IEN;
			pLCDC1->LCTRL |= HwLCTRL_LEN;
		}
		else
		{
			//clk_disable(lcdc1_clk);
		}
	}
	else
	{
		// Save the setting value for clock from LCDC1 to LCDC0
		clk_enable(lcdc0_clk);
		regl = readl(&pCKC->PCLK_LCD1);
		writel(regl | (0x10000000), &pCKC->PCLK_LCD0); // EN: 0
		
		// Disable LCDC1 and image
		pLCDC1->LCTRL &= (~HwLCTRL_LEN);
		pLCDC1_FB_CH->LIC &= (~HwLIC_IEN);

		// Disable LCDC0 and image
		pLCDC0->LCTRL &= (~HwLCTRL_LEN);
		pLCDC0->LI1C &= (~HwLIC_IEN);

		msleep(30);

		// Save the setting value from LCDC1 to LCDC0
		TCC_LCDC_Change_ChannelReg(pLCDC1_FB_CH, pLCDC0_FB_CH);
		TCC_LCDC_Change_CtrlReg(pLCDC1, pLCDC0);

		if(OnOff)
		{
			pLCDC0_FB_CH->LIC |= HwLIC_IEN;
			pLCDC0->LCTRL |= HwLCTRL_LEN;
		}
		else
		{
			//clk_disable(lcdc0_clk);
		}
	}	

	return 0;
}

char TCC_HDMI_OnOff(char onoff)
{
	dprintk("%s onoff:%d Output_SelectMode:%d \n",__func__, onoff, Output_SelectMode);


	if(onoff && (Output_SelectMode == OUTPUT_SELECT_HDMI_ENABLE || Output_SelectMode == OUTPUT_SELECT_HDMI_TIMING_SET))
		return 0;

	if((!onoff) && (Output_SelectMode == OUTPUT_SELECT_NONE))
		return 0;

	if(onoff)	{
		#ifndef CONFIG_TCC_OUTPUT_STARTER
			TCC_FB_LCDC_NumSet(0);
		#endif
		TCC_HDMI_LCDC_OnOff(1, 1);
		Output_SelectMode = OUTPUT_SELECT_HDMI_ENABLE;
		BITSET(HwDDI_CONFIG->HDMI_CTRL, HwDDIC_HDMI_CTRL_EN);
	}
	else {
		BITCLR(HwDDI_CONFIG->HDMI_CTRL, HwDDIC_HDMI_CTRL_EN);
		#ifndef CONFIG_TCC_OUTPUT_STARTER
			TCC_FB_LCDC_NumSet(1);
		#endif
		TCC_HDMI_LCDC_OnOff(0, 0);
		Output_SelectMode= OUTPUT_SELECT_NONE;
	}

	return 1;	
}

void tcc92xxfb_hdmi_starter(char hdmi_lcdc_num, struct lcdc_timimg_parms_t *lcdc_timing)
{
	PLCDC pLCDC0, pLCDC1;
	
	dprintk("%s  \n",__func__);

	pLCDC0 = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
	pLCDC1 = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);

	#ifdef CONFIG_TCC_OUTPUT_STARTER
		TCC_FB_LCDC_NumSet(0);				
		TCC_HDMI_LCDC_OnOff(1, 1);
		BITSET(HwDDI_CONFIG->HDMI_CTRL, HwDDIC_HDMI_CTRL_EN);
	#else
		TCC_HDMI_OnOff(TRUE);
	#endif
	
	tcc_ddi_cache_setting();
	TCC_HDMI_LCDC_Timing(lcdc_timing);

	#ifdef CONFIG_TCC_OUTPUT_STARTER
		pLCDC1->LI2C = HwLIC_IEN | TCC_LCDC_IMG_FMT_RGB565;
	#else
		Output_SelectMode = OUTPUT_SELECT_HDMI_TIMING_SET;
		pLCDC1->LI2C = pLCDC0->LI0C;
	#endif
}

void tcc_lcd_interrupt_reg(char lcdc, char SetClear)
{

	dprintk("%s  %d %d\n",__func__, lcdc, SetClear);

	if(SetClear)
	{
		#ifdef USE_EXT_INTERRUPT
	    BITCLR(HwGPIOF->GPFN3, Hw8-Hw4);
	    BITCLR(HwGPIOF->GPEN, Hw25);
		
		BITCSET(HwEINTSEL->EINTSEL1, HwEINTSEL1_EINT6_MASK, HwEINTSEL1_EINT6(SEL_GPIOF25));

		BITSET(HwPIC->POL0, HwINT0_EI6);
		BITCLR(HwPIC->MODE0, HwINT0_EI6);
		BITCLR(HwPIC->MODEA0, HwINT0_EI6);
		BITSET(HwPIC->SEL0, HwINT0_EI6);	
		BITCLR(HwPIC->INTMSK0, HwINT0_EI6);	

		if (request_irq(IRQ_EI6, tcc_lcd_handler,	IRQF_SHARED,
						"TCC_LCD",	tcc_lcd_handler))	{
			printk(KERN_ERR "tcc_lcd_interrupt_reg request_irq ERROR!\n");
		}
		#else
		
		if(lcdc){
			if (request_irq(IRQ_LCD1, tcc_lcd_handler,	IRQF_SHARED,
						"TCC_LCD",tcc_lcd_handler)){
				printk(KERN_ERR "tcc_lcdc1_interrupt_reg request_irq ERROR!\n");
			}
		}
		else{
			if (request_irq(IRQ_LCD0, tcc_lcd_handler,	IRQF_SHARED,
					"TCC_LCD",	tcc_lcd_handler)){
				printk(KERN_ERR "tcc_lcdc0_interrupt_reg request_irq ERROR!\n");
			}
		}
		#endif//
	}
	else
	{
	#ifdef USE_EXT_INTERRUPT
		free_irq(IRQ_EI6, tcc_lcd_handler);
	#else
		if(lcdc)
			free_irq(IRQ_LCD1, tcc_lcd_handler);
		else
			free_irq(IRQ_LCD0, tcc_lcd_handler);
	#endif//USE_EXT_INTERRUPT
	}
}


static void tcc_lcdc_interrupt(char onoff, char lcdc)
{
#ifdef USE_EXT_INTERRUPT
	if(onoff == 1)	{
		BITSET(HwPIC->CLR0, HwINT0_EI6);	
		BITSET(HwPIC->INTMSK0, HwINT0_EI6);	
		BITSET(HwPIC->IEN0, HwINT0_EI6);	
	}
	else	{
		BITCLR(HwPIC->INTMSK0, HwINT0_EI6);	
		BITCLR(HwPIC->IEN0, HwINT0_EI6);	

		BITSET(HwPIC->CLR0, HwINT0_EI6);	
	}
#else
	unsigned int LCDC_INTbit;
	if(lcdc)
		LCDC_INTbit = HwINT0_LCD1;
	else
		LCDC_INTbit = HwINT0_LCD0;

	if(onoff == 1)
	{
		BITCSET(Fb_pLCDC->LIM, 0xFFFFFFFF, HwLSTATUS_DER);
		BITSET(HwPIC->POL0, LCDC_INTbit);
		BITSET(HwPIC->SEL0, LCDC_INTbit);
		BITSET(HwPIC->MODE0, LCDC_INTbit);
		BITSET(HwPIC->IEN0, LCDC_INTbit);
	}
	else
	{
		BITCLR(HwPIC->IEN0, LCDC_INTbit);
	}
#endif //USE_EXT_INTERRUPT
}



static int tccfb_check_var(struct fb_var_screeninfo *var,
			       struct fb_info *info)
{
//	struct tccfb_info *fbi = info->par;

//	dprintk(" %s:  yoffset:%d \n", __FUNCTION__, var->yoffset);

	/* validate bpp */
	if (var->bits_per_pixel > 32)
		var->bits_per_pixel = 32;
	else if (var->bits_per_pixel < 16)
		var->bits_per_pixel = 16;
	
	/* set r/g/b positions */
	if (var->bits_per_pixel == 16) {
		var->red.offset 	= 11;
		var->green.offset	= 5;
		var->blue.offset	= 0;
		var->red.length 	= 5;
		var->green.length	= 6;
		var->blue.length	= 5;
		var->transp.length	= 0;
	} else if (var->bits_per_pixel == 32) {
		var->red.offset 	= 16;
		var->green.offset	= 8;
		var->blue.offset	= 0;
		var->transp.offset	= 24;
		var->red.length 	= 8;
		var->green.length	= 8;
		var->blue.length	= 8;
		var->transp.length	= 8;
	} else {
		var->red.length 	= var->bits_per_pixel;
		var->red.offset 	= 0;
		var->green.length	= var->bits_per_pixel;
		var->green.offset	= 0;
		var->blue.length	= var->bits_per_pixel;
		var->blue.offset	= 0;
		var->transp.length	= 0;
	}
	if (var->yres_virtual < var->yoffset + var->yres)
		var->yres_virtual = var->yoffset + var->yres;

	return 0;
}


#ifdef FB_M2M_COPY
void tcc_scaler_copy(unsigned int left, unsigned int top, unsigned int eright, unsigned int ebottom, struct tccfb_info *fbi)
{
	SCALER_TYPE fbscaler;
	unsigned int src_addr, dest_addr;
	unsigned int offset, size;

	offset = top * (fbi->fb->var.xres) * (fbi->fb->var.bits_per_pixel / 8);
	
	size = (ebottom - top) * (fbi->fb->var.xres) * (fbi->fb->var.bits_per_pixel / 8);
	src_addr = fbi->map_dma + fbi->fb->var.xres * fbi->fb->var.yoffset *(fbi->fb->var.bits_per_pixel / 8);

	
	if(fbi->fb->var.yoffset > fbi->fb->var.yres)	{
		src_addr = PAGE_ALIGN(src_addr);
	}
	
	sprintk("%s left:%d  top:%d right:%d ebottom:%d addr:0x%x addr:0x%x \n", __func__, left, top, eright, ebottom, Gmap_dma, src_addr);

	src_addr = src_addr + offset;
	dest_addr = Gmap_dma + offset;
	

	fbscaler.responsetype = SCALER_NOWAIT;

	fbscaler.src_Yaddr = (char *)src_addr;
	fbscaler.src_Uaddr = 0;
	fbscaler.src_Vaddr = 0;
	fbscaler.src_fmt = SCALER_YUV422_sq0;
	fbscaler.src_ImgWidth = (eright - left)* 2;
	fbscaler.src_ImgHeight = (ebottom - top);

	fbscaler.src_winLeft = 0;
	fbscaler.src_winTop = 0;
	fbscaler.src_winRight = (eright - left)* 2;
	fbscaler.src_winBottom = (ebottom - top);

	fbscaler.dest_Yaddr = (char *)dest_addr;	// destination image address
	fbscaler.dest_Uaddr = 0;	// destination image address
	fbscaler.dest_Vaddr = 0;	// destination image address
	fbscaler.dest_fmt = SCALER_YUV422_sq0;		// destination image format
	fbscaler.dest_ImgWidth = (eright - left)* 2;	// destination image width
	fbscaler.dest_ImgHeight = (ebottom - top); // destination image height

	fbscaler.dest_winLeft = 0;
	fbscaler.dest_winTop = 0;
	fbscaler.dest_winRight = (eright - left)* 2;
	fbscaler.dest_winBottom = (ebottom - top);

	M2M_Scaler0_Ctrl_External(&fbscaler);

}

void tcc_setup_interrupt(struct tccfb_info *fbi)
{
	int ret;
	unsigned int status = Fb_pLCDC->LSTATUS;
	
	if(tccfb_parial.update == 1)	{
		lcdc_struct.state = 0;

		ret = wait_event_interruptible_timeout(lcdc_struct.waitq, lcdc_struct.state == 1, msecs_to_jiffies(50));
		if(!ret)
		{
			printk("  [%d]: tcc_setup_interrupt timed_out!! \n", ret);
			tccfb_parial.update = 0;
		}

	}

	if((fbi->fb->var.reserved[0] == 0x54445055) && (fbi->fb->var.reserved[2] != 0))	{
		tccfb_parial.fbi = *fbi;
		tccfb_parial.update = 1;
		tccfb_parial.left = fbi->fb->var.reserved[1] & 0xffff;
		tccfb_parial.top = fbi->fb->var.reserved[1] >> 16;
		tccfb_parial.eright= fbi->fb->var.reserved[2] & 0xffff;
		tccfb_parial.ebottom= fbi->fb->var.reserved[2] >> 16;
	}
	else {
		tccfb_parial.fbi = *fbi;
		tccfb_parial.update = 1;
		tccfb_parial.left = 0;
		tccfb_parial.top = 0;
		tccfb_parial.eright= fbi->fb->var.xres;
		tccfb_parial.ebottom= fbi->fb->var.yres;
	}
	
	Fb_pLCDC->LSTATUS = status | HwLSTATUS_DER; 
	tcc_lcdc_interrupt(TRUE, Fb_Lcdc_num);
}
#endif//


/* tccfb_pan_display
 *
 * pandisplay (set) the controller from the given framebuffer
 * information
*/

static int tccfb_pan_display(struct fb_var_screeninfo *var, struct fb_info *info)
{
	struct tccfb_info *fbi =(struct tccfb_info *) info->par;
	
	unsigned int status, regl;
	unsigned int base_addr = 0;
	unsigned int lcd_width, lcd_height, img_width, img_height;
	unsigned int bpp = fbi->fb->var.bits_per_pixel / 8;	/* bytes per pixel */
	unsigned int update = 1;
	unsigned int output_type = TCC_OUTPUT_COMPOSITE;
#ifdef USE_VSYNC_POLLING
	unsigned long buffer_size = fbi->fb->var.xres * bpp * fbi->fb->var.yres
#endif

	base_addr = Output_BaseAddr = fbi->map_dma +fbi->fb->var.xres * bpp * var->yoffset;
	if(var->yoffset > var->yres)	{
		base_addr = Output_BaseAddr = PAGE_ALIGN(base_addr);
	}
	last_framebufferaddr = base_addr;
	
	if(!fb_power_state) 	{
		printk("%s  power down state %d\n", __func__, fb_power_state);
		return 1;
	}
	
	status = Fb_pLCDC->LSTATUS;

	regl = Fb_pLCDC->LDS; // get LCD size 
	
	#ifdef FB_M2M_COPY
	if(!tccxxx_overlay_use())
		base_addr = Gmap_dma;
	#endif//


	sprintk("%s: fb%d addr:0x%x addr:0x%x bpp:%d xres:%d, yres:%d yoffset:%3d \n", __FUNCTION__, 
		fbi->fb->node, base_addr, pLCDC_FB_CH->LIBA0, var->bits_per_pixel, fbi->fb->var.xres,fbi->fb->var.yres, var->yoffset);

	#ifndef CONFIG_TCC_OUTPUT_STARTER
	lcd_width = (regl & 0xFFFF);
	lcd_height = ((regl>>16) & 0xFFFF);
	img_width = fbi->fb->var.xres;
	img_height = fbi->fb->var.yres;

	if(img_width > lcd_width)
		img_width = lcd_width;
	
	if(img_height > lcd_height)
		img_height = lcd_height;

	if(Output_SelectMode != OUTPUT_SELECT_COMPONENT_ENABLE)
	{
		#ifdef FB_M2M_COPY
		if(tccxxx_overlay_use() || (!fb_updating))
		#endif// FB_M2M_COPY
		{
			pLCDC_FB_CH->LIBA0 = base_addr;
			pLCDC_FB_CH->LISR = 0;
			pLCDC_FB_CH->LIS = (img_height<< 16) | (img_width); //Size
			pLCDC_FB_CH->LIO = fbi->fb->var.xres * bpp;
			pLCDC_FB_CH->LIC |= (1<<28); //  image enable
		}
	}
	#endif //CONFIG_TCC_OUTPUT_STARTER

	#ifdef LOCK_SCREEN
	if(!fb_updating)
	{
		if(backlight_level != tca_bkl_getpowerval())
		{
			msleep(10); //first frame appllied to lcd
			tca_bkl_setpowerval(backlight_level);
			printk("%s setting backlight level %d\n", __func__, backlight_level);
		}
	}
	#endif//	LOCK_SCREEN

	fb_updating = 1;

	#ifndef CONFIG_TCC_OUTPUT_STARTER
	if(Fb_pLCDC->LCTRL & HwLCTRL_LEN)
		update = 1;
	else
		update = 0;
	#endif

	if(update)
	{
#if 0//defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
		// in case RGB888, Alpha blending enable...
		if(HDMI_video_mode && fbi->fb->var.bits_per_pixel == 32)
		{
			TCC_HDMI_FB_Set_AlphaBlending();
		}
#endif /* CONFIG_TCC_HDMI_UI_SIZE_1280_720 */

#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
	#if 0//def CONFIG_TCC_OUTPUT_STARTER
		/* Update HDMI,COMPOSITE,COMPONENT data */
		TCC_HDMI_FB_update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr);
		TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, TCC_OUTPUT_COMPOSITE);
		TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, TCC_OUTPUT_COMPONENT);
	#else //CONFIG_TCC_OUTPUT_STARTER
		switch(Output_SelectMode)
		{
			case OUTPUT_SELECT_HDMI_TIMING_SET:
				{
					TCC_HDMI_FB_update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr);
				}
				break;
			case OUTPUT_SELECT_COMPOSITE_ENABLE:
				{
					output_type = TCC_OUTPUT_COMPOSITE;
					TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, TCC_OUTPUT_COMPOSITE);
				}
				break;
			case OUTPUT_SELECT_COMPONENT_ENABLE:
				{
					output_type = TCC_OUTPUT_COMPONENT;
					TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, TCC_OUTPUT_COMPONENT);
				}
				break;
			default:
				break;
		}
	#endif //CONFIG_TCC_OUTPUT_STARTER
#endif /* CONFIG_TCC_HDMI_UI_DISPLAY_OFF */

	#if 0//def CONFIG_TCC_OUTPUT_STARTER
		#if 0
		if( (Output_SelectMode != OUTPUT_SELECT_HDMI_TIMING_SET) &&
			(Output_SelectMode != OUTPUT_SELECT_COMPOSITE_ENABLE) &&
			(Output_SelectMode != OUTPUT_SELECT_COMPONENT_ENABLE) )
		{
			return 0;
		}
		#endif
	
		/* Update HDMI,COMPOSITE,COMPONENT screen */
		TCC_HDMI_FB_update_sync();
		TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_COMPOSITE);
		TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_COMPONENT);
	#else
		switch(Output_SelectMode)
		{
		#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
			case OUTPUT_SELECT_HDMI_TIMING_SET:
				TCC_HDMI_FB_update_sync();
				#ifdef FB_M2M_COPY
					tcc_setup_interrupt(fbi);
				#endif//FB_M2M_COPY
				break;
			case OUTPUT_SELECT_COMPOSITE_ENABLE:
			case OUTPUT_SELECT_COMPONENT_ENABLE:
				TCC_OUTPUT_FB_UpdateSync(output_type);
				#ifdef FB_M2M_COPY
					if(Output_SelectMode == OUTPUT_SELECT_COMPOSITE_ENABLE)
						tcc_setup_interrupt(fbi);
				#endif//FB_M2M_COPY
				break;
		#endif /* CONFIG_TCC_HDMI_UI_DISPLAY_OFF */
			default:
			#ifdef USE_VSYNC_INTERRUPT
				#ifdef FB_M2M_COPY
				if(!tccxxx_overlay_use())
					tcc_setup_interrupt(fbi);
				#else
					if(Fb_Lcdc_num) {
						Fb_pLCDC->LSTATUS = status | HwLSTATUS_DER; 
						tcc_lcdc_interrupt(TRUE, Fb_Lcdc_num);
						lcdc_struct.state = 0;
						wait_event_interruptible(lcdc_struct.waitq, lcdc_struct.state == 1);
					}
				#endif//FB_M2M_COPY
			#endif

			#ifdef USE_VSYNC_POLLING
				while(1) {
					unsigned long n = pLCDC_FB_CH->LICA;
					if ((n >= base_addr) && (n < (base_addr + buffer_size))) break;
				}
			#endif
				break;
		}
	#endif
	}

	return 0;
}
/* tccfb_activate_var
 *
 * activate (set) the controller from the given framebuffer
 * information
*/

static void tccfb_activate_var(struct tccfb_info *fbi,
				   struct fb_var_screeninfo *var)
{
//	dprintk("activate_var\n");

	unsigned int imgch = 0;
	unsigned int bpp_value;
	unsigned int tmp_value;

//	dprintk(" %s:fb%d bpp:%d %3d, yoffset:%3d \n", __FUNCTION__, fbi->fb->node, var->bits_per_pixel, fbi->fb->var.xres, var->yoffset);

	if((0 <= fbi->fb->node) && (fbi->fb->node < CONFIG_FB_TCC_DEVS))
		imgch = fbi->fb->node;
	else
		return;

	if(fbi->fb->var.bits_per_pixel == 32)
		bpp_value = 0x000C; //RGB888
	else if(fbi->fb->var.bits_per_pixel == 16)
		bpp_value = 0x000A; //RGB565
	else
	{
		dprintk("%s:fb%d Not Supported BPP!\n", __FUNCTION__, fbi->fb->node);
		return;
	}

	/* write new registers */
	switch(imgch)
	{
		case 0:
			tmp_value = pLCDC_FB_CH->LIC;
			pLCDC_FB_CH->LIC = (tmp_value & 0xFFFFFFE0) | (bpp_value & 0x0000001F);
			pLCDC_FB_CH->LIO = fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel/8);
			break;

		case 1:
			tmp_value = pLCDC_FB_CH->LIC;
			pLCDC_FB_CH->LIC = (tmp_value & 0xFFFFFFE0) | (bpp_value & 0x0000001F);
			pLCDC_FB_CH->LIO = fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel/8);
			break;

		case 2:
			tmp_value = pLCDC_FB_CH->LIC;
			pLCDC_FB_CH->LIC = (tmp_value & 0xFFFFFFE0) | (bpp_value & 0x0000001F);
			pLCDC_FB_CH->LIO = fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel/8);
			break;
	}
	return;
}


/*
 *      tccfb_set_par - Optional function. Alters the hardware state.
 *      @info: frame buffer structure that represents a single frame buffer
 *
 */
static int tccfb_set_par(struct fb_info *info)
{
	struct tccfb_info *fbi = info->par;
	struct fb_var_screeninfo *var = &info->var;

//	dprintk("- tccfb_set_par\n");
	
	if (var->bits_per_pixel == 16)
		fbi->fb->fix.visual = FB_VISUAL_TRUECOLOR;
	else if (var->bits_per_pixel == 32)
		fbi->fb->fix.visual = FB_VISUAL_TRUECOLOR;
	else
		fbi->fb->fix.visual = FB_VISUAL_PSEUDOCOLOR;

	fbi->fb->fix.line_length = (var->xres*var->bits_per_pixel)/8;

	#ifndef CONFIG_TCC_OUTPUT_STARTER
	/* activate this new configuration */
   	if( fb_power_state && Output_SelectMode != OUTPUT_SELECT_COMPONENT_ENABLE ) 
		tccfb_activate_var(fbi, var);
	#endif

	return 0;
}


static int tccfb_ioctl(struct fb_info *info, unsigned int cmd,unsigned long arg)
{
	unsigned int imgch=0;

	if((0 <= info->node) && (info->node < CONFIG_FB_TCC_DEVS))
	{
		imgch = info->node;
	}
	else
	{
		dprintk("ioctl: Error - fix.id[%d]\n", info->node);
		return 0;
	}


	switch(cmd)
	{
		case TCC_FB_FLUSH:
			local_flush_tlb_all();
			flush_cache_all();
			break;

		case TCC_LCD_BL_SET:
			{
				unsigned long bl_val;
						
				if (copy_from_user((void *)&bl_val, (const void *)arg, sizeof(unsigned long)))
					return -EFAULT;
				
				dprintk("ioctl: TCC_LCD_BL_SET %lu %d %d \n", bl_val, backlight_level, fb_power_state);

				backlight_level = bl_val;
				#ifdef LOCK_SCREEN
				if(fb_updating) 
				#endif//
				{
					tca_bkl_setpowerval(backlight_level);
				}
			}
			break;

 
		case TCC_LCDC_HDMI_START:
			dprintk("TCC_LCDC_HDMI_START: \n");
			
			if(!fb_power_state)
				return -1;
			
			TCC_HDMI_OnOff(TRUE);
			break;
			
		case TCC_LCDC_HDMI_TIMING:
			{
				#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
				struct tccfb_info *fbi =(struct tccfb_info *) info->par;
				#endif//

				struct lcdc_timimg_parms_t lcdc_timing;
				printk("TCC_LCDC_HDMI_TIMING: \n");

				if(!fb_power_state)
					return -1;

				if (copy_from_user((void*)&lcdc_timing, (const void*)arg, sizeof(struct lcdc_timimg_parms_t)))				{
					return -EFAULT;
				}
				dprintk("%s: TCC_LCDC_HDMI_TIMING id:%d iv:%d lpc:%d flc:%d  \n", __func__, lcdc_timing.id, lcdc_timing.iv, lcdc_timing.lpc, lcdc_timing.flc);	
				TCC_HDMI_LCDC_Timing(&lcdc_timing);	
				Output_SelectMode = OUTPUT_SELECT_HDMI_TIMING_SET;

#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
				// current framebuffer display to HDMI
				TCC_HDMI_FB_update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr);
				TCC_HDMI_FB_update_sync();	
#endif /*CONFIG_TCC_HDMI_UI_DISPLAY_OFF*/
			}
			break;
			
		case TCC_LCDC_HDMI_DISPLAY:
			{
				struct tcc_lcdc_image_update ImageInfo;
				if (copy_from_user((void *)&ImageInfo, (const void *)arg, sizeof(struct tcc_lcdc_image_update))){
					return -EFAULT;
				}
		
				if(Output_SelectMode == OUTPUT_SELECT_HDMI_TIMING_SET)
					TCC_HDMI_DISPLAY_UPDATE((struct tcc_lcdc_image_update *)&ImageInfo);
			}
			break;

		case TCC_LCDC_HDMI_END:
			dprintk("TCC_LCDC_HDMI_END:fb_power_state:%d\n", fb_power_state);
			if(fb_power_state)
			{
				TCC_HDMI_OnOff(FALSE);
			}
			break;

		case TCC_LCDC_HDMI_CHECK:
			{
				unsigned int ret_mode = 0;
				if(
					(!fb_power_state) || HDMI_pause
					#if (SCREEN_WIDTH < SCREEN_HEIGHT)
					|| (!HDMI_video_mode)
					#endif//
				)
					ret_mode = 1;
				
				put_user(ret_mode, (unsigned int __user*)arg);
			}
			break;

		case TCC_LCDC_HDMI_MODE_SET:
 			{
				TCC_HDMI_M uiHdmi;
				
				if(get_user(uiHdmi, (int __user *) arg))
					return -EFAULT;

				dprintk("%s: TCC_LCDC_HDMI_MODE_SET [%d] video_M:%d Output_SelectMode:%d   \n", __func__ , uiHdmi , HDMI_video_mode, Output_SelectMode);

				switch(uiHdmi)
				{
					case TCC_HDMI_SUSEPNED:
						HDMI_pause = 1;
						break;
					case TCC_HDMI_RESUME:
						HDMI_pause = 0;
						break;
					case TCC_HDMI_VIDEO_START:
						HDMI_video_mode = 1;
						break;
					case TCC_HDMI_VIDEO_END:
						HDMI_video_mode = 0;
						break;
					default:
						break;
				}
 			}
			break;

		case TCC_LCDC_HDMI_SET_SIZE:
			{
				tcc_hdmisize HdmiSize;
				if(copy_from_user((void *)&HdmiSize, (const void *)arg, sizeof(tcc_hdmisize))){
					return -EFAULT;
				}

				HDMI_video_width = HdmiSize.width;
				HDMI_video_height = HdmiSize.height;

				dprintk("%s: TCC_LCDC_HDMI_SET_SIZE -  HDMI_video_width:%d HDMI_video_height:%d   \n", __func__ , HDMI_video_width, HDMI_video_height);
			}
			break;

		case TCC_LCDC_HDMI_GET_SIZE:
			{
				tcc_hdmisize HdmiSize;
				HdmiSize.width = HDMI_video_width;
				HdmiSize.height = HDMI_video_height;
				if(copy_to_user((tcc_hdmisize *)arg, &HdmiSize, sizeof(HdmiSize))){
					return -EFAULT;			
				}

				dprintk("%s: TCC_LCDC_HDMI_GET_SIZE -  HDMI_video_width:%d HDMI_video_height:%d   \n", __func__ , HDMI_video_width, HDMI_video_height);
			}
			break;

		case TCC_LCDC_HDMI_STATUS_SET:
			{
				LCDC_HDMI_MODE HdmiMode;

				if(copy_from_user((void *)&HdmiMode, (const void *)arg, sizeof(LCDC_HDMI_MODE))){
					return -EFAULT;			
				}
				
				if(HdmiMode == LCDC_HDMI_UI_MODE)
				{
					Output_SelectMode = OUTPUT_SELECT_HDMI_ENABLE;
				}
				else if(HdmiMode == LCDC_HDMI_NONE_MODE)
				{
					Output_SelectMode = OUTPUT_SELECT_NONE;
				}
			}
			break;

		case TCC_LCDC_COMPOSITE_CHECK:
			break;
			
		case TCC_LCDC_COMPOSITE_MODE_SET:
			{
				LCDC_COMPOSITE_MODE composite_mode;
				struct tccfb_info *fbi =(struct tccfb_info *) info->par;

				if(copy_from_user((void *)&composite_mode, (const void *)arg, sizeof(LCDC_COMPOSITE_MODE))){
					return -EFAULT;			
				}
				
				if(composite_mode == LCDC_COMPOSITE_UI_MODE)
				{
					Output_SelectMode = OUTPUT_SELECT_COMPOSITE_ENABLE;

					#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
						TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, TCC_OUTPUT_COMPOSITE);
						TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_COMPOSITE);					
					#endif /*CONFIG_TCC_HDMI_UI_DISPLAY_OFF*/
				}
				else if(composite_mode == LCDC_COMPOSITE_NONE_MODE)
				{
					Output_SelectMode = OUTPUT_SELECT_NONE;
				}
			}
			break;

		case TCC_LCDC_COMPONENT_CHECK:
			break;
			
		case TCC_LCDC_COMPONENT_MODE_SET:
			{
				LCDC_COMPONENT_MODE component_mode;
				struct tccfb_info *fbi = (struct tccfb_info *)info->par;

				if(copy_from_user((void *)&component_mode, (const void *)arg, sizeof(LCDC_COMPONENT_MODE))){
					return -EFAULT;			
				}
				
				if(component_mode == LCDC_COMPONENT_UI_MODE)
				{
					Output_SelectMode = OUTPUT_SELECT_COMPONENT_ENABLE;
					
					#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
						TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, TCC_OUTPUT_COMPONENT);					
						TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_COMPONENT);
					#endif /*CONFIG_TCC_HDMI_UI_DISPLAY_OFF*/
				}
				else if(component_mode == LCDC_COMPONENT_NONE_MODE)
				{
					Output_SelectMode = OUTPUT_SELECT_NONE;
				}
			}
			break;

		case TCC_LCDC_OUTPUT_MODE_CHECK:
			{
				unsigned int uiOutputSelectMode;

				switch(Output_SelectMode)
				{
					case OUTPUT_SELECT_NONE:
						uiOutputSelectMode = OUTPUT_NONE;
						break;
					case OUTPUT_SELECT_HDMI_ENABLE:
					case OUTPUT_SELECT_HDMI_TIMING_SET:
						uiOutputSelectMode = OUTPUT_HDMI;			
						break;
					case OUTPUT_SELECT_COMPOSITE_ENABLE:
						uiOutputSelectMode = OUTPUT_COMPOSITE;			
						break;
					case OUTPUT_SELECT_COMPONENT_ENABLE:
						uiOutputSelectMode = OUTPUT_COMPONENT;			
						break;
				}

				if (put_user(uiOutputSelectMode, (unsigned int __user*)arg))
					return -EFAULT;
			}
			break;

		case TCC_LCDC_OUTPUT_MODE_SET:
			{
				unsigned int uiOutputSelectMode;

				if (copy_from_user((void *)&uiOutputSelectMode, (const void *)arg, sizeof(unsigned int)))
					return -EFAULT;

				switch(uiOutputSelectMode)
				{
					case OUTPUT_NONE:
						Output_SelectMode = OUTPUT_SELECT_NONE;
						break;
					case OUTPUT_HDMI:
						Output_SelectMode = OUTPUT_SELECT_HDMI_ENABLE;			
						break;
					case OUTPUT_COMPOSITE:
						Output_SelectMode = OUTPUT_SELECT_COMPOSITE_ENABLE;			
						break;
					case OUTPUT_COMPONENT:
						Output_SelectMode = OUTPUT_SELECT_COMPONENT_ENABLE;			
						break;
				}
			}
			break;

		case TCC_LCDC_OUTPUT_HDMI_CHECK:
			{
				unsigned int uiOutputSelectMode;

				switch(Output_SelectMode)
				{
					case OUTPUT_SELECT_NONE:
						uiOutputSelectMode = OUTPUT_NONE;
						break;
					case OUTPUT_SELECT_HDMI_ENABLE:
					case OUTPUT_SELECT_HDMI_TIMING_SET:
						uiOutputSelectMode = OUTPUT_HDMI;			
						break;
					case OUTPUT_SELECT_COMPOSITE_ENABLE:
						uiOutputSelectMode = OUTPUT_COMPOSITE;			
						break;
					case OUTPUT_SELECT_COMPONENT_ENABLE:
						uiOutputSelectMode = OUTPUT_COMPONENT;			
						break;
				}

				if((uiOutputSelectMode == OUTPUT_HDMI) || (uiOutputSelectMode == OUTPUT_NONE))
				{
					if(TCC_HDMI_Cable_Detect() == false)
 			 			uiOutputSelectMode = OUTPUT_COMPOSITE;
					else
 			 			uiOutputSelectMode = OUTPUT_HDMI;
  			 	}
				else
				{
					if(TCC_HDMI_Cable_Detect() == true)
 			 			uiOutputSelectMode = OUTPUT_HDMI;
					else
 			 			uiOutputSelectMode = OUTPUT_COMPOSITE;
  				}

				if (put_user(uiOutputSelectMode, (unsigned int __user*)arg))
					return -EFAULT;
			}
			break;
			
        case TCC_LCDC_HDMI_SET_SETTING:
	        {
	        	unsigned int val;
				dprintk("TCC_LCDC_HDMI_SET_SETTING\n");

				if (get_user(val, (unsigned int __user *) arg))
				return -EFAULT;

				hdmi_set_setting_flag(val);

				hdmi_get_setting_flag(&val);
				
				printk("gHdmiSettingFlag = %d\n", val);
	        }
			break;

        case TCC_LCDC_HDMI_GET_SETTING:
	        {
	        	unsigned int val;
				printk(KERN_INFO "HDMI: ioctl(HDMI_IOC_GET_HDMI_SETTING)\n");
				hdmi_get_setting_flag(&val);
				put_user(val,(unsigned int __user*)arg);

				printk("gHdmiSettingFlag = %d\n", val);
	        }
			break;

		case TCC_LCDC_SET_ENABLE:
			{
				char lcdc_num;

				if (copy_from_user((void *)&lcdc_num, (const void *)arg, sizeof(char))){
					return -EFAULT;			
				}
				//tcc_LCDC_enable(lcdc_num, 1);
				tcc_LCDC_ctrl_layer(1, 2, 1);
			}
			break;
			
		case TCC_LCDC_SET_DISABLE:
			{
				char lcdc_num;

				if (copy_from_user((void *)&lcdc_num, (const void *)arg, sizeof(char))){
					return -EFAULT;			
				}
				//tcc_LCDC_enable(lcdc_num, 0);
				tcc_LCDC_ctrl_layer(1, 2, 0);
			}
			break;
			
		case TCC_LCDC_LAYER_ENABLE:
			{
				lcdc_layerctrl_params layer_ctrl;
				if (copy_from_user((void *)&layer_ctrl, (const void *)arg, sizeof(lcdc_layerctrl_params))){
					return -EFAULT;			
				}
				tcc_LCDC_ctrl_layer(layer_ctrl.lcdc_num, layer_ctrl.layer_num, 1);
			}
			break;
			
		case TCC_LCDC_LAYER_DISABLE:
			{
				lcdc_layerctrl_params layer_ctrl;
				if (copy_from_user((void *)&layer_ctrl, (const void *)arg, sizeof(lcdc_layerctrl_params))){
					return -EFAULT;			
				}
				tcc_LCDC_ctrl_layer(layer_ctrl.lcdc_num, layer_ctrl.layer_num, 0);
			}
			break;
			
		case TCC_LCDC_SET_BGCOLOR:
			{
				lcdc_bgcolor_params lcdc_bgclor;
				if (copy_from_user((void *)&lcdc_bgclor, (const void *)arg, sizeof(lcdc_bgcolor_params))){
					return -EFAULT;			
				}
				tcc_LCDC_set_bgcolor(lcdc_bgclor.lcdc_num, lcdc_bgclor.red, lcdc_bgclor.green, lcdc_bgclor.blue);
			}
			break;
			
		case TCC_LCDC_SET_ALPHA:
			{
				lcdc_alpha_params lcdc_alpha;
				if (copy_from_user((void *)&lcdc_alpha, (const void *)arg, sizeof(lcdc_alpha_params))){
					return -EFAULT;			
				}
				tcc_LCDC_set_alpha(lcdc_alpha.lcdc_num, lcdc_alpha.layer_num, lcdc_alpha.layer_alpha);
			}
			break;
			
		default:
			dprintk("ioctl: Unknown [%d/0x%X]", cmd, cmd);
			break;			
	}


	return 0;
}

static void schedule_palette_update(struct tccfb_info *fbi,
				    unsigned int regno, unsigned int val)
{
	unsigned long flags;

	local_irq_save(flags);

	local_irq_restore(flags);
}

/* from pxafb.c */
static inline unsigned int chan_to_field(unsigned int chan, struct fb_bitfield *bf)
{
	chan &= 0xffff;
	chan >>= 16 - bf->length;
	return chan << bf->offset;
}

static int tccfb_setcolreg(unsigned regno,
			       unsigned red, unsigned green, unsigned blue,
			       unsigned transp, struct fb_info *info)
{
	struct tccfb_info *fbi = info->par;
	unsigned int val;

	/* dprintk("setcol: regno=%d, rgb=%d,%d,%d\n", regno, red, green, blue); */

	switch (fbi->fb->fix.visual) {
	case FB_VISUAL_TRUECOLOR:
		/* true-colour, use pseuo-palette */

		if (regno < 16) {
			u32 *pal = fbi->fb->pseudo_palette;

			val  = chan_to_field(red,   &fbi->fb->var.red);
			val |= chan_to_field(green, &fbi->fb->var.green);
			val |= chan_to_field(blue,  &fbi->fb->var.blue);

			pal[regno] = val;
		}
		break;

	case FB_VISUAL_PSEUDOCOLOR:
		if (regno < 256) {
			/* currently assume RGB 5-6-5 mode */

			val  = ((red   >>  0) & 0xf800);
			val |= ((green >>  5) & 0x07e0);
			val |= ((blue  >> 11) & 0x001f);

			//writel(val, S3C2410_TFTPAL(regno));
			schedule_palette_update(fbi, regno, val);
		}

		break;

	default:
		return 1;   /* unknown type */
	}

	return 0;
}


/**
 *      tccfb_blank
 *	@blank_mode: the blank mode we want.
 *	@info: frame buffer structure that represents a single frame buffer
 *
 *	Blank the screen if blank_mode != 0, else unblank. Return 0 if
 *	blanking succeeded, != 0 if un-/blanking failed due to e.g. a
 *	video mode which doesn't support it. Implements VESA suspend
 *	and powerdown modes on hardware that supports disabling hsync/vsync:
 *	blank_mode == 2: suspend vsync
 *	blank_mode == 3: suspend hsync
 *	blank_mode == 4: powerdown
 *
 *	Returns negative errno on error, or zero on success.
 *
 */
static int tccfb_blank(int blank_mode, struct fb_info *info)
{
	dprintk("blank(mode=%d, info=%p)\n", blank_mode, info);

	return 0;
}


#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc92xx_fb_early_suspend(struct early_suspend *h)
{
	printk("%s: fb_lcdc_num:%d ", __FUNCTION__, Fb_Lcdc_num);

	if(Output_SelectMode == OUTPUT_SELECT_HDMI_ENABLE || Output_SelectMode == OUTPUT_SELECT_HDMI_TIMING_SET)
	{
		TCC_HDMI_OnOff(FALSE);
	}

	fb_power_state = 0;

  	tca_bkl_powerdown();

	clk_disable(ddi_cache);

#ifdef CONFIG_LCD_CPU_INTERFACE
	clk_disable(lcd_si);
#endif//
	if(Fb_Lcdc_num)
		clk_disable(lcdc1_clk);
	else
		clk_disable(lcdc0_clk);

#ifdef FB_M2M_COPY
	clk_disable(scaler0_clk);
#endif//
	

printk(" clock off");

	tcc_cpufreq_set_limit_table(&stFBClockLimitTable, TCC_FREQ_LIMIT_FB, 0);
	fb_updating = 0;
	
	printk("%s end lcdc_num:%d \n ", __FUNCTION__, Fb_Lcdc_num);

}

static void tcc92xx_fb_late_resume(struct early_suspend *h)
{
	printk("tcc92fb %s: ", __FUNCTION__);

	tcc_cpufreq_set_limit_table(&stFBClockLimitTable, TCC_FREQ_LIMIT_FB, 1);

	if(Fb_Lcdc_num)
		clk_enable(lcdc1_clk);
	else
		clk_enable(lcdc0_clk);
	
	clk_enable(ddi_cache);

	#ifdef FB_M2M_COPY
	clk_enable(scaler0_clk);
	#endif//
	
	tcc_ddi_cache_setting();
#ifdef CONFIG_LCD_CPU_INTERFACE
	clk_enable(lcd_si);
#endif//

  	tca_bkl_powerup(last_framebufferaddr);
	
	printk("~ clock");
	
	#ifndef LOCK_SCREEN
	if(backlight_level != tca_bkl_getpowerval())
		tca_bkl_setpowerval(backlight_level);
	#endif//

	fb_power_state = 1;
	
	printk(" end \n ");
}
#endif



static struct fb_ops tccfb_ops = {
	.owner			= THIS_MODULE,
	.fb_check_var	= tccfb_check_var,
	.fb_set_par		= tccfb_set_par,
	.fb_blank		= tccfb_blank,
	.fb_setcolreg	= tccfb_setcolreg,
	.fb_fillrect	= cfb_fillrect,
	.fb_copyarea	= cfb_copyarea,
	.fb_imageblit	= cfb_imageblit,
	.fb_ioctl		= tccfb_ioctl,
	.fb_pan_display = tccfb_pan_display,
};


/*
 * tccfb_map_video_memory():
 *	Allocates the DRAM memory for the frame buffer.  This buffer is
 *	remapped into a non-cached, non-buffered, memory region to
 *	allow palette and pixel writes to occur without flushing the
 *	cache.  Once this area is remapped, all virtual memory
 *	access to the video memory should occur at the new region.
 */
 
static int __init tccfb_map_video_memory(struct tccfb_info *fbi, int plane)
{

	fbi->map_size = PAGE_ALIGN(fbi->fb->fix.smem_len + PAGE_SIZE);

	dprintk("map_video_memory(fbi=%p), map_size:%08x\n", fbi, fbi->map_size);


	if(FB_VIDEO_MEM_SIZE == 0)
	{
		printk("map_video_memory (fbi=%p) kenel memoy, map_size:%08x\n", fbi, fbi->map_size);

		fbi->map_cpu  = dma_alloc_writecombine(fbi->dev, fbi->map_size,
						       &fbi->map_dma, GFP_KERNEL);
	}
	else
	{
		printk("map_video_memory (fbi=%p) used map memoy, map_size:%08x\n", fbi, fbi->map_size);
		fbi->map_dma =  FB_VIDEO_MEM_BASE;
		fbi->map_cpu = ioremap_nocache(fbi->map_dma, fbi->map_size);
	}

	fbi->map_size = fbi->fb->fix.smem_len;

	if (fbi->map_cpu) {
		/* prevent initial garbage on screen */
		dprintk("map_video_memory: clear %p:%08x\n",
			fbi->map_cpu, fbi->map_size);
		memset(fbi->map_cpu, 0x00, fbi->map_size);

		fbi->screen_dma		= fbi->map_dma;
		fbi->fb->screen_base	= fbi->map_cpu;
		fbi->fb->fix.smem_start  = fbi->screen_dma;

		// Set the LCD frame buffer start address
		switch (plane)
		{
			case 2:	// IMG2
				pLCDC_FB_CH->LIBA0 = fbi->map_dma;
				fb_mem_vaddr[plane] = fbi->map_cpu;
				fb_mem_size [plane] = fbi->map_size;
				break;
			case 1:	// IMG1
				pLCDC_FB_CH->LIBA0 = fbi->map_dma;
				fb_mem_vaddr[plane] = fbi->map_cpu;
				fb_mem_size [plane] = fbi->map_size;
				break;
			case 0:	// IMG0
				pLCDC_FB_CH->LIBA0 = fbi->map_dma;
				fb_mem_vaddr[plane] = fbi->map_cpu;
				fb_mem_size [plane] = fbi->map_size;
				break;
		}
		dprintk("map_video_memory: dma=%08x cpu=%p size=%08x\n",
			fbi->map_dma, fbi->map_cpu, fbi->fb->fix.smem_len);
	}

	return fbi->map_cpu ? 0 : -ENOMEM;
}

static inline void tccfb_unmap_video_memory(struct tccfb_info *fbi)
{
	dma_free_writecombine(fbi->dev,fbi->map_size,fbi->map_cpu, fbi->map_dma);

	fb_mem_vaddr[fbi->imgch] = (u_char*)NULL;
	fb_mem_size [fbi->imgch] = (u_int)NULL;
}

static irqreturn_t tcc_lcd_handler(int irq, void *dev_id)
{
	#ifdef FB_M2M_COPY
	unsigned int src_addr = 0;
	struct tccfb_info *fbi = &tccfb_parial.fbi;
	#endif//

 	sprintk("%s lcdc_struct.state:%d \n",__func__, lcdc_struct.state);	

	#ifdef USE_EXT_INTERRUPT
		#ifdef FB_M2M_COPY
		if(tccfb_parial.update)		{
			// check scaler using 
			if(!tccxxx_overlay_use())	{
				tcc_scaler_copy(tccfb_parial.left, tccfb_parial.top, tccfb_parial.eright, tccfb_parial.ebottom, &tccfb_parial.fbi);
				pLCDC_FB_CH->LIBA0 	= Gmap_dma;			
			}
			else	{
				sprintk("%s address:0x%x overaly used \n",__func__, src_addr);

				src_addr = fbi->map_dma + fbi->fb->var.xres * fbi->fb->var.yoffset *(fbi->fb->var.bits_per_pixel / 8);
		
				if(fbi->fb->var.yoffset > fbi->fb->var.yres)	{
					src_addr = PAGE_ALIGN(src_addr);
				}
				pLCDC_FB_CH->LIBA0 	= src_addr;			
			}
			pLCDC_FB_CH->LISR = 0;
			pLCDC_FB_CH->LIS = (fbi->fb->var.yres<< 16) | (fbi->fb->var.xres); //Size
			pLCDC_FB_CH->LIO = fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel / 8);
			pLCDC_FB_CH->LIC |= (1<<28); //  image enable

			tccfb_parial.update = 0;
		}
		#endif//

		tcc_lcdc_interrupt(FALSE, Fb_Lcdc_num);
		if(lcdc_struct.state == 0)	{
			lcdc_struct.state = 1;
			wake_up_interruptible(&lcdc_struct.waitq);
		}
	#else
		if(Fb_pLCDC->LSTATUS & Hw3)
		{
			#ifdef FB_M2M_COPY
			if(tccfb_parial.update)		{
				// check scaler using 
				if(!tccxxx_overlay_use())	{
					tcc_scaler_copy(tccfb_parial.left, tccfb_parial.top, tccfb_parial.eright, tccfb_parial.ebottom, &tccfb_parial.fbi);
					pLCDC_FB_CH->LIBA0 	= Gmap_dma;			
				}
				else	{
					src_addr = fbi->map_dma + fbi->fb->var.xres * fbi->fb->var.yoffset *(fbi->fb->var.bits_per_pixel / 8);
					
					if(fbi->fb->var.yoffset > fbi->fb->var.yres)	{
						src_addr = PAGE_ALIGN(src_addr);
					}
					pLCDC_FB_CH->LIBA0 	= src_addr;			
				}
				pLCDC_FB_CH->LISR = 0;
				pLCDC_FB_CH->LIS = (fbi->fb->var.yres<< 16) | (fbi->fb->var.xres); //Size
				pLCDC_FB_CH->LIO = fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel / 8);
				pLCDC_FB_CH->LIC |= (1<<28); //  image enable

				tccfb_parial.update = 0;
			}
			#endif//

			tcc_lcdc_interrupt(FALSE, Fb_Lcdc_num);

			if(lcdc_struct.state == 0)		{
				lcdc_struct.state = 1;
				wake_up_interruptible(&lcdc_struct.waitq);
			}
		}
	#endif//
	

	return IRQ_HANDLED;
}

static char tccfb_driver_name[]="tccfb";
static int __init tccfb_probe(struct platform_device *pdev)
{
	struct tccfb_info *info;
	struct fb_info *fbinfo;
	int ret = 0, plane = 0;
	unsigned int screen_width, screen_height;

        screen_width      = SCREEN_WIDTH;
        screen_height     = SCREEN_HEIGHT;

#if defined(CONFIG_TCC_HDMI_UI_SIZE_1280_720)
        if(tcc_display_data.resolution == 1)
        {
                screen_width      = 720;
                screen_height     = 576;
        }	
        else if(tcc_display_data.resolution == 2)
        {
                screen_width 	  = 800;
                screen_height 	  = 480;
        }	
#endif

	printk("tcc92xxx fb +probe\n");

	for (plane = 0; plane < CONFIG_FB_TCC_DEVS; plane++)
	{
		fbinfo = framebuffer_alloc(sizeof(struct tccfb_info), &pdev->dev);

		info = fbinfo->par;
		info->fb = fbinfo;
		//platform_set_drvdata(pdev, fbinfo);

		strcpy(fbinfo->fix.id, tccfb_driver_name);

		//info->mach_info		    = pdev->dev.platform_data;

		fbinfo->fix.type			= FB_TYPE_PACKED_PIXELS;
		fbinfo->fix.type_aux		= 0;
		fbinfo->fix.xpanstep		= 0;
		fbinfo->fix.ypanstep		= 1;
		fbinfo->fix.ywrapstep		= 0;
		fbinfo->fix.accel		= FB_ACCEL_NONE;

		fbinfo->var.nonstd		= 0;
		fbinfo->var.activate		= FB_ACTIVATE_NOW;

		fbinfo->var.accel_flags		= 0;
		fbinfo->var.vmode		= FB_VMODE_NONINTERLACED;

		fbinfo->fbops			= &tccfb_ops;
		fbinfo->flags			= FBINFO_FLAG_DEFAULT;

		fbinfo->var.xres		= screen_width;
		fbinfo->var.xres_virtual	= fbinfo->var.xres;
		fbinfo->var.yres		= screen_height;
		
#ifdef TCC_FB_DOUBLE
		fbinfo->var.yres_virtual	= fbinfo->var.yres * 2;
#else
		fbinfo->var.yres_virtual	= fbinfo->var.yres;
#endif//
		fbinfo->var.bits_per_pixel	= default_scn_depth[plane];
		fbinfo->fix.line_length 	= fbinfo->var.xres * fbinfo->var.bits_per_pixel/8;

		tccfb_check_var(&fbinfo->var, fbinfo);

		// the memory size that LCD should occupy
		fbinfo->fix.smem_len = fbinfo->var.xres * fbinfo->var.yres_virtual * SCREEN_DEPTH_MAX / 8;

		info->imgch = plane;

		/* Initialize video memory */
		ret = tccfb_map_video_memory(info, plane);
		if (ret) {
			dprintk( KERN_ERR "Failed to allocate video RAM: %d\n", ret);
			ret = -ENOMEM;
		}

		ret = register_framebuffer(fbinfo);
		if (ret < 0) {
			dprintk(KERN_ERR "Failed to register framebuffer device: %d\n", ret);
			goto free_video_memory;
		}

//		fbinfo->var.reserved[0] = 0x54445055;

		tccfb_set_par(fbinfo);
		
		if (plane == (CONFIG_FB_TCC_DEVS-1))	// top layer
			#ifdef FB_M2M_COPY
			if(FB_VIDEO_MEM_SIZE == 0)
			{
				Gmap_cpu  = dma_alloc_writecombine(info->dev, SCREEN_WIDTH* SCREEN_HEIGHT *4 , &Gmap_dma, GFP_KERNEL);
			}
			else
			{
				Gmap_dma =	FB_VIDEO_MEM_BASE + (SCREEN_WIDTH* SCREEN_HEIGHT * 4 * 2);
				Gmap_cpu = ioremap_nocache(Gmap_dma, SCREEN_WIDTH* SCREEN_HEIGHT * 4);
			}
			#endif//

			if (fb_prepare_logo(fbinfo,	FB_ROTATE_UR)) {
			/* Start display and show logo on boot */
			fb_set_cmap(&fbinfo->cmap, fbinfo);

			dprintk("fb_show_logo\n");
			fb_show_logo(fbinfo, FB_ROTATE_UR);
        }

		printk(KERN_INFO "fb%d: %s frame buffer device\n",	fbinfo->node, fbinfo->fix.id);
	}


    init_waitqueue_head(&lcdc_struct.waitq);
	tcc_lcd_interrupt_reg(Fb_Lcdc_num, TRUE);
	
#ifdef CONFIG_HAS_EARLYSUSPEND
	info->early_suspend.suspend = tcc92xx_fb_early_suspend;
	info->early_suspend.resume 	= tcc92xx_fb_late_resume;
	info->early_suspend.level 	= EARLY_SUSPEND_LEVEL_BLANK_SCREEN;
	register_early_suspend(&info->early_suspend);
#endif

	return 0;

free_video_memory:
	tccfb_unmap_video_memory(info);
	dprintk("TCC92xx fb init failed.\n");
	return ret;
}


/*
 *  Cleanup
 */
static int tccfb_remove(struct platform_device *pdev)
{
	struct fb_info	   *fbinfo = platform_get_drvdata(pdev);
	struct tccfb_info *info = fbinfo->par;

	dprintk(" %s LCDC:%d \n", __func__, Fb_Lcdc_num);

	tcc_lcd_interrupt_reg(Fb_Lcdc_num, FALSE);

	tccfb_unmap_video_memory(info);

	//release_mem_region((unsigned long)S3C24XX_VA_LCD, S3C24XX_SZ_LCD);
	unregister_framebuffer(fbinfo);

	return 0;
}

#ifdef CONFIG_PM

/* suspend and resume support for the lcd controller */

static int tccfb_suspend(struct platform_device *dev, pm_message_t state)
{
	printk("%s:  \n", __FUNCTION__);

	return 0;
}

static int tccfb_resume(struct platform_device *dev)
{
	printk("tcc92fb %s:  \n", __FUNCTION__);

	return 0;
}

static void tccfb_shutdown(struct platform_device *dev)
{
	pm_message_t state = {0};
	tccfb_suspend(dev, state);
}

#else
#define tccfb_suspend	NULL
#define tccfb_resume	NULL
#define tccfb_shutdown	NULL
#endif

static struct platform_driver tccfb_driver = {
	.probe		= tccfb_probe,
	.remove		= tccfb_remove,
	.suspend	= tccfb_suspend,
	.shutdown	= tccfb_shutdown,
	.resume		= tccfb_resume,
	.driver		= {
		.name	= "tccxxx-lcd",
		.owner	= THIS_MODULE,
	},
};

//int __devinit tccfb_init(void)
static int __init tccfb_init(void)
{
    printk(KERN_INFO " %s (built %s %s)\n", __func__, __DATE__, __TIME__);

	fb_power_state = 1;
	fb_updating = 1;
	tcc_cpufreq_set_limit_table(&stFBClockLimitTable, TCC_FREQ_LIMIT_FB, 1);

	lcdc0_clk = clk_get(0, "lcdc0");
	lcdc1_clk = clk_get(0, "lcdc1");
	ddi_cache = clk_get(0, "ddi_cache");
#ifdef CONFIG_LCD_CPU_INTERFACE
	lcd_si = clk_get(0, "lcdsi");
#endif//

#ifdef FB_M2M_COPY
	scaler0_clk = clk_get(0, "m2m0");
	clk_enable(scaler0_clk);
#endif//

	BUG_ON(lcdc0_clk == NULL);
	BUG_ON(lcdc1_clk == NULL);
	BUG_ON(ddi_cache == NULL);
#ifdef CONFIG_LCD_CPU_INTERFACE
	BUG_ON(lcd_si == NULL);
#endif//


#ifdef TCC_LCDC1_USE
	TCC_Set_pLCDC(1);
	clk_enable(lcdc1_clk);
#else
	TCC_Set_pLCDC(0);
	clk_enable(lcdc0_clk);
#endif//	TCC_LCDC1_USE

	clk_enable(ddi_cache);
#ifdef CONFIG_LCD_CPU_INTERFACE
	clk_enable(lcd_si);
#endif//

	// backlight init
	tca_bkl_init(Fb_Lcdc_num);

	tcc_ddi_cache_setting();

	TCC_HDMI_LCDC_init();
	TCC_OUTPUT_LCDC_Init();

	#ifdef CONFIG_TCC_OUTPUT_STARTER
	/* Disable UI screen on LCD */
	BITCLR(pLCDC_FB_CH->LIC, Hw28);
	#endif

	printk(" %s LCDC:%d \n", __func__, Fb_Lcdc_num);

	return platform_driver_register(&tccfb_driver);
}

static void __exit tccfb_cleanup(void)
{
	dprintk(" %s LCDC:%d \n", __func__, Fb_Lcdc_num);

	clk_put(lcdc0_clk);
	platform_driver_unregister(&tccfb_driver);
}


module_init(tccfb_init);
module_exit(tccfb_cleanup);

MODULE_AUTHOR("linux <linux@telechips.com>");
MODULE_DESCRIPTION("Telechips TCC Framebuffer driver");
MODULE_LICENSE("GPL");
