#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif

#include <mach/bsp.h>
#include <mach/tca_ckc.h>

#ifdef CONFIG_I2C
#include <linux/i2c.h>
#endif
#include <linux/spi/spi.h>

#ifdef CONFIG_FB_TCC92XX_WVGA_DX08D11VM0AAA
#include "TCC_LCD_Interface.h"
#include "TCC_LCD_DriverCtrl.h"


#if 1
#define dprintk(msg...)	 { printk( "DX08D11VM0AAA: " msg); }
#else
#define dprintk(msg...)	 
#endif


#define WAIT_FLAG	0xFFFF
#define END_FLAG	0xFFFE

unsigned short gLCDInit_Table[][2]=
{
		{0x002, 0x0000}, 
		{0x003, 0x0000},
		{0x004, 0x0000},
		{0x010, 0x0210}, 
		{0x020, 0x0000},
		{0x021, 0x2831},
		{0x022, 0x003E},
		{0x023, 0x7400},
		{0x024, 0x7400},
		{0x025, 0x6A06},
		{0x026, 0x7400},
		{0x027, 0x1C06},
		{0x028, 0x1C28},
		{0x029, 0x1C4A},
		{0x02A, 0x0666},
		{0x100, 0x0533},
		{0x101, 0x0003},
		{0x102, 0x3700},
#if 1		
		{0x300, 0x7777}, 
		{0x301, 0x0717},
		{0x302, 0xC114},
		{0x303, 0x0071}, 
		{0x304, 0x7110}, 
		{0x305, 0x01D4},
#else
		{0x300, 0x5757}, //0x7777
		{0x301, 0x0537}, //0x0717
		{0x302, 0xC114},
		{0x303, 0x0053}, //0x0071
		{0x304, 0x5130}, //0x7110
		{0x305, 0xC317}, //0x01D4
#endif
		{0x402, 0x0000},
		{0x501, 0xFFFF},
		{0x502, 0xFFFF},
		{0x503, 0xFFFF},
		{0x504, 0x00FF},
		{0x002, 0x0200},
		{WAIT_FLAG, 2},//wait 10ms
		{0x001, 0x0001},
		{WAIT_FLAG, 1},//wait 2ms
		{0x002, 0x8210},
		{WAIT_FLAG, 4},//wait 20ms
		{0x002, 0x8310},
		{WAIT_FLAG, 4},//wait 20ms
		{0x002, 0x0710},
		{WAIT_FLAG, 4},//wait 20ms
		{0x002, 0x1730},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001, 0x0012},
		{0x001, 0x0032},
		{0x023, 0x0000},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001, 0x0033},
		{0x023, 0x7400},
		{END_FLAG, 0x0}
};

unsigned short gLCDOff_Table[][2]=
{
		{0x203, 0x0000},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001 ,0x032},
		{0x023 ,0x7400},
		{WAIT_FLAG, 4},//wait 20ms
		{0x023, 0x0000},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001 ,0x0012},
		{WAIT_FLAG, 4},//wait 20ms
		{0x001 ,0x0000},
		{WAIT_FLAG, 1},//wait 1ms
		{0x002 ,0x0730},
		{0x100 ,0x0000},
		{0x002 ,0x0000},
		{WAIT_FLAG, 1},//wait 1ms
		{0x002 ,0x0001},
		{WAIT_FLAG, 1},//wait 1ms
		{END_FLAG, 0x0}
};


volatile unsigned int gReg = 0x100000;
volatile unsigned int gWriteData = 0x120000;
volatile unsigned int gReadData = 0x130000;



// Unit 100hz
#define LCDC_CLK		(230000)
#define LCD_DIVIDER		2

#define LCD_BUS_WIDTH	24

#define LPW		4	// line pulse width
#define LPC		480	// line pulse count (active horizontal pixel - 1)
#define LSWC	8	// line start wait clock (the number of dummy pixel clock - 1)
#define LEWC	5	// line end wait clock (the number of dummy pixel clock - 1)
#define VDB		0	// Back porch Vsync delay
#define VDF		0 	// front porch of Vsync delay
#define FPW1	4	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC1	800	// frmae line count is the number of lines in each frmae on the screen
#define FSWC1	2	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC1	11	// frame start wait cycle is the number of lines to insert at the begining each frame
#define FPW2	4	// TFT/TV : Frame pulse width is the pulse width of frmae clock
#define FLC2	800	// frmae line count is the number of lines in each frmae on the screen
#define FSWC2	2	// frmae start wait cycle is the number of lines to insert at the end each frame
#define FEWC2 	11	// frame start wait cycle is the number of lines to insert at the begining each frame

#define SYNC_INVET (ID_INVERT_EN|IV_INVERT_EN|IH_INVERT_EN|IP_INVERT_EN)
	

void GPSB_Port_disable(void)
{
	BITCSET(HwGPIOF->GPFN3, HwPORTCFG_GPFN0_MASK | HwPORTCFG_GPFN2_MASK | HwPORTCFG_GPFN3_MASK,
							 HwPORTCFG_GPFN0(0)|HwPORTCFG_GPFN2(0)| HwPORTCFG_GPFN3(0));

	BITCSET(HwGPIOF->GPEN, Hw24|Hw26|Hw27, Hw24|Hw26|Hw27);
	BITCLR(HwGPIOF->GPDAT, Hw24|Hw26|Hw27);
}


#define I2C_BD6083GUL_SLAVE 		(0xEC>>1)
static unsigned short probe_i2c[] = {0, I2C_BD6083GUL_SLAVE, I2C_CLIENT_END};	/* { i2c_num, i2c_addr } */
static unsigned short dummy[] = {I2C_CLIENT_END};

static struct i2c_client_address_data addr_data = {
	.normal_i2c = dummy,
	.probe = probe_i2c,
	.ignore = dummy,
};

struct i2c_client *backlight_i2c_client = NULL;
static struct i2c_driver backlight_i2c_driver;

static int backlight_i2c_probe(struct i2c_adapter *adap, int addr, int kind)
{
	struct i2c_client *i2c;
	
	i2c = kzalloc(sizeof(*i2c), GFP_KERNEL);
	if (!i2c)
		return -ENOMEM;

	dprintk("DX08D11VM %s ,addr:0x%x  \n", __func__, addr);

	strcpy(i2c->name, "LCD_backlight");
	i2c->flags = 0;
	i2c->addr = addr;
	i2c->adapter = adap;
	i2c->driver = &backlight_i2c_driver;

	backlight_i2c_client = i2c;

	return i2c_attach_client(i2c);
	
}
static int backlight_i2c_detach(struct i2c_client *client)
{
	dprintk("DX08D11VM %s ,  \n", __func__);

	i2c_detach_client(client);
	kfree(backlight_i2c_client);
	return 0;
}

static int backlight_i2c_attach(struct i2c_adapter *adap)
{
	dprintk("DX08D11VM %s ,  \n", __func__);

	return i2c_probe(adap, &addr_data, backlight_i2c_probe);
}

static struct i2c_driver backlight_i2c_driver = {
	.driver = {
		.name = "LCD_backlight",
		.owner = THIS_MODULE,
	},
	.id = I2C_DRIVERID_LCD_BACKLIGHT,
	.attach_adapter = backlight_i2c_attach,
	.detach_client = backlight_i2c_detach,
};


typedef struct{
		char reg;
		char value;
}BD6083GUL_CTRL_TYPE;


//N-1) Power ON change to Normal mode
BD6083GUL_CTRL_TYPE BD6083_init[] = {{0x00, 0x01}, {0x01, 0x0E}, {0x03, 0x63},
											{0x09, 0x00},{0x02, 0x01}};

BD6083GUL_CTRL_TYPE BD6083_NormaleToALCD[] = {{0x01, 0x0F}, {0x09, 0x87},	{0x0A, 0x01},
													{0x0B, 0x01}, {0x0D, 0x13},	{0x0E, 0x63}, {0x01, 0x0F} };

BD6083GUL_CTRL_TYPE BD6083_ALCToNormal[] = {{0x01, 0x0E},	{0x09, 0x00},{0x02, 0x01}};

BD6083GUL_CTRL_TYPE BD6083_NormalToDiming[] = {{0x03, 0x04}};

BD6083GUL_CTRL_TYPE BD6083_DimingToSleep[] = {{0x02, 0x00}};

BD6083GUL_CTRL_TYPE BD6083_SleepToNomal[] = {{0x06, 0x63}, {0x02, 0x01}};

BD6083GUL_CTRL_TYPE BD6083_DimingToNomal[] = {{0x03, 0x63}};


static char BD6083_state = 1;
int DX08D11VM0AAA_BL_I2C_Write(BD6083GUL_CTRL_TYPE *BL_ctrl,  unsigned data_bytes)
{
 	int i;
	char i2c_write_buf[2];
	
	for(i = 0; i < data_bytes; i++)
	{
		if(BL_ctrl[i].reg == 0xFF)
			udelay(BL_ctrl[i].value);
		else
		{
			i2c_write_buf[0] = BL_ctrl[i].reg;
			i2c_write_buf[1] = BL_ctrl[i].value;
			i2c_master_send(backlight_i2c_client, i2c_write_buf, 2);
		}

	}
}

void BD6083_BackLight (unsigned int BLlevel)
{
	if(BLlevel)
	{
		if(!BD6083_state)
			DX08D11VM0AAA_BL_I2C_Write(BD6083_init, sizeof(BD6083_init)/sizeof(BD6083GUL_CTRL_TYPE));
		else
		{
			BD6083_NormalToDiming[0].value = (BLlevel>>1);
			DX08D11VM0AAA_BL_I2C_Write(BD6083_NormalToDiming, sizeof(BD6083_NormalToDiming)/sizeof(BD6083GUL_CTRL_TYPE));
		}
		
		BD6083_state = 1;
	}
	else
	{
		BD6083_state = 0;
		DX08D11VM0AAA_BL_I2C_Write(BD6083_DimingToSleep, sizeof(BD6083_DimingToSleep)/sizeof(BD6083GUL_CTRL_TYPE));
	}

}


#ifdef CONFIG_MACH_TCC9200S
//E3 : LCD_BL_EN, C29: LCD_RESET	C30:LCD_DISP  C31:LCD_PWRCTL
#define LCD_PROT_INT() 		BITCSET(HwGPIOC->GPEN, Hw29 | Hw30| Hw31, Hw29 | Hw30| Hw31)//;// 	\
							//BITCSET(HwGPIOC->GPDAT, Hw29 | Hw30| Hw31, 0)

#define LCD_POWER_ON()		BITCSET(HwGPIOC->GPDAT, Hw31, Hw31)
#define LCD_POWER_OFF()		BITCSET(HwGPIOC->GPDAT, Hw31, 0)
#define LCD_DISPLAYON()		BITCSET(HwGPIOC->GPDAT, Hw30, Hw30)
#define LCD_DISPLAYOFF()	BITCSET(HwGPIOC->GPDAT, Hw30, 0)

#define LCD_RESET_H()		BITCSET(HwGPIOC->GPDAT, Hw29, Hw29)
#define LCD_RESET_L()		BITCSET(HwGPIOC->GPDAT, Hw29, 0)

#elif defined(CONFIG_MACH_TCC9200) || defined(CONFIG_MACH_TCC9201) || defined(CONFIG_MACH_TCC8900)

void Power_OnOff(char onoff)
{
}

//  A7 : LCD_bl_en ,  D22 : LCD_RST,   C28:LCD_DISP ,  extension: LCD_PWRCTL
#define LCD_PROT_INT() 		BITCSET(HwGPIOA->GPEN, Hw7, Hw7); 	\
							BITCSET(HwGPIOD->GPEN, Hw22, Hw22); \
							BITCSET(HwGPIOC->GPEN, Hw28 , Hw28); \
					 		BITCSET(HwGPIOA->GPDAT, Hw7, 0);  \
							BITCSET(HwGPIOD->GPDAT, Hw22, 0); \
							BITCSET(HwGPIOC->GPDAT, Hw28 , 0); 

#define LCD_POWER_ON()		Power_OnOff(1)
#define LCD_POWER_OFF()		Power_OnOff(0)
#define LCD_DISPLAYON()		BITCSET(HwGPIOC->GPDAT, Hw28 ,Hw28)
#define LCD_DISPLAYOFF()	BITCSET(HwGPIOC->GPDAT, Hw28, 0)
#define LCD_RESET_H()		BITCSET(HwGPIOD->GPDAT, Hw22 ,Hw22)
#define LCD_RESET_L()		BITCSET(HwGPIOD->GPDAT, Hw22, 0)
#endif//

struct spi_device *tcc_lcd_spi;



void DX08D11VM0AAA_Spi_write(unsigned int Reg, unsigned int Data)
{
	volatile unsigned int gReg = 0x100000;
	volatile unsigned int gWriteData = 0x120000;
	volatile unsigned int gReadData = 0x130000;
	
	unsigned int value;
	
	value = gReg  | Reg;
//	DEV_LCD_SPIO_TxRxData(&value, 0, 1);
	spi_write(tcc_lcd_spi, &value, 1);

	value = gWriteData | Data;
//	DEV_LCD_SPIO_TxRxData(&value, 0, 1);
	spi_write(tcc_lcd_spi, &value, 1);
}




int DX08D11VM0AA_Module_initialize(char LCDC_num)
{

	int i = 0;
	DEV_LCD_SPIO_Config_type lcd_spio; 


	LCD_RESET_H();
	LCD_Driver_udelay(10);

#if 0	
	lcd_spio.clock = 60000;
	lcd_spio.data_length = 24;
	lcd_spio.port = LCD_GPSB_PORT9;

	DEV_LCD_SPIO_Config(&lcd_spio);	
#endif//	

	 /* open */
    tcc_lcd_spi->master->tcc_open(tcc_lcd_spi);

	while(gLCDInit_Table[i][0] != END_FLAG)
	{
		if(gLCDInit_Table[i][0] == WAIT_FLAG)
			msleep(gLCDInit_Table[i][1]);
		else
			DX08D11VM0AAA_Spi_write(gLCDInit_Table[i][0], gLCDInit_Table[i][1]);

		i++;			
	}
    tcc_lcd_spi->master->tcc_close(tcc_lcd_spi);

	LCDC_IO_Set(LCDC_num, LCD_BUS_WIDTH);

	dprintk("DX08D11VM %s \n", __func__);
}


int DX08D11VM0AA_Module_Off(void)
{

	int i = 0;
	DEV_LCD_SPIO_Config_type lcd_spio; 

	LCDC_IO_Disable(0, LCD_BUS_WIDTH);

	 /* open */
    tcc_lcd_spi->master->tcc_open(tcc_lcd_spi);

#if 0
	lcd_spio.clock = 60000;
	lcd_spio.data_length = 24;
	lcd_spio.port = LCD_GPSB_PORT9;
	DEV_LCD_SPIO_Config(&lcd_spio);	dprintk("DX08D11VM %s \n", __func__);
#endif//
	while(gLCDOff_Table[i][0] != END_FLAG)
	{
		if(gLCDOff_Table[i][0] == WAIT_FLAG)
			msleep(gLCDOff_Table[i][1]);
		else
			DX08D11VM0AAA_Spi_write(gLCDOff_Table[i][0], gLCDOff_Table[i][1]);

		i++;			
	}
	
    tcc_lcd_spi->master->tcc_close(tcc_lcd_spi);

	GPSB_Port_disable();

	LCD_DISPLAYOFF();
	
	LCD_Driver_mdelay(10);

	LCD_PROT_INT();
	
	return 1;
}

int DX08D11VM0AA_Module_On(char LCDC_num)
{
	int i = 0;
	DEV_LCD_SPIO_Config_type lcd_spio; 


    /* open */
    tcc_lcd_spi->master->tcc_open(tcc_lcd_spi);

	LCD_RESET_H();
	LCD_Driver_udelay(10);
#if 0
	lcd_spio.clock = 60000;
	lcd_spio.data_length = 24;
	lcd_spio.port = LCD_GPSB_PORT9;
	
	dprintk("DX08D11VM %s \n", __func__);
	
	DEV_LCD_SPIO_Config(&lcd_spio);	
#endif//
	while(gLCDInit_Table[i][0] != END_FLAG)
	{
		if(gLCDInit_Table[i][0] == WAIT_FLAG)
			msleep(gLCDInit_Table[i][1]);
		else
			DX08D11VM0AAA_Spi_write(gLCDInit_Table[i][0], gLCDInit_Table[i][1]);

		i++;			
	}

    tcc_lcd_spi->master->tcc_close(tcc_lcd_spi);

	LCDC_IO_Set(LCDC_num, LCD_BUS_WIDTH);

	LCD_DISPLAYON();
	return 1;
}

void DX08D11VM0AA_Module_Spec(LCD_MODULE_SPEC_TYPE* spec)
{
	spec->lcdc_clk = LCDC_CLK;
	spec->lcdc_clk_div = LCD_DIVIDER;
	spec->lcd_bus_width = LCD_BUS_WIDTH;
	
	spec->lcd_width = 480;
	spec->lcd_height = 800;
	spec->lcd_LPW = LPW;		// line pulse width
	spec->lcd_LPC = LPC;		// line pulse count (active horizontal pixel - 1)
	spec->lcd_LSWC = LSWC;		// line start wait clock (the number of dummy pixel clock - 1)
	spec->lcd_LEWC = LEWC;		// line end wait clock (the number of dummy pixel clock - 1)
	spec->lcd_VDB = VDB;		// Back porch Vsync delay
	spec->lcd_VDF = VDF;		// front porch of Vsync delay

	spec->lcd_FPW1 = FPW1;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC1 = FLC1;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC1 = FSWC1;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC1 = FEWC1;		// frame start wait cycle is the number of lines to insert at the begining each frame

	spec->lcd_FPW2 = FPW2;		// TFT/TV : Frame pulse width is the pulse width of frmae clock
	spec->lcd_FLC2 = FLC2;		// frmae line count is the number of lines in each frmae on the screen
	spec->lcd_FSWC2 = FSWC2;		// frmae start wait cycle is the number of lines to insert at the end each frame
	spec->lcd_FEWC2 = FEWC2;		// frame start wait cycle is the number of lines to insert at the begining each frame
	spec->SyncSignal_Invert = SYNC_INVET;
}



static int tcc_lcd_spi_probe(struct spi_device *spi)
{
	int ret = 0;

	spi->bits_per_word = 24;
	spi->master->bus_num = 9;
	spi->max_speed_hz = 6000000;
	spi->mode = SPI_MODE_2 | SPI_3WIRE;		
	
	ret = spi_setup(spi);
    printk("%s id [%d] ret:%d \n",__func__ ,spi->master->bus_num, ret);

	tcc_lcd_spi = kzalloc(sizeof(struct spi_device), GFP_KERNEL);
	if (!tcc_lcd_spi) {
		dev_err(&spi->dev, "failed to allocate memory\n");
		return -ENOMEM;
	}
	tcc_lcd_spi = spi;
	
    printk("%s id [%d]\n",__func__, spi->master->bus_num);

    return 0;
}

static int __devexit tcc_lcd_spi_remove(struct spi_device *spi)
{
	dprintk("~~~~ %s \n ", __func__);
	kfree(tcc_lcd_spi);
	return 0;
}


static struct spi_driver lcd_spi_driver = {
	.driver = {
		.name		= "lcdspi",
		.owner		= THIS_MODULE,
	},
	.probe		= tcc_lcd_spi_probe,
	.remove		= __devexit_p(tcc_lcd_spi_remove),
};


int DX08D11VM0AA_Module_probe(void)
{
    int ret;

	i2c_add_driver(&backlight_i2c_driver);
	
    ret = spi_register_driver(&lcd_spi_driver);

	dprintk("DX08D11VM %s  %d \n", __func__, ret);

	return 0;
}


void DX08D11VM0AAA_Module_func(LCD_MODULE_FUNC_TYPE *func)
{
	func->Lcd_Module_probe = DX08D11VM0AA_Module_probe;
	func->Lcd_Initailize = DX08D11VM0AA_Module_initialize;
	func->Lcd_on = DX08D11VM0AA_Module_On;
	func->Lcd_off = DX08D11VM0AA_Module_Off;
	func->Backligth_ctrl = BD6083_BackLight;
	func->Lcd_GetModuleSpec = DX08D11VM0AA_Module_Spec;
}

EXPORT_SYMBOL(DX08D11VM0AAA_Module_func);

#endif//FB_TCC92XX_WVGA_DX08D11VM0AAA
