/*
 * linux/drivers/video/tcc93xxfb.c
 *
 * Based on:    Based on s3c2410fb.c, sa1100fb.c and others
 * Author:  <linux@telechips.com>
 * Created: June 10, 2008
 * Description: TCC LCD Controller Frame Buffer Driver
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/errno.h>
#include <linux/string.h>
#include <linux/mm.h>
#include <linux/slab.h>
#include <linux/delay.h>
#include <linux/fb.h>
#include <linux/init.h>
#include <linux/dma-mapping.h>
#include <linux/interrupt.h>
#include <linux/workqueue.h>
#include <linux/wait.h>
#include <linux/platform_device.h>
#include <linux/clk.h>
#include <linux/cpufreq.h>

#include <asm/io.h>
#include <asm/uaccess.h>
#include <asm/div64.h>
#include <asm/mach/map.h>
#ifdef CONFIG_PM
#include <linux/pm.h>
#endif
//#include <mach/tcc_used_mem.h>
#include <asm/tlbflush.h>
#include <asm/cacheflush.h>

#include <mach/globals.h>
#include <mach/reg_physical.h>
#include <mach/tca_ckc.h>
#ifdef CONFIG_HAS_EARLYSUSPEND
#include <linux/earlysuspend.h>
#endif
#include "tcc93xxfb.h"
#include <mach/tcc_scaler_ctrl.h>

#include <mach/tcc92xxfb_ioctrl.h>
#include <mach/tcc_composite_ioctl.h>
#include <mach/tcc_component_ioctl.h>

#include <mach/tca_lcdc.h>
#include <mach/tca_backlight.h>
#include <mach/tca_fb_hdmi.h>
#include <mach/tca_fb_output.h>

extern int tca_fb_pan_display(struct fb_var_screeninfo *var, struct fb_info *info);
extern void tca_fb_pan_display_sync(void);

extern void tca_fb_activate_var(struct tccfb_info *fbi,  struct fb_var_screeninfo *var);
#ifdef CONFIG_HAS_EARLYSUSPEND
extern void tca_fb_early_suspend(struct early_suspend *h);
extern void tca_fb_earlier_suspend(struct early_suspend *h);
extern void tca_fb_late_resume(struct early_suspend *h);
extern void tca_fb_later_resume(struct early_suspend *h);
#endif
extern int tca_fb_suspend(struct platform_device *dev, pm_message_t state);
extern int tca_fb_resume(struct platform_device *dev);
extern irqreturn_t tca_fb_lcdc_handler(void);
extern int tca_fb_init(void);
extern void tca_fb_cleanup(void);

extern int hdmi_set_setting_flag(unsigned int hdmi_setting);
extern int hdmi_get_setting_flag(unsigned int *hdmi_setting);

/* Debugging stuff */
static int debug = 0;
#define dprintk(msg...)	if (debug) { printk( "tccfb: " msg); }


//#define POST_SET

#define CONFIG_FB_TCC_DEVS_MAX	3	// do not change!
#define CONFIG_FB_TCC_DEVS		1

#if (CONFIG_FB_TCC_DEVS > CONFIG_FB_TCC_DEVS_MAX)
	#undef CONFIG_FB_TCC_DEVS
	#define CONFIG_FB_TCC_DEVS	CONFIG_FB_TCC_DEVS_MAX
#endif


#define SCREEN_DEPTH_MAX	32	// 32 or 16 
//								 : 32 - 32bpp(alpah+rgb888)
//								 : 16 - 16bpp(rgb565)


const unsigned int default_scn_depth[CONFIG_FB_TCC_DEVS_MAX] = 
{
/* fb0, Layer0: bottom */  (16), // 32 or 16
/* fb1, Layer1: middle */  (16), //  "
/* fb2, Layer2: top    */  (16)  //  "
};




static char HDMI_pause;
static char HDMI_video_mode = 0;

TCC_OUTPUT_TYPE	Output_SelectMode =  TCC_OUTPUT_NONE;
static unsigned int Output_BaseAddr;

static unsigned int HDMI_video_width = 0;
static unsigned int HDMI_video_height = 0;



char fb_power_state;
static unsigned int backlight_level;

// for frame buffer clear
static u_char *fb_mem_vaddr[CONFIG_FB_TCC_DEVS_MAX]= {0,};
static u_int   fb_mem_size [CONFIG_FB_TCC_DEVS_MAX]= {0,};


void tcc92xxfb_hdmi_starter(char hdmi_lcdc_num, struct lcdc_timimg_parms_t *lcdc_timing)
{
	PLCDC pLCDC0, pLCDC1;
	PLCDC pHDMI_LCDC, pLCD_LCDC;
	struct clk *lcdc_clk;
	
	dprintk("%s, lcdc_num=%d\n", __func__, hdmi_lcdc_num);

	if(hdmi_lcdc_num)
	{
		pHDMI_LCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);
		pLCD_LCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);

		lcdc_clk = clk_get(0, "lcdc1");
		clk_enable(lcdc_clk);
	}
	else
	{
		pHDMI_LCDC = (volatile PLCDC)tcc_p2v(HwLCDC0_BASE);
		pLCD_LCDC = (volatile PLCDC)tcc_p2v(HwLCDC1_BASE);

		lcdc_clk = clk_get(0, "lcdc0");
		clk_enable(lcdc_clk);
	}

	TCC_OUTPUT_LCDC_OnOff(TCC_OUTPUT_HDMI, hdmi_lcdc_num, 1);

	tcc_ddi_cache_setting();
	TCC_HDMI_LCDC_Timing(lcdc_timing);
	
	if(lcdc_timing->dp == 1)
		pHDMI_LCDC->LI2S = ((lcdc_timing->flc+1)<<16) | ((lcdc_timing->lpc+1)>>1);
	else
		pHDMI_LCDC->LI2S = ((lcdc_timing->flc+1)<<16) | (lcdc_timing->lpc+1);
	
	#if 0 //TBD
	pHDMI_LCDC->LI2BA0 = pLCDC1->LI2BA0;
	pHDMI_LCDC->LI2O = pLCDC1->LI2O;
	#endif
	
	#ifdef CONFIG_TCC_OUTPUT_STARTER
		pHDMI_LCDC->LI2C = HwLIC_IEN | HwLIC_UPD | TCC_LCDC_IMG_FMT_RGB565;
	#else
		pHDMI_LCDC->LI2C = HwLIC_UPD | pLCD_LCDC->LI2C;
	#endif
	
	Output_SelectMode = TCC_OUTPUT_HDMI;
}

static irqreturn_t tcc_lcd_handler(int irq, void *dev_id)
{
	tca_fb_lcdc_handler();
	return IRQ_HANDLED;
}


void tcc_lcd_interrupt_reg(char SetClear)
{

	dprintk("%s  %d\n",__func__, SetClear);

	if(SetClear)
	{
		#if 0
		if(!lcdc){
			tca_lcdc_interrupt_onoff(0, 1);
			
			request_irq(IRQ_LCD0, tcc_lcd_handler,	IRQF_SHARED,
						"TCC_LCD",tcc_lcd_handler);
		}
		else
		#endif//
		{
			tca_lcdc_interrupt_onoff(0, 0);
			request_irq(IRQ_LCD1, tcc_lcd_handler,	IRQF_SHARED,
					"TCC_LCD",	tcc_lcd_handler);
		}
	}
	else
	{
		#if 0	
		if(!lcdc)
			free_irq(IRQ_LCD0, tcc_lcd_handler);
		else
		#endif//
			free_irq(IRQ_LCD1, tcc_lcd_handler);
	}
}



static int tccfb_check_var(struct fb_var_screeninfo *var,
			       struct fb_info *info)
{
	/* validate bpp */
	if (var->bits_per_pixel > 32)
		var->bits_per_pixel = 32;
	else if (var->bits_per_pixel < 16)
		var->bits_per_pixel = 16;
	
	/* set r/g/b positions */
	if (var->bits_per_pixel == 16) {
		var->red.offset 	= 11;
		var->green.offset	= 5;
		var->blue.offset	= 0;
		var->red.length 	= 5;
		var->green.length	= 6;
		var->blue.length	= 5;
		var->transp.length	= 0;
	} else if (var->bits_per_pixel == 32) {
		var->red.offset 	= 16;
		var->green.offset	= 8;
		var->blue.offset	= 0;
		var->transp.offset	= 24;
		var->red.length 	= 8;
		var->green.length	= 8;
		var->blue.length	= 8;
		var->transp.length	= 8;
	} else {
		var->red.length 	= var->bits_per_pixel;
		var->red.offset 	= 0;
		var->green.length	= var->bits_per_pixel;
		var->green.offset	= 0;
		var->blue.length	= var->bits_per_pixel;
		var->blue.offset	= 0;
		var->transp.length	= 0;
	}
	if (var->yres_virtual < var->yoffset + var->yres)
		var->yres_virtual = var->yoffset + var->yres;

	return 0;
}




/* tccfb_pan_display
 *
 * pandisplay (set) the controller from the given framebuffer
 * information
*/
static int tccfb_pan_display(struct fb_var_screeninfo *var, struct fb_info *info)
{
	char tcc_output_ret =0;
	unsigned int base_addr = 0;
	struct tccfb_info *fbi =(struct tccfb_info *) info->par;

 	base_addr = Output_BaseAddr = fbi->map_dma +fbi->fb->var.xres * (fbi->fb->var.bits_per_pixel/8) * var->yoffset;
	if(var->yoffset > var->yres)	{
		base_addr = Output_BaseAddr = PAGE_ALIGN(base_addr);
	}

#ifdef CONFIG_TCC_OUTPUT_STARTER
	TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, base_addr, TCC_OUTPUT_COMPOSITE);
	TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_COMPOSITE);
	TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, base_addr, TCC_OUTPUT_COMPONENT);
	TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_COMPONENT);
	TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, base_addr, TCC_OUTPUT_HDMI);
	TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_HDMI);
#else
	tca_fb_pan_display(var, info);

	#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
	if(Output_SelectMode)
		tcc_output_ret = TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, base_addr, Output_SelectMode);
	#endif//CONFIG_TCC_HDMI_UI_DISPLAY_OFF

	tca_fb_pan_display_sync();

	#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
	if(Output_SelectMode && tcc_output_ret )
		TCC_OUTPUT_FB_UpdateSync(Output_SelectMode);
	#endif//CONFIG_TCC_HDMI_UI_DISPLAY_OFF
#endif//CONFIG_TCC_OUTPUT_STARTER
	return 0;
}

/* tccfb_activate_var
 *
 * activate (set) the controller from the given framebuffer
 * information
*/
static void tccfb_activate_var(struct tccfb_info *fbi,  struct fb_var_screeninfo *var)
{
	unsigned int imgch = 0;

	if((0 <= fbi->fb->node) && (fbi->fb->node < CONFIG_FB_TCC_DEVS))
		imgch = fbi->fb->node;
	else
		return;

	tca_fb_activate_var(fbi, var);
}


/*
 *      tccfb_set_par - Optional function. Alters the hardware state.
 *      @info: frame buffer structure that represents a single frame buffer
 *
 */
static int tccfb_set_par(struct fb_info *info)
{
	struct tccfb_info *fbi = info->par;
	struct fb_var_screeninfo *var = &info->var;

//	dprintk("- tccfb_set_par\n");
	
	if (var->bits_per_pixel == 16)
		fbi->fb->fix.visual = FB_VISUAL_TRUECOLOR;
	else if (var->bits_per_pixel == 32)
		fbi->fb->fix.visual = FB_VISUAL_TRUECOLOR;
	else
		fbi->fb->fix.visual = FB_VISUAL_PSEUDOCOLOR;

	fbi->fb->fix.line_length = (var->xres*var->bits_per_pixel)/8;

	#ifndef CONFIG_TCC_OUTPUT_STARTER
	/* activate this new configuration */
   	if( fb_power_state && Output_SelectMode != TCC_OUTPUT_COMPONENT ) 
		tccfb_activate_var(fbi, var);
	#endif

	return 0;
}


static int tccfb_ioctl(struct fb_info *info, unsigned int cmd,unsigned long arg)
{
	unsigned int imgch=0;
	void __user *argp = (void __user *) arg;
	
	unsigned int prev_fmt[CONFIG_FB_TCC_DEVS], prev_pd[CONFIG_FB_TCC_DEVS];
	unsigned int curr_fmt[CONFIG_FB_TCC_DEVS], curr_pd[CONFIG_FB_TCC_DEVS];

	if((0 <= info->node) && (info->node < CONFIG_FB_TCC_DEVS))
	{
		imgch = info->node;
	}
	else	{
		dprintk("ioctl: Error - fix.id[%d]\n", info->node);
		return 0;
	}


	switch(cmd)
	{
		case TCC_FB_FLUSH:
			local_flush_tlb_all();
			flush_cache_all();
			break;

		case TCC_LCD_BL_SET:
			{
				unsigned int bl_val;
				if(copy_from_user((void *)&bl_val, (const void *)arg, sizeof(unsigned long)))
					return -EFAULT;
				
				dprintk("ioctl: TCC_LCD_BL_SET %u %u %d \n", bl_val, backlight_level, fb_power_state);

			#ifndef CONFIG_MACH_TCC9300ST
				backlight_level = bl_val;
				if(fb_power_state)
				tca_bkl_setpowerval(backlight_level);
			#endif
			}
			break;

	
		case TCC_LCDC_HDMI_TIMING:
			{
				struct lcdc_timimg_parms_t lcdc_timing;
				dprintk(" TCC_LCDC_HDMI_TIMING: ");
				if (copy_from_user((void*)&lcdc_timing, (const void*)arg, sizeof(struct lcdc_timimg_parms_t)))
					return -EFAULT;

				TCC_HDMI_LCDC_Timing(&lcdc_timing);	

				Output_SelectMode = TCC_OUTPUT_HDMI;

				#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
				{
					struct tccfb_info *fbi =(struct tccfb_info *) info->par;				
					
					TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, Output_SelectMode);
					TCC_OUTPUT_FB_UpdateSync(Output_SelectMode);
				}
				#endif//
			}
			break;
			
		case TCC_LCDC_HDMI_DISPLAY:
			{
				struct tcc_lcdc_image_update ImageInfo;
				if (copy_from_user((void *)&ImageInfo, (const void *)arg, sizeof(struct tcc_lcdc_image_update)))
					return -EFAULT;
		
				if(Output_SelectMode == TCC_OUTPUT_HDMI)
					TCC_HDMI_DISPLAY_UPDATE((struct tcc_lcdc_image_update *)&ImageInfo);
			}
			break;

		case TCC_LCDC_HDMI_START:
			TCC_OUTPUT_LCDC_OnOff(TCC_OUTPUT_HDMI, 0, 1);
			break;
			
		case TCC_LCDC_HDMI_END:
			Output_SelectMode = TCC_OUTPUT_NONE;
			TCC_OUTPUT_LCDC_OnOff(TCC_OUTPUT_HDMI, 0, 0);
			break;

		case TCC_LCDC_HDMI_CHECK:
			{ 
				unsigned int ret_mode = 0; // 0 : UI display to external display device 
				if((!fb_power_state) || HDMI_pause
					#if (SCREEN_WIDTH < SCREEN_HEIGHT)
					|| (!HDMI_video_mode)
					#endif//
					)
					ret_mode = 1;
				
				put_user(ret_mode, (unsigned int __user*)arg);
			}
			break;

		case TCC_LCDC_HDMI_MODE_SET:
 			{
				TCC_HDMI_M uiHdmi;
				
				if(get_user(uiHdmi, (int __user *) arg))
					return -EFAULT;

				dprintk("%s: TCC_LCDC_HDMI_MODE_SET [%d] video_M:%d Output_SelectMode:%d   \n", __func__ , uiHdmi , HDMI_video_mode, Output_SelectMode);

				switch(uiHdmi)
				{
					case TCC_HDMI_SUSEPNED:
						HDMI_pause = 1;
						break;
					case TCC_HDMI_RESUME:
						HDMI_pause = 0;
						break;
					case TCC_HDMI_VIDEO_START:
						HDMI_video_mode = 1;
						break;
					case TCC_HDMI_VIDEO_END:
						HDMI_video_mode = 0;
						break;
					default:
						break;
				}
 			}
			break;

		case TCC_LCDC_HDMI_SET_SIZE:
			{
				tcc_hdmisize HdmiSize;
				if (copy_from_user((void *)&HdmiSize, (const void *)arg, sizeof(tcc_hdmisize)))
					return -EFAULT;

				HDMI_video_width = HdmiSize.width;
				HDMI_video_height = HdmiSize.height;

				dprintk("%s: TCC_LCDC_HDMI_SET_SIZE -  HDMI_video_width:%d HDMI_video_height:%d   \n", __func__ , HDMI_video_width, HDMI_video_height);
			}
			break;

		case TCC_LCDC_HDMI_GET_SIZE:
			{
				tcc_hdmisize HdmiSize;
				HdmiSize.width = HDMI_video_width;
				HdmiSize.height = HDMI_video_height;

				dprintk("%s: TCC_LCDC_HDMI_GET_SIZE -  HDMI_video_width:%d HDMI_video_height:%d   \n", __func__ , HDMI_video_width, HDMI_video_height);
				if (copy_to_user((tcc_hdmisize *)arg, &HdmiSize, sizeof(HdmiSize)))
					return -EFAULT;
			}
			break;

		case TCC_LCDC_COMPOSITE_MODE_SET:
			{
				LCDC_COMPOSITE_MODE composite_mode;
				struct tccfb_info *fbi =(struct tccfb_info *) info->par;

				if (copy_from_user((void *)&composite_mode, (const void *)arg, sizeof(LCDC_COMPOSITE_MODE)))
					return -EFAULT;				
				
				if(composite_mode == LCDC_COMPOSITE_UI_MODE)
				{
					#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
					TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, TCC_OUTPUT_COMPOSITE);
					TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_COMPOSITE);					
					#endif /*CONFIG_TCC_HDMI_UI_DISPLAY_OFF*/

					Output_SelectMode = TCC_OUTPUT_COMPOSITE;

				}
				else if(composite_mode == LCDC_COMPOSITE_NONE_MODE)
				{
					Output_SelectMode = TCC_OUTPUT_NONE;
				}
			}
			break;

		case TCC_LCDC_COMPONENT_MODE_SET:
			{
				LCDC_COMPONENT_MODE component_mode;
				struct tccfb_info *fbi = (struct tccfb_info *)info->par;

				if (copy_from_user((void *)&component_mode, (const void *)arg, sizeof(LCDC_COMPONENT_MODE)))
					return -EFAULT;					
				
				if(component_mode == LCDC_COMPONENT_UI_MODE)
				{
					#if !defined(CONFIG_TCC_HDMI_UI_DISPLAY_OFF)
					TCC_OUTPUT_FB_Update(fbi->fb->var.xres, fbi->fb->var.yres, fbi->fb->var.bits_per_pixel, Output_BaseAddr, TCC_OUTPUT_COMPONENT);
					TCC_OUTPUT_FB_UpdateSync(TCC_OUTPUT_COMPONENT);					
					#endif /*CONFIG_TCC_HDMI_UI_DISPLAY_OFF*/

					Output_SelectMode = TCC_OUTPUT_COMPONENT;

				}
				else if(component_mode == LCDC_COMPONENT_NONE_MODE)
				{
					Output_SelectMode = TCC_OUTPUT_NONE;
				}
			}
			break;

		case TCC_LCDC_HDMI_SET_SETTING:
	        {
	        	unsigned int val;
				dprintk("TCC_LCDC_HDMI_SET_SETTING\n");

				if (get_user(val, (unsigned int __user *) arg))
				return -EFAULT;

				hdmi_set_setting_flag(val);

				hdmi_get_setting_flag(&val);
				
				printk("gHdmiSettingFlag = %d\n", val);
	        }
			break;

        case TCC_LCDC_HDMI_GET_SETTING:
	        {
	        	unsigned int val;
				printk(KERN_INFO "HDMI: ioctl(HDMI_IOC_GET_HDMI_SETTING)\n");
				hdmi_get_setting_flag(&val);
				put_user(val,(unsigned int __user*)arg);

				printk("gHdmiSettingFlag = %d\n", val);
	        }
			break;

		case TCC_LCDC_OUTPUT_MODE_CHECK:
			{
				unsigned int uiOutputSelectMode;

				uiOutputSelectMode = Output_SelectMode;
				if (put_user(uiOutputSelectMode, (unsigned int __user*)arg))
					return -EFAULT;
			}
			break;
			
		case TCC_LCDC_OUTPUT_MODE_SET:
			{
				unsigned int uiOutputSelectMode;

				if (copy_from_user((void *)&uiOutputSelectMode, (const void *)arg, sizeof(unsigned int)))
					return -EFAULT;

				switch(uiOutputSelectMode)
				{
					case OUTPUT_NONE:
						Output_SelectMode = TCC_OUTPUT_NONE;			
						break;
					case OUTPUT_HDMI:
						Output_SelectMode = TCC_OUTPUT_HDMI;			
						break;
					case OUTPUT_COMPOSITE:
						Output_SelectMode = TCC_OUTPUT_COMPOSITE;			
						break;
					case OUTPUT_COMPONENT:
						Output_SelectMode = TCC_OUTPUT_COMPONENT;			
						break;
				}
			}
			break;

		case TCC_LCDC_OUTPUT_HDMI_CHECK:
			{
				unsigned int uiOutputSelectMode;

				uiOutputSelectMode = TCC_OUTPUT_LCDC_GetOutput();
				if((uiOutputSelectMode == OUTPUT_HDMI) || (uiOutputSelectMode == OUTPUT_NONE))
				{
					if(TCC_HDMI_Cable_Detect() == false)
			 			uiOutputSelectMode = OUTPUT_COMPOSITE;
			 	}
				else
				{
					if(TCC_HDMI_Cable_Detect() == true)
			 			uiOutputSelectMode = OUTPUT_HDMI;
				}

				if (put_user(uiOutputSelectMode, (unsigned int __user*)arg))
					return -EFAULT;
			}
			break;
			
		case TCC_LCDC_SET_ENABLE:
			{
				char lcdc_num;

				if(copy_from_user((void *)&lcdc_num, (const void *)arg, sizeof(char)))
					return -EFAULT;
				tcc_LCDC_ctrl_layer(1, 2, 1);
			}
			break;
			
		case TCC_LCDC_SET_DISABLE:
			{
				char lcdc_num;

				if(copy_from_user((void *)&lcdc_num, (const void *)arg, sizeof(char)))
					return -EFAULT;
				tcc_LCDC_ctrl_layer(1, 2, 0);
			}
			break;
			
		case TCC_LCDC_LAYER_ENABLE:
			{
				lcdc_layerctrl_params layer_ctrl;
				if(copy_from_user((void *)&layer_ctrl, (const void *)arg, sizeof(lcdc_layerctrl_params)))
					return -EFAULT;
				tcc_LCDC_ctrl_layer(layer_ctrl.lcdc_num, layer_ctrl.layer_num, 1);
			}
			break;
			
		case TCC_LCDC_LAYER_DISABLE:
			{
				lcdc_layerctrl_params layer_ctrl;
				if(copy_from_user((void *)&layer_ctrl, (const void *)arg, sizeof(lcdc_layerctrl_params)))
					return -EFAULT;
				tcc_LCDC_ctrl_layer(layer_ctrl.lcdc_num, layer_ctrl.layer_num, 0);
			}
			break;
			
		case TCC_LCDC_SET_BGCOLOR:
			{
				lcdc_bgcolor_params lcdc_bgclor;
				if(copy_from_user((void *)&lcdc_bgclor, (const void *)arg, sizeof(lcdc_bgcolor_params)))
					return -EFAULT;
				tcc_LCDC_set_bgcolor(lcdc_bgclor.lcdc_num, lcdc_bgclor.red, lcdc_bgclor.green, lcdc_bgclor.blue);
			}
			break;
			
		case TCC_LCDC_SET_ALPHA:
			{
				lcdc_alpha_params lcdc_alpha;
				if(copy_from_user((void *)&lcdc_alpha, (const void *)arg, sizeof(lcdc_alpha_params)))
					return -EFAULT;
				tcc_LCDC_set_alpha(lcdc_alpha.lcdc_num, lcdc_alpha.layer_num, lcdc_alpha.layer_alpha);
			}
			break;
			
		default:
			dprintk("ioctl: Unknown [%d/0x%X]", cmd, cmd);
			break;			
	}


	return 0;
}

static void schedule_palette_update(struct tccfb_info *fbi,
				    unsigned int regno, unsigned int val)
{
	unsigned long flags;

	local_irq_save(flags);

	local_irq_restore(flags);
}

/* from pxafb.c */
static inline unsigned int chan_to_field(unsigned int chan, struct fb_bitfield *bf)
{
	chan &= 0xffff;
	chan >>= 16 - bf->length;
	return chan << bf->offset;
}

static int tccfb_setcolreg(unsigned regno,
			       unsigned red, unsigned green, unsigned blue,
			       unsigned transp, struct fb_info *info)
{
	struct tccfb_info *fbi = info->par;
	unsigned int val;

	/* dprintk("setcol: regno=%d, rgb=%d,%d,%d\n", regno, red, green, blue); */

	switch (fbi->fb->fix.visual) {
		case FB_VISUAL_TRUECOLOR:
			/* true-colour, use pseuo-palette */

			if (regno < 16) {
				u32 *pal = fbi->fb->pseudo_palette;

				val  = chan_to_field(red,   &fbi->fb->var.red);
				val |= chan_to_field(green, &fbi->fb->var.green);
				val |= chan_to_field(blue,  &fbi->fb->var.blue);

				pal[regno] = val;
			}
			break;

		case FB_VISUAL_PSEUDOCOLOR:
			if (regno < 256) {
				/* currently assume RGB 5-6-5 mode */

				val  = ((red   >>  0) & 0xf800);
				val |= ((green >>  5) & 0x07e0);
				val |= ((blue  >> 11) & 0x001f);

				//writel(val, S3C2410_TFTPAL(regno));
				schedule_palette_update(fbi, regno, val);
			}
			break;

		default:
			return 1;   /* unknown type */
	}

	return 0;
}


/**
 *      tccfb_blank
 *	@blank_mode: the blank mode we want.
 *	@info: frame buffer structure that represents a single frame buffer
 *
 *	Blank the screen if blank_mode != 0, else unblank. Return 0 if
 *	blanking succeeded, != 0 if un-/blanking failed due to e.g. a
 *	video mode which doesn't support it. Implements VESA suspend
 *	and powerdown modes on hardware that supports disabling hsync/vsync:
 *	blank_mode == 2: suspend vsync
 *	blank_mode == 3: suspend hsync
 *	blank_mode == 4: powerdown
 *
 *	Returns negative errno on error, or zero on success.
 *
 */
static int tccfb_blank(int blank_mode, struct fb_info *info)
{
	dprintk("blank(mode=%d, info=%p)\n", blank_mode, info);
	return 0;
}


#ifdef CONFIG_HAS_EARLYSUSPEND
static void tcc93xx_fb_early_suspend(struct early_suspend *h)
{
	printk("%s: START Fb_Lcdc_num ", __FUNCTION__);

	tca_fb_early_suspend(h);
}

static void tcc92xx_fb_earlier_suspend(struct early_suspend *h)
{
	printk("%s: START Fb_Lcdc_num ", __FUNCTION__);
	fb_power_state = 0;
	tca_fb_earlier_suspend(h);
}
static void tcc93xx_fb_late_resume(struct early_suspend *h)
{
	tca_fb_late_resume(h);
}

static void tcc93xx_fb_later_resume(struct early_suspend *h)
{
	tca_fb_later_resume(h);
	fb_power_state = 1;
}
#endif


#ifdef CONFIG_HAS_EARLYSUSPEND

#endif

#ifdef CONFIG_PM
/* suspend and resume support for the lcd controller */
static int tccfb_suspend(struct platform_device *dev, pm_message_t state)
{
	tca_fb_suspend(dev, state);
	return 0;
}

static int tccfb_resume(struct platform_device *dev)
{
	tca_fb_resume(dev);
	return 0;
}

static void tccfb_shutdown(struct platform_device *dev)
{
	pm_message_t state = {0};
	tccfb_suspend(dev, state);
}

#else
#define tccfb_suspend	NULL
#define tccfb_resume	NULL
#define tccfb_shutdown	NULL
#endif


static struct fb_ops tccfb_ops = {
	.owner			= THIS_MODULE,
	.fb_check_var	= tccfb_check_var,
	.fb_set_par		= tccfb_set_par,
	.fb_blank		= tccfb_blank,
	.fb_setcolreg	= tccfb_setcolreg,
	.fb_fillrect	= cfb_fillrect,
	.fb_copyarea	= cfb_copyarea,
	.fb_imageblit	= cfb_imageblit,
	.fb_ioctl		= tccfb_ioctl,
	.fb_pan_display = tccfb_pan_display,
};


/*
 * tccfb_map_video_memory():
 *	Allocates the DRAM memory for the frame buffer.  This buffer is
 *	remapped into a non-cached, non-buffered, memory region to
 *	allow palette and pixel writes to occur without flushing the
 *	cache.  Once this area is remapped, all virtual memory
 *	access to the video memory should occur at the new region.
 */
 
static int __init tccfb_map_video_memory(struct tccfb_info *fbi, int plane)
{

	fbi->map_size = PAGE_ALIGN(fbi->fb->fix.smem_len + PAGE_SIZE);

	dprintk("map_video_memory(fbi=%p), map_size:%08x\n", fbi, fbi->map_size);

		// LCD Output
	#ifdef CONFIG_TCC_HDMI_UI_SIZE_1280_720
		fbi->map_dma =  FB_VIDEO_MEM_BASE;
		fbi->map_cpu = ioremap_nocache(fbi->map_dma, fbi->map_size);
	#else
		fbi->map_cpu  = dma_alloc_writecombine(fbi->dev, fbi->map_size,
						       &fbi->map_dma, GFP_KERNEL);
	#endif//

	fbi->map_size = fbi->fb->fix.smem_len;

	if (fbi->map_cpu) {
		/* prevent initial garbage on screen */
		dprintk("map_video_memory: clear %p:%08x\n", fbi->map_cpu, fbi->map_size);
		
		memset(fbi->map_cpu, 0x00, fbi->map_size);

		fbi->screen_dma		= fbi->map_dma;
		fbi->fb->screen_base	= fbi->map_cpu;
		fbi->fb->fix.smem_start  = fbi->screen_dma;

		// Set the LCD frame buffer start address
		switch (plane)
		{
			case 2:	// IMG2
				fb_mem_vaddr[plane] = fbi->map_cpu;
				fb_mem_size [plane] = fbi->map_size;
				break;
			case 1:	// IMG1
				fb_mem_vaddr[plane] = fbi->map_cpu;
				fb_mem_size [plane] = fbi->map_size;
				break;
			case 0:	// IMG0
				fb_mem_vaddr[plane] = fbi->map_cpu;
				fb_mem_size [plane] = fbi->map_size;
				break;
		}
		
		dprintk("map_video_memory: dma=%08x cpu=%p size=%08x\n",
					fbi->map_dma, fbi->map_cpu, fbi->fb->fix.smem_len);
	}

	return fbi->map_cpu ? 0 : -ENOMEM;
}

static inline void tccfb_unmap_video_memory(struct tccfb_info *fbi)
{
	dma_free_writecombine(fbi->dev,fbi->map_size,fbi->map_cpu, fbi->map_dma);

	fb_mem_vaddr[fbi->imgch] = (u_char*)NULL;
	fb_mem_size [fbi->imgch] = (u_int)NULL;
}

static char tccfb_driver_name[]="tccfb";
static int __init tccfb_probe(struct platform_device *pdev)
{
	struct tccfb_info *info;
	struct fb_info *fbinfo;
	int ret;
	int plane = 0;
//	unsigned int layer_en = 0;
    unsigned int tmp_value= 0;
	unsigned int screen_width, screen_height;

    screen_width      = SCREEN_WIDTH;
    screen_height     = SCREEN_HEIGHT;


	printk("%s, screen_width=%d, screen_height=%d\n", __func__, screen_width, screen_height);

	for (plane = 0; plane < CONFIG_FB_TCC_DEVS; plane++)
	{
		fbinfo = framebuffer_alloc(sizeof(struct tccfb_info), &pdev->dev);

		info = fbinfo->par;
		info->fb = fbinfo;
		//platform_set_drvdata(pdev, fbinfo);

		strcpy(fbinfo->fix.id, tccfb_driver_name);

		//info->mach_info		    = pdev->dev.platform_data;

		fbinfo->fix.type			= FB_TYPE_PACKED_PIXELS;
		fbinfo->fix.type_aux		= 0;
		fbinfo->fix.xpanstep		= 0;
#ifdef TCC_FB_DOUBLE
			fbinfo->fix.ypanstep	= 1;
#else
		fbinfo->fix.ypanstep		= 0;
#endif
		fbinfo->fix.ywrapstep		= 0;
		fbinfo->fix.accel		= FB_ACCEL_NONE;

		fbinfo->var.nonstd		= 0;
		fbinfo->var.activate		= FB_ACTIVATE_NOW;

		fbinfo->var.accel_flags		= 0;
		fbinfo->var.vmode		= FB_VMODE_NONINTERLACED;

		fbinfo->fbops			= &tccfb_ops;
		fbinfo->flags			= FBINFO_FLAG_DEFAULT;

		fbinfo->var.xres		= screen_width;
		fbinfo->var.xres_virtual	= fbinfo->var.xres;
		fbinfo->var.yres		= screen_height;
		
#ifdef TCC_FB_DOUBLE
		fbinfo->var.yres_virtual	= fbinfo->var.yres * 2;
#else
		fbinfo->var.yres_virtual	= fbinfo->var.yres;
#endif//
		fbinfo->var.bits_per_pixel	= default_scn_depth[plane];
		fbinfo->fix.line_length 	= fbinfo->var.xres * fbinfo->var.bits_per_pixel/8;

		tccfb_check_var(&fbinfo->var, fbinfo);

		// the memory size that LCD should occupy
		fbinfo->fix.smem_len = fbinfo->var.xres * fbinfo->var.yres_virtual * SCREEN_DEPTH_MAX / 8;

		info->imgch = plane;

		/* Initialize video memory */
		ret = tccfb_map_video_memory(info, plane);

		if (ret) {
			dprintk( KERN_ERR "Failed to allocate video RAM: %d\n", ret);
			ret = -ENOMEM;
		}

		ret = register_framebuffer(fbinfo);
		if (ret < 0) {
			dprintk(KERN_ERR "Failed to register framebuffer device: %d\n", ret);
			goto free_video_memory;
		}

//		fbinfo->var.reserved[0] = 0x54445055;

		tccfb_set_par(fbinfo);
		
		if (plane == 0)	// top layer
			if (fb_prepare_logo(fbinfo,	FB_ROTATE_UR)) {
			/* Start display and show logo on boot */
			fb_set_cmap(&fbinfo->cmap, fbinfo);

			dprintk("fb_show_logo\n");
			fb_show_logo(fbinfo, FB_ROTATE_UR);
        }
		printk(KERN_INFO "fb%d: %s frame buffer device\n",	fbinfo->node, fbinfo->fix.id);
	}


	tcc_lcd_interrupt_reg(TRUE);
	
#ifdef CONFIG_HAS_EARLYSUSPEND
	info->early_suspend.suspend = tcc93xx_fb_early_suspend;
	info->early_suspend.resume 	= tcc93xx_fb_late_resume;
	info->early_suspend.level 	= EARLY_SUSPEND_LEVEL_STOP_DRAWING;
	register_early_suspend(&info->early_suspend);
	
	info->earlier_suspend.suspend = tcc92xx_fb_earlier_suspend;
	info->earlier_suspend.resume 	= tcc93xx_fb_later_resume;
	info->earlier_suspend.level 	= EARLY_SUSPEND_LEVEL_BLANK_SCREEN;
	register_early_suspend(&info->earlier_suspend);
#endif

	return 0;

free_video_memory:
	tccfb_unmap_video_memory(info);
	dprintk("TCC93xx fb init failed.\n");
	return ret;
}


/*
 *  Cleanup
 */
static int tccfb_remove(struct platform_device *pdev)
{
	struct fb_info	   *fbinfo = platform_get_drvdata(pdev);
	struct tccfb_info *info = fbinfo->par;

	dprintk(" %s  \n", __func__);

	tcc_lcd_interrupt_reg(FALSE);

	tccfb_unmap_video_memory(info);

	//release_mem_region((unsigned long)S3C24XX_VA_LCD, S3C24XX_SZ_LCD);
	unregister_framebuffer(fbinfo);

	return 0;
}


static struct platform_driver tccfb_driver = {
	.probe		= tccfb_probe,
	.remove		= tccfb_remove,
	.suspend	= tccfb_suspend,
	.shutdown	= tccfb_shutdown,
	.resume		= tccfb_resume,
	.driver		= {
		.name	= "tccxxx-lcd",
		.owner	= THIS_MODULE,
	},
};

//int __devinit tccfb_init(void)
static int __init tccfb_init(void)
{
    printk(KERN_INFO " tcc93xx (built %s %s)\n", __func__, __DATE__, __TIME__);

	fb_power_state = 1;

	tca_fb_init();

	printk(" %s end \n", __func__);

	return platform_driver_register(&tccfb_driver);
}

static void __exit tccfb_cleanup(void)
{
	dprintk(" %s \n", __func__);

	tca_fb_cleanup();
	platform_driver_unregister(&tccfb_driver);
}


module_init(tccfb_init);
module_exit(tccfb_cleanup);

MODULE_AUTHOR("linux <linux@telechips.com>");
MODULE_DESCRIPTION("Telechips TCC Framebuffer driver");
MODULE_LICENSE("GPL");
