/*
 * include/asm-arm/arch-tcc79x/dpm.h
 *
 * Author:  <linux@telechips.com>
 * Created: June 10, 2008
 * Description: DPM for Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 * Copyright (C) 2002-2004, MontaVista Software <source@mvista.com>
 * Copyright (C) 2008-2009 Telechips 
 *
 * Initially based on include/asm-arm/arch-omap/omap_dpm.h
 */

#ifndef __ASM_ARCH_DPM_H__
#define __ASM_ARCH_DPM_H__

#include <asm/io.h>
#include <asm/timex.h>

#if defined(_TCC79x_)
#include <mach/tcc79x.h>
//#include <asm/arch/tcc79x.h>
#elif defined(_TCC83x_)
#include <asm/arch/tcc83x_virt.h>
#endif
#include <asm/plat-tcc/tcc_ckc.h>

/*!
 * machine dependent operating state
 *
 * An operating state is a cpu execution state that has implications for power
 * management. The DPM will select operating points based largely on the
 * current operating state.
 *
 * DPM_STATES is the number of supported operating states. Valid operating
 * states are from 0 to DPM_STATES-1 but when setting an operating state the
 * kernel should only specify a state from the set of "base states" and should
 * do so by name.  During the context switch the new operating state is simply
 * extracted from current->dpm_state.
 *
 * task states:
 *
 * APIs that reference task states use the range -(DPM_TASK_STATE_LIMIT + 1)
 * through +DPM_TASK_STATE_LIMIT.  This value is added to DPM_TASK_STATE to
 * obtain the downward or upward adjusted task state value. The
 * -(DPM_TASK_STATE_LIMIT + 1) value is interpreted specially, and equates to
 * DPM_NO_STATE.
 *
 * Tasks inherit their task operating states across calls to
 * fork(). DPM_TASK_STATE is the default operating state for all tasks, and is
 * inherited from init.  Tasks can change (or have changed) their tasks states
 * using the DPM_SET_TASK_STATE variant of the sys_dpm() system call.
 */
#define DPM_IDLE_TASK_STATE  0
#define DPM_IDLE_STATE       1
#define DPM_SLEEP_STATE      2
#define DPM_BASE_STATES      3

#define DPM_TASK_STATE_LIMIT 4
#define DPM_TASK_STATE       (DPM_BASE_STATES + DPM_TASK_STATE_LIMIT) /* 7 */
#define DPM_STATES           (DPM_TASK_STATE + DPM_TASK_STATE_LIMIT + 1) /* 11 */
#define DPM_TASK_STATES      (DPM_STATES - DPM_BASE_STATES) /* 5 */

#define DPM_STATE_NAMES                  \
{ "idle-task", "idle", "sleep",          \
  "task-4", "task-3", "task-2", "task-1",\
  "task",                                \
  "task+1", "task+2", "task+3", "task+4" \
}


#if defined(_TCC79x_)
#define DPM_PARAM_NAMES				\
{   "sys_cpu", "sys_bus", "peri_lcd", "peri_bl", "peri_i2c", \
    "peri_uart0", "peri_uart1", "peri_uart2", "peri_uart3", \
    "peri_usb", "peri_sdmmc", "peri_cam", "peri_scaler",  \
    /*"peri_adc",*/ "peri_gpsb0", "peri_gpsb1",  \
}

#elif defined(_TCC83x_)
#define DPM_PARAM_NAMES				\
{   "sys_cpu", "sys_bus", "peri_lcd", "peri_bl", "peri_i2c", \
    "peri_uart0","peri_uart1", "peri_uart2", "peri_uart3", \
    "peri_usb", "peri_sdmmc", "peri_cam", "peri_scaler",  \
    "peri_spi0", "peri_spi1",    \
}

#endif


/*!
 * MD operating point parameters
 */
enum {
    DPM_MD_SYS_CPU,     /* 0. core voltage */
    DPM_MD_SYS_BUS,     /* 1. system bus clock */
    DPM_MD_PERI_LCD,    /* 2. lcd */
    DPM_MD_PERI_BACKLIGHT,    /* 3. backlight */
    DPM_MD_PERI_I2C,    /* 4. i2c */
    DPM_MD_PERI_UART0,  /* 5. uart0 */
    DPM_MD_PERI_UART1,  /* 6. uart1*/
    DPM_MD_PERI_UART2,  /* 7. uart2*/
    DPM_MD_PERI_UART3,  /* 8. uart3*/
    DPM_MD_PERI_USB,    /* 9. usb */
    DPM_MD_PERI_SDMMC,  /* 10. sdmmc */
    DPM_MD_PERI_CAM,    /* 11.  cam */
    DPM_MD_PERI_SCALER, /* 12. scaler */
    //DPM_MD_PERI_ADC,    /* 13. adc */
#if defined(_TCC79x_)
    DPM_MD_PERI_GPSB0,     /* 14. gpsb0 */
    DPM_MD_PERI_GPSB1,     /* 15. gpsb1 */
#if 0
    DPM_MD_PERI_GPSB2,     /* gpsb2 */
    DPM_MD_PERI_GPSB3,     /* gpsb3 */
#endif
#elif defined(_TCC83x_)
    DPM_MD_PERI_SPIMS0,    /* spi0 */
    DPM_MD_PERI_SPIMS1,    /* spi1*/
#endif
};

#if defined(_TCC79x_)
#define DPM_PP_NBR 15 
#elif defined(_TCC83x_)
#define DPM_PP_NBR 15
#endif


#ifndef __ASSEMBLER__

#include <linux/types.h>
#include <linux/proc_fs.h>

#define DPM_MD_STATS
typedef __u64 dpm_md_count_t;
typedef __u64 dpm_md_time_t;


#define dpm_time() get_cycles() 
#define dpm_time_to_usec(ticks) ({ \
         unsigned long long quot = (unsigned long long) ticks * 10; \
         do_div(quot, (unsigned long) (1000*2)); \
           quot; })


/*!
 * Instances of this structure define valid Innovator operating points for DPM.
 * Voltages are represented in mV, and frequencies are represented in KHz.
 */ 
/* TODO */
/* Operating Points */
struct dpm_md_opt {
        unsigned int sys_cpu;         /* 0. system cpu clock*/
        unsigned int sys_bus;         /* 1. system bus clock */
        unsigned int peri_lcd;        /* 2. in KHz */
        unsigned int peri_bl;        /* 3. in KHz */
        unsigned int peri_i2c;        /* 4. in KHz */
        unsigned int peri_uart0;        /* 5. in KHz */
        unsigned int peri_uart1;        /* 6. in KHz */
        unsigned int peri_uart2;        /* 7. in KHz */
        unsigned int peri_uart3;        /* 8. in KHz */
        unsigned int peri_usb;        /* 9. in KHz */
        unsigned int peri_sdmmc;        /* 10. in KHz */
        unsigned int peri_cam;        /* 11. in KHz */
        unsigned int peri_scaler;        /* 12. in KHz */
        //unsigned int peri_adc;        /* 13. in KHz */
#if defined(_TCC79x_)
        unsigned int peri_gpsb0;        /* 14. in KHz */
        unsigned int peri_gpsb1;        /* 15. in KHz */
#if 0
        unsigned int peri_gpsb2;        /* in KHz */
        unsigned int peri_gpsb3;        /* in KHz */
#endif
#elif defined(_TCC83x_)
        unsigned int peri_spi0;        /* in KHz */
        unsigned int peri_spi1;        /* in KHz */
#endif
};

#endif /* __ASSEMBLER__ */
#endif /* __ASM_ARCH_DPM_H__ */
