/* sound/oss/tcc7901-oss.c
 *
 * Copyright (c) 2007,Telechips
 *	Mac	Chen, <604sworder@16.com>
 *
 * This	program	is free	software; you can redistribute it and/or modify
 * it under	the	terms of the GNU General Public	License	version	2 as
 * published by	the	Free Software Foundation.
 *
*/


#include <linux/module.h>
#include <linux/sched.h>
#include <linux/init.h>
#include <linux/slab.h>
#include <linux/poll.h>
#include <linux/delay.h>
#include <linux/device.h>
#include <linux/interrupt.h>
#include <linux/sound.h>
#include <linux/soundcard.h>
#include <linux/dma-mapping.h>
#include <linux/platform_device.h>
#include <linux/calc64.h>
#include <linux/types.h>

#include <asm/io.h>
#include <asm/arch/tcc79x.h>

#ifdef CONFIG_I2C
#include <linux/i2c.h>
#else
#include "tcc7901_i2c.h"
#endif
#include "tcc7901_i2s.h"

//#define DEBUG
#ifdef	DEBUG
#define	DPRINTK( x... )	 printk( x )
#else
#define	DPRINTK( x... )
#endif

#define	AUDIO_NAME				"WM8731"
#define	AUDIO_NAME_VERBOSE		"WM8731	audio driver"

#define	MIC_IN
//#define	LINE_IN

// supported audio formats
#define AUDIO_FMT_MASK          (AFMT_S16_LE)
#define AUDIO_FMT_DEFAULT       (AFMT_S16_LE)

#define	AUDIO_CHANNELS_DEFAULT	2
#define	AUDIO_RATE_DEFAULT		44100

#define	AUDIO_NBFRAGS_DEFAULT	32
#define	AUDIO_FRAGSIZE_DEFAULT	2048

#if 0
// To detect earphone jack
#define PERIOD_JACK_DETECT		100
static struct timer_list tmr_jack_detect;
#endif

static struct device *this_dev;

typedef	struct {
	int	size;		/* buffer size */
	char *start;		/* point to	actual buffer */
	dma_addr_t dma_addr;	/* physical	buffer address */
	struct semaphore sem;	/* down	before touching	the	buffer */
	int	master;		/* owner for buffer	allocation,	contain	size when true */
	int	dma_size;
} audio_buf_t;

typedef	struct {
	audio_buf_t	*buffers;	/* pointer to audio	buffer structures */
	audio_buf_t	*buf;	/* current buffer used by read/write */
	u_int buf_idx;		/* index for the pointer above */
	u_int play_idx;     /* the current buffer that playing */
	u_int fragsize;		/* fragment	i.e. buffer	size */
	u_int nbfrags;		/* nbr of fragments	*/
} audio_stream_t;


typedef struct {
	char *addr;
	dma_addr_t dma_addr;
	int  buf_size;	// total size of DMA
	int  page_size;	// size of each page
	int  dma_half;	// 0 or 1, mark the next available buffer
}dma_buf_t;

static dma_buf_t dma_out_buffer, dma_in_buffer;	// DMA buffer

static audio_stream_t output_stream;	/* output */
static audio_stream_t input_stream;		/* input */
static u_int rd_buf_tail;
static u_int wr_buf_tail;
static u_int play_dma_running, record_dma_running;

#define	NEXT_BUF(_s_,_b_) {	\
		(_s_)->_b_##_idx++;	\
		(_s_)->_b_##_idx %=	(_s_)->nbfrags;	\
		(_s_)->_b_ = (_s_)->buffers	+ (_s_)->_b_##_idx;	}


static u_int audio_rate;		// 44100Hz
static int audio_channels;		// 2 channels
static int audio_fmt;			// unsigned 16bit
static u_int audio_fragsize;
static u_int audio_nbfrags;

static int audio_dev_dsp;
static int audio_dev_mixer;

static int audio_rd_refcount;
static int audio_wr_refcount;
#define	audio_active			(audio_rd_refcount | audio_wr_refcount)

static int g_volume;	// SPK volume
static int mic_volume=0xDFDF;	// MIC volume
static int mixer_igain = 0x4;	/* -6db*/

// mark the DMA status
#define DMA_IDLE	0
#define DMA_INPUT	1
#define DMA_OUTPUT	2

#define DMA_CH_OUT	1
#define DMA_CH_IN	2

static void copy_data_to_dma(void);
static void copy_data_from_dma(void);
static void malloc_output_dma_buffer(int buf_size, int page_size);
static void malloc_input_dma_buffer(int buf_size, int page_size);

#ifdef CONFIG_I2C
/*
 * WM8731 2 wire address is determined by GPIO5
 * state during powerup.
 *    low  = 0x1a
 *    high = 0x1b
 */
static unsigned short normal_i2c[] = { 0x1a, I2C_CLIENT_END };
static unsigned short dummy[] = {I2C_CLIENT_END};

static struct i2c_client_address_data addr_data = {
	.normal_i2c = normal_i2c,
	.probe 		= dummy,
	.ignore 	= dummy,
};

static struct i2c_driver wm8731_i2c_driver;
static struct i2c_client *wm8731_i2c_client;

static void CODEC_SEND_CMD(unsigned char reg, unsigned char val)
{
	unsigned char cmd[2];

	/* data is
	 *  D15..D9 WM8731 register offset
	 *  D8...D0 register data
	 */
	cmd[0] = reg;
	cmd[1] = val;

	i2c_master_send(wm8731_i2c_client, cmd, 2);
}
#endif

static void	init_tcc7901_iis_bus_rx(void)
{
#ifdef MIC_IN
	CODEC_SEND_CMD(0x00,0x9f);	// left
	CODEC_SEND_CMD(0x02,0x9f);	// right
	CODEC_SEND_CMD(0x08,0x1d);	// line in select
	CODEC_SEND_CMD(0x0c,0x61);	// powerdown control
#else
	CODEC_SEND_CMD(0x00,0x1f);	// left
	CODEC_SEND_CMD(0x02,0x1f);	// right
	CODEC_SEND_CMD(0x08,0x18);	// line in select
	CODEC_SEND_CMD(0x0c,0x62);	// powerdown control
#endif
}

static void	init_tcc7901_iis_bus_tx(void)
{
	CODEC_SEND_CMD(0x00,0x9f);	// left
	CODEC_SEND_CMD(0x02,0x9f);	// right
	CODEC_SEND_CMD(0x08,0x12);	// line out select
	CODEC_SEND_CMD(0x0c,0x66);	// powerdown control
}

/*
static void codec_channel(int mode)
{
}
*/


static void	audio_clear_buf(audio_stream_t * s)
{
	DPRINTK("audio_clear_buf, frags:%d\n", s->nbfrags);

	if (s->buffers)	{
		int	frag;

		for	(frag =	0; frag	< s->nbfrags; frag++) {
			if (!s->buffers[frag].master)	//master = size
				continue;
			kfree(s->buffers[frag].start);

		}
		kfree(s->buffers);
		s->buffers = NULL;
	}

	s->buf_idx = 0;
	s->play_idx = 0;
	s->buf = NULL;

	dma_out_buffer.dma_half = 0;
}

static int audio_setup_buf(audio_stream_t *s, int init_v)
{
	int	frag;
	char *dmabuf = 0;
	dma_addr_t dmaphys = 0;

	if (s->buffers)
		return -EBUSY;

	wr_buf_tail	= rd_buf_tail =	play_dma_running = record_dma_running = 0;

	s->nbfrags = audio_nbfrags;
	s->fragsize	= audio_fragsize;

#if 0
	if (s == &input_stream)
		printk("audio_setup_input_buf, fragsize:%d, nbfrags:%d\n", s->fragsize, s->nbfrags);
	else
		printk("audio_setup_output_buf, fragsize:%d, nbfrags:%d\n", s->fragsize, s->nbfrags);
#endif

	// malloc the buffer
	s->buffers = (audio_buf_t *)
		kmalloc(sizeof(audio_buf_t)	* s->nbfrags, GFP_KERNEL);
	if (!s->buffers)
		goto err;
	memset(s->buffers, 0, sizeof(audio_buf_t) *	s->nbfrags);

	// allocate the buffer
	for	(frag =	0; frag	< s->nbfrags; frag++) {
		audio_buf_t	*b = &s->buffers[frag];

		DPRINTK("malloc fragment:%d\n", s->fragsize);
		//dmabuf = dma_alloc_writecombine(0, s->fragsize, &dmaphys, GFP_KERNEL);
		dmabuf = kmalloc(s->fragsize, GFP_KERNEL);
		if(!dmabuf)	{
			DPRINTK("Unable to get fragment buffer\n");
			goto err;
		}
		b->master =	s->fragsize;

		b->start = dmabuf;
		b->dma_addr	= dmaphys;
		sema_init(&b->sem, init_v);
		DPRINTK("buf %d: start %p dma %p, size: %d\n", frag, b->start, b->dma_addr, s->fragsize);
	}

	s->buf_idx = 0;
	s->play_idx = 0;
	s->buf = &s->buffers[0];

	return 0;

err:
	printk(AUDIO_NAME ": unable	to allocate	audio memory\n ");
	audio_clear_buf(s);
	return -ENOMEM;
}

//
// DMA input & output share one interrupt
//
static irqreturn_t iis_dma_done_handler(int irq, void *dev_id)
{
	audio_stream_t *s;
	static int cnt;
	short dmaInterruptSource=0;
	int k;

	if (HwCHCTRL0 & HwCHCTRL_FLAG)	// output interrupt
		dmaInterruptSource |= DMA_CH_OUT;
	if (HwCHCTRL1 & HwCHCTRL_FLAG)	// input interrupt
		dmaInterruptSource |= DMA_CH_IN;

	DPRINTK("HwCHCTRL0:%p, HwCHCTRL1:%p\n", HwCHCTRL0, HwCHCTRL1);
	// output
	if((dmaInterruptSource & DMA_CH_OUT) )
	{
		s = &output_stream;
		DPRINTK("Output interrupt tail:%d, play_idx:%d\n", wr_buf_tail, s->play_idx);
		if (s->buffers)
		{
			audio_buf_t *b;

			wr_buf_tail = s->play_idx;

			// move to next block
			wr_buf_tail++;
			if(wr_buf_tail >= s->nbfrags) wr_buf_tail = 0;

			b = s->buffers + wr_buf_tail;
			k = 0;//atomic_read(&s->buffers[s->play_idx].sem.count);

			if ((b != s->buf) && (k == 0))
			{
				copy_data_to_dma();
				HwCHCTRL0 |= HwCHCTRL_FLAG;
				//b->dma_size	= 0;
			}
			else	// buffer is empty
			{
				cnt = 0;
				DPRINTK("iis_dma_done_handler, STOP\n");
				StopI2SOutputClock();
				play_dma_running = 0;
			}

		}
	}

	// input
	if (dmaInterruptSource & DMA_CH_IN)
	{
		s = &input_stream;
#if 0
		{
			int i;
			unsigned char *p=dma_in_buffer.addr;
			printk("audio input:%p(", p);
			for (i = 0; i < 8; i++)
				printk("%02x,", *(p+i));
			printk("\n");
		}
#endif
		if (s->buffers)
		{
/*
			audio_buf_t	*b = s->buffers+rd_buf_tail;
			rd_buf_tail++;
			if(rd_buf_tail>=s->nbfrags) rd_buf_tail = 0;

			if(s->buf_idx == rd_buf_tail)	{
				s->buf_idx ++;
				if(s->buf_idx >= s->nbfrags) s->buf_idx = 0;
				s->buf = s->buffers + s->buf_idx;
			}

			//if(b->size == 0)
			up(&(b->sem));
			b->size	= s->fragsize;
			//printk("b->size:%d\n", b->size);
*/
			copy_data_from_dma();
			HwCHCTRL1 |= HwCHCTRL_FLAG;
			//b->dma_size	= 0;
		}
	}

	return IRQ_HANDLED;
}

 /*	using when write */
static int audio_sync(struct file *file)
{
	audio_stream_t *s =	&output_stream;
	audio_buf_t	*b = s->buf;

	DPRINTK("-->audio_sync\n");

	if (!s->buffers)
		return 0;
#if 1
	//audio_buf_t *b;

	wr_buf_tail = s->play_idx;

	if (!play_dma_running)
	{
		DPRINTK("audio_sync::wr_buf_tail:%d, s->buf_idx:%d\n", wr_buf_tail, s->buf_idx);
		b = s->buffers + wr_buf_tail;
		if (b->size > 9)
		{
			DPRINTK("audio_sync::play the frag:%d\n", wr_buf_tail);
			copy_data_to_dma();
			copy_data_to_dma();
			StartI2SOutputClock();
			SetDMASourceBuffer(dma_out_buffer.dma_addr, dma_out_buffer.buf_size);
			HwCHCTRL0 |= HwCHCTRL_FLAG;
			mdelay(200);
			//b->dma_size = 0;
		}

	}
#endif
	dma_out_buffer.dma_half = 0;
	DPRINTK("<--audio_sync done\n");
	return 0;
}

static inline int copy_from_user_mono_stereo(char *to, const char *from, int count)
{
	u_int *dst = (u_int	*)to;
	const char *end	= from + count;

#if	0
	if (verify_area(VERIFY_READ, from, count))
		return -EFAULT;
#endif
	if ((int)from &	0x2) {
		u_int v;
		__get_user(v, (const u_short *)from); from += 2;
		*dst++ = v | (v	<< 16);
	}

	while (from	< end-2) {
		u_int v, x,	y;
		__get_user(v, (const u_int *)from);	from +=	4;
		x =	v << 16;
		x |= x >> 16;
		y =	v >> 16;
		y |= y << 16;
		*dst++ = x;
		*dst++ = y;
	}

	if (from < end)	{
		u_int v;
		__get_user(v, (const u_short *)from);
		*dst = v | (v << 16);
	}

	return 0;
}

/*
static void fill_empty_channel(unsigned short *p, int t)
{
	int i;
	unsigned short *pt;

	pt = p;
	for (i = 0; i < t/4;i++)
	{
		*(pt) = *(pt+1);
		pt += 2;
	}

}
*/

// copy data from DMA buffer to 1st level buffer
// top = 0, the top half is target
// top = 1, the bottom half is target
static void copy_data_from_dma(void)
{
	int t, k, cnt;
	audio_stream_t *s =	&input_stream;
	audio_buf_t *b;
	unsigned char *p;

	//b = s->buffers+rd_buf_tail;

	cnt = 0;

	while (cnt < dma_in_buffer.page_size)
	{
		b = s->buffers + s->play_idx;
		b->size	= s->fragsize;

		k = atomic_read(&s->buffers[s->play_idx].sem.count);
		if (k > 0)
		{
			StopI2SInputClock();
			record_dma_running = 0;
			break;
		}

		t = s->fragsize;
		if (t > dma_in_buffer.page_size) t = dma_in_buffer.page_size;

		p = dma_in_buffer.addr+(dma_in_buffer.dma_half*dma_in_buffer.page_size) + cnt;
		// need to fill the channel without data to avoid the noise???
		memcpy(b->start, p, t);

		//printk("memcpy:(%p->%p(%d),%d)\n",p, b->start, s->play_idx, t);
#if 0
		{
			int i;
			//printk("memcpy:(%p->%p,%d)\n",p, b->start, t);

			printk("DMA addr:%p(%d),buffer:%p,size:%d\n(", dma_in_buffer.addr, dma_in_buffer.dma_half,p, t);
			/*
			for (i = 0;i < 8; i++)
				printk("%02X,",*(b->start+i));
			printk(")\n");
			*/
			for (i = 0;i < 8; i++)
				printk("%02X,",*(p+i));
			printk(")\n");


		}
#endif

		b->dma_size = 0;

		// move to next 1st level buffer
		if (t <= dma_in_buffer.page_size)
			s->play_idx = (s->play_idx+1) % s->nbfrags;

		cnt += s->fragsize;

		up(&(b->sem));
	}

	//printk("done\n");
	// move to next half
	dma_in_buffer.dma_half = (dma_in_buffer.dma_half+1) % 2;
}

void * memcpy_ex(void * dst, void *src, size_t len)
{
	long * plDst = (long *) dst;
	long const * plSrc = (long const *) src;

	while (len >= 4)
	{
		*plDst++ = *plSrc++;
		len -= 4;
	}

	return (dst);
}


// copy data from 1st level buffer to DMA buffer
// top = 0, the top half is target
// top = 1, the bottom half is target
static void copy_data_to_dma(void)
{
	int t, cnt=0,k;
	audio_stream_t *s =	&output_stream;
	audio_buf_t *b;
	unsigned char *p;


	DPRINTK("copy_data_to_dma\n");
	do {
		b = s->buffers + s->play_idx;
		// no data in buffer
		k = atomic_read(&s->buffers[s->play_idx].sem.count);
		if (b->size == 0)
		{
			DPRINTK("buffer %d is empty %d\n", s->play_idx, k);
			break;
		}

		t = b->dma_size;
		if (t > dma_out_buffer.page_size) t = dma_out_buffer.page_size;

		p = dma_out_buffer.addr+(dma_out_buffer.dma_half*dma_out_buffer.page_size) + cnt;
		memcpy(p, b->start, t);
		b->size = 0;
#if 0
		{
			int i;
			printk("memcpy:(%p,%p,%d)\n",p, b->start, t);

			printk("b->start:%p,(", b->start);
			for (i = 0;i < 8; i++)
				printk("%02X,",*(b->start+i));
			printk(")\n");
/*
			printk("dma_buffer:(");
			for (i = 0;i < 8; i++)
				printk("%02X,",*(p+i));
			printk(")\n\n");
*/
		}
#endif

		DPRINTK("copy %d , sz:%d to dma\n", s->play_idx,t);
		// move to next 1st level buffer
		//if (t <= dma_out_buffer.page_size)
		s->play_idx = (s->play_idx+1) % s->nbfrags;

		up(&b->sem);
		cnt += b->dma_size;
	} while(cnt < dma_out_buffer.page_size);

	//printk("--------%d--------\n", dma_out_buffer.dma_half);
	// move to next half
	dma_out_buffer.dma_half = (dma_out_buffer.dma_half+1) % 2;
}

// check whether the data is enough to fill DMA buffer
// return the valid fragment number
int ready_to_fill_dmabuf(audio_stream_t *s)
{
	int i, frags=0;

	for (i = 0; i < s->nbfrags; i++){
		if (atomic_read(&s->buffers[i].sem.count) <= 0)
		{
			frags++;
    	}
	}

	DPRINTK("avail. frags:%d, total frags:%d, fragsize:%d\n",
		frags, s->nbfrags, s->fragsize);

	if (frags*s->fragsize >= dma_out_buffer.buf_size)
		return frags;
	else
		return 0;
}

static void full_fill_dmabuf(audio_stream_t *s, int n)
{
	int i;

	for (i = 0; i < n; i++)
	{
		copy_data_to_dma();
		if (s->play_idx >= n) break;
	}
}

static ssize_t tcc7901_audio_write(struct file *file, const	char *buffer,
					size_t count, loff_t * ppos)
{
	const char *buffer0	= buffer;
	audio_stream_t *s =	&output_stream;
	int	chunksize, ret = 0, frags;

	//printk("-->audio_write : count=%d,(%02x,%02x,%02x)\n", count, buffer[0], buffer[1], buffer[2]);

	switch (file->f_flags &	O_ACCMODE) {
		case O_WRONLY:
		case O_RDWR:
			break;
		default:
			return -EPERM;
	}

	if (!s->buffers	&& audio_setup_buf(s, 1))
	{
		return -ENOMEM;
    }
	count &= ~0x03;

	DPRINTK("audio_write:%d bytes to idx:%d\n",count, s->buf_idx);
	while (count > 0)
	{
		audio_buf_t	*b;
		b = s->buf;
		//DPRINTK("Start to move data to buffer, audio channel:%d\n", audio_channels);

		if (file->f_flags &	O_NONBLOCK)	{
			ret	= -EAGAIN;
			if (down_trylock(&b->sem))
				break;
		} else {
			ret	= -ERESTARTSYS;
			if (down_interruptible(&b->sem))
				break;
		}

		if (audio_channels == 2)
		//if (audio_fmt == 16)
		{
			chunksize =	s->fragsize - b->size;
			if (chunksize >	count) chunksize = count;
			DPRINTK("copy_from_user(2 channel):%p-->%p, chunksizz=%d, count:%d\n", buffer, b->start+b->size,
				chunksize, count, s->buf_idx);
			if (copy_from_user(b->start	+ b->size, buffer, chunksize))
			{
				printk("copy failed:%p-->%p, buf_idx:%d\n", buffer, b->start+b->size, s->buf_idx);
				up(&b->sem);
				return -EFAULT;
			}
			else
				DPRINTK("copy success:%p-->%p, buf_idx:%d\n", buffer, b->start+b->size, s->buf_idx);
			b->size	+= chunksize;
			//printk("write %d to %d, fragsize:%d, b->size:%d\n", chunksize, s->buf_idx, s->fragsize, b->size);
		} else
		{
			chunksize =	(s->fragsize - b->size)	>> 1;
			if (chunksize >	count) chunksize = count;
			DPRINTK("copy_from_user(1 channel):%p-->%p, chunksizz=%d, count:%d\n", buffer, b->start+b->size, chunksize, count);

			if (copy_from_user_mono_stereo(b->start	+ b->size,
								   buffer, chunksize)) {
				up(&b->sem);
				return -EFAULT;
			}

			b->size	+= chunksize*2;
		}

		buffer += chunksize;
		count -= chunksize;

		b->dma_size	= b->size;
		DPRINTK("write:%d, sz=%d, dma_size:%d\n", s->buf_idx, chunksize, b->dma_size);

		if (b->size < s->fragsize) {
			up(&b->sem);
			break;
		}

		//local_irq_save(flags);
		// if DMA not running, fill the whole buffer and startup DMA
		frags = ready_to_fill_dmabuf(s);
		if ((!play_dma_running) && (frags*s->fragsize >= dma_out_buffer.buf_size))
		{
			play_dma_running = 1;
			dma_out_buffer.dma_half = 0;
			full_fill_dmabuf(s, frags);
			// Start address of DMA
			//SetDMASourceBuffer(b->dma_addr, b->dma_size);
			StopI2SOutputClock();
			SetDMASourceBuffer(dma_out_buffer.dma_addr, dma_out_buffer.buf_size);
			DPRINTK("StartI2SOutputClock @ %p, page size:%d, buf size:%d\n", HwST_SADR0,
				dma_out_buffer.page_size, dma_out_buffer.buf_size);
			StartI2SOutputClock();	// enable DMA interrupt
			HwCHCTRL0 |= HwCHCTRL_FLAG;
		}

		//b->size = 0;
		NEXT_BUF(s,	buf);
		//local_irq_restore(flags);
	}	// end of while

	if ((buffer	- buffer0))
		ret	= buffer - buffer0;

	DPRINTK("audio_write done:%d, s->buf_idx:%d\n\n",ret, s->buf_idx);

	return ret;
}

static ssize_t tcc7901_audio_read(struct file *file, char *buffer,
										size_t count, loff_t * ppos)
{
	const char *buffer0	= buffer;
	audio_stream_t *s =	&input_stream;
	int	chunksize, ret = 0;

	//printk("audio_read: count=%d\n", count);

	//if (ppos != &file->f_pos)
	//	return -ESPIPE;

	if (!s->buffers) {
		int	i;

		if (audio_setup_buf(s,0))
			return -ENOMEM;

		for	(i = 0;	i <	s->nbfrags;	i++) {
			audio_buf_t	*b = s->buf;
			down(&b->sem);
			NEXT_BUF(s,	buf);
		}

	}

	if (!record_dma_running)
	{
		StartI2SInputClock();
		record_dma_running = 1;
		//printk("StartI2SInputclock()\n");
	}

	while (count > 0)
	{
		audio_buf_t	*b = s->buf;

//		if (b->size <= 0) continue;
		/* Wait	for	a buffer become full	*/
		if (file->f_flags &	O_NONBLOCK)
		{
			ret	= -EAGAIN;

			if (down_trylock(&b->sem))
				break;

		} else {
			ret	= -ERESTARTSYS;

			if (down_interruptible(&b->sem))
				break;
		}

		chunksize =	b->size;
		if (chunksize >	count) chunksize = count;
		//printk("read %d from %d, %p\n", chunksize, s->buf_idx, b->start);
		if (copy_to_user(buffer, b->start + s->fragsize - b->size, chunksize))
		{
			DPRINTK("Failed to copy data from %p to %p\n", b->start + s->fragsize - b->size, buffer);
			up(&b->sem);
			return -EFAULT;
		}
#if 0
		{
			int j;
			unsigned char *p=b->start + s->fragsize - b->size;
			printk("Get(");
			for (j = 0; j < 16; j++)
				printk("%2x,", *(p+j));
			printk(")\n");
		}
#endif
		b->size	-= chunksize;
		buffer += chunksize;
		count -= chunksize;
		if (b->size	> 0) {
			up(&b->sem);
			break;
		}
		NEXT_BUF(s,	buf);
	}

	if ((buffer	- buffer0))
		ret	= buffer - buffer0;

	//printk("audio_read: return=%d\n", ret);

	return ret;
}

static unsigned	int	tcc7901_audio_poll(struct file *file,
					struct poll_table_struct *wait)
{
	unsigned int mask =	0;
	int	i;

	DPRINTK("audio_poll(): mode=%s\n",
		(file->f_mode &	FMODE_WRITE) ? "w" : "");

	if (file->f_mode & FMODE_READ) {
		if (!input_stream.buffers && audio_setup_buf(&input_stream,0))
				return -ENOMEM;
		poll_wait(file,	&input_stream.buf->sem.wait, wait);

		for	(i = 0;	i <	input_stream.nbfrags; i++) {

			if (atomic_read(&input_stream.buffers[i].sem.count)	> 0)
				mask |=	POLLIN | POLLWRNORM;

			break;
		}
	}

	if (file->f_mode & FMODE_WRITE)	{
		if (!output_stream.buffers && audio_setup_buf(&output_stream,1))
			return -ENOMEM;
		poll_wait(file,	&output_stream.buf->sem.wait, wait);

		for	(i = 0;	i <	output_stream.nbfrags; i++)	{
			if (atomic_read(&output_stream.buffers[i].sem.count) > 0)
				mask |=	POLLOUT	| POLLWRNORM;
				break;
		}
	}

	DPRINTK("audio_poll() returned mask	of %s\n",
		(mask &	POLLOUT) ? "w" : "");

	return mask;
}

static loff_t tcc7901_audio_llseek(struct file *file, loff_t offset,
					int	origin)
{
	return -ESPIPE;
}

static int audio_set_dsp_speed(__u32 val)
{
	unsigned long f_pll0;
	unsigned long /*m, p, s,*/ div, n;

	/*
	m = (HwPLL0CFG & 0xff00)>>8;
	s = (HwPLL0CFG & 0x00030000) >> 16;
	p = (HwPLL0CFG & 0x1f);
	//f_pll0 = (2*m*12000000)/(p*(2^s));
	f_pll0 = (2*12000000*(m+8)/(p+2))>>s;
	*/
	f_pll0 = tcc79x_get_pll0();
	DPRINTK("PLL0:%X, %d\n", HwPLL0CFG, f_pll0);

	switch (val) {
	case 8000:
	case 11025:
	case 16000:
	case 22050:
	case 24000:
	case 32000:
	case 44100:
	case 48000:
		div = val*256/(f_pll0/0x10000);
		n = HwPCK_EN_EN + HwPCK_SEL_PLL0 + 0x10000 - div;
		DPRINTK("Set Sample rate:%d, HwPCK_DAI:%X-->%X\n",val, HwPCK_DAI, n);
		HwPCK_DAI = n;
		break;
	default:
		printk("Invalid	value\n");
		return -1;
	}

	return val;
}

static int tcc7901_mixer_ioctl(struct inode	*inode,	struct file	*file,
								unsigned int cmd, unsigned long	arg)
{
	static __u32 audio_mix_modcnt =	0;
	int	ret;
	long val = 0;
	int vol;

	audio_mix_modcnt++;

	switch (cmd) {
		case SOUND_MIXER_INFO:
		{
			mixer_info info;
			memset(&info, 0, sizeof(info));
			strncpy(info.id, AUDIO_NAME, sizeof(info.id));
			strncpy(info.name,AUDIO_NAME, sizeof(info.name));
			info.modify_counter	= audio_mix_modcnt;
			return copy_to_user((void *)arg, &info,	sizeof(info));
		}
		case SOUND_OLD_MIXER_INFO:
		{
			_old_mixer_info	info;
			memset(&info, 0, sizeof(info));
			strncpy(info.id, AUDIO_NAME, sizeof(info.id));
			strncpy(info.name,AUDIO_NAME, sizeof(info.name));
			return copy_to_user((void *)arg, &info,	sizeof(info));
		}

		case SOUND_MIXER_READ_STEREODEVS:
			return put_user(0, (long *)	arg);

		case SOUND_MIXER_READ_CAPS:
			val	= SOUND_CAP_EXCL_INPUT;
			return put_user(val, (long *) arg);

		// MIC volume
		case SOUND_MIXER_READ_MIC:
			return put_user(mic_volume, (long *) arg);
		break;
		case SOUND_MIXER_WRITE_MIC:
			ret	= get_user(val,	(long *) arg);
			if (ret)
				return ret;

			mic_volume = val;
		break;

		case SOUND_MIXER_WRITE_VOLUME:
			ret	= get_user(val,	(long *) arg);
			if (ret)
				return ret;
			//printk("SetVolume:%d\n", val);
			g_volume = (val&0xffff);

			vol = (g_volume & 0xff)*3/4;
			CODEC_SEND_CMD(0x04, 175+vol);
			vol = ((g_volume &0xff00)*3/4) >> 8;
			CODEC_SEND_CMD(0x06, 175+vol);
			break;

		case SOUND_MIXER_READ_VOLUME:
			val	= g_volume;
			return put_user(val, (long *) arg);

		case SOUND_MIXER_READ_IGAIN:
			val	= ((31-	mixer_igain) * 100)	/ 31;
			return put_user(val, (int *) arg);

		case SOUND_MIXER_WRITE_IGAIN:
			ret	= get_user(val,	(int *)	arg);
			if (ret)
				return ret;
			mixer_igain	= 31 - (val	* 31 / 100);
			/* use mixer gain channel 1*/
			break;

		default:
			printk("mixer ioctl %08x unknown\n",	cmd);
			return -ENOSYS;
	}

	return 0;
}

static int tcc7901_audio_ioctl(struct inode	*inode,	struct file	*file,
								uint cmd, ulong	arg)
{
	long val;
	int k;

	//printk("tcc7901_audio_ioctl:%08X,", cmd);
	switch (cmd) {
		case OSS_GETVERSION:
			DPRINTK("OSS_GETVERSION:%X\n", SOUND_VERSION);
			return put_user(SOUND_VERSION, (int	*)arg);
		case SNDCTL_DSP_SETFMT:
			get_user(val, (long *) arg);
			DPRINTK("SNDCTL_DSP_SETFMT:%d\n", val);
			if (val & AUDIO_FMT_MASK) {
				audio_fmt = val;
				break;
			} else
				return -EINVAL;
		break;
		case SNDCTL_DSP_CHANNELS:
			DPRINTK("SNDCTL_DSP_CHANNELS\n");
		break;
		case SNDCTL_DSP_STEREO:
			get_user(val, (long	*) arg);
			if (cmd	== SNDCTL_DSP_STEREO)
				val	= val ?	2 :	1;
			if (val	!= 1 &&	val	!= 2)
				return -EINVAL;
			audio_channels = val;
			DPRINTK("SNDCTL_CTL_DSP_STEREO:%d\n", audio_channels);
		break;

		case SOUND_PCM_READ_CHANNELS:
			DPRINTK("SOUND_PCM_READ_CHANNELS\n");
			put_user(audio_channels, (long *) arg);
		break;

		// set codec sample rate
		case SNDCTL_DSP_SPEED:
			get_user(val, (long	*) arg);
			DPRINTK("SNDCTL_DSP_SPEED:%d\n", val);

			val	= audio_set_dsp_speed(val);
			audio_rate = val;
			if (val	< 0)
				return -EINVAL;
			put_user(val, (long	*) arg);
		break;

		case SOUND_PCM_READ_RATE:
			DPRINTK("SOUND_PCM_READ_RATE\n");
			put_user(audio_rate, (long *) arg);
		break;

		case SNDCTL_DSP_GETFMTS:
			DPRINTK("SNDCTL_DSP_GETFMTS\n");
		  	put_user(AUDIO_FMT_MASK, (long *) arg);

			break;

		case SNDCTL_DSP_GETBLKSIZE:
			DPRINTK("SNDCTL_DSP_GETBLKSIZE\n");
			if(file->f_mode	& FMODE_WRITE)
				return put_user(audio_fragsize,	(long *) arg);
			else
				return put_user(audio_fragsize,	(int *)	arg);

		case SNDCTL_DSP_SETFRAGMENT:
			DPRINTK("SNDCTL_PCM_SETFRAGMENT\n");
			if (file->f_mode & FMODE_WRITE)	{
				if (output_stream.buffers)
				{
					audio_clear_buf(&output_stream);
					//return -EBUSY;
				}

    			get_user(val, (long	*) arg);
    			audio_fragsize = 1 << (val & 0xFFFF);
    			if (audio_fragsize < 16) audio_fragsize = 16;
    			if (audio_fragsize > 16384) audio_fragsize = 16384;
    			audio_nbfrags =	(val >>	16)	& 0x7FFF;
				audio_nbfrags = (audio_nbfrags<4)?4:audio_nbfrags;
				if (audio_nbfrags *	audio_fragsize > 128 * 1024)
					audio_nbfrags =	128	* 1024 / audio_fragsize;
    			if (audio_setup_buf(&output_stream,1))
				{
    				return -ENOMEM;
				}

				malloc_output_dma_buffer(audio_fragsize*AUDIO_DMA_PAGE_NUMBER, audio_fragsize);
				SetDMASourceBuffer(dma_out_buffer.dma_addr, dma_out_buffer.buf_size);

				DPRINTK("Set fragments(WRITE), sz=%d, nr=%d\n", audio_fragsize, audio_nbfrags);
			}
			if (file->f_mode & FMODE_READ) {
				if (input_stream.buffers)
				{
					audio_clear_buf(&input_stream);
					//return -EBUSY;
				}
				get_user(val, (int *) arg);
				audio_fragsize =  1	<< (val	& 0xFFFF);
				if (audio_fragsize < 16) audio_fragsize = 16;
				if (audio_fragsize > 16384) audio_fragsize = 16384;
    			audio_nbfrags =	(val >>	16)	& 0x7FFF;
				if (audio_nbfrags *	audio_fragsize > 128 * 1024)
					audio_nbfrags =	128	* 1024 / audio_fragsize;

				if (audio_setup_buf(&input_stream, 0))
					return -ENOMEM;

				malloc_input_dma_buffer(audio_fragsize*AUDIO_DMA_PAGE_NUMBER, audio_fragsize);
				SetInputDMADestBuffer(dma_in_buffer.dma_addr, dma_in_buffer.buf_size);

			}
			return 0;

		case SNDCTL_DSP_SYNC:
			DPRINTK("SNDCTL_DSP_SYNC\n");
			return audio_sync(file);

		case SNDCTL_DSP_GETOSPACE:
			DPRINTK("SNDCTL_DSP_GETOSPACE\n");
		{
			audio_stream_t *s =	&output_stream;
			audio_buf_info *inf	= (audio_buf_info *) arg;
			int ok = access_ok(VERIFY_WRITE, inf, sizeof(*inf));
			int	i;
			int	frags =	0, bytes = 0;

			if (!ok) return -1;

			for	(i = 0;	i <	s->nbfrags;	i++)
			{
				k = atomic_read(&s->buffers[i].sem.count);
				DPRINTK("frag:%d, sem=%d, size=%d\n", i, k, s->buffers[i].size);
				if (k >	0)
				{
					if (s->buffers[i].size == 0) frags++;
					bytes += s->fragsize - s->buffers[i].size;
				}
			}

			DPRINTK("OSPACE avail. frags:%d, total frags:%d, fragsize:%d, bytes:%d, HwCHCTRL0:%p, HwCHCTRL1:%p\n",
				frags, s->nbfrags, s->fragsize, bytes, HwCHCTRL0, HwCHCTRL1);
			put_user(frags,	&inf->fragments);
			put_user(s->nbfrags, &inf->fragstotal);
			put_user(s->fragsize, &inf->fragsize);
			put_user(bytes,	&inf->bytes);

			return 0;
		}

		case SNDCTL_DSP_GETISPACE:
			DPRINTK("SNDCTL_DSP_GETISPACE\n");
            {
                audio_stream_t *s =	&input_stream;
                audio_buf_info *inf	= (audio_buf_info *) arg;

                int	i;
                int	frags =	0, bytes = 0;

                if (!(file->f_mode & FMODE_READ))
                    return -EINVAL;

                for(i =	0; i < s->nbfrags; i++){
                    if (atomic_read(&s->buffers[i].sem.count) >	0)
                    {
                        if (s->buffers[i].size == s->fragsize)
                            frags++;
                        bytes += s->buffers[i].size;
                    }
                }
                put_user(frags,	&inf->fragments);
                put_user(s->nbfrags, &inf->fragstotal);
                put_user(s->fragsize, &inf->fragsize);
                put_user(bytes,	&inf->bytes);

                DPRINTK("ISPACE avail. frags:%d, total frags:%d, fragsize:%d, bytes:%d, HwCHCTRL0:%p, HwCHCTRL1:%p\n",
                    frags, s->nbfrags, s->fragsize, bytes, HwCHCTRL0, HwCHCTRL1);
            }

			return 0;
		case SNDCTL_DSP_RESET:
			DPRINTK("SNDCTL_DSP_RESET\n");
			if (file->f_mode & FMODE_READ) {
				audio_clear_buf(&input_stream);
			}
			if (file->f_mode & FMODE_WRITE)	{
				audio_clear_buf(&output_stream);
			}
			StopI2SOutputClock();
			return 0;
		case SNDCTL_DSP_NONBLOCK:
			DPRINTK("SNDCTL_DSP_NONBLOCK\n");
			file->f_flags |= O_NONBLOCK;
			return 0;
		case SNDCTL_DSP_POST:
		case SNDCTL_DSP_SUBDIVIDE:
		case SNDCTL_DSP_GETCAPS:
		case SNDCTL_DSP_GETTRIGGER:
		case SNDCTL_DSP_SETTRIGGER:
		case SNDCTL_DSP_GETIPTR:
		case SNDCTL_DSP_GETOPTR:
		case SNDCTL_DSP_MAPINBUF:
		case SNDCTL_DSP_MAPOUTBUF:
		case SNDCTL_DSP_SETSYNCRO:
		case SNDCTL_DSP_SETDUPLEX:
		return -ENOSYS;
		default:
			//printk("mixer_ioctl\n");
			return tcc7901_mixer_ioctl(inode, file,	cmd, arg);
	}

	return 0;
}


static int tcc7901_audio_open(struct inode *inode, struct file *file)
{
	int	cold = !audio_active;

	if ((file->f_flags & O_ACCMODE)	== O_RDONLY) {
		if (audio_rd_refcount || audio_wr_refcount)
			return -EBUSY;
		audio_rd_refcount++;
	} else if ((file->f_flags &	O_ACCMODE) == O_WRONLY)	{
		if (audio_rd_refcount || audio_wr_refcount)
			return -EBUSY;
		audio_wr_refcount++;
	} else if ((file->f_flags &	O_ACCMODE) == O_RDWR) {
		if (audio_rd_refcount || audio_wr_refcount)
			return -EBUSY;
		audio_rd_refcount++;
		audio_wr_refcount++;
	} else
		return -EINVAL;

	// open the device at the 1st time, need to initialize the buffer
	if (cold) {
		audio_rate = AUDIO_RATE_DEFAULT;
		audio_channels = AUDIO_CHANNELS_DEFAULT;
		audio_fragsize = AUDIO_FRAGSIZE_DEFAULT;
		audio_nbfrags =	AUDIO_NBFRAGS_DEFAULT;
		if ((file->f_mode &	FMODE_WRITE)){
			DPRINTK("Open in WRITE mode\n");
			init_tcc7901_iis_bus_tx();
			audio_clear_buf(&output_stream);
			audio_setup_buf(&output_stream, 1);
			// malloc in & out buffer for DMA
			malloc_output_dma_buffer(AUDIO_DMA_OUT_BUF_SIZE, AUDIO_DMA_OUT_PAGE_SIZE);
		}
		if ((file->f_mode &	FMODE_READ)){
			DPRINTK("Open in READ mode\n");
			init_tcc7901_iis_bus_rx();
			audio_clear_buf(&input_stream);
			audio_setup_buf(&input_stream, 0);

			malloc_input_dma_buffer(AUDIO_DMA_IN_BUF_SIZE, AUDIO_DMA_IN_PAGE_SIZE);
			SetInputDMADestBuffer(dma_in_buffer.dma_addr, dma_in_buffer.buf_size);
			DPRINTK("Malloc  input DMA buffer @%p(Phy=%p), size:%d\n", dma_in_buffer.addr, dma_in_buffer.dma_addr, dma_in_buffer.buf_size);

			// enable audio input
			StartI2SInputClock();
			HwCHCTRL1 |= HwCHCTRL_EN_EN;
		}


		EnableDAI();
	}

	return 0;
}

static int tcc7901_audio_release(struct	inode *inode, struct file *file)
{
	DPRINTK("audio_release, rd_refcount:%d, wr_refcount:%d\n", audio_rd_refcount, audio_wr_refcount);

	if (file->f_mode & FMODE_READ) {
			if (audio_rd_refcount == 1)	{
				audio_clear_buf(&input_stream);
				// Stop audio in
				StopI2SInputClock();
				HwCHCTRL1 &= ~HwCHCTRL_EN_EN;
			}
			audio_rd_refcount --;
	}

	if(file->f_mode	& FMODE_WRITE) {
			if (audio_wr_refcount == 1)	{
					audio_sync(file);
					audio_clear_buf(&output_stream);
					// Stop audio out
					StopI2SOutputClock();
					HwCHCTRL0 &= ~HwCHCTRL_EN_EN;
			}
			audio_wr_refcount --;
		}

	DisableDAI();
	return 0;
}

static struct file_operations tcc7901_audio_fops = {
	.owner		= THIS_MODULE,
	.llseek		= tcc7901_audio_llseek,
	.write		= tcc7901_audio_write,
	.read		= tcc7901_audio_read,
	.poll		= tcc7901_audio_poll,
	.ioctl		= tcc7901_audio_ioctl,
	.open		= tcc7901_audio_open,
	.release	= tcc7901_audio_release,
};

static int tcc7901_mixer_open(struct inode *inode, struct file *file)
{
	return 0;
}

static int tcc7901_mixer_release(struct	inode *inode, struct file *file)
{
	return 0;
}

static struct file_operations tcc7901_mixer_fops = {
	.owner		= THIS_MODULE,
	.llseek		= no_llseek,
	.ioctl		= tcc7901_mixer_ioctl,
	.open		= tcc7901_mixer_open,
	.release	= tcc7901_mixer_release,
};

#ifdef CONFIG_I2C
static int wm8731_i2c_probe(struct i2c_adapter *adap, int addr, int kind)
{
	struct i2c_client *i2c;
	int ret;

	i2c = kzalloc(sizeof(*i2c), GFP_KERNEL);
	if (!i2c)
		return -ENOMEM;

	strcpy(i2c->name, "WM8731");
	i2c->flags = 0;
	i2c->addr = addr;
	i2c->adapter = adap;
	i2c->driver = &wm8731_i2c_driver;

	wm8731_i2c_client = i2c;

	/* attach i2c client device */
	ret = i2c_attach_client(i2c);
	if (ret < 0) {
		printk("%s: failed to attach codec at addr %x\n", __func__, addr);
		goto err;
	}

	return ret;
err:
	kfree(i2c);
	return ret;
}

static int wm8731_i2c_detach(struct i2c_client *client)
{
	i2c_detach_client(client);
	kfree(wm8731_i2c_client);
	return 0;
}

static int wm8731_i2c_attach(struct i2c_adapter *adap)
{
	int ret;
	if (adap->nr == 0) 	/* WM8731 use the I2C Channel 0 */
		ret = i2c_probe(adap, &addr_data, wm8731_i2c_probe);
	else				/* if I2C Channel 1, return error */
		ret = -1;
	return ret;
}

/* corgi i2c codec control layer */
static struct i2c_driver wm8731_i2c_driver = {
    .driver	= {
        .name	= "WM8731 I2C Codec",
        .owner	= THIS_MODULE,
    },
    .id				= I2C_DRIVERID_WM8731,
    .attach_adapter	= wm8731_i2c_attach,
    .detach_client	= wm8731_i2c_detach,
    .command		= NULL,
};

/* TODO: XXX
static struct i2c_client wm8731_i2c_client = {
    .name =   "WM8731",
    .driver = &wm8731_i2c_driver,
};
*/
#endif	/* CONFIG_I2C */

static void	init_wm8731(void)
{
	int ret = 0;

#ifdef CONFIG_I2C
	ret = i2c_add_driver(&wm8731_i2c_driver);
	if (ret != 0)
		printk("%s: can't add i2c driver", __func__);
#else
    DDI_I2C_Initialize();

    //HwGPEDAT  &= ~Hw20;
    //HwGPBDAT    &= ~Hw21;   // Mute Ctrl OFF

    i2c_init();
#endif

    CODEC_SEND_CMD(0x1E,0x00);  // reset

    CODEC_SEND_CMD(0x00,0x97);  // Llinein register control
    CODEC_SEND_CMD(0x02,0x97);  // Rlinein register control
    CODEC_SEND_CMD(0x04,0xe0);	// volume, left channel
    CODEC_SEND_CMD(0x06,0xe0);	// volume, right channel
    CODEC_SEND_CMD(0x08,0x02);  // Lineout select
    CODEC_SEND_CMD(0x0A,0x01);  // Highpass filter Enable
    CODEC_SEND_CMD(0x0C,0xFE);  // Powerdown control
    CODEC_SEND_CMD(0x0E,0x02);    // Slave MCLK of CODEC
    //CODEC_SEND_CMD(0x0E,0x42);  // Master MCLK of CODEC
    CODEC_SEND_CMD(0x10,0x20);
    CODEC_SEND_CMD(0x12,0x01);  // Active

    CODEC_SEND_CMD(0x0A,0x00);  //CODEC_Mute(0);
    //CODEC_Power(2);
    CODEC_SEND_CMD(0x08,0x12);  // Lineout select
    CODEC_SEND_CMD(0x0C,0x66);  // Powerdown control

	// I2S & DMA
	I2S_Init();

	// disable audio input
	StopI2SInputClock();
	HwCHCTRL1 &= ~HwCHCTRL_EN_EN;

}

static void malloc_output_dma_buffer(int buf_size, int page_size)
{
	// free DMA buffer if already allocated
	if (dma_out_buffer.dma_addr != 0)
	{
		dma_free_writecombine(0, dma_out_buffer.buf_size,dma_out_buffer.addr,dma_out_buffer.dma_addr);
		dma_out_buffer.dma_addr = 0;
	}
	dma_out_buffer.buf_size = buf_size;
	dma_out_buffer.page_size = page_size;
	dma_out_buffer.addr = dma_alloc_writecombine(0, dma_out_buffer.buf_size, &dma_out_buffer.dma_addr, GFP_KERNEL);
	DPRINTK("Malloc output DMA buffer @%p(Phy=%p), size:%d\n", dma_out_buffer.addr, dma_out_buffer.dma_addr, dma_out_buffer.buf_size);
}
static void malloc_input_dma_buffer(int buf_size, int page_size)
{
	// free DMA buffer if already allocated
	if (dma_in_buffer.dma_addr != 0)
	{
		dma_free_writecombine(0, dma_in_buffer.buf_size,dma_in_buffer.addr,dma_in_buffer.dma_addr);
		dma_in_buffer.dma_addr = 0;
	}
	dma_in_buffer.buf_size = buf_size;
	dma_in_buffer.page_size = page_size;
	dma_in_buffer.addr = dma_alloc_writecombine(0, dma_in_buffer.buf_size, &dma_in_buffer.dma_addr, GFP_KERNEL);

	DPRINTK("Malloc input DMA buffer @%p(Phy=%p), size:%d\n", dma_in_buffer.addr, dma_in_buffer.dma_addr, dma_in_buffer.buf_size);
}

static int __init tcc7901_audio_probe(struct platform_device *dev)
{
	this_dev = &(dev->dev);

	printk("TCC7901" AUDIO_NAME " Probe.\n");


	if(request_irq(INT_DMA, iis_dma_done_handler, IRQF_SHARED,
					"IIS Play",	dev) <	0) {
		printk(	KERN_WARNING AUDIO_NAME_VERBOSE
					": unable to get DMA interrupts	for	play\n"	);
		return -EBUSY;
	}

	init_wm8731();

	audio_dev_dsp =	register_sound_dsp(&tcc7901_audio_fops,	-1);
	audio_dev_mixer	= register_sound_mixer(&tcc7901_mixer_fops,	-1);

	printk(AUDIO_NAME_VERBOSE " initialized\n");

	return 0;
}

static int tcc7901_audio_remove(struct platform_device *dev)
{
	unregister_sound_dsp(audio_dev_dsp);
	unregister_sound_mixer(audio_dev_mixer);

	// Free DMA buffer
	dma_free_writecombine(&(dev->dev),
                          dma_out_buffer.buf_size,
                          dma_out_buffer.addr,
                          dma_out_buffer.dma_addr);
	dma_free_writecombine(&(dev->dev),
                          dma_in_buffer.buf_size,
                          dma_in_buffer.addr,
                          dma_in_buffer.dma_addr);
	dma_out_buffer.dma_addr = 0;
	dma_in_buffer.dma_addr = 0;

#ifdef CONFIG_I2C
	i2c_del_driver(&wm8731_i2c_driver);
	kfree(wm8731_i2c_client);
#endif

	return 0;
}

#ifdef CONFIG_PM
static int tcc7901_wm8731_suspend(struct platform_device *pdev, pm_message_t state)
{
	//printk(AUDIO_NAME " suspend\n");

	// Stop output DMA
	StopI2SOutputClock();
	HwCHCTRL0 &= ~HwCHCTRL_EN_EN;

	// Stop input DMA
	StopI2SInputClock();
	HwCHCTRL1 &= ~HwCHCTRL_EN_EN;

	pdev->dev.power.power_state = PMSG_SUSPEND;

	return 0;
}

static int tcc7901_wm8731_resume(struct platform_device *pdev)
{
	pdev->dev.power.power_state = PMSG_ON;

	I2S_Init();

	//printk(AUDIO_NAME " activated\n");
	return 0;
}
#endif

static struct platform_driver tcc7901_wm8731_driver	= {
	.probe		= tcc7901_audio_probe,
	.remove		= tcc7901_audio_remove,
#ifdef CONFIG_PM
	.suspend	= tcc7901_wm8731_suspend,
	.resume		= tcc7901_wm8731_resume,
#endif
	.driver		= {
		.name	= "tcc7901-audio",
		.owner	= THIS_MODULE,
	},
};

static int __init tcc7901_wm8731_init(void)
{
	int	ret;

	ret	= platform_driver_register(&tcc7901_wm8731_driver);
	if(ret)
		printk("Register device	driver failed, return code is %d\n", ret);
	return ret;
}

static void	__exit tcc7901_wm8731_exit(void)
{
	platform_driver_unregister(&tcc7901_wm8731_driver);
}

#ifdef	MODULE
MODULE_AUTHOR("Mac Chen");
MODULE_LICENSE("GPL");
MODULE_DESCRIPTION(AUDIO_NAME " OSS driver for Telechips TCC7901");
#endif

module_init(tcc7901_wm8731_init);
module_exit(tcc7901_wm8731_exit);

