/****************************************************************************
 *   
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
****************************************************************************/
#include <asm/arch/tcc83x_virt.h>
#include "tcc83xx_i2c.h"



#define 		I2C_WR		0
#define 		I2C_RD		1

static char I2C_use_flag;

//Functions
static unsigned char i2c_read(unsigned int i2ctgt, unsigned char SlaveAddr, char addr, unsigned char *ptr, unsigned char bytes);
static unsigned char i2c_write(unsigned int i2ctgt, unsigned char SlaveAddr, char addr, unsigned char *ptr, unsigned char bytes);
static void i2c_init(void);
static int i2c_register(unsigned int i2ctgt);
static int	DDI_I2C_Initialize(unsigned int uiFlag);


typedef struct stI2C_Vector
{
	unsigned long int tgtaddr;	
	unsigned long int port;
}tI2C_Vector;

static unsigned int	guiI2CTgt;

//I2C: Clock , Data
static const tI2C_Vector I2C_VectorTable[I2C_MAX_VECTOR][I2C_SIGNAL_BUS]	= 
{
#if defined(TCC78X)
	{{GPIO_A, Hw0}, {GPIO_A, Hw1}},
	{{GPIO_A, Hw0}, {GPIO_A, Hw1}},
	{{GPIO_A, Hw0}, {GPIO_A, Hw1}},
	{{GPIO_A, Hw0}, {GPIO_A, Hw1}},
#endif	
#if defined(TCC82XX) || defined(TCC83XX)
	{{GPIO_E, Hw6}, {GPIO_E, Hw5}},	//I2C_CODEC
	{{GPIO_F, Hw15}, {GPIO_F, Hw14}},	//I2C_TVOUT
	{{GPIO_F, Hw15}, {GPIO_F, Hw14}},	//I2C_TOUCH
	{{GPIO_F, Hw15}, {GPIO_F, Hw14}},	//I2C_MAX_VECTOR
#endif	

};
static tI2C_Vector I2C_Vector[I2C_SIGNAL_BUS];

static void delay760(unsigned int count)
{
	#define DELAYCNT	22
	for(count=DELAYCNT;count>0;count--) {
		;
	}
	#undef	DELAYCNT
}

// start bit
static void i2c_comm_s(void)
{	
	i2c_wr;
	i2c_clk_hi;
	i2c_data_hi;
	delay760(i2c_Tbuf);
	
	
	i2c_wr;
	i2c_data_lo;
	delay760(i2c_Thold);
	i2c_clk_lo;
	delay760(i2c_Thold);
}

// data out
static void i2c_out_byte(unsigned char *ptr)
{	
	unsigned char i, temp;

	i2c_wr;
	for(i=0;i<8;i++)
	{
		temp = *ptr << i;
		temp &= 0x80;
		if(temp){
			i2c_data_hi;
		}else{
			i2c_data_lo;
		}
		delay760(i2c_Tsetup);
		
		i2c_clk_hi;
		delay760(i2c_Thold);
		
		i2c_clk_lo;
		delay760(i2c_Thold);
	}
}

// get acknowledge
static unsigned char i2c_get_ack(void)
{	
	unsigned char i=0;

	i2c_rd;
	
	i2c_clk_hi;
	delay760(i2c_Tcatch);

	i = (unsigned char)((*(volatile unsigned long int *)I2C_Vector[I2C_DATA].tgtaddr) & (I2C_Vector[I2C_DATA].port)); 	   
	delay760(i2c_Tcatch);
	
	i2c_clk_lo;
	delay760(i2c_Thold);
	
	return i;
}


// Input Byte
static void i2c_in_byte(unsigned char *ptr)
{
	unsigned int i, temp=0;
	
	i2c_rd;

	for(i=0;i<8;i++)
	{
		i2c_clk_hi;
		delay760(i2c_Tcatch);

		temp = (unsigned int)(  ( (*(volatile unsigned long int *)(I2C_Vector[I2C_DATA].tgtaddr)) & (I2C_Vector[I2C_DATA].port))) ? 1 : 0;

		if(i == 0) *ptr = temp;
		else *ptr = (*ptr << 1) | temp;
		delay760(i2c_Tcatch);
		
		i2c_clk_lo;
		delay760(i2c_Thold);
	}
}

// put acknowledge
static void i2c_do_ack(unsigned char bit)
{

	i2c_wr;

	if(bit){
		i2c_data_hi;
	}else{
		i2c_data_lo;
	}
	delay760(i2c_Tsetup);
		
	i2c_clk_hi;
	delay760(i2c_Thold);
	
	i2c_clk_lo;
	delay760(i2c_Thold);
}

static void i2c_comm_p(void)
{
	i2c_wr;
	i2c_data_lo;
	delay760(i2c_Tsetup);
	
	i2c_clk_hi;
	delay760(i2c_Thold);
	
	i2c_data_hi;
	delay760(i2c_Tbuf);
}

static void i2c_init(void)
{
	// transfer rate setting... 8bit
	I2C_use_flag = I2C_FREE;
}

static void TC_TimeDly(int ticks){
	int ms;
	while(ticks --){
		ms = 100;
		while(ms --);
	}
}

static unsigned char i2c_write(unsigned int i2ctgt, unsigned char SlaveAddr, char subaddr, unsigned char *ptr, unsigned char bytes )
{
	unsigned char SA = SlaveAddr | I2C_WR ;
	unsigned char i;

	//RETAILMSG(TRUE, (TEXT("Wavedev:::i2c_write\r\n")));
	
	/* for prevent from access on same time */
	while( I2C_use_flag == I2C_USED ) {
		TC_TimeDly(1);
	}
	I2C_use_flag = I2C_USED;

	i2c_register(i2ctgt);

	i2c_wr;
	i2c_clk_hi;
	i2c_data_hi;
	delay760(i2c_Tbuf);
	
	i2c_comm_s();
	i2c_out_byte(&SA); // slave address | write

	if(i2c_get_ack() != i2c_ack) 
	{
		i2c_comm_p();
		I2C_use_flag = I2C_FREE;
		return i2c_err;
	}

	if( subaddr != (char)I2C_SUBADDR_NOUSE )
	{
		i2c_out_byte((unsigned char*)&subaddr); // address

		if(i2c_get_ack() != i2c_ack) 
		{
			i2c_comm_p();
			I2C_use_flag = I2C_FREE;
			return i2c_err;
		}
	}

	for(i=0; i<bytes ; i++)
	{		
		i2c_out_byte(ptr);
		
		if(i2c_get_ack()) ;
		else		  ;
		
		ptr++;
	}
	
	i2c_comm_p();
	
	I2C_use_flag = I2C_FREE;
	return i2c_ok;
}

static unsigned char i2c_read(unsigned int i2ctgt, unsigned char SlaveAddr, char subaddr, unsigned char *ptr, unsigned char bytes)
{
	unsigned char i;
	unsigned char SA = SlaveAddr | I2C_WR ;

	/* for prevent from access on same time */
	while( I2C_use_flag == I2C_USED ) {
		TC_TimeDly(1);
	}
	I2C_use_flag = I2C_USED;

	i2c_register(i2ctgt);

	if( subaddr != (char)I2C_SUBADDR_NOUSE )
	{
		i2c_wr;
		i2c_clk_hi;
		i2c_data_hi;
		delay760(i2c_Tbuf);
	
		i2c_comm_s();
		i2c_out_byte(&SA); // address | write

		if(i2c_get_ack() != i2c_ack) 
		{
			i2c_comm_p();
			I2C_use_flag = I2C_FREE;
			return i2c_err;
		}

		i2c_out_byte((unsigned char*)&subaddr);
		
		if(i2c_get_ack() != i2c_ack) 
		{
			i2c_comm_p();
			I2C_use_flag = I2C_FREE;
			return i2c_err;
		}
	}

	i2c_wr;
	i2c_data_hi;
	i2c_clk_hi;
	delay760(i2c_Tbuf);

	SA = SlaveAddr | I2C_RD ;	
	i2c_comm_s();
	i2c_out_byte(&SA); // Addr | Read
	
	if(i2c_get_ack() != i2c_ack) 
	{
		i2c_comm_p();
		I2C_use_flag = I2C_FREE;
		return i2c_err;
	}
	
	for(i=0;i<bytes;i++)
	{		
		i2c_in_byte(ptr);

		if(i < (bytes-1)) i2c_do_ack(i2c_ack);
		else  i2c_do_ack(i2c_noack);
		
		ptr++;
	}
	
	i2c_comm_p();

	I2C_use_flag = I2C_FREE;
	return i2c_ok;
}

static int i2c_register(unsigned int i2ctgt)
{
	guiI2CTgt = i2ctgt;

	I2C_Vector[I2C_CLK] = I2C_VectorTable[i2ctgt][I2C_CLK];
	I2C_Vector[I2C_DATA] = I2C_VectorTable[i2ctgt][I2C_DATA];

	return 1;
}


/*******************************************************************
 *	CKC functions
 *
 ******************************************************************/
static void IO_CKC_SetPeriClock(void *upAddr, int iP0P1, int iSrc, int iDiv)
{
#if defined(TCC78X)
		if (iDiv)
			iDiv	= HwPCK_EN_EN | iSrc | ((iDiv-1) & 0xFFFF);
		else
			iDiv	= HwPCK_SEL_XTIN | 0x7FF;		// when disabled, change the source clock into XTIN (save power)

		*(volatile unsigned long *)upAddr	= iDiv;
#endif
}

static void IO_CKC_SetI2CClock(int iFreq)
{
	unsigned	div;
//	unsigned	uCPSR;

#if defined(TCC78X)
	//uCPSR	= IO_INT_DisableINT();

	if (iFreq != 0)
		div	= (IO_CKC_Fxin + (iFreq>>1)) / iFreq;
	else
		div	= 0;

	IO_CKC_SetPeriClock((void*)&HwPCK_I2C, 0, HwPCK_I2C_SEL_XIN, div);
	
	//IO_INT_RestoreINT(uCPSR);
#endif
}

/********************************************************************
 *	I2C output functions
 *
*********************************************************************/
/*
static void WaitDoneI2C(void)
{
	int iCnt;

	while (1) 
	{
		if((HwI2CMCH0_SR & HwI2CMCH0_SR_TIP) == 0)
		{
			break;
		}	
	}
	for(iCnt=0;iCnt<7000;iCnt++);
}

static void IIC_START (unsigned int uiDeviceAddr)
{
	BITSET(HwI2CMCH0_CMD, HwI2CMCH0_CMD_IACK_CLR);

	BITCSET(HwI2CMCH0_TXR, 0xFFFFFFFF, uiDeviceAddr);	// Camera I2C Write address
	BITSET(HwI2CMCH0_CMD, (HwI2CMCH0_CMD_STA_EN|HwI2CMCH0_CMD_WR_EN));
	WaitDoneI2C();
	BITSET(HwI2CMCH0_CMD, HwI2CMCH0_CMD_IACK_CLR);
}

static void IIC_STOP(void)
{
	BITSET(HwI2CMCH0_CMD, HwI2CMCH0_CMD_STO_EN);
	BITSET(HwI2CMCH0_CMD, HwI2CMCH0_CMD_IACK_CLR);
	WaitDoneI2C();
}
*/

static void OpenI2C(unsigned int uiFlag)
{
	int iCnt;
	
#if defined(TCC78X)
	IO_CKC_EnableBUS_I2C();
	BITCLR(HwPCK_I2C, HwPCK_I2C_EN_EN);
	IO_CKC_SetI2CClock(41000);		// 196.8Mhz / 48 = 4.1Mhz

	BITSET(HwSWRESET, HwSWRESET_I2C_ON);
	BITCLR(HwSWRESET, HwSWRESET_I2C_ON);

	BITCSET(HwI2CMCH0_PRES, 0xFFFFFFFF, 0x0000003C);	// 4.1Mhz / 61 = 68.33Khz
	BITSET(HwI2CMCH0_CTRL, HwI2CMCH0_CTRL_EN_ON);

	for (iCnt=0;iCnt<10;iCnt++);

	BITCLR(HwI2CMCH0_CTRL, HwI2CMCH0_CTRL_EN_ON);
	BITCLR(HwPCK_I2C, HwPCK_I2C_EN_EN);
	IO_CKC_DisableBUS_I2C();
#endif
#if defined(TCC83XX)
	if(uiFlag == I2C_CODEC)
	{
		HwGSEL_E |= HwGSEL_E_I2C;
		HwNGSEL_E |= (1<<5)|(1<<6);
		HwGIOCON_E |= (1<<5)|(1<<6);
	}
	else if(uiFlag == I2C_TOUCH)
	{
		HwGSEL_F |= HwGSEL_F_I2C;
		HwNGSEL_F |= (1<<14)|(1<<15);
		HwGIOCON_F |= (1<<14)|(1<<15);
	}
#endif
}

/*
static void CloseI2C(void)
{
	IIC_STOP();

	BITCLR(HwI2CMCH0_CTRL, HwI2CMCH0_CTRL_IEN_ON);
	BITCLR(HwI2CMCH0_CTRL, HwI2CMCH0_CTRL_EN_ON);
	BITCLR(HwPCK_I2C, HwPCK_I2C_EN_EN);
	IO_CKC_DisableBUS_I2C();
}
*/

static int	DDI_I2C_Initialize(unsigned int uiFlag)
{
	OpenI2C(uiFlag);
	
	return 0;
}

/*
static int	DDI_I2C_Terminate(void)
{
	CloseI2C();
	
	return 0;
}
*/


static int	DDI_I2C_Write(unsigned int uiFlag, unsigned int uiHbyte, unsigned int uiLbyte)
{
//	I2C_SW_WRITE_t	*I2C_SwWrite;
	int iReturn = 0;
	unsigned char ucSzDataBuff[2];//,  ucSize;
	char  cSubAddr;

	if(uiFlag == I2C_CODEC)
	{
		cSubAddr = (char)uiHbyte;
		ucSzDataBuff[0] = (char)uiLbyte;
#if defined(TCC78X)
		BITCSET(HwPORTCFG0, HwPORTCFG0_GPA0_1,HwPORTCFG0_GPA0_0);
#endif
		iReturn = i2c_write(uiFlag, (unsigned char)codec_slaveaddr, cSubAddr, ucSzDataBuff, 1 );
	}
	else if(uiFlag == I2C_TOUCH)
	{
		cSubAddr = (char)I2C_SUBADDR_NOUSE;
#if defined(TCC78X)
		BITCSET(HwPORTCFG0, HwPORTCFG0_GPA0_1,HwPORTCFG0_GPA0_0);
#endif
		iReturn = i2c_write(uiFlag, (unsigned char)touch_slaveaddr, cSubAddr, (unsigned char*)uiLbyte, 1 );
	}
	
	return 0;
}

static int	DDI_I2C_Read(unsigned int uiFlag, unsigned int uiHbyte, unsigned int uiLbyte)
{
//	I2C_SW_WRITE_t	*I2C_SwWrite;
	int iReturn = 0;
	unsigned char  ucSize, *ptrDest;
	char  cSubAddr;

	if(uiFlag == I2C_TOUCH)
	{
		cSubAddr = (char)I2C_SUBADDR_NOUSE;
		ptrDest = (unsigned char *)uiLbyte;
		ucSize = (unsigned char)2;
		
#if defined(TCC78X)
		BITCSET(HwPORTCFG0, HwPORTCFG0_GPA0_1,HwPORTCFG0_GPA0_0);
#endif
		iReturn = i2c_read(uiFlag, (unsigned char)touch_slaveaddr,	cSubAddr, ptrDest, ucSize);
	}
	
	return iReturn;
}

//***********************************************************
void CODEC_IIC_INIT(void)
{
	DDI_I2C_Initialize(I2C_CODEC);
	i2c_init();
}

void CODEC_SEND_CMD(unsigned char hb, unsigned char lb)
{
	DDI_I2C_Write(I2C_CODEC, hb, lb);
	TC_TimeDly(1);
}


/* end of file */

