
/****************************************************************************
 *   FileName    : tca_wm8731control.c
 *   Description : 
 ****************************************************************************
 *
 *   TCC Version 1.0
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
 ****************************************************************************/
/*****************************************************************************
*
* includes
*
******************************************************************************/
#ifdef __KERNEL__
#include <asm/plat-tcc/tca_errorcode.h>
#else
#include "tca_errorcode.h"
#endif

#include "tca_wm8731control.h"

/*****************************************************************************
* Function Name : tca_wm8731_initcodec()
******************************************************************************
* Desription    : send wm8731 init command
* Parameter     : void
* Return        : success(SUCCESS) 
******************************************************************************/

unsigned int tca_wm8731_initcodec(void * hI2C)
{

    tea_wave_i2c_write( hI2C, 0x1E,0x00);  // reset
	
    tea_wave_i2c_write( hI2C, 0x00,0x97);  // Llinein register control
    tea_wave_i2c_write( hI2C, 0x02,0x97);  // Rlinein register control
    tea_wave_i2c_write( hI2C, 0x04,0x80);
    tea_wave_i2c_write( hI2C, 0x06,0x80);
    tea_wave_i2c_write( hI2C, 0x08,0x02);  // Lineout select
    tea_wave_i2c_write( hI2C, 0x0A,0x01);  // Highpass filter Enable
    //tea_wave_i2c_write(0x0C,0xFE);  // Powerdown control
    tea_wave_i2c_write( hI2C, 0x0E,0x02);    // Slave MCLK of CODEC
    //tea_wave_i2c_write( hI2C, 0x0E,0x42);  // Master MCLK of CODEC
    tea_wave_i2c_write( hI2C, 0x10,0x20);
    tea_wave_i2c_write( hI2C, 0x12,0x01);  // Active
    tea_wave_i2c_write( hI2C, 0x0A,0x00);  //CODEC_Mute(0);

    tea_wave_i2c_write( hI2C, 0x08,0x12);  // Lineout select
    tea_wave_i2c_write( hI2C, 0x0C,0x66);  // Powerdown control

	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_wm8731_setinputoutputmode()
******************************************************************************
* Desription    : send wm8731 input and output mode
* Parameter     : void
* Return        : success(SUCCESS) 
******************************************************************************/
 //Line In record or MIC record option
unsigned int tca_wm8731_setinputoutputmode(void *hI2C, unsigned int nMode)
{
	tea_wave_i2c_write( hI2C, 0x0A,0x00);  // mute off

#ifdef _LINE_IN_RECORD_
		tea_wave_i2c_write( hI2C, 0x00,0x1F);  // Llinein register control
		tea_wave_i2c_write( hI2C, 0x02,0x1F);  // Rlinein register control
		tea_wave_i2c_write( hI2C, 0x08,0x12);  // Lineout select
		tea_wave_i2c_write( hI2C, 0x0C,0x62);  // Powerdown control
#else
		tea_wave_i2c_write( hI2C, 0x00,0x9F);  // Llinein register control
		tea_wave_i2c_write( hI2C, 0x02,0x9F);  // Rlinein register control
		tea_wave_i2c_write( hI2C, 0x04,0xF9);  // HP Left Output
		tea_wave_i2c_write( hI2C, 0x06,0xF9);  // HP Right Output
#if defined(_BT_MODE_)
		tea_wave_i2c_write( hI2C, 0x08,0x14);  // Record monitering On
#else
		tea_wave_i2c_write( hI2C, 0x08,0x34);  // Record monitering On
#endif
		tea_wave_i2c_write( hI2C, 0x0C,0x60);  // Powerdown control
#endif
	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_wm8731_setinputmode()
******************************************************************************
* Desription    : send wm8731 input mode
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/
 //Line In record or MIC record option
unsigned int tca_wm8731_setinputmode(void *hI2C )
{
#ifdef _LINE_IN_RECORD_
		tea_wave_i2c_write( hI2C, 0x0A,0x00);  // mute off
		tea_wave_i2c_write( hI2C, 0x00,0x1F);  // Llinein register control
		tea_wave_i2c_write( hI2C, 0x02,0x1F);  // Rlinein register control
		tea_wave_i2c_write( hI2C, 0x08,0x18);  // Linein select
		tea_wave_i2c_write( hI2C, 0x0C,0x62);  // Powerdown control
#else
		tea_wave_i2c_write( hI2C, 0x0A,0x00);  // mute off
		tea_wave_i2c_write( hI2C, 0x00,0x97);  // Llinein register control
		tea_wave_i2c_write( hI2C, 0x02,0x97);  // Rlinein register control
		tea_wave_i2c_write( hI2C, 0x04,0xF9);  // HP Left Output
		tea_wave_i2c_write( hI2C, 0x06,0xF9);  // HP Right Output
#if defined(_BT_MODE_)
		tea_wave_i2c_write( hI2C, 0x08,0x14);  // Record monitering On
#else
		tea_wave_i2c_write( hI2C, 0x08,0x24);  // Record monitering On
#endif
		tea_wave_i2c_write( hI2C, 0x0C,0x68);  // Powerdown control
		tea_wave_i2c_write( hI2C, 0x0A,0x08);  // DA Path
#endif
	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_wm8731_setoutputmode()
******************************************************************************
* Desription    : send wm8731 gain control command
* Parameter     : nMode // FMR output Mode ON or OFF
* Return        : success(SUCCESS) 
******************************************************************************/

unsigned int tca_wm8731_setoutputmode(void *hI2C, unsigned int nMode)
{

	tea_wave_i2c_write( hI2C, 0x0A,0x00);  // mute off
	if(nMode) // using FMR output
	{
		tea_wave_i2c_write( hI2C, 0x00,0x1F);  // Llinein register control
		tea_wave_i2c_write( hI2C, 0x02,0x1F);  // Rlinein register control
		tea_wave_i2c_write( hI2C, 0x08,0x18);  // Linein select , bypass on/DAC On
		tea_wave_i2c_write( hI2C, 0x0C,0x62);  // Powerdown control , Line in/ Mic Off/ ADC on/DAC on/output on/
	}
	else	//not using FMR output
	{
		tea_wave_i2c_write( hI2C, 0x00,0x9F);  // Llinein register control
		tea_wave_i2c_write( hI2C, 0x02,0x9F);  // Rlinein register control
		tea_wave_i2c_write( hI2C, 0x08,0x12);  // Lineout select
		tea_wave_i2c_write( hI2C, 0x0C,0x66);  // Powerdown control
	}
	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_wm8731_setnormalmode()
******************************************************************************
* Desription    : send wm8731 gain control command
* Parameter     : 
* Return        : success(SUCCESS) 
******************************************************************************/

unsigned int tca_wm8731_setnormalmode(void * hI2C)
{
	tea_wave_i2c_write( hI2C, 0x08,0x12);  // Lineout select
	tea_wave_i2c_write( hI2C, 0x0C,0x66);  // Powerdown control
	// tea_wave_i2c_write( hI2C, 0x0A,0x08); //mute on
	return SUCCESS;	
}

/*****************************************************************************
* Function Name : tca_wm8731_setoutputmute()
******************************************************************************
* Desription    : send wm8731 gain control command
* Parameter     : nMode Mute on or off
* Return        : success(SUCCESS) 
******************************************************************************/

unsigned int tca_wm8731_setoutputmute(void *hI2C, unsigned int nMode)
{
	if(nMode) //mute on
		tea_wave_i2c_write( hI2C, 0x0A,0x08);	
	else      //mute off
		tea_wave_i2c_write( hI2C, 0x0A,0x00);	
	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_wm8731_setinputmute()
******************************************************************************
* Desription    : send wm8731 gain control command
* Parameter     : nMode Mute on or off
* Return        : success(SUCCESS) 
******************************************************************************/

unsigned int tca_wm8731_setinputmute(void *hI2C, unsigned int nMode)
{
	if(nMode) //mute on
	{
		tea_wave_i2c_write( hI2C, 0x00,0x80);
		tea_wave_i2c_write( hI2C, 0x02,0x80);
	}
	else      //mute off
	{
		tea_wave_i2c_write( hI2C, 0x00,0);
		tea_wave_i2c_write( hI2C, 0x02,0);
	}
	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_wm8731_setinputgain()
******************************************************************************
* Desription    : send wm8731 gain control command
* Parameter     : nGain //gain valude
* Return        : success(SUCCESS) 
******************************************************************************/

unsigned int tca_wm8731_setinputgain(void *hI2C, unsigned int nGain)
{
    tea_wave_i2c_write( hI2C, 0x00, nGain);    // LINVol register control
    tea_wave_i2c_write( hI2C, 0x02, nGain);    // RINVol register control

	return SUCCESS;
}

/*****************************************************************************
* Function Name : tca_wm8731_setoutputgain()
******************************************************************************
* Desription    : send wm8731 gain control command
* Parameter     : nGain //gain valude
* Return        : success(SUCCESS) 
******************************************************************************/
unsigned int tca_wm8731_setoutputgain(void *hI2C, unsigned int nGain)
{
	
	unsigned int ucGain;
	unsigned int idx;
	const unsigned char arrVol[80] = 
		{0,13,20,25,29,33,36,38,40,42,		
		44,45,47,48,49,51,52,53,54,55,		
		56,56,57,58,59,59,60,61,61,62,		
		63,63,64,64,65,65,66,66,67,67,		
		68,68,69,69,69,70,70,71,71,71,		
		72,72,72,73,73,73,74,74,74,75,		
		75,75,76,76,76,76,77,77,77,78,		
		78,78,78,79,79,79,79,80,80,80};

	idx	   = nGain*79 / 0xffff;
	ucGain = arrVol[idx] + 0x2f;
	
   	tea_wave_i2c_write( hI2C, 0x04,ucGain);    // LHPVol register control
   	tea_wave_i2c_write( hI2C, 0x06,ucGain);    // RHPVol register control
	
	return SUCCESS;
}

