/*
 * linux/sound/soc/codecs/tcc250_dac.h
 *
 * Author:  <linux@telechips.com>
 * Created: June 10, 2008
 * Description: 
 *
 * Copyright (C) 2008-2009 Telechips 
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see the file COPYING, or write
 * to the Free Software Foundation, Inc.,
 * 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */
#ifndef _TCC250DAC_H_
#define _TCC250DAC_H_

#define	ANA_LDO_INCLUDE
#define	TCC250_CODEC_I2S	// I2S Interface - Undef  Left Justfied (MSB First)
#define MCLK_TO_TCC250_INCLUDE
#define FRAME_TO_TCC250_INCLUDE
#define _TCC250_DVS_INCLUDE

/***********************
Audio Register Mapping
************************/
#define	TCC250_ANA_CTRL_REG			0x20		//LDOANA Regulator Management	Read / Write	0x0C
#define	TCC250_AUTOSTART1_REG		0x10	//Audio Automatic Start Up Sequence	Read / Write	0x00
#define	TCC250_AUTOSTART2_REG		0x11	//Audio Automatic Start Up Sequence	Read / Write	0x0F
#define	TCC250_PUP_OFF_INPUT_REG	0x12	//Audio Power Up-Down ? Inputs	Read / Write	0x50
#define	TCC250_PUP_OFF_OUTPUT_REG	0x13	//Audio Power Up-Down ? Outputs	Read / Write	0x00
#define	TCC250_MIC_CTRL_REG			0x14	//Microphone Control	Read / Write	0x00
#define	TCC250_ISSCTRL_REG			0x15	//Digital Audio Interface Clock Control	Read / Write	0x02
#define	TCC250_FRAME_CTRL_REG		0x16	//Digital Audio Interface Frame Control	Read / Write	0x49
#define	TCC250_LRMUTE_REG			0x17	//Signal Path Muting, Left/Right Channel	Read / Write	0x7F
#define	TCC250_MICVOL_REG			0x18	//Microphone Volume and Mute	Read / Write	0x00
#define	TCC250_LINVOL_REG			0x19	//Left Channel (LINE/AUX) Volume and Mute	Read / Write	0x00
#define	TCC250_RINVOL_REG			0x1A	//Right Channel (LINE) Volume and Mute	Read / Write	0x00
#define	TCC250_HSOUTL_REG			0x1B	//Headset Left Channel Volume and Mute	Read / Write	0x00
#define	TCC250_HSOUTR_REG			0x1C	//Headset Right Channel Volume and Mute	Read / Write	0x00
#define	TCC250_PLAYBACK_REG			0x1D	//Left Channel Sidetone and Bypass Control	Read / Write	0x00
#define	TCC250_PLBCK_REG			0x1E	//Playback Control Parameters	Read / Write	0x10
		
#define	TCC250_ISSCTRL_FRAMEGEN	0x04
#define	TCC250_ISSCTRL_112896	0x02
#define	TCC250_ISSCTRL_MCLKGEN	0x01

#if defined(MCLK_TO_TCC250_INCLUDE) && defined(FRAME_TO_TCC250_INCLUDE)// MCLK,LR,BCLK : TCC8xx --> TCC250
    #define	TCC250_ISSCTRL_VALUE	(TCC250_ISSCTRL_112896) //Full Slave Mode
#endif

#if defined(MCLK_TO_TCC250_INCLUDE) && !defined(FRAME_TO_TCC250_INCLUDE)// MCLK : TCC8xx --> TCC250, LR,BCLK : TCC250 --> TCC8xx
    #define	TCC250_ISSCTRL_VALUE	(TCC250_ISSCTRL_112896|TCC250_ISSCTRL_FRAMEGEN) // Slave/Master Mode
#endif

#if !defined(MCLK_TO_TCC250_INCLUDE) && defined(FRAME_TO_TCC250_INCLUDE)// MCLK : TCC250 --> TCC8xx, LR,BCLK : TCC8xx --> TCC250
    #define	TCC250_ISSCTRL_VALUE	(TCC250_ISSCTRL_112896|TCC250_ISSCTRL_MCLKGEN) //Master/Slave Mode
#endif

#if !defined(MCLK_TO_TCC250_INCLUDE) && !defined(FRAME_TO_TCC250_INCLUDE) // MCLK,LR,BCLK : TCC250 --> TCC8xx
    #define	TCC250_ISSCTRL_VALUE	(TCC250_ISSCTRL_112896|TCC250_ISSCTRL_FRAMEGEN|TCC250_ISSCTRL_MCLKGEN) //Full Master Mode
#endif

extern void TCC250_SEND_CMD(void *handle, unsigned char hb, unsigned char lb);
extern unsigned char TCC250_READ_CMD(void *handle, unsigned char hb);
extern void TCC250_CODEC_Init(void *handle);
extern void TCC250_SetToLDOANA(void *handle, unsigned short mV);
//void TCC250_OutPutMute(unsigned char ucOnOff);
extern void TCC250_InPutMute(void *handle, unsigned char ucOnOff);
extern void TCC250_SoftMute(void *handle, unsigned char ucOnOff);
extern void TCC250_ModeLineIn(void *handle);
extern void TCC250_ModeLineInOut(void *handle);
extern void TCC250_ModePlay(void *handle);

extern void TCC250_ModeAuxIn(void *handle);
extern void TCC250_SetAudioInVolume(void *handle, unsigned char ucVolume);
extern void TCC250_SetAudioOutVolume(void *handle, unsigned char ucVolume);
extern int TCC250_GetAudioOutVolume(void *handle);
extern int TCC250_GetAudioInVolume(void *handle);

#endif

