/****************************************************************************
 *   
 *   Copyright (c) Telechips, Inc.
 *   ALL RIGHTS RESERVED
 *
****************************************************************************/
#include <asm/hardware.h>
#include <linux/delay.h>
#ifdef CONFIG_ARCH_TCC83X
#include <asm/arch/tcc83x_virt.h>
#elif CONFIG_ARCH_TCC79X
#include <asm/arch/tcc79x.h>
#endif
#include "tcc8300_i2c.h"



#define 		I2C_WR		0
#define 		I2C_RD		1

#define I2C_CODEC  0

static char I2C_use_flag;

typedef struct stI2C_Vector
{
	unsigned long int tgtaddr;	
	unsigned long int port;
}tI2C_Vector;

unsigned int	guiI2CTgt;

const tI2C_Vector I2C_VectorTable[I2C_MAX_VECTOR][I2C_SIGNAL_BUS]	= 
{
	{{0xF1005040, Hw6}, {0xF1005040, Hw5}},
	{{0xF1005040, Hw6}, {0xF1005040, Hw5}},
	{{0xF1005040, Hw6}, {0xF1005040, Hw5}},
	{{0xF1005040, Hw6}, {0xF1005040, Hw5}},
};
tI2C_Vector I2C_Vector[I2C_SIGNAL_BUS];

void delay760(unsigned int count)
{
	udelay(5);
/*
	#define DELAYCNT	30
	for(count=DELAYCNT;count>0;count--) {
		;
	}
	#undef	DELAYCNT
*/
}

// start bit
static void i2c_comm_s(void)
{	
	i2c_wr;
	i2c_clk_hi;
	i2c_data_hi;
	delay760(i2c_Tbuf);
		
	i2c_wr;
	i2c_data_lo;
	delay760(i2c_Thold);
	i2c_clk_lo;
	delay760(i2c_Thold);
}

// data out
static void i2c_out_byte(unsigned char *ptr)
{	
	unsigned char i, temp;

	i2c_wr;
	for(i=0;i<8;i++)
	{
		temp = *ptr << i;
		temp &= 0x80;
		if(temp){
			i2c_data_hi;
		}else{
			i2c_data_lo;
		}
		delay760(i2c_Tsetup);
		
		i2c_clk_hi;
		delay760(i2c_Thold);
		
		i2c_clk_lo;
		delay760(i2c_Thold);
	}
}

// get acknowledge
static unsigned char i2c_get_ack(void)
{	
	unsigned char i=0;

	i2c_rd;
	
	i2c_clk_hi;
	delay760(i2c_Tcatch);

	i = (unsigned char)((*(volatile unsigned long int *)I2C_Vector[I2C_DATA].tgtaddr) & (I2C_Vector[I2C_DATA].port)); 	   
	delay760(i2c_Tcatch);
	
	i2c_clk_lo;
	delay760(i2c_Thold);
	
	return i;
}


// Input Byte
static void i2c_in_byte(unsigned char *ptr)
{
	unsigned int i, temp=0;
	
	i2c_rd;

	for(i=0;i<8;i++)
	{
		i2c_clk_hi;
		delay760(i2c_Tcatch);

		temp = (unsigned int)(  ( (*(volatile unsigned long int *)(I2C_Vector[I2C_DATA].tgtaddr)) & (I2C_Vector[I2C_DATA].port))) ? 1 : 0;

		if(i == 0) *ptr = temp;
		else *ptr = (*ptr << 1) | temp;
		delay760(i2c_Tcatch);
		
		i2c_clk_lo;
		delay760(i2c_Thold);
	}
}

// put acknowledge
static void i2c_do_ack(unsigned char bit)
{

	i2c_wr;

	if(bit){
		i2c_data_hi;
	}else{
		i2c_data_lo;
	}
	delay760(i2c_Tsetup);
		
	i2c_clk_hi;
	delay760(i2c_Thold);
	
	i2c_clk_lo;
	delay760(i2c_Thold);
}

// stop bit	HwGDATA_A |= CODEC_SDIN_SET;
static void i2c_comm_p(void)
{
	i2c_wr;
	i2c_data_lo;
	delay760(i2c_Tsetup);
	
	i2c_clk_hi;
	delay760(i2c_Thold);
	
	i2c_data_hi;
	delay760(i2c_Tbuf);
}

void alsa_i2c_init(void)
{
	// transfer rate setting... 8bit
	I2C_use_flag = I2C_FREE;
}

void alsa_TC_TimeDly(int ticks){
	int ms;
	while(ticks --){
		ms = 1000000;
		while(ms --);
	}
}

unsigned char alsa_i2c_write(unsigned int i2ctgt, unsigned char SlaveAddr, char subaddr, unsigned char *ptr, unsigned char bytes )
{
	unsigned char SA = SlaveAddr | I2C_WR ;
	unsigned char i;

	//RETAILMSG(TRUE, (TEXT("Wavedev:::alsa_i2c_write\r\n")));
	
	/* for prevent from access on same time */
	while( I2C_use_flag == I2C_USED ) {
		alsa_TC_TimeDly(1);
	}
	I2C_use_flag = I2C_USED;

	alsa_i2c_register(i2ctgt);

	i2c_wr;
	i2c_clk_hi;
	i2c_data_hi;
	delay760(i2c_Tbuf);
	
	i2c_comm_s();
	i2c_out_byte(&SA); // slave address | write

	if(i2c_get_ack() != i2c_ack) 
	{
		i2c_comm_p();
		I2C_use_flag = I2C_FREE;
		return i2c_err;
	}

	if( subaddr != (char)I2C_SUBADDR_NOUSE )
	{
		i2c_out_byte((unsigned char*)&subaddr); // address

		if(i2c_get_ack() != i2c_ack) 
		{
			i2c_comm_p();
			I2C_use_flag = I2C_FREE;
			return i2c_err;
		}
	}

	for(i=0; i<bytes ; i++)
	{		
		i2c_out_byte(ptr);
		
		if(i2c_get_ack()) ;
		else		  ;
		
		ptr++;
	}
	
	i2c_comm_p();
	
	I2C_use_flag = I2C_FREE;
	return i2c_ok;
}

unsigned char alsa_i2c_read(unsigned int i2ctgt, unsigned char SlaveAddr, char subaddr, unsigned char *ptr, unsigned char bytes)
{
	unsigned char i;
	unsigned char SA = SlaveAddr | I2C_WR ;

	/* for prevent from access on same time */
	while( I2C_use_flag == I2C_USED ) {
		alsa_TC_TimeDly(1);
	}
	I2C_use_flag = I2C_USED;

	alsa_i2c_register(i2ctgt);

	if( subaddr != (char)I2C_SUBADDR_NOUSE )
	{
		i2c_wr;
		i2c_clk_hi;
		i2c_data_hi;
		delay760(i2c_Tbuf);
	
		i2c_comm_s();
		i2c_out_byte(&SA); // address | write

		if(i2c_get_ack() != i2c_ack) 
		{
			i2c_comm_p();
			I2C_use_flag = I2C_FREE;
			return i2c_err;
		}

		i2c_out_byte((unsigned char*)&subaddr);
		
		if(i2c_get_ack() != i2c_ack) 
		{
			i2c_comm_p();
			I2C_use_flag = I2C_FREE;
			return i2c_err;
		}
	}

	i2c_wr;
	i2c_data_hi;
	i2c_clk_hi;
	delay760(i2c_Tbuf);

	SA = SlaveAddr | I2C_RD ;	
	i2c_comm_s();
	i2c_out_byte(&SA); // Addr | Read
	
	if(i2c_get_ack() != i2c_ack) 
	{
		i2c_comm_p();
		I2C_use_flag = I2C_FREE;
		return i2c_err;
	}
	
	for(i=0;i<bytes;i++)
	{		
		i2c_in_byte(ptr);

		if(i < (bytes-1)) i2c_do_ack(i2c_ack);
		else  i2c_do_ack(i2c_noack);
		
		ptr++;
	}
	
	i2c_comm_p();

	I2C_use_flag = I2C_FREE;
	return i2c_ok;
}

int alsa_i2c_register(unsigned int i2ctgt)
{
	guiI2CTgt = i2ctgt;

	I2C_Vector[I2C_CLK] = I2C_VectorTable[i2ctgt][I2C_CLK];
	I2C_Vector[I2C_DATA] = I2C_VectorTable[i2ctgt][I2C_DATA];

	return 1;
}

int	alsa_DDI_I2C_Initialize(void)
{	
	return 0;
}

int	alsa_DDI_I2C_Terminate(void)
{	
	return 0;
}


int	alsa_DDI_I2C_Write(unsigned int uiFlag, unsigned int uiHbyte, unsigned int uiLbyte)
{
//	I2C_SW_WRITE_t	*I2C_SwWrite;
	int iReturn = 0;
	unsigned char ucSzDataBuff[2];//,  ucSize;
	char  cSubAddr;
        						
	if(uiFlag == I2C_CODEC)
	{
		cSubAddr = (char)uiHbyte;
		ucSzDataBuff[0] = (char)uiLbyte;
		BITSET(HwNGSEL_E, HwGSEL_E_GPIO5|HwGSEL_E_GPIO6);        
		iReturn = alsa_i2c_write(uiFlag, (unsigned char)codec_slaveaddr, cSubAddr, ucSzDataBuff, 1 );
	}
       else if(uiFlag == I2C_CAMERA)
       {
	      cSubAddr = (char)I2C_SUBADDR_NOUSE;
          ucSzDataBuff[0] = (char)uiLbyte;
          BITSET(HwNGSEL_E, HwGSEL_E_GPIO5|HwGSEL_E_GPIO6);        
	      iReturn = alsa_i2c_write(uiFlag, (unsigned char)0xBA, cSubAddr, ucSzDataBuff, 1 );        
       }
    
	
	return 0;
}

int	alsa_DDI_I2C_Read(unsigned int uiFlag, unsigned int uiHbyte, unsigned int uiLbyte)
{
	return 0;
}


//***********************************************************
void alsa_CODEC_SEND_CMD(unsigned char hb, unsigned char lb)
{
	alsa_DDI_I2C_Write(I2C_CODEC, hb, lb);
	alsa_TC_TimeDly(1);
}

/* end of file */

